@@ -10,13 +10,12 @@
 #endif
 
 #include <gtk/gtk.h>
-
 #include "gettext_macros.h"
 
 /**
  * Program version number.
  */
-#define VERS "1.9.0"
+#define VERS "1.9.2"
 
 /** Home dir name */
 #define HOMEDIRNAME ".bygfoot-1.9"
@@ -38,6 +37,7 @@
 #define cup_id_new (counters[COUNT_CUP_ID]++)
 #define league_id_new (counters[COUNT_LEAGUE_ID]++)
 #define fixture_id_new (counters[COUNT_FIX_ID]++)
+#define lg_commentary_id_new (counters[COUNT_LG_COMM_ID]++)
 
 /** Convenience abbreviation. */
 #define ligs country.leagues
@@ -62,7 +62,7 @@
 #define stat4 status[4]
 #define stat5 status[5]
 
-#define debug opt_int("int_opt_debug")
+#define debug const_int("int_debug")
 
 /**
  * Exit codes.
@@ -70,6 +70,7 @@
 enum ExitCodes
 {
     EXIT_OK = 0, /**< Normal exit. */
+    EXIT_GENERAL, /**< A general, multi-purpose exit code. */
     EXIT_NO_COUNTRY_FILES, /**< Exit when no country files have been found. */
     EXIT_FILE_OPEN_FAILED, /**< A file could not be opened. */
     EXIT_PRINT_ERROR, /**< Exit when the print_error function is called on a set error.*/
@@ -81,8 +82,12 @@ enum ExitCodes
     EXIT_OPTION_NOT_FOUND, /**< An option couldn't be found. */
     EXIT_POINTER_NOT_FOUND, /**< We didn't find a pointer needed. */
     EXIT_INT_NOT_FOUND, /**< We didn't find an integer value (mostly indices). */
+    EXIT_FILE_NOT_FOUND, /**< Some file couldn't be found. */
     EXIT_NO_SUPPORT_DIR, /**< No support directory found. */
     EXIT_CUP_ROUND_ERROR, /**< Too few cup rounds. */
+    EXIT_LOAD_TEAM_DEF, /**< Error loading a team definition file. */
+    EXIT_DEF_SORT, /**< Error sorting a team after loading the definition. */
+    EXIT_PROM_REL, /**< Error when executing the promotion/relegation stuff. */
     EXIT_END
 };
 
@@ -91,7 +96,7 @@ enum ExitCodes
  */
 typedef struct
 {
-    GString *name, /**< Name of the country. */
+    gchar *name, /**< Name of the country. */
 	*symbol, /**< Symbol of the country, eg a flag pixmap. */
 	*sid; /**< Id of the country, eg 'england'. */
 
@@ -120,9 +125,12 @@ typedef struct
 	*file_chooser,
 	*contract,
 	*menu_player,
+	*menu_youth,
 	*user_management,
 	*wdebug,
 	*help,
-	*transfer_dialog;
+	*transfer_dialog,
+	*sponsors,
+	*mmatches;
 } Windows;
 #endif
