@@ -4,6 +4,7 @@
 #include "finance.h"
 #include "fixture.h"
 #include "game_gui.h"
+#include "gui.h"
 #include "league.h"
 #include "live_game.h"
 #include "maths.h"
@@ -72,16 +73,16 @@ callback_player_clicked(gint idx, GdkEventButton *event)
     /* Only accept single-clicks right now. */
     if(event->type != GDK_BUTTON_PRESS)
 	return;
-    
+
     if(event->button == 1)
     {
-	if(selected_row[0] == -1)
+	if(selected_row == -1)
 	{
-	    selected_row[0] = idx;
+	    selected_row = idx;
 	    return;
 	}
 
-	player_swap(current_user.tm, selected_row[0],
+	player_swap(current_user.tm, selected_row,
 		    current_user.tm, idx);
 	if(opt_user_int("int_opt_user_swap_adapts") == 1 &&
 	   current_user.tm->structure !=
@@ -94,7 +95,7 @@ callback_player_clicked(gint idx, GdkEventButton *event)
 
 	game_gui_write_av_skills();
 
-	selected_row[0] = -1;
+	selected_row = -1;
 
 	treeview_show_user_player_list();
 	if(stat0 == STATUS_MAIN)
@@ -104,13 +105,18 @@ callback_player_clicked(gint idx, GdkEventButton *event)
     {
 	if(stat0 == STATUS_SHOW_TRANSFER_LIST)
 	{
-	    selected_row[0] = -1;
+	    selected_row = -1;
 	    transfer_add_remove_user_player(player_of_idx_team(current_user.tm, idx));
 	}
+	else if(stat0 == STATUS_SHOW_YA)
+	{
+	    selected_row = idx;
+	    on_menu_move_to_youth_academy_activate(NULL, NULL);
+	}
 	else
 	{
+	    selected_row = idx;
 	    window_show_menu_player((GdkEvent*)event);
-	    selected_row[0] = idx;
 	}
     }
 
@@ -120,7 +126,7 @@ callback_player_clicked(gint idx, GdkEventButton *event)
 /** Show the last match of the current user. 
     @param start Whether we start the replay from the beginning or continue it. */
 void
-callback_show_last_match(gboolean start)
+callback_show_last_match(gboolean start, LiveGame *lg)
 {
     gint i;
 
@@ -129,16 +135,13 @@ callback_show_last_match(gboolean start)
     if(start)
     {
 	stat2 = cur_user;
-	statp = &current_user.live_game;
+	statp = lg;
 
 	window_create(WINDOW_LIVE);
 
-	current_user.live_game.fix = 
-	    fixture_from_id(current_user.live_game.fix_id);
+	gui_set_sensitive_lg_meters(FALSE);
 
-	treeview_show_game_stats(GTK_TREE_VIEW(lookup_widget(window.live, "treeview_stats")),
-				 &current_user.live_game);
-	live_game_set_match(&current_user.live_game);
+	treeview_show_game_stats(GTK_TREE_VIEW(lookup_widget(window.live, "treeview_stats")), lg);
     }
     else
     {
@@ -146,9 +149,9 @@ callback_show_last_match(gboolean start)
 	gtk_widget_set_sensitive(lookup_widget(window.live, "button_resume"), FALSE);
     }
 
-    for(i=stat3;i<current_user.live_game.units->len;i++)
+    for(i=stat3;i<lg->units->len;i++)
     {
-	game_gui_live_game_show_unit(&g_array_index(current_user.live_game.units, LiveGameUnit, i));
+	game_gui_live_game_show_unit(&g_array_index(lg->units, LiveGameUnit, i));
 
 	if(stat4 == STATUS_SHOW_LAST_MATCH_PAUSE ||
 	   stat4 == STATUS_SHOW_LAST_MATCH_ABORT)
@@ -248,28 +251,30 @@ callback_show_tables(gint type)
     gint clid = -1;
 
     if(type == SHOW_CURRENT)
-	clid = current_user.tm->clid;
+	clid = team_get_table_clid(current_user.tm);
     else if(type == SHOW_NEXT_LEAGUE)
-    {
-	clid = league_cup_get_next_clid(stat1);
-	while(clid >= ID_CUP_START && cup_has_tables(clid) == -1)
-	    clid = league_cup_get_next_clid(clid);
-    }
+	clid = league_cup_get_next_clid(stat1, FALSE);
     else if(type == SHOW_PREVIOUS_LEAGUE)
-    {
-	clid = league_cup_get_previous_clid(stat1);
-	while(clid >= ID_CUP_START && cup_has_tables(clid) == -1)
-	    clid = league_cup_get_previous_clid(clid);
-    }
+	clid = league_cup_get_previous_clid(stat1, FALSE);
     else
     {
 	g_warning("callback_show_tables: unknown type %d \n", type);
 	return;
     }
 
-    stat1 = clid;    
+    while((clid < ID_CUP_START && !league_from_clid(clid)->active) ||
+	  (clid >= ID_CUP_START && cup_has_tables(clid) == -1))
+    {
+	if(type == SHOW_PREVIOUS_LEAGUE)
+	    clid = league_cup_get_previous_clid(clid, FALSE);
+	else
+	    clid = league_cup_get_next_clid(clid, FALSE);
+    }
 
-    treeview_show_table(GTK_TREE_VIEW(lookup_widget(window.main, "treeview_right")), clid);
+    stat1 = clid;
+
+    treeview_show_table(GTK_TREE_VIEW(lookup_widget(window.main, "treeview_right")), 
+			clid);
 }
 
 /** Open the digits window to get a loan. */
@@ -287,7 +292,7 @@ callback_get_loan(void)
     }
 
     stat1 = STATUS_GET_LOAN;
-    misc_print_grouped_int(max_loan, buf2, FALSE);
+    misc_print_grouped_int(max_loan, buf2);
     sprintf(buf, _("You can take out at most %s."), buf2);
 
     window_show_digits(buf, _("Loan"), max_loan, NULL, 0);
@@ -313,7 +318,7 @@ callback_pay_loan(void)
     }
 
     stat1 = STATUS_PAY_LOAN;
-    misc_print_grouped_int(max_payback, buf2, FALSE);
+    misc_print_grouped_int(max_payback, buf2);
     sprintf(buf, _("You can pay back at most %s"), buf2);
 
     window_show_digits(buf, _("Payback"), max_payback, NULL, 0);
@@ -340,22 +345,22 @@ callback_transfer_list_user(gint button, gint idx)
 	if(trans(idx).offers->len == 0)
 	    game_gui_print_message(_("There are no offers for the player."));
 	else if(trans(idx).offers->len > 0 &&
-		!transoff(idx, 0).accepted)
-	    game_gui_print_message(_("There are some offers for the player which you will see next week."));
+		transoff(idx, 0).status != TRANSFER_OFFER_ACCEPTED)
+	    game_gui_print_message(_("There are some offers for the player which you rejected or will see next week."));
 	else
 	{
-	    misc_print_grouped_int(transoff(idx, 0).fee, buf2, FALSE);
+	    misc_print_grouped_int(transoff(idx, 0).fee, buf2);
 	    misc_print_grouped_int(ABS(transoff(idx, 0).fee -
 				       player_of_id_team(current_user.tm,
-							 trans(idx).id)->value), buf3, FALSE);
+							 trans(idx).id)->value), buf3);
 	    if(transoff(idx, 0).fee -
 	       player_of_id_team(current_user.tm, trans(idx).id)->value > 0)
 		strcat(buf3, _(" more"));
 	    else
 		strcat(buf3, _(" less"));
 		
-	    sprintf(buf, _("%s would like to buy %s. They offer %s for him, which is %s than the player's value. Do you accept?"), transoff(idx, 0).tm->name->str,
-		    player_of_id_team(current_user.tm, trans(idx).id)->name->str,
+	    sprintf(buf, _("%s would like to buy %s. They offer %s for him, which is %s than the player's value. Do you accept?"), transoff(idx, 0).tm->name,
+		    player_of_id_team(current_user.tm, trans(idx).id)->name,
 		    buf2, buf3);
 	    stat1 = STATUS_TRANSFER_OFFER_USER;
 	    stat2 = idx;
@@ -391,11 +396,11 @@ callback_transfer_list_cpu(gint button, gint idx)
 	return;
     }
 
-    misc_print_grouped_int(transoff(idx, 0).fee, buf2, FALSE);
-    misc_print_grouped_int(transoff(idx, 0).wage, buf3, FALSE);
+    misc_print_grouped_int(transoff(idx, 0).fee, buf2);
+    misc_print_grouped_int(transoff(idx, 0).wage, buf3);
 
     sprintf(buf, _("You offered a transfer fee of %s and a wage of %s for %s. The owners and the player are satisfied with your offer. Do you still want to buy the player?"), 
-	    buf2, buf3, player_of_id_team(trans(idx).tm, trans(idx).id)->name->str);
+	    buf2, buf3, player_of_id_team(trans(idx).tm, trans(idx).id)->name);
     stat1 = STATUS_TRANSFER_OFFER_CPU;
     stat2 = idx;
     window_show_transfer_dialog(buf);
@@ -409,6 +414,7 @@ callback_transfer_list_clicked(gint button, gint idx)
 {
     gchar buf[SMALL];
     Transfer *tr = &trans(idx);
+    gint old_fee, old_wage = -1;
 
     if(tr->tm == current_user.tm)
     {
@@ -416,15 +422,14 @@ callback_transfer_list_clicked(gint button, gint idx)
 	return;
     }
     else if(tr->offers->len > 0 &&
-	    transoff(idx, 0).accepted)
+	    transoff(idx, 0).status == TRANSFER_OFFER_ACCEPTED)
     {
 	if(transoff(idx, 0).tm == current_user.tm)
 	{
 	    if(team_is_user(tr->tm) != -1)
 	    {
-		sprintf(buf, _("User %s didn't consider your offer yet."),
-			user_from_team(tr->tm)->name->str);
-		game_gui_print_message(buf);
+		game_gui_print_message(_("User %s didn't consider your offer yet."),
+			user_from_team(tr->tm)->name);
 	    }
 	    else
 		callback_transfer_list_cpu(button, idx);
@@ -435,13 +440,33 @@ callback_transfer_list_clicked(gint button, gint idx)
 	return;
     }
     
-    sprintf(buf, _("You are making an offer for %s. Your scout's recommendations for value and wage are preset."),
-	    player_of_id_team(tr->tm, tr->id)->name->str);
+    if(week >= transfer_get_deadline())
+    {
+	game_gui_print_message(_("The transfer deadline is over."));
+	return;
+    }
+    
     stat1 = STATUS_SHOW_TRANSFER_LIST;
     stat2 = idx;
+    
+    if(tr->offers->len > 0)
+	transfer_get_previous_offer(tr, current_user.tm, &old_fee, &old_wage);
 
-    window_show_digits(buf, _("Fee"), tr->fee[current_user.scout % 10],
-		       _("Wage"), tr->wage[current_user.scout % 10]);
+    if(old_wage == -1)
+    {
+	sprintf(buf, _("You are making an offer for %s. Your scout's recommendations for fee and wage are preset."),
+		player_of_id_team(tr->tm, tr->id)->name);
+	
+	window_show_digits(buf, _("Fee"), tr->fee[current_user.scout % 10],
+			   _("Wage"), tr->wage[current_user.scout % 10]);
+    }
+    else
+    {
+	sprintf(buf, _("You are making an offer for %s again. Your previous values for fee and wage are preset."),
+		player_of_id_team(tr->tm, tr->id)->name);
+	
+	window_show_digits(buf, _("Fee"), old_fee, _("Wage"), old_wage);
+    }
 }
 
 /** Show the contract window for the player with the specified index. */
@@ -477,7 +502,8 @@ callback_offer_new_contract(gint idx)
 
     window_create(WINDOW_CONTRACT);
 
-    sprintf(buf, _("You are negotiating with %s about a new contract. Pay attention to what you're doing; if you don't come to terms with him within %d offers, he's going to leave your team after his current contract expires (unless you sell him).\nYour scout's recommendations are preset:"), pl->name->str,
+    sprintf(buf, _("You are negotiating with %s about a new contract. Pay attention to what you're doing; if you don't come to terms with him within %d offers, he's going to leave your team after his current contract expires (unless you sell him). You may only abort BEFORE making the first offer.\nYour scout's recommendations are preset:"), 
+	    pl->name,
 	    const_int("int_contract_max_offers"));
     gtk_label_set_text(GTK_LABEL(lookup_widget(window.contract, "label_contract")), buf);
 
@@ -516,15 +542,15 @@ callback_show_team(gint type)
     {
 	if(type == SHOW_NEXT_LEAGUE)
 	{
-	    stat2 = league_cup_get_next_clid(stat2);
+	    stat2 = league_cup_get_next_clid(stat2, TRUE);
 	    while(stat2 >= ID_CUP_START && cup_from_clid(stat2)->teams->len == 0)
-		stat2 = league_cup_get_next_clid(stat2);
+		stat2 = league_cup_get_next_clid(stat2, TRUE);
 	}
 	else if(type == SHOW_PREVIOUS_LEAGUE)
 	{
-	    stat2 = league_cup_get_previous_clid(stat2);
+	    stat2 = league_cup_get_previous_clid(stat2, TRUE);
 	    while(stat2 >= ID_CUP_START && cup_from_clid(stat2)->teams->len == 0)
-		stat2 = league_cup_get_previous_clid(stat2);
+		stat2 = league_cup_get_previous_clid(stat2, TRUE);
 	}
 
 	if(stat2 < ID_CUP_START)
@@ -575,10 +601,14 @@ callback_show_player_list(gint type)
 	    stat1 = current_user.tm->clid;
 	    break;
 	case SHOW_NEXT_LEAGUE:
-	    stat1 = league_cup_get_next_clid(stat1);
+	    stat1 = league_cup_get_next_clid(stat1, TRUE);
+	    while(stat1 >= ID_CUP_START && cup_from_clid(stat1)->teams->len == 0)
+		stat1 = league_cup_get_next_clid(stat1, TRUE);
 	    break;
 	case SHOW_PREVIOUS_LEAGUE:
-	    stat1 = league_cup_get_previous_clid(stat1);
+	    stat1 = league_cup_get_previous_clid(stat1, TRUE);
+	    while(stat1 >= ID_CUP_START && cup_from_clid(stat1)->teams->len == 0)
+		stat1 = league_cup_get_previous_clid(stat1, TRUE);
 	    break;
     }
 
@@ -596,9 +626,9 @@ callback_fire_player(gint idx)
     stat2 = idx;
     stat3 = (gint)rint(pl->wage * const_float("float_player_fire_wage_factor") * pl->contract);
 
-    misc_print_grouped_int(stat3, buf2, FALSE);
+    misc_print_grouped_int(stat3, buf2);
 
-    sprintf(buf, _("You want to fire %s. Since his contract expires in %.1f years, he demands a compensation of %s. Do you accept?"), pl->name->str, pl->contract, buf2);
+    sprintf(buf, _("You want to fire %s. Since his contract expires in %.1f years, he demands a compensation of %s. Do you accept?"), pl->name, pl->contract, buf2);
 
     window_show_yesno(buf);
 }
@@ -616,16 +646,19 @@ callback_show_league_stats(gint type)
 	    break;
 	case SHOW_CURRENT:
 	    stat1 = current_user.tm->clid;
+	    while(stat1 >= ID_CUP_START ||
+		  !league_from_clid(stat1)->active)
+		stat1 = league_cup_get_next_clid(stat1, FALSE);
 	    break;
 	case SHOW_NEXT_LEAGUE:
-	    stat1 = league_cup_get_next_clid(stat1);
+	    stat1 = league_cup_get_next_clid(stat1, FALSE);
 	    while(stat1 >= ID_CUP_START)
-		stat1 = league_cup_get_next_clid(stat1);
+		stat1 = league_cup_get_next_clid(stat1, FALSE);
 	    break;
 	case SHOW_PREVIOUS_LEAGUE:
-	    stat1 = league_cup_get_previous_clid(stat1);
+	    stat1 = league_cup_get_previous_clid(stat1, FALSE);
 	    while(stat1 >= ID_CUP_START)
-		stat1 = league_cup_get_previous_clid(stat1);
+		stat1 = league_cup_get_previous_clid(stat1, FALSE);
 	    break;
     }
 
@@ -709,3 +742,29 @@ callback_show_player_team(void)
 
     treeview_show_player_list_team(treeview_right, pl->team, current_user.scout % 10);    
 }
+
+/** Show the youth players of the current user. */
+void
+callback_show_youth_academy(void)
+{
+    gint i;
+    PlayerListAttribute attributes;
+
+    for(i=0;i<PLAYER_LIST_ATTRIBUTE_END;i++)
+	attributes.on_off[i] = 0;
+
+    attributes.on_off[PLAYER_LIST_ATTRIBUTE_NAME] =
+	attributes.on_off[PLAYER_LIST_ATTRIBUTE_POS] =
+	attributes.on_off[PLAYER_LIST_ATTRIBUTE_SKILL] =
+	attributes.on_off[PLAYER_LIST_ATTRIBUTE_FITNESS] =
+	attributes.on_off[PLAYER_LIST_ATTRIBUTE_STATUS] =
+	attributes.on_off[PLAYER_LIST_ATTRIBUTE_AGE] =
+	attributes.on_off[PLAYER_LIST_ATTRIBUTE_ETAL] = 1;
+
+    if(stat0 != STATUS_SHOW_YA)
+	game_gui_print_message(_("Right click to move players to and from the youth academy; left click for context menu."));
+
+    treeview_show_player_list(GTK_TREE_VIEW(lookup_widget(window.main, "treeview_right")),
+			      player_get_pointers_from_array(current_user.youth_academy.players),
+			      attributes, FALSE);
+}
