@@ -2,6 +2,7 @@
 #define CALLBACK_FUNC_H
 
 #include "bygfoot.h"
+#include "live_game_struct.h"
 
 void
 callback_show_next_live_game(void);
@@ -10,7 +11,7 @@ void
 callback_player_clicked(gint idx, GdkEventButton *event);
 
 void
-callback_show_last_match(gboolean start);
+callback_show_last_match(gboolean start, LiveGame *lg);
 
 void
 callback_show_fixtures(gint type);
@@ -57,4 +58,7 @@ callback_show_next_opponent(void);
 void
 callback_show_player_team(void);
 
+void
+callback_show_youth_academy(void);
+
 #endif
