@@ -5,13 +5,16 @@
 #include "game.h"
 #include "game_gui.h"
 #include "gui.h"
+#include "league.h"
 #include "load_save.h"
 #include "main.h"
 #include "option.h"
 #include "player.h"
+#include "table.h"
 #include "team.h"
 #include "transfer.h"
 #include "treeview.h"
+#include "treeview2.h"
 #include "treeview_helper.h"
 #include "user.h"
 #include "window.h"
@@ -33,8 +36,8 @@ on_menu_new_activate                   (GtkMenuItem     *menuitem,
 {
     window_destroy(&window.main, FALSE);
     free_memory();
-
-    main_init(-1, NULL);
+    
+    main_init(NULL, NULL);
     window_show_startup();
     stat0 = STATUS_TEAM_SELECTION;
 }
@@ -44,7 +47,7 @@ void
 on_menu_open_activate                  (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
-    stat1 = STATUS_LOAD_GAME;
+    stat5 = STATUS_LOAD_GAME;
     window_show_file_sel();
 }
 
@@ -54,10 +57,10 @@ on_menu_save_activate                  (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
     if(!opt_int("int_opt_save_will_overwrite") ||
-       strlen(save_file->str) == 0)
+       save_file == NULL)
 	on_menu_save_as_activate(NULL, NULL);
     else
-	load_save_save_game(save_file->str);
+	load_save_save_game(save_file);
 }
 
 
@@ -65,7 +68,7 @@ void
 on_menu_save_as_activate               (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
-    stat1 = STATUS_SAVE_GAME;
+    stat5 = STATUS_SAVE_GAME;
     window_show_file_sel();
 }
 
@@ -142,10 +145,13 @@ void
 on_button_transfers_clicked            (GtkButton       *button,
                                         gpointer         user_data)
 {
-    if(week < transfer_get_deadline())
+    if(sett_int("int_opt_disable_transfers"))
+	game_gui_print_message(_("Transfers are disabled in this country definition."));
+    else if(week < transfer_get_deadline() || transfer_list->len > 0)
     {
 	stat0 = STATUS_SHOW_TRANSFER_LIST;
 	game_gui_print_message(_("Left click to make an offer. Right click to remove offer."));
+	game_gui_print_message_with_delay(_("Transfer deadline is Week %d"), transfer_get_deadline());
 	treeview_show_transfer_list(GTK_TREE_VIEW(lookup_widget(window.main, "treeview_right")));
 	gtk_notebook_set_current_page(GTK_NOTEBOOK(lookup_widget(window.main, "notebook_player")), 1);
 	
@@ -202,9 +208,9 @@ on_player_list1_button_press_event     (GtkWidget       *widget,
     if(treeview_helper_select_row(GTK_TREE_VIEW(widget), event))
 	idx = treeview_helper_get_index(GTK_TREE_VIEW(widget), 0);
 
-    if(idx < 0 || idx - 1 == selected_row[0])
+    if(idx < 0 || idx - 1 == selected_row)
     {
-	selected_row[0] = -1;
+	selected_row = -1;
 	return FALSE;
     }
 
@@ -229,7 +235,7 @@ on_button_browse_forward_clicked       (GtkButton       *button,
 	    callback_show_team(SHOW_NEXT);
 	    break;
 	case STATUS_SHOW_PLAYER_INFO:
-	    selected_row[0] = (selected_row[0] + 1) % current_user.tm->players->len;
+	    selected_row = (selected_row + 1) % current_user.tm->players->len;
 	    on_menu_show_info_activate(NULL, NULL);
 	    break;
 	case STATUS_SHOW_SEASON_HISTORY:
@@ -255,7 +261,7 @@ on_button_browse_back_clicked          (GtkButton       *button,
 	    callback_show_team(SHOW_PREVIOUS);
 	    break;
 	case STATUS_SHOW_PLAYER_INFO:
-	    selected_row[0] = (selected_row[0] == 0) ? current_user.tm->players->len - 1 : selected_row[0] - 1;
+	    selected_row = (selected_row == 0) ? current_user.tm->players->len - 1 : selected_row - 1;
 	    on_menu_show_info_activate(NULL, NULL);
 	    break;
 	case STATUS_SHOW_SEASON_HISTORY:
@@ -273,7 +279,7 @@ on_button_cl_back_clicked              (GtkButton       *button,
 	case STATUS_SHOW_FIXTURES:
 	    callback_show_fixtures(SHOW_PREVIOUS_LEAGUE);
 	    break;
-	case STATUS_SHOW_TABLES:
+	case STATUS_SHOW_TABLES:	    
 	    callback_show_tables(SHOW_PREVIOUS_LEAGUE);
 	    break;
 	case STATUS_BROWSE_TEAMS:
@@ -291,7 +297,6 @@ on_button_cl_back_clicked              (GtkButton       *button,
     }
 }
 
-
 void
 on_button_cl_forward_clicked           (GtkButton       *button,
                                         gpointer         user_data)
@@ -352,6 +357,14 @@ void
 on_menu_tables_activate                (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
+    /* No tables in this country? */
+    if(!query_tables_in_country())
+    {
+	game_gui_print_message(
+	    _("There are no leagues or cups with tables in this country definition."));
+	return;
+    }
+
     stat0 = STATUS_SHOW_TABLES;
     callback_show_tables(SHOW_CURRENT);
 
@@ -363,6 +376,13 @@ void
 on_menu_league_stats_activate          (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
+    if(!query_leagues_active_in_country())
+    {
+	game_gui_print_message(
+	    _("There are no leagues in this country definition (only cups)."));
+	return;	
+    }
+
     stat0 = STATUS_SHOW_LEAGUE_STATS;
     callback_show_league_stats(SHOW_CURRENT);
 
@@ -391,14 +411,16 @@ void
 on_menu_put_on_transfer_list_activate  (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
-    if(selected_row[0] == -1)
+    if(sett_int("int_opt_disable_transfers"))
+	game_gui_print_message(_("Transfers are disabled in this country definition."));
+    else if(selected_row == -1)
 	game_gui_print_message(_("You haven't selected a player."));
-    else if(query_transfer_player_is_on_list(player_of_idx_team(current_user.tm, selected_row[0])))
+    else if(query_transfer_player_is_on_list(player_of_idx_team(current_user.tm, selected_row)))
 	game_gui_print_message(_("The player is already on the list."));
     else
     {
 	setsav0;
-	transfer_add_remove_user_player(player_of_idx_team(current_user.tm, selected_row[0]));
+	transfer_add_remove_user_player(player_of_idx_team(current_user.tm, selected_row));
     }
 }
 
@@ -407,14 +429,14 @@ void
 on_menu_remove_from_transfer_list_activate (GtkMenuItem     *menuitem,
 					    gpointer         user_data)
 {
-    if(selected_row[0] == -1)
+    if(selected_row == -1)
 	game_gui_print_message(_("You haven't selected a player."));
-    else if(!query_transfer_player_is_on_list(player_of_idx_team(current_user.tm, selected_row[0])))
+    else if(!query_transfer_player_is_on_list(player_of_idx_team(current_user.tm, selected_row)))
 	game_gui_print_message(_("The player is not on the list."));
     else
     {
 	setsav0;
-	transfer_add_remove_user_player(player_of_idx_team(current_user.tm, selected_row[0]));
+	transfer_add_remove_user_player(player_of_idx_team(current_user.tm, selected_row));
     }
 }
 
@@ -423,14 +445,14 @@ void
 on_menu_fire_activate                  (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
-    if(selected_row[0] == -1)
+    if(selected_row == -1)
 	game_gui_print_message(_("You haven't selected a player."));
     else if(current_user.tm->players->len == 11)
 	game_gui_show_warning(_("Your team can't have less than 11 players."));
     else
     {
-	callback_fire_player(selected_row[0]);
-	selected_row[0] = -1;
+	callback_fire_player(selected_row);
+	selected_row = -1;
     }
 }
 
@@ -439,11 +461,9 @@ void
 on_menu_shoots_penalties_activate      (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
-    gchar buf[SMALL];
-
-    if(selected_row[0] == -1)
+    if(selected_row == -1)
 	game_gui_print_message(_("You haven't selected a player."));
-    else if(player_of_idx_team(current_user.tm, selected_row[0])->id ==
+    else if(player_of_idx_team(current_user.tm, selected_row)->id ==
 	    opt_user_int("int_opt_user_penalty_shooter"))
     {
 	opt_user_set_int("int_opt_user_penalty_shooter", -1);
@@ -453,16 +473,53 @@ on_menu_shoots_penalties_activate      (GtkMenuItem     *menuitem,
     }
     else
     {
-	sprintf(buf, _("%s will shoot penalties and free kicks when he plays."),
-		player_of_idx_team(current_user.tm, selected_row[0])->name->str);
+	game_gui_print_message(_("%s will shoot penalties and free kicks when he plays."),
+			       player_of_idx_team(current_user.tm, selected_row)->name);
 	opt_user_set_int("int_opt_user_penalty_shooter",
-			 player_of_idx_team(current_user.tm, selected_row[0])->id);
-	game_gui_print_message(buf);
+			 player_of_idx_team(current_user.tm, selected_row)->id);
 	treeview_show_user_player_list();
 	setsav0;
     }
 }
 
+
+void
+on_menu_move_to_youth_academy_activate (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+    Player *pl;
+
+    if(sett_int("int_opt_disable_ya"))
+    {
+	game_gui_print_message(
+	    _("Youth academy is disabled in this country definition."));
+	return;
+    }
+
+    if(selected_row == -1)
+	game_gui_print_message(_("You haven't selected a player."));
+    else
+    {
+	pl = player_of_idx_team(current_user.tm, selected_row);
+
+	if(pl->age > const_float("float_player_age_lower"))
+	    game_gui_print_message(_("The player is too old for the youth academy."));
+	else if(current_user.tm->players->len <= 11)
+	    game_gui_print_message(_("You can't move the player, there are too few players in your team."));
+	else if(current_user.youth_academy.players->len ==
+		const_int("int_youth_academy_max_youths"))
+	    game_gui_print_message(_("There is no room in your youth academy."));
+	else
+	{
+	    player_move_to_ya(selected_row);
+	    treeview_show_user_player_list();
+	    on_menu_show_youth_academy_activate(NULL, NULL);
+	    selected_row = -1;
+	}
+    }
+}
+
+
 void
 on_menu_my_league_results_activate     (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
@@ -474,6 +531,17 @@ on_menu_my_league_results_activate     (GtkMenuItem     *menuitem,
 }
 
 
+
+void
+on_menu_season_results_activate        (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+    stat0 = STATUS_SHOW_SEASON_RESULTS;
+    treeview2_show_season_results();
+    gui_set_arrows();
+}
+
+
 void
 on_menu_browse_teams_activate          (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
@@ -520,7 +588,7 @@ on_treeview_right_button_press_event   (GtkWidget       *widget,
     {
 	case STATUS_SHOW_TRANSFER_LIST:
 	    if(trans(idx - 1).tm == current_user.tm ||
-	       (trans(idx - 1).offers->len > 0 && transoff(idx - 1, 0).accepted) ||
+	       (trans(idx - 1).offers->len > 0 && transoff(idx - 1, 0).status == TRANSFER_OFFER_ACCEPTED) ||
 	       event->button == 1)
 		callback_transfer_list_clicked(event->button, idx - 1);
 	    else if(event->button == 3)
@@ -541,6 +609,13 @@ on_treeview_right_button_press_event   (GtkWidget       *widget,
 	case STATUS_SHOW_PLAYER_LIST:
 	    callback_show_player_team();
 	    break;
+	case STATUS_SHOW_YA:
+	    selected_row = idx - 1;
+	    if(event->button == 3)
+		on_menu_youth_move_to_team_activate(NULL, NULL);
+	    else
+		window_show_menu_youth((GdkEvent*)event);
+	    break;
     }
 
     gui_set_arrows();
@@ -593,9 +668,9 @@ on_menu_custom_structure_activate      (GtkMenuItem     *menuitem,
 
 
 gboolean
-on_menu_team_button_press_event        (GtkWidget       *widget,
-                                        GdkEventButton  *event,
-                                        gpointer         user_data)
+on_menu_team_button_release_event        (GtkWidget       *widget,
+					  GdkEventButton  *event,
+					  gpointer         user_data)
 {
     game_gui_read_radio_items(widget);
     setsav0;
@@ -627,7 +702,7 @@ on_menu_user_show_last_match_activate  (GtkMenuItem     *menuitem,
 
     stat1 = STATUS_SHOW_LAST_MATCH;
     stat3 = 0;
-    callback_show_last_match(TRUE);
+    callback_show_last_match(TRUE, &current_user.live_game);
 }
 
 
@@ -681,7 +756,7 @@ on_eventbox_style_button_press_event   (GtkWidget       *widget,
 
     team_change_attribute_with_message(current_user.tm, TEAM_ATTRIBUTE_STYLE, new_style);
 
-    game_gui_write_meters();
+    game_gui_write_meters(current_user.tm);
     game_gui_write_radio_items();
     
     if(stat0 == STATUS_MAIN)
@@ -712,9 +787,12 @@ on_eventbox_boost_button_press_event   (GtkWidget       *widget,
     else
 	return FALSE;
 
-    team_change_attribute_with_message(current_user.tm, TEAM_ATTRIBUTE_BOOST, new_boost);
+    if(new_boost == 1 && sett_int("int_opt_disable_boost_on"))
+	game_gui_print_message(_("Boost ON is disabled in this country definition."));
+    else
+	team_change_attribute_with_message(current_user.tm, TEAM_ATTRIBUTE_BOOST, new_boost);
 
-    game_gui_write_meters();
+    game_gui_write_meters(current_user.tm);
     game_gui_write_radio_items();
 
     if(stat0 == STATUS_MAIN)
@@ -729,6 +807,12 @@ void
 on_menu_show_finances_activate         (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
+    if(sett_int("int_opt_disable_finances"))
+    {
+	game_gui_print_message(_("Finances are disabled in this country definition."));
+	return;
+    }
+
     stat0 = STATUS_SHOW_FINANCES;
     game_gui_print_message(_("Left-click: get loan; Right-click: pay back; Middle click: stadium window."));
     treeview_show_finances(GTK_TREE_VIEW(lookup_widget(window.main, "treeview_right")),
@@ -742,6 +826,13 @@ void
 on_menu_show_stadium_activate          (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
+    if(sett_int("int_opt_disable_stadium"))
+    {
+	game_gui_print_message(
+	    _("Stadium management is disabled in this country definition."));
+	return;
+    }
+
     window_show_stadium();
 }
 
@@ -761,13 +852,19 @@ void
 on_menu_offer_new_contract_activate    (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
-    if(selected_row[0] == -1)
+    if(sett_int("int_opt_disable_contracts"))
+    {
+	game_gui_print_message(_("Contracts are disabled in this country definition."));
+	return;
+    }
+
+    if(selected_row == -1)
     {
 	game_gui_print_message(_("You haven't selected a player."));
 	return;
     }
 
-    callback_offer_new_contract(selected_row[0]);
+    callback_offer_new_contract(selected_row);
 
     setsav0;
 }
@@ -777,7 +874,7 @@ void
 on_menu_show_info_activate      (GtkMenuItem     *menuitem,
 				 gpointer         user_data)
 {
-    if(selected_row[0] == -1)
+    if(selected_row == -1)
     {
 	game_gui_print_message(_("You haven't selected a player."));
 	return;
@@ -785,7 +882,7 @@ on_menu_show_info_activate      (GtkMenuItem     *menuitem,
 
     if(stat0 != STATUS_LIVE_GAME_PAUSE)
 	stat0 = STATUS_SHOW_PLAYER_INFO;
-    treeview_show_player_info(player_of_idx_team(current_user.tm, selected_row[0]));
+    treeview_show_player_info(player_of_idx_team(current_user.tm, selected_row));
 
     gui_set_arrows();
 }
@@ -834,6 +931,15 @@ on_player_menu_shoots_penalties_activate
     on_menu_shoots_penalties_activate(NULL, NULL);
 }
 
+void
+on_player_menu_move_to_youth_academy_activate
+                                        (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+    on_menu_move_to_youth_academy_activate(NULL, NULL);
+}
+
+
 void
 on_menu_browse_players_activate        (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
@@ -865,7 +971,7 @@ void
 on_menu_load_last_save_activate        (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
-    if(load_save_load_game("last_save"))
+    if(load_save_load_game("last_save", FALSE))
     {
 	cur_user = 0;
 	on_button_back_to_main_clicked(NULL, NULL);
@@ -930,3 +1036,118 @@ on_button_quit_button_press_event      (GtkWidget       *widget,
     return FALSE;
 }
 
+
+void
+on_menu_show_youth_academy_activate    (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+    if(sett_int("int_opt_disable_ya"))
+    {
+	game_gui_print_message(
+	    _("Youth academy is disabled in this country definition."));
+	return;
+    }
+
+    callback_show_youth_academy();
+    stat0 = STATUS_SHOW_YA;
+}
+
+void
+on_menu_set_investment_activate        (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+    if(sett_int("int_opt_disable_ya"))
+    {
+	game_gui_print_message(
+	    _("Youth academy is disabled in this country definition."));
+	return;
+    }
+
+    stat1 = STATUS_SET_YA_PERCENTAGE;
+    window_show_digits(
+	_("Set the percentage of your income you want to devote to your youth academy."),
+	NULL, -1, "%", current_user.youth_academy.percentage);
+}
+
+void
+on_menu_youth_move_to_team_activate    (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+    if(sett_int("int_opt_disable_ya"))
+    {
+	game_gui_print_message(
+	    _("Youth academy is disabled in this country definition."));
+	return;
+    }
+
+    if(current_user.tm->players->len == const_int("int_team_max_players"))
+	game_gui_print_message(_("You can't have more than %d players in the team."),
+			       const_int("int_team_max_players"));
+    else
+    {
+	player_move_from_ya(selected_row);
+	treeview_show_user_player_list();
+	on_menu_show_youth_academy_activate(NULL, NULL);
+	selected_row = -1;
+    }
+}
+
+
+void
+on_menu_youth_kick_out_of_academy_activate
+                                        (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+    if(opt_user_int("int_opt_user_confirm_youth"))
+    {
+	stat1 = STATUS_QUERY_KICK_YOUTH;
+	window_show_yesno(_("Do you really want to kick the poor boy out of your academy?"));
+    }
+    else
+    {
+	free_player(&g_array_index(current_user.youth_academy.players, Player, selected_row));
+	g_array_remove_index(current_user.youth_academy.players, selected_row);
+	on_menu_show_youth_academy_activate(NULL, NULL);
+	selected_row = -1;
+    }
+}
+
+void
+on_mm_add_last_match_activate          (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+    if(current_user.live_game.units->len == 0)
+    {
+	game_gui_show_warning(_("No match stored."));
+	return;
+    }
+
+    if(current_user.mmatches_file == NULL)
+    {
+	stat5 = STATUS_SELECT_MM_FILE_ADD;
+	window_show_file_sel();
+    }
+    else
+	user_mm_add_last_match(FALSE, TRUE);
+}
+
+
+void
+on_mm_manage_matches_activate          (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+    if(current_user.mmatches_file == NULL)
+    {
+	stat5 = STATUS_SELECT_MM_FILE_LOAD;
+	window_show_file_sel();
+    }
+    else
+	window_show_mmatches();
+}
+
+void
+on_menu_save_window_geometry_activate  (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+    window_main_save_geometry();
+}
