@@ -135,7 +135,7 @@ on_player_list1_button_press_event     (GtkWidget       *widget,
                                         gpointer         user_data);
 
 gboolean
-on_menu_team_button_press_event        (GtkWidget       *widget,
+on_menu_team_button_release_event        (GtkWidget       *widget,
                                         GdkEventButton  *event,
                                         gpointer         user_data);
 
@@ -248,7 +248,6 @@ on_menu_reset_players_activate         (GtkMenuItem     *menuitem,
 void
 on_menu_league_stats_activate          (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
-#endif
 
 gboolean
 on_button_quit_button_press_event      (GtkWidget       *widget,
@@ -267,3 +266,46 @@ void
 on_menu_user_show_coming_matches_activate
                                         (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
+
+void
+on_menu_show_youth_academy_activate    (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_menu_set_investment_activate        (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_menu_move_to_youth_academy_activate (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_player_menu_move_to_youth_academy_activate
+                                        (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_menu_youth_move_to_team_activate    (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_menu_youth_kick_out_of_academy_activate
+                                        (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_mm_add_last_match_activate          (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_mm_manage_matches_activate          (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_menu_save_window_geometry_activate  (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_menu_season_results_activate        (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+#endif
