@@ -21,10 +21,10 @@ cup_new(gboolean new_id)
 {
     Cup new;
 
-    new.name = g_string_new("");
-    new.short_name = g_string_new("");
-    new.symbol = g_string_new("");
-    new.sid = g_string_new("");
+    new.name = NULL;
+    new.short_name = NULL;
+    new.symbol = NULL;
+    new.sid = NULL;
 
     new.id = (new_id) ? cup_id_new : -1;
     new.group = -1;
@@ -33,7 +33,7 @@ cup_new(gboolean new_id)
     new.week_gap = 1;
 
     new.yellow_red = 1000;
-    new.skill_diff = 0;
+    new.talent_diff = 0;
 
     new.rounds = g_array_new(FALSE, FALSE, sizeof(CupRound));
     new.teams = g_ptr_array_new();
@@ -55,7 +55,7 @@ cup_choose_team_new(void)
 {
     CupChooseTeam new;
 
-    new.sid = g_string_new("");
+    new.sid = NULL;
     new.number_of_teams = -1;
     new.start_idx = new.end_idx = -1;
     new.randomly = FALSE;
@@ -75,12 +75,14 @@ cup_round_new(void)
     new.home_away = TRUE;
     new.replay = 0;
     new.neutral = FALSE;
+    new.randomise_teams = TRUE;
     new.round_robin_number_of_groups = 0;
-    new.round_robin_number_of_advance = 1;
+    new.round_robin_number_of_advance = 0;
     new.round_robin_number_of_best_advance = 0;
     new.tables = g_array_new(FALSE, FALSE, sizeof(Table));
     new.choose_teams = g_array_new(FALSE, FALSE, sizeof(CupChooseTeam));
     new.teams = g_array_new(FALSE, FALSE, sizeof(Team));
+    new.team_ptrs = g_ptr_array_new();
 
     return new;
 }
@@ -91,7 +93,7 @@ cup_reset(Cup *cup)
 {
     gint i, j;
     
-    g_ptr_array_free(cup->team_names, TRUE);
+    free_gchar_array(&cup->team_names);
     cup->team_names = g_ptr_array_new();
 
     if(cup->teams->len > 0)
@@ -110,14 +112,19 @@ cup_reset(Cup *cup)
 	if(g_array_index(cup->rounds, CupRound, i).tables->len > 0)
 	{
 	    for(j=0;j<g_array_index(cup->rounds, CupRound, i).tables->len;j++)
-		free_table(&g_array_index(g_array_index(cup->rounds, CupRound, i).tables, Table, j));	
+		free_table(&g_array_index(
+			       g_array_index(cup->rounds, CupRound, i).tables, Table, j));	
 	 
 	    g_array_free(g_array_index(cup->rounds, CupRound, i).tables, TRUE);
-	    g_array_index(cup->rounds, CupRound, i).tables = g_array_new(FALSE, FALSE, sizeof(Table));
+	    g_array_index(cup->rounds, CupRound, i).tables = 
+		g_array_new(FALSE, FALSE, sizeof(Table));
 	}
 
 	if(g_array_index(cup->rounds, CupRound, i).teams->len > 0)
 	    free_teams_array(&g_array_index(cup->rounds, CupRound, i).teams, TRUE);
+
+	g_ptr_array_free(g_array_index(cup->rounds, CupRound, i).team_ptrs, TRUE);
+	g_array_index(cup->rounds, CupRound, i).team_ptrs = g_ptr_array_new();
     }
 }
 
@@ -130,7 +137,7 @@ query_cup_choose_team_is_league(const gchar *sid)
     gint i;
 
     for(i=0;i<ligs->len;i++)
-	if(strcmp(lig(i).sid->str, sid) == 0)
+	if(strcmp(lig(i).sid, sid) == 0)
 	    return TRUE;
 
     return FALSE;
@@ -147,14 +154,14 @@ cup_get_choose_team_league_cup(const CupChooseTeam *ct,
     *league = NULL;
     *cup = NULL;
 
-    sscanf(ct->sid->str, "%[^0-9]%d", trash, &idx);
+    sscanf(ct->sid, "%[^0-9]%d", trash, &idx);
 
-    if(g_str_has_prefix(ct->sid->str, "LEAGUE"))
+    if(g_str_has_prefix(ct->sid, "LEAGUE"))
     {
 	*league = &lig(idx - 1);
 	*cup = NULL;
     }
-    else if(g_str_has_prefix(ct->sid->str, "CUP"))
+    else if(g_str_has_prefix(ct->sid, "CUP"))
     {
 	*cup = &cp(idx - 1);
 	*league = NULL;
@@ -162,16 +169,18 @@ cup_get_choose_team_league_cup(const CupChooseTeam *ct,
     else
     {
 	for(i=0;i<ligs->len;i++)
-	    if(strcmp(lig(i).sid->str, ct->sid->str) == 0)
+	{
+	    if(strcmp(lig(i).sid, ct->sid) == 0)
 	    {
 		*league = &lig(i);
 		*cup = NULL;
 		break;
 	    }
+	}
 
 	for(i=0;i<cps->len;i++)
 	{
-	    if(strcmp(cp(i).sid->str, ct->sid->str) == 0)
+	    if(strcmp(cp(i).sid, ct->sid) == 0)
 	    {
 		*cup = &cp(i);
 		*league = NULL;
@@ -179,20 +188,31 @@ cup_get_choose_team_league_cup(const CupChooseTeam *ct,
 	    }
 	}
     }
+
+    if(*league == NULL && *cup == NULL)
+    {
+	g_warning("cup_get_choose_team_league_cup: no league nor cup found for chooseteam %s",
+		  ct->sid);
+	main_exit_program(EXIT_CHOOSE_TEAM_ERROR, NULL);
+    }
 }
 
-/** Return the pointers to the teams participating in the 
-    cup. If necessary, teams are generated and stored in the teams
-    array of the cup. */
-GPtrArray*
+/** Load the pointers to the teams participating in the 
+    cup round. If necessary, teams are generated and stored in the teams
+    array of the cup round. */
+void
 cup_get_team_pointers(Cup *cup, gint round)
 {
     gint i;
-    GPtrArray *teams = g_ptr_array_new();
     CupRound *cup_round = &g_array_index(cup->rounds, CupRound, round);
+    GPtrArray *teams = cup_round->team_ptrs;
 
     if(debug > 60)
-	printf("cup_get_team_pointers %s \n", cup->name->str);
+	printf("cup_get_team_pointers %s \n", cup->name);
+
+    if(teams->len > 0)
+	g_warning("cup_get_team_pointers: round %d in cup %s has non-empty team pointers array.",
+		  round, cup->name);
 
     for(i=0;i<cup_round->choose_teams->len;i++)
 	if(g_array_index(cup_round->choose_teams, CupChooseTeam, i).generate)
@@ -202,21 +222,23 @@ cup_get_team_pointers(Cup *cup, gint round)
 	    cup_load_choose_team(cup, teams, 
 				 &g_array_index(cup_round->choose_teams, CupChooseTeam, i));
 
-    for(i=0;i<cup_round->teams->len;i++)
-    {
-	if(teams->len < cup_round->new_teams)
-	{
-	    team_generate_players_stadium(&g_array_index(cup_round->teams, Team, i));
-	    g_ptr_array_add(cup->teams, &g_array_index(cup_round->teams, Team, i));	    
-	    g_ptr_array_add(teams, &g_array_index(cup_round->teams, Team, i));
+    if(cup_round->teams->len > 0)
+	while(teams->len + cup_round->teams->len > cup_round->new_teams)
+	{		
+	    free_team(&g_array_index(cup_round->teams, Team, cup_round->teams->len - 1));
+	    g_array_remove_index(cup_round->teams, cup_round->teams->len - 1);
 	}
+
+    for(i=0;i<cup_round->teams->len;i++)
+    {	
+	team_generate_players_stadium(&g_array_index(cup_round->teams, Team, i));
+	g_ptr_array_add(cup->teams, &g_array_index(cup_round->teams, Team, i));	    
+	g_ptr_array_add(teams, &g_array_index(cup_round->teams, Team, i));
     }
 
     if(debug > 70)
 	for(i=0;i<teams->len;i++)
-	    printf("%d %s \n", i, ((Team*)g_ptr_array_index(teams, i))->name->str);
-
-    return teams;
+	    printf("cup_get_team_pointers: %d %s \n", i, ((Team*)g_ptr_array_index(teams, i))->name);
 }
 
 /** Get the pointers to the teams (already generated, in one of the leagues or cups)
@@ -232,17 +254,25 @@ cup_load_choose_team(Cup *cup, GPtrArray *teams, const CupChooseTeam *ct)
     const Cup *cup_temp = NULL;
 
     if(debug > 60)
-	printf("cup_load_choose_team: %s, %s \n", cup->name->str,
-	       ct->sid->str);
+	printf("cup_load_choose_team: %s, %s \n", cup->name,
+	       ct->sid);
 
     cup_get_choose_team_league_cup(ct, &league, &cup_temp);
 
     if(cup_temp == NULL)
     {	    
 	if(ct->number_of_teams == -1)
+	{
 	    for(j=0;j<league->table.elements->len;j++)
-		g_ptr_array_add(teams, 
-				team_of_id(g_array_index(league->table.elements, TableElement, j).team_id));
+	    {
+		g_ptr_array_add(
+		    teams, team_of_id(
+			g_array_index(league->table.elements, TableElement, j).team_id));
+		g_ptr_array_add(
+		    cup->team_names, 
+		    g_strdup(team_of_id(g_array_index(league->table.elements, TableElement, j).team_id)->name));
+	    }
+	}
 	else
 	{
 	    start = ct->start_idx - 1;
@@ -259,18 +289,22 @@ cup_load_choose_team(Cup *cup, GPtrArray *teams, const CupChooseTeam *ct)
 	    {
 		if(debug > 80)
 		    printf("team %s isinint %d numteams %d\n",
-			   team_of_id(g_array_index(league->table.elements, TableElement, order[j]).team_id)->name->str,
-			   query_team_is_in_international_cups(
-			       team_of_id(g_array_index(league->table.elements, TableElement, order[j]).team_id), cup->group),
+			   team_of_id(g_array_index(league->table.elements, 
+						    TableElement, order[j]).team_id)->name,
+			   query_team_is_in_cups(
+			       team_of_id(g_array_index(league->table.elements, 
+							TableElement, order[j]).team_id),
+			       cup->group),
 			   number_of_teams);
 
-		if(!query_team_is_in_international_cups(
+		if(!query_team_is_in_cups(
 		       team_of_id(g_array_index(league->table.elements, TableElement, order[j]).team_id), cup->group))
 		{
 		    g_ptr_array_add(teams, 
 				    team_of_id(g_array_index(league->table.elements, TableElement, order[j]).team_id));
-		    g_ptr_array_add(cup->team_names, 
-				    team_of_id(g_array_index(league->table.elements, TableElement, order[j]).team_id)->name);
+		    g_ptr_array_add(
+			cup->team_names, 
+			g_strdup(team_of_id(g_array_index(league->table.elements, TableElement, order[j]).team_id)->name));
 		    number_of_teams++;
 
 		    if(number_of_teams == ct->number_of_teams)
@@ -282,8 +316,8 @@ cup_load_choose_team(Cup *cup, GPtrArray *teams, const CupChooseTeam *ct)
 	if(ct->number_of_teams != -1 &&
 	   number_of_teams != ct->number_of_teams)
 	{
-	    g_warning("cup_load_choose_team (+): not enough teams (that don't participate in international cups yet) found in chooseteam %s for cup %s (%d specified, %d found) cup group %d.\n ",
-		      ct->sid->str, cup->name->str, ct->number_of_teams, number_of_teams, cup->group);
+	    g_warning("cup_load_choose_team (1): not enough teams (that don't participate in international cups yet) found in chooseteam %s for cup %s (%d specified, %d found) cup group %d.\n ",
+		      ct->sid, cup->name, ct->number_of_teams, number_of_teams, cup->group);
 	    main_exit_program(EXIT_CHOOSE_TEAM_ERROR, NULL);
 	}
     }
@@ -294,7 +328,7 @@ cup_load_choose_team(Cup *cup, GPtrArray *teams, const CupChooseTeam *ct)
 	    if(lig(0).teams->len < ct->number_of_teams)
 	    {
 		g_warning("cup_load_choose_team: not enough teams in league 0 for chooseteam %s (%d; required: %d) in cup %s\n",
-			  ct->sid->str, lig(0).teams->len, ct->number_of_teams, cup->name->str);
+			  ct->sid, lig(0).teams->len, ct->number_of_teams, cup->name);
 		main_exit_program(EXIT_CHOOSE_TEAM_ERROR, NULL);
 	    }
 
@@ -303,15 +337,15 @@ cup_load_choose_team(Cup *cup, GPtrArray *teams, const CupChooseTeam *ct)
 
 	    for(i = ct->start_idx - 1; i <= ct->end_idx - 1; i++)
 	    {
-		if(!query_team_is_in_international_cups(
+		if(!query_team_is_in_cups(
 		       &g_array_index(lig(0).teams,
 				      Team, permutation[i - ct->start_idx + 1]), cup->group))
 		{
 		    g_ptr_array_add(teams, &g_array_index(lig(0).teams,
 							  Team, permutation[i - ct->start_idx + 1]));
 		    g_ptr_array_add(cup->team_names,
-				    g_array_index(lig(0).teams,
-						  Team, permutation[i - ct->start_idx + 1]).name);
+				    g_strdup(g_array_index(lig(0).teams,
+							   Team, permutation[i - ct->start_idx + 1]).name));
 		    number_of_teams++;
 		}
 		
@@ -321,8 +355,8 @@ cup_load_choose_team(Cup *cup, GPtrArray *teams, const CupChooseTeam *ct)
 	    
 	    if(number_of_teams != ct->number_of_teams)
 	    {
-		g_warning("cup_load_choose_team (*): not enough teams found in league 0 for chooseteam %s (%d; required: %d) in cup %s (group %d)\n",
-			  ct->sid->str, number_of_teams, ct->number_of_teams, cup->name->str, cup->group);
+		g_warning("cup_load_choose_team (2): not enough teams found in league 0 for chooseteam %s (%d; required: %d) in cup %s (group %d)\n",
+			  ct->sid, number_of_teams, ct->number_of_teams, cup->name, cup->group);
 		main_exit_program(EXIT_CHOOSE_TEAM_ERROR, NULL);
 	    }
 	}
@@ -343,11 +377,12 @@ cup_load_choose_team(Cup *cup, GPtrArray *teams, const CupChooseTeam *ct)
 
 	    for(j = start; j < end; j++)
 	    {
-		if(!query_team_is_in_international_cups(
+		if(!query_team_is_in_cups(
 		       (Team*)g_ptr_array_index(cup_teams_sorted, j), cup->group))
 		{
 		    g_ptr_array_add(teams, g_ptr_array_index(cup_teams_sorted, j));
-		    g_ptr_array_add(cup->team_names, ((Team*)g_ptr_array_index(cup_teams_sorted, j))->name);
+		    g_ptr_array_add(cup->team_names, 
+				    g_strdup(((Team*)g_ptr_array_index(cup_teams_sorted, j))->name));
 		    number_of_teams++;
 
 		    if(number_of_teams == ct->number_of_teams)
@@ -360,8 +395,8 @@ cup_load_choose_team(Cup *cup, GPtrArray *teams, const CupChooseTeam *ct)
 	    if(ct->number_of_teams != -1 &&
 	       number_of_teams != ct->number_of_teams)
 	    {
-		g_warning("cup_load_choose_team(*): not enough teams (that don't participate in international cups yet) found in chooseteam %s for cup %s (%d specified, %d found) cup group %d.\n ",
-			  ct->sid->str, cup->name->str, ct->number_of_teams, number_of_teams, cup->group);
+		g_warning("cup_load_choose_team(3): not enough teams (that don't participate in international cups yet) found in chooseteam %s for cup %s (%d specified, %d found) cup group %d.\n ",
+			  ct->sid, cup->name, ct->number_of_teams, number_of_teams, cup->group);
 		main_exit_program(EXIT_CHOOSE_TEAM_ERROR, NULL);
 	    }
 	}
@@ -369,7 +404,7 @@ cup_load_choose_team(Cup *cup, GPtrArray *teams, const CupChooseTeam *ct)
 
     if(debug > 80)
 	for(i=debug_num;i<teams->len;i++)
-	    printf("%d %s \n", i, ((Team*)g_ptr_array_index(teams, i))->name->str);
+	    printf("cup_load_choose_team: %d %s \n", i, ((Team*)g_ptr_array_index(teams, i))->name);
 }
 
 /** Load the teams specified in the chooseteam from a non-country league. */
@@ -383,18 +418,18 @@ cup_load_choose_team_generate(Cup *cup, CupRound *cup_round, const CupChooseTeam
     GPtrArray *sids = NULL;
 
     if(debug > 60)
-	printf("cup_load_choose_team_generate: %s, %s \n", cup->name->str,
-	       ct->sid->str);
+	printf("cup_load_choose_team_generate: %s, %s \n", cup->name,
+	       ct->sid);
 
     leagues = g_array_new(FALSE, FALSE, sizeof(League));
     teams_local = g_array_new(FALSE, FALSE, sizeof(Team));
-    sids = misc_separate_strings(ct->sid->str);
+    sids = misc_separate_strings(ct->sid);
 
     for(j=0;j<sids->len;j++)
     {
-	if(!query_cup_choose_team_is_league(((GString*)g_ptr_array_index(sids, j))->str))
+	if(!query_cup_choose_team_is_league((gchar*)g_ptr_array_index(sids, j)))
 	{
-	    xml_league_read(((GString*)g_ptr_array_index(sids, j))->str, leagues);
+	    xml_league_read((gchar*)g_ptr_array_index(sids, j), leagues);
 		    
 	    for(k=0; k < g_array_index(leagues, League, leagues->len - 1).teams->len; k++)
 		g_array_append_val(teams_local, g_array_index(
@@ -406,6 +441,14 @@ cup_load_choose_team_generate(Cup *cup, CupRound *cup_round, const CupChooseTeam
     }
 
     g_array_free(leagues, TRUE);
+    free_gchar_array(&sids);
+
+    /** No teams found. */
+    if(teams_local->len == 0)
+    {
+	free_teams_array(&teams_local, FALSE);
+	return;
+    }
     
     gint permutation[teams_local->len];
     for(j=0;j<teams_local->len;j++)
@@ -429,19 +472,20 @@ cup_load_choose_team_generate(Cup *cup, CupRound *cup_round, const CupChooseTeam
     if(teams_local->len < end_idx)
     {
 	g_warning("cup_load_choose_team_generate: not enough teams (%d) in chooseteam %s in cup %s (%d are specified) \n",
-		  teams_local->len, ct->sid->str, cup->name->str, end_idx);
+		  teams_local->len, ct->sid, cup->name, end_idx);
 	
+	free_teams_array(&teams_local, FALSE);
 	main_exit_program(EXIT_CHOOSE_TEAM_ERROR, NULL);
     }
 
     for(j = 0; j < end_idx; j++)
     {
-	if(!query_team_is_in_international_cups(&g_array_index(teams_local, Team, permutation[j]), cup->group))
+	if(!query_team_is_in_cups(&g_array_index(teams_local, Team, permutation[j]), cup->group))
 	{
 	    g_array_append_val(cup_round->teams, g_array_index(teams_local, Team, permutation[j]));
 	    g_array_index(cup_round->teams, Team, cup_round->teams->len - 1).clid = cup->id;
 	    g_ptr_array_add(cup->team_names, 
-				   g_array_index(cup_round->teams, Team, cup_round->teams->len - 1).name);
+			    g_strdup(g_array_index(cup_round->teams, Team, cup_round->teams->len - 1).name));
 
 	    number_of_teams++;
 	}
@@ -454,7 +498,7 @@ cup_load_choose_team_generate(Cup *cup, CupRound *cup_round, const CupChooseTeam
 	(ct->number_of_teams == -1 && number_of_teams != teams_local->len)) &&
        teams_local->len > 0)
     {
-	g_warning("cup_load_choose_team_generate: not enough teams (that don't participate in international cups yet) found in chooseteam %s for cup %s (%d specified, %d found).\n ", ct->sid->str, cup->name->str, ct->number_of_teams, number_of_teams);
+	g_warning("cup_load_choose_team_generate: not enough teams (that don't participate in international cups yet) found in chooseteam %s for cup %s (%d specified, %d found).\n ", ct->sid, cup->name, ct->number_of_teams, number_of_teams);
 	main_exit_program(EXIT_CHOOSE_TEAM_ERROR, NULL);
     }
 
@@ -462,7 +506,6 @@ cup_load_choose_team_generate(Cup *cup, CupRound *cup_round, const CupChooseTeam
 	if(query_team_is_in_cup(&g_array_index(teams_local, Team, j), cup))
 	    g_array_remove_index(teams_local, j);
 	
-    free_g_string_array(&sids);
     free_teams_array(&teams_local, FALSE);
 }
 
@@ -504,7 +547,6 @@ cup_compare_success(gconstpointer a, gconstpointer b, gpointer data)
     const GArray *fixtures = cup->fixtures;
     const Team *team1 = *(const Team**)a;
     const Team *team2 = *(const Team**)b;
-    const Fixture *last_fix = &g_array_index(fixtures, Fixture, fixtures->len - 1);
     gint round_reached1 = cup_get_round_reached(team1, fixtures),
 	round_reached2 = cup_get_round_reached(team2, fixtures);
     gint return_value = 0;
@@ -522,20 +564,46 @@ cup_compare_success(gconstpointer a, gconstpointer b, gpointer data)
 	if(cupround->tables->len > 0)
 	    return_value = 
 		cup_compare_success_tables(team1, team2, cup, round_reached1);
-	else if(round_reached1 != last_fix->round)
-	    return_value = 0;
 	else
-	{
-	    if(GPOINTER_TO_INT(fixture_winner_of(last_fix, TRUE)) ==  team1->id)
-		return_value = -1;
-	    else
-		return_value = 1;
-	}
+	    return_value = cup_compare_success_knockout(team1, team2, cup->fixtures);
     }
 
     return return_value;
 }
 
+/** Compare two teams that reached the same knockout round. */
+gint
+cup_compare_success_knockout(const Team *tm1, const Team *tm2, const GArray *fixtures)
+{
+    gint i;
+    gint winner1 = -1, winner2 = -1;
+    gint return_value = 0;
+
+    for(i=fixtures->len - 1; i>=0; i--)
+    {
+	if(winner1 == -1 &&
+	   query_fixture_team_involved((&g_array_index(fixtures, Fixture, i)), tm1->id))
+	    winner1 = GPOINTER_TO_INT(
+		fixture_winner_of(&g_array_index(fixtures, Fixture, i), TRUE));
+
+	if(winner2 == -1 &&
+	   query_fixture_team_involved((&g_array_index(fixtures, Fixture, i)), tm2->id))
+	    winner2 = GPOINTER_TO_INT(
+		fixture_winner_of(&g_array_index(fixtures, Fixture, i), TRUE));
+	
+	if(winner1 != -1 && winner2 != -1)
+	    break;
+    }
+
+    if(winner1 == tm1->id && winner2 != tm2->id)
+	return_value = -1;
+    else if(winner1 != tm1->id && winner2 == tm2->id)
+	return_value = 1;
+
+    return return_value;
+}
+
+
 /** Compare two teams in cup tables. */
 gint
 cup_compare_success_tables(const Team *tm1, const Team *tm2, const Cup *cup, gint round)
@@ -545,9 +613,11 @@ cup_compare_success_tables(const Team *tm1, const Team *tm2, const Cup *cup, gin
     const CupRound *cupround = &g_array_index(cup->rounds, CupRound, round);
     const TableElement *elem1 = NULL, *elem2 = NULL;
 
-    if(team_get_cup_rank(tm1, cupround) > team_get_cup_rank(tm2, cupround))
+    if(team_get_cup_rank(tm1, cupround, TRUE) > 
+       team_get_cup_rank(tm2, cupround, TRUE))
 	return_value = 1;
-    else if(team_get_cup_rank(tm1, cupround) < team_get_cup_rank(tm2, cupround))
+    else if(team_get_cup_rank(tm1, cupround, TRUE) < 
+	    team_get_cup_rank(tm2, cupround, TRUE))
 	return_value = -1;
     else
     {
@@ -599,7 +669,7 @@ cup_get_first_week_of_cup_round(const Cup *cup, gint cup_round)
 
     if(week_number <= 0)
     {
-	g_warning("cup_get_first_week_of_cup_round: first week of cup %s cup round %d is not positive (%d).\nPlease lower the week gap or set a later last week.\n", cup->name->str, cup_round, week_number);
+	g_warning("cup_get_first_week_of_cup_round: first week of cup %s cup round %d is not positive (%d).\nPlease lower the week gap or set a later last week.\n", cup->name, cup_round, week_number);
 	main_exit_program(EXIT_FIRST_WEEK_ERROR, NULL);
     }
 
@@ -624,20 +694,26 @@ cup_get_last_week_from_first(const Cup *cup, gint first_week)
     @param cup_round The index of the cup round.
     @return The number of matchdays, mostly 1 or 2. */
 gint
-cup_get_matchdays_in_cup_round(const Cup *cup, gint cup_round)
+cup_get_matchdays_in_cup_round(const Cup *cup, gint round)
 {
+    const CupRound *cup_round = &g_array_index(cup->rounds, CupRound, round);    
     gint number_of_teams = -1;
     gint number_of_matchdays = -1;
     
-    if(g_array_index(cup->rounds, CupRound, cup_round).
-       round_robin_number_of_groups > 0)
+    if(cup_round->round_robin_number_of_groups > 0)
     {
-	number_of_teams = cup_round_robin_get_number_of_teams(cup, cup_round);
-	number_of_matchdays = (number_of_teams  % 2 == 0) ?
-	    2 * (number_of_teams  - 1) :
-	    2 * number_of_teams;	
+	number_of_teams = cup_round_get_number_of_teams(cup, round);
+	
+	if(number_of_teams % cup_round->round_robin_number_of_groups == 0)
+	    number_of_matchdays = 
+		((number_of_teams / cup_round->round_robin_number_of_groups) - 1) * 2;
+	else
+	    number_of_matchdays =
+		((number_of_teams - 
+		  (number_of_teams % cup_round->round_robin_number_of_groups)) /
+		 cup_round->round_robin_number_of_groups) * 2;
     }
-    else if(g_array_index(cup->rounds, CupRound, cup_round).home_away)
+    else if(g_array_index(cup->rounds, CupRound, round).home_away)
 	number_of_matchdays = 2;
     else
 	number_of_matchdays = 1;
@@ -645,13 +721,13 @@ cup_get_matchdays_in_cup_round(const Cup *cup, gint cup_round)
     return number_of_matchdays;
 }
 
-/** Return the number of teams playing in a group of 
-    the given cup round with round robin.
+/** Return the number of teams playing in
+    the given cup round.
     @param cup The cup we examine.
     @param cup_round The index of the cup round.
-    @return The number teams in one group. */
+    @return The number teams in the round. */
 gint
-cup_round_robin_get_number_of_teams(const Cup *cup, gint round)
+cup_round_get_number_of_teams(const Cup *cup, gint round)
 {
     const CupRound *cup_round = &g_array_index(cup->rounds, CupRound, round);
     gint number_of_teams = 0;
@@ -664,13 +740,47 @@ cup_round_robin_get_number_of_teams(const Cup *cup, gint round)
 	    (g_array_index(cup->rounds, CupRound, round - 1).round_robin_number_of_groups *
 	     g_array_index(cup->rounds, CupRound, round - 1).round_robin_number_of_advance) +
 	    g_array_index(cup->rounds, CupRound, round - 1).round_robin_number_of_best_advance +
-	    cup_round_get_new_teams(cup_round);
+	    cup_round_get_new_teams(cup_round) + cup_round_get_byes(cup, round - 1);
+    }
+    else
+	number_of_teams = cup_round_get_number_of_teams(cup, round - 1) / 2 +
+	    cup_round_get_new_teams(cup_round) + cup_round_get_byes(cup, round - 1);
+
+    return number_of_teams;
+}
+
+/** Get the number of byes in the given cup round. */
+gint
+cup_round_get_byes(const Cup *cup, gint round)
+{
+    const CupRound *cup_round = &g_array_index(cup->rounds, CupRound, round);
+    gint number_of_byes = 0, new_teams = 0;
+
+    if(cup_round->byes != -1)
+	number_of_byes = cup_round->byes;
+    else
+    {
+	if(round == 0)
+	    new_teams = cup_round_get_new_teams(cup_round);
+	else
+	    new_teams = cup_round_get_number_of_teams(cup, round);
+
+	number_of_byes = (cup_round->round_robin_number_of_groups == 0) ?
+	    math_get_bye_len(new_teams) : 0;
+
+/* 	if(cup_round->round_robin_number_of_groups == 0) */
+/* 	    number_of_byes = math_get_bye_len(new_teams); */
+/* 	else */
+/* 	{ */
+/* 	    while(new_teams % cup_round->round_robin_number_of_groups != 0) */
+/* 	    { */
+/* 		new_teams--; */
+/* 		number_of_byes++; */
+/* 	    } */
+/* 	} */
     }
 
-    while(number_of_teams % cup_round->round_robin_number_of_groups != 0)
-	number_of_teams--;
-
-    return number_of_teams / cup_round->round_robin_number_of_groups;
+    return number_of_byes;
 }
 
 /** Return the number of new teams that come into the
@@ -689,12 +799,16 @@ cup_round_get_new_teams(const CupRound *cup_round)
     {
 	for(i=0;i<cup_round->choose_teams->len;i++)
 	{
-	    if(g_array_index(cup_round->choose_teams, CupChooseTeam, i).number_of_teams != -1)
-		new_teams += g_array_index(cup_round->choose_teams, CupChooseTeam, i).number_of_teams;
+	    if(g_array_index(cup_round->choose_teams, CupChooseTeam, i).
+	       number_of_teams != -1)
+		new_teams += 
+		    g_array_index(cup_round->choose_teams, CupChooseTeam, i).
+		    number_of_teams;
 	    else
 	    {
-		cup_get_choose_team_league_cup(&g_array_index(cup_round->choose_teams, CupChooseTeam, i),
-					       &league, &cup_temp);
+		cup_get_choose_team_league_cup(
+		    &g_array_index(cup_round->choose_teams, CupChooseTeam, i),
+		    &league, &cup_temp);
 		if(cup_temp == NULL)
 		    new_teams += league->teams->len;
 		else
@@ -736,7 +850,7 @@ cup_from_sid(const gchar *sid)
     gint i;
 
     for(i=0;i<cps->len;i++)
-	if(strcmp(cp(i).sid->str, sid) == 0)
+	if(strcmp(cp(i).sid, sid) == 0)
 	    return &cp(i);
 
     g_warning("cup_from_sid: didn't find cup with sid %s \n", sid);
@@ -786,6 +900,7 @@ cup_get_round_name(const Cup *cup, gint round, gchar *buf)
     switch(cup->rounds->len - round)
     {
 	default:
+	    /* A cup stage, e.g. Last 32 when there are only 32 teams left. */
 	    sprintf(buf, _("Last %d"), (gint)rint(powf(2, cup->rounds->len - round)));
 	    break;
 	case 1:
@@ -849,10 +964,11 @@ query_cup_begins(const Cup *cup)
 			       CupChooseTeam, i), &league, &cup_temp);
 
 	    if((cup_temp == NULL && 
-		g_array_index(league->fixtures, Fixture,
-			      league->fixtures->len - 1).week_number == week &&
-		g_array_index(league->fixtures, Fixture,
-			      league->fixtures->len - 1).week_round_number == week_round) ||
+		(!league->active || 
+		 (g_array_index(league->fixtures, Fixture,
+				league->fixtures->len - 1).week_number == week &&
+		  g_array_index(league->fixtures, Fixture,
+				league->fixtures->len - 1).week_round_number == week_round))) ||
 	       (league == NULL && 
 		(cup_temp->fixtures->len > 0 &&
 		 g_array_index(cup_temp->fixtures, Fixture,
@@ -872,12 +988,14 @@ query_cup_begins(const Cup *cup)
 		&g_array_index(cup_round->choose_teams,
 			       CupChooseTeam, i), &league, &cup_temp);
 	    if((cup_temp == NULL &&
-		g_array_index(league->fixtures, Fixture,
-			      league->fixtures->len - 1).attendance == -1) ||
+		(league->active &&
+		 g_array_index(league->fixtures, Fixture,
+			       league->fixtures->len - 1).attendance == -1)) ||
 	       (league == NULL &&
-		(cup_temp->fixtures->len > 0 && 
-		 g_array_index(cup_temp->fixtures, Fixture,
-			       cup_temp->fixtures->len - 1).attendance == -1)))
+		(cup_temp->fixtures->len == 0 || 
+		 (cup_temp->fixtures->len > 0 && 
+		  g_array_index(cup_temp->fixtures, Fixture,
+				cup_temp->fixtures->len - 1).attendance == -1))))
 		return FALSE;
 	}
 
@@ -906,10 +1024,10 @@ cup_get_highlight_colour(const Cup *cup)
     gchar buf[SMALL];
 
     for(i=0;i<cup->properties->len;i++)
-	if(g_str_has_prefix(((GString*)g_ptr_array_index(cup->properties, i))->str, "highlight"))
+	if(g_str_has_prefix((gchar*)g_ptr_array_index(cup->properties, i), "highlight"))
 	{
 	    sprintf(buf, "string_cup_%s",
-		    ((GString*)g_ptr_array_index(cup->properties, i))->str);
+		    (gchar*)g_ptr_array_index(cup->properties, i));
 	    return const_app(buf);
 	}
 
