@@ -40,14 +40,19 @@ fixture_write_league_fixtures(League *league)
 void
 fixture_write_cup_fixtures(Cup *cup)
 {
-    GPtrArray *teams = NULL;
+    gint i;
 
-    teams = cup_get_team_pointers(cup, 0);
+    for(i=0;i<cup->rounds->len;i++)
+	cup_get_team_pointers(cup, i);
 
     if(g_array_index(cup->rounds, CupRound, 0).round_robin_number_of_groups > 0)
-	fixture_write_cup_round_robin(cup, 0, teams);
+	fixture_write_cup_round_robin(cup, 0, 
+				      misc_copy_ptr_array(
+					  g_array_index(cup->rounds, CupRound, 0).team_ptrs));
     else
-	fixture_write_knockout_round(cup, 0, teams);
+	fixture_write_knockout_round(cup, 0, 
+				     misc_copy_ptr_array(
+					 g_array_index(cup->rounds, CupRound, 0).team_ptrs));
 }
 
 /** Update the fixtures for the given cup. 
@@ -59,7 +64,7 @@ fixture_update(Cup *cup)
     GArray *fixtures = cup->fixtures;
     gint round = g_array_index(fixtures, Fixture, fixtures->len - 1).round;
     gint replay = g_array_index(cup->rounds, CupRound, round).replay;
-    GPtrArray *teams = NULL, *teams_new = NULL;
+    GPtrArray *teams = NULL;
     const CupRound *new_round = NULL;
 
     if(replay != 0 && 
@@ -70,27 +75,29 @@ fixture_update(Cup *cup)
     teams = fixture_get_cup_round_winners(cup);
 
     if(round == cup->rounds->len - 1 && teams->len < 2)
+    {
+	g_ptr_array_free(teams, TRUE);
         return;
+    }
 
     if(round + 1 > cup->rounds->len - 1)
     {
 	g_warning("fixture_update: round index %d too high for round array (%d) in cup %s\n",
-		  round + 1, cup->rounds->len - 1, cup->name->str);
+		  round + 1, cup->rounds->len - 1, cup->name);
+	g_ptr_array_free(teams, TRUE);
 	main_exit_program(EXIT_CUP_ROUND_ERROR, NULL);
     }
 
     new_round = &g_array_index(cup->rounds, CupRound, round + 1);
 
-    if(new_round->choose_teams->len > 0)
-    {
-	teams_new = cup_get_team_pointers(cup, round + 1);
-	for(i=0;i<teams_new->len;i++)
-	    g_ptr_array_add(teams, g_ptr_array_index(teams_new, i));
-	g_ptr_array_free(teams_new, TRUE);
-    }
+    for(i=0;i<new_round->team_ptrs->len;i++)
+	g_ptr_array_add(teams, g_ptr_array_index(new_round->team_ptrs, i));
     
     if(teams->len < 2)
+    {
+	g_ptr_array_free(teams, TRUE);
 	return;
+    }
     
     if(cup->bye != NULL && cup->bye->len != 0)
     {
@@ -163,48 +170,42 @@ fixture_get_cup_round_winners(const Cup *cup)
 GPtrArray*
 fixture_get_round_robin_advance(const Cup *cup, gint round)
 {
-    gint i, j;
-    GPtrArray *array = g_ptr_array_new();
     const CupRound *cupround = &g_array_index(cup->rounds, CupRound, round);
-    GArray *best_advance = g_array_new(FALSE, FALSE, sizeof(TableElement));
-    
-    for(i=0;i<cupround->tables->len;i++)
-	for(j=0;j<g_array_index(cupround->tables, Table, i).elements->len;j++)
-	{
-	    if(j < cupround->round_robin_number_of_advance)
-		g_ptr_array_add(array, g_array_index(
-				    g_array_index(cupround->tables, Table, i).elements,
-				    TableElement, j).team);
-	    else
-		g_array_append_val(best_advance,
-				   g_array_index(g_array_index(cupround->tables, Table, i).elements,
-						 TableElement, j));
-	}
-    
-    g_array_sort_with_data(best_advance,
-			   (GCompareDataFunc)table_element_compare_func,
-			   GINT_TO_POINTER(cup->id));
-    
-    for(i=0;i<cupround->round_robin_number_of_best_advance;i++)
-	g_ptr_array_add(array, g_array_index(best_advance, TableElement, i).team);
+    GPtrArray *teams = cup_get_teams_sorted(cup);
+    gint i, remove = teams->len -
+	(cupround->round_robin_number_of_groups *
+	 cupround->round_robin_number_of_advance) -
+	cupround->round_robin_number_of_best_advance;  
 
-    g_array_free(best_advance, TRUE);
+    for(i=0;i<remove;i++)
+	g_ptr_array_remove_index(teams, teams->len - 1);    
 
-    return array;
+    return teams;
 }
 
 /** Return the pointer of the team that won the encounter.
     @param fix The fixture we examine.
     @param team_name Whether to return a team pointer or the
-    team name of the winner.
+    team id of the winner.
     @return A team pointer or a coded integer. */
 gpointer
 fixture_winner_of(const Fixture *fix, gboolean team_id)
 {
     gint winner_idx = -1;
-    const Fixture *first_leg;
-    const CupRound *cupround = 
-	&g_array_index(cup_from_clid(fix->clid)->rounds, CupRound, fix->round);
+    const Fixture *first_leg = NULL;
+    const CupRound *cupround = NULL;
+
+    if(fix->clid < ID_CUP_START)
+    {
+	winner_idx = (fix->result[0][0] < fix->result[1][0]);
+	if(team_id)
+	    return GINT_TO_POINTER(fix->team_ids[winner_idx]);
+	else
+	    return (gpointer)fix->teams[winner_idx];
+    }
+
+    cupround = &g_array_index(cup_from_clid(fix->clid)->rounds, 
+			      CupRound, fix->round);
 
     if(cupround->replay != 0 || !cupround->home_away)
 	winner_idx = (math_sum_int_array(&(fix->result[0][0]), 3) <
@@ -212,18 +213,20 @@ fixture_winner_of(const Fixture *fix, gboolean team_id)
     else
     {
 	first_leg = fixture_get_first_leg(fix);
-
-	if(fix->result[0][0] + first_leg->result[1][0] >
-	   fix->result[1][0] + first_leg->result[0][0])
-	    winner_idx = 0;
-	else if(fix->result[0][0] + first_leg->result[1][0] <
-		fix->result[1][0] + first_leg->result[0][0])
-	    winner_idx = 1;
-	else if(fix->result[1][0] > first_leg->result[1][0])
-	    winner_idx = 1;
+	
+	if(fix->result[0][2] + fix->result[1][2] != 0)
+	    winner_idx = (fix->result[0][2] < fix->result[1][2]);
+	else if(fix->result[0][0] + fix->result[0][1] +
+		 first_leg->result[1][0] + first_leg->result[1][1] !=
+		 fix->result[1][0] + fix->result[1][1] +
+		first_leg->result[0][0] + first_leg->result[0][1])
+	    winner_idx = (fix->result[0][0] + fix->result[0][1] +
+			  first_leg->result[1][0] + first_leg->result[1][1] <
+			  fix->result[1][0] + fix->result[1][1] +
+			  first_leg->result[0][0] + first_leg->result[0][1]);
 	else
-	    winner_idx = (fix->result[0][1] + fix->result[0][2] <
-			  fix->result[1][1] + fix->result[1][2]);
+	    winner_idx = (fix->result[1][0] + fix->result[1][1] >
+			  first_leg->result[1][0] + first_leg->result[1][1]);
     }
 
     if(team_id)
@@ -273,49 +276,66 @@ fixture_update_write_replays(Cup *cup)
 void
 fixture_write_cup_round_robin(Cup *cup, gint cup_round, GPtrArray *teams)
 {
-    gint i, j;
+    gint i, j, cnt;
     CupRound *cupround = &g_array_index(cup->rounds, CupRound, cup_round);
     gint number_of_groups = cupround->round_robin_number_of_groups;
-    gint teams_per_group = -1;
-    GPtrArray *teams_group = NULL;
-    Table new_table;
+    gint team_num = (teams->len - (teams->len % number_of_groups)) / number_of_groups;
+    gint plus_div = teams->len % number_of_groups;
+    GPtrArray *teams_group[number_of_groups];
+    Table table_group[number_of_groups];
     TableElement new_table_element;
 
-    teams = misc_randomise_g_pointer_array(teams);
-
-    if(teams->len % number_of_groups != 0)
+    if(teams->len < number_of_groups)
     {
-	cup->bye = g_ptr_array_new();
-
-	while(teams->len % number_of_groups != 0)
-	{
-	    g_ptr_array_add(cup->bye, g_ptr_array_index(teams, teams->len - 1));
-	    g_ptr_array_remove_index(teams, teams->len - 1);
-	}
+	g_warning("fixture_write_cup_round_robin: cup %s round %d: number of teams (%d) less than number of groups (%d)\n", cup->name, cup_round, teams->len, number_of_groups);
+	main_exit_program(EXIT_FIXTURE_WRITE_ERROR, NULL);
     }
 
-    teams_per_group = teams->len / number_of_groups;    
+    if(cupround->randomise_teams)
+	g_ptr_array_sort_with_data(teams, (GCompareDataFunc)team_compare_func,
+			       GINT_TO_POINTER(TEAM_COMPARE_AV_SKILL));
 
     for(i=0;i<number_of_groups;i++)
     {
-	new_table.name = g_string_new(cup->name->str);
-	new_table.clid = cup->id;
-	new_table.round = cup_round;
-	new_table.elements = g_array_new(FALSE, FALSE, sizeof(TableElement));
+	table_group[i].name = NULL;
+	misc_string_assign(&table_group[i].name, cup->name);
+	table_group[i].clid = cup->id;
+	table_group[i].round = cup_round;
+	table_group[i].elements = g_array_new(FALSE, FALSE, sizeof(TableElement));    
+	teams_group[i] = g_ptr_array_new();
+    }
     
-	teams_group = g_ptr_array_new();
-
-	for(j=0;j<teams_per_group;j++)
+    cnt = 0;
+    if(!cupround->randomise_teams)
+    {
+	for(i=0;i<number_of_groups;i++)
+	    for(j=0;j<team_num + (i < plus_div);j++)
+	    {
+		g_ptr_array_add(teams_group[i],
+				g_ptr_array_index(teams, cnt));
+		new_table_element = 
+		    table_element_new((Team*)g_ptr_array_index(teams, cnt), table_group[i].elements->len);
+		g_array_append_val(table_group[i].elements, new_table_element);
+		cnt++;
+	    }	
+    }
+    else
+    {
+	for(i=0;i<teams->len;i++)
 	{
-	    g_ptr_array_add(teams_group, g_ptr_array_index(teams, j + i * teams_per_group));
+	    g_ptr_array_add(teams_group[cnt],
+			    g_ptr_array_index(teams, i));
 	    new_table_element = 
-		table_element_new((Team*)g_ptr_array_index(teams, j + i * teams_per_group), j);
-	    g_array_append_val(new_table.elements, new_table_element);
+		table_element_new((Team*)g_ptr_array_index(teams, i), table_group[cnt].elements->len);
+	    g_array_append_val(table_group[cnt].elements, new_table_element);
+	    cnt = (cnt + 1) % number_of_groups;
 	}
+    }
 
-	g_array_append_val(cupround->tables, new_table);
-
-	fixture_write_round_robin((gpointer)cup, cup_round, teams_group, !cupround->home_away);
+    for(i=0;i<number_of_groups;i++)
+    {
+	g_array_append_val(cupround->tables, table_group[i]);
+	fixture_write_round_robin((gpointer)cup, cup_round, teams_group[i], !cupround->home_away);	
     }
 
     g_ptr_array_free(teams, TRUE);
@@ -483,8 +503,6 @@ fixture_write_knockout_round(Cup *cup, gint cup_round, GPtrArray *teams)
     gint bye_len = (round->byes == -1) ?
 	math_get_bye_len(len) : round->byes;
 
-    teams = misc_randomise_g_pointer_array(teams);
-
     if(bye_len != 0)
     {
 	cup->bye = g_ptr_array_new();
@@ -500,12 +518,14 @@ fixture_write_knockout_round(Cup *cup, gint cup_round, GPtrArray *teams)
 
 	teams = misc_randomise_g_pointer_array(teams);
     }
+    else if(round->randomise_teams)
+	teams = misc_randomise_g_pointer_array(teams);
 
     week_round_number =
 	fixture_get_free_round(first_week, teams, -1, -1);
-    for(i=0; i<teams->len / 2; i++)
-	fixture_write(cup->fixtures, (Team*)g_ptr_array_index(teams, i),
-		      (Team*)g_ptr_array_index(teams, i + teams->len / 2), first_week,
+    for(i=0; i<=(teams->len - 2) / 2; i++)
+	fixture_write(cup->fixtures, (Team*)g_ptr_array_index(teams, 2 * i),
+		      (Team*)g_ptr_array_index(teams, 2 * i + 1), first_week,
 		      week_round_number, cup->id, cup_round, 0,
 		      !round->neutral, FALSE, (!round->home_away && round->replay == 0));
 
@@ -513,9 +533,9 @@ fixture_write_knockout_round(Cup *cup, gint cup_round, GPtrArray *teams)
     {
 	week_round_number =
 	    fixture_get_free_round(first_week + cup->week_gap, teams, -1, -1);
-	for(i=0; i<teams->len / 2; i++)
-	    fixture_write(cup->fixtures, (Team*)g_ptr_array_index(teams, i + teams->len / 2),
-			  (Team*)g_ptr_array_index(teams, i), first_week + cup->week_gap,
+	for(i=0; i<=(teams->len - 2) / 2; i++)
+	    fixture_write(cup->fixtures, (Team*)g_ptr_array_index(teams, 2 * i + 1),
+			  (Team*)g_ptr_array_index(teams, 2 * i), first_week + cup->week_gap,
 			  week_round_number, cup->id, cup_round, 0,
 			  !round->neutral, TRUE, TRUE);
     }
@@ -664,10 +684,9 @@ query_fixture_is_draw(const Fixture *fix)
 
     first_leg = fixture_get_first_leg(fix);
 
-    return (fix->result[0][0] + first_leg->result[1][0] ==
-	    fix->result[1][0] + first_leg->result[0][0] &&
+    return (fix->result[0][1] + fix->result[1][1] == 0 &&
 	    fix->result[0][0] == first_leg->result[0][0] &&
-	    fix->result[0][1] == fix->result[1][1]);
+	    fix->result[1][0] == first_leg->result[1][0]);
 }
 
 /** Check whether a user's team participates.
@@ -734,7 +753,7 @@ fixture_get_first_leg(const Fixture *fix)
 
     if(first_leg == NULL)
 	g_warning("fixture_get_first_leg: didn't find first leg match; cup %s round %d\n",
-		  cup_from_clid(fix->clid)->name->str, fix->round);
+		  cup_from_clid(fix->clid)->name, fix->round);
 
     return first_leg;
 }
@@ -780,30 +799,40 @@ fixture_get_week_list(gint week_number, gint week_round_number)
     return fixtures;
 }
 
-/** Print the result of the fixture into a buffer. */
+/** Print the result of the fixture into a buffer.
+    @param swap Whether to swap the two teams' goals. */
 void
-fixture_result_to_buf(const Fixture *fix, gchar *buf)
+fixture_result_to_buf(const Fixture *fix, gchar *buf, gboolean swap)
 {
     gchar local_buf[SMALL];
+    gint idx0 = 0, idx1 = 1;
+
+    if(swap)
+    {
+	idx0 = 1;
+	idx1 = 0;
+    }
 
     if(fix->attendance < 0)
 	strcpy(buf, "-- : --");
     else
     {
-	sprintf(local_buf, "%d - %d", math_sum_int_array(fix->result[0], 3),
-		math_sum_int_array(fix->result[1], 3));
-	if(fix->result[0][2] + fix->result[1][2] != 0)
+	sprintf(local_buf, "%d - %d", math_sum_int_array(fix->result[idx0], 3),
+		math_sum_int_array(fix->result[idx1], 3));
+	if(fix->result[idx0][2] + fix->result[idx1][2] != 0)
+	    /* Game was decided in penalty shoot-out. */
 	    strcat(local_buf, _(" p."));
-	else if(fix->result[0][1] + fix->result[1][1] != 0)
+	else if(fix->result[idx0][1] + fix->result[idx1][1] != 0)
+	    /* Game was decided in extra time. */
 	    strcat(local_buf, _(" e.t."));
 
 	if(fix->second_leg)
 	    sprintf(buf, "%s (%d - %d)", local_buf,
-		    fixture_get_first_leg(fix)->result[1][0],
-		    fixture_get_first_leg(fix)->result[0][0]);
+		    fixture_get_first_leg(fix)->result[idx1][0],
+		    fixture_get_first_leg(fix)->result[idx0][0]);
 	else
 	    strcpy(buf, local_buf);
-    }    
+    }
 }
 
 /** Return the number of fixtures in a given week round.
@@ -882,7 +911,8 @@ fixture_get_previous(gint clid, gint week_number, gint week_round_number)
     @param tm A team pointer (for the case SHOW_TEAM).
     @return A fixture pointer or NULL. */
 Fixture*
-fixture_get(gint type, gint clid, gint week_number, gint week_round_number, const Team *tm)
+fixture_get(gint type, gint clid, gint week_number, 
+	    gint week_round_number, const Team *tm)
 {
     Fixture *fix = NULL;
     gint new_clid = -1;
@@ -907,8 +937,8 @@ fixture_get(gint type, gint clid, gint week_number, gint week_round_number, cons
 	    type == SHOW_PREVIOUS_LEAGUE)
     {
 	    new_clid = (type == SHOW_NEXT_LEAGUE) ?
-		league_cup_get_next_clid(clid):
-		league_cup_get_previous_clid(clid);
+		league_cup_get_next_clid(clid, FALSE):
+		league_cup_get_previous_clid(clid, FALSE);
 
 	    fix = fixture_get(SHOW_CURRENT, new_clid, week, week_round, NULL);
     }
@@ -1103,18 +1133,22 @@ fixture_get_next_week(gint *week_number, gint *week_round_number)
     *week_number = *week_round_number = 1000;
 
     for(i=0;i<ligs->len;i++)
-    {
-	fix = fixture_get_next(lig(i).id, local_week, local_round);
-	if((fix->week_number > local_week ||
-	   (fix->week_number == local_week && fix->week_round_number > local_round)) &&
-	   (fix->week_number < *week_number ||
-	    (fix->week_number == *week_number && fix->week_round_number < *week_round_number)) &&
-	   (fix->clid == current_user.tm->clid || opt_user_int("int_opt_user_show_all_leagues")))
+	if(lig(i).active)
 	{
-	    *week_number = fix->week_number;
-	    *week_round_number = 1;
-	}	    
-    }
+	    fix = fixture_get_next(lig(i).id, local_week, local_round);
+	    if((fix->week_number > local_week ||
+		(fix->week_number == local_week && 
+		 fix->week_round_number > local_round)) &&
+	       (fix->week_number < *week_number ||
+		(fix->week_number == *week_number && 
+		 fix->week_round_number < *week_round_number)) &&
+	       (fix->clid == current_user.tm->clid || 
+		opt_user_int("int_opt_user_show_all_leagues")))
+	    {
+		*week_number = fix->week_number;
+		*week_round_number = 1;
+	    }	    
+	}
 
     for(i=0;i<acps->len;i++)
     {
@@ -1146,18 +1180,22 @@ fixture_get_previous_week(gint *week_number, gint *week_round_number)
     *week_number = *week_round_number = -1;
 
     for(i=0;i<ligs->len;i++)
-    {
-	fix = fixture_get_previous(lig(i).id, local_week, local_round);
-	if((fix->week_number < local_week ||
-	   (fix->week_number == local_week && fix->week_round_number < local_round)) &&
-	   (fix->week_number > *week_number ||
-	    (fix->week_number == *week_number && fix->week_round_number > *week_round_number)) &&
-	   (fix->clid == current_user.tm->clid || opt_user_int("int_opt_user_show_all_leagues")))
+	if(lig(i).active)
 	{
-	    *week_number = fix->week_number;
-	    *week_round_number = 1;
+	    fix = fixture_get_previous(lig(i).id, local_week, local_round);
+	    if((fix->week_number < local_week ||
+		(fix->week_number == local_week && 
+		 fix->week_round_number < local_round)) &&
+	       (fix->week_number > *week_number ||
+		(fix->week_number == *week_number && 
+		 fix->week_round_number > *week_round_number)) &&
+	       (fix->clid == current_user.tm->clid || 
+		opt_user_int("int_opt_user_show_all_leagues")))
+	    {
+		*week_number = fix->week_number;
+		*week_round_number = 1;
+	    }
 	}
-    }
 
     for(i=0;i<acps->len;i++)
     {
@@ -1178,3 +1216,36 @@ fixture_get_previous_week(gint *week_number, gint *week_round_number)
 	*week_round_number = local_round;
     }
 }
+
+/** Return an array with the user's fixtures in this season. */
+GPtrArray*
+fixture_get_season_results(void)
+{
+    gint i, j;
+    GPtrArray *results = g_ptr_array_new();
+    GArray *fixtures = NULL;
+    
+    if(league_from_clid(current_user.tm->clid)->active)
+    {
+	fixtures = league_from_clid(current_user.tm->clid)->fixtures;
+	for(i=0;i<fixtures->len;i++)
+	    if(query_fixture_team_involved((&g_array_index(fixtures, Fixture, i)),
+					   current_user.tm->id))
+		g_ptr_array_add(results, &g_array_index(fixtures, Fixture, i));
+    }
+
+    for(i=0;i<acps->len;i++)
+	if(query_team_is_in_cup(current_user.tm, acp(i)))
+	{
+	    fixtures = acp(i)->fixtures;
+	    for(j=0;j<fixtures->len;j++)
+		if(query_fixture_team_involved((&g_array_index(fixtures, Fixture, j)),
+					       current_user.tm->id))
+		    g_ptr_array_add(results, &g_array_index(fixtures, Fixture, j));
+	}
+
+    g_ptr_array_sort_with_data(results, fixture_compare_func,
+			       GINT_TO_POINTER(FIXTURE_COMPARE_DATE));
+
+    return results;
+}
