@@ -10,6 +10,8 @@
 #include "team_struct.h"
 #include "user_struct.h"
 
+#define free_gchar_ptr(string) {g_free(string); string = NULL;}
+
 void
 free_g_array(GArray **array);
 
@@ -56,7 +58,10 @@ void
 free_variables(void);
 
 void
-free_g_string_array(GPtrArray **array);
+free_gchar_array(GPtrArray **array);
+
+void
+free_player_array(GArray **players);
 
 void
 free_player(Player *pl);
@@ -64,6 +69,9 @@ free_player(Player *pl);
 void
 free_live_game(LiveGame *match);
 
+void
+free_lg_commentary(gboolean reset);
+
 void
 free_option_list(OptionList *optionlist, gboolean reset);
 
@@ -91,4 +99,7 @@ free_name_list(NameList *namelist, gboolean reset);
 void
 free_names(gboolean reset);
 
+void
+free_mmatches(GArray **mmatches, gboolean reset);
+
 #endif
