@@ -86,7 +86,8 @@ game_get_player_contribution(const Player *pl, gint type)
 	  const_float("float_player_team_weight_forward_attack")}};
 
     return player_get_game_skill(pl, FALSE) *
-	player_weights[pl->cpos - 1][type - GAME_TEAM_VALUE_DEFEND];
+	player_weights[pl->cpos - 1][type - GAME_TEAM_VALUE_DEFEND] *
+	(1 + (gfloat)pl->streak * const_float("float_player_streak_influence_skill"));
 }
 
 /** Return a random attacking or defending player
@@ -108,24 +109,23 @@ game_get_player(const Team *tm, gint player_type,
     gfloat probs[10];
     gfloat rndom;
 
-    /*todo move to constants file?*/
     if(player_type == GAME_PLAYER_TYPE_ATTACK)
     {
-	weights[0] = 0.25;
-	weights[1] = 0.5;
-	weights[2] = 1;
+	weights[0] = const_float("float_game_player_weight_attack_def");
+	weights[1] = const_float("float_game_player_weight_attack_mid");
+	weights[2] = const_float("float_game_player_weight_attack_fwd");
     }
     else if(player_type == GAME_PLAYER_TYPE_MIDFIELD)
     {
-	weights[0] = 0.5;
-	weights[1] = 1;
-	weights[2] = 0.5;
+	weights[0] = const_float("float_game_player_weight_midfield_def");
+	weights[1] = const_float("float_game_player_weight_midfield_mid");
+	weights[2] = const_float("float_game_player_weight_midfield_fwd");
     }
     else if(player_type == GAME_PLAYER_TYPE_DEFEND)
     {
-	weights[0] = 1;
-	weights[1] = 0.5;
-	weights[2] = 0.25;
+	weights[0] = const_float("float_game_player_weight_defend_def");
+	weights[1] = const_float("float_game_player_weight_defend_mid");
+	weights[2] = const_float("float_game_player_weight_defend_fwd");
     }
     else if(player_type == GAME_PLAYER_TYPE_PENALTY)
 	return game_get_penalty_taker(tm, last_penalty);
@@ -154,12 +154,12 @@ game_get_player(const Team *tm, gint player_type,
     else
     {
 	g_warning("game_get_player: All players injured or banned, apparently.\n");
-	printf("player list:\n");
+	printf("%s %s player list:\n", league_cup_get_name_string(tm->clid), tm->name);
 	for(i=0;i<tm->players->len;i++)
 	{
 	    if(i < 10)
 		printf("prob %.3f  ", probs[i]);
-	    printf("%d %20s health %d cskill %.2f\n", i, player_of_idx_team(tm, i)->name->str,
+	    printf("%d %20s health %d cskill %.2f\n", i, player_of_idx_team(tm, i)->name,
 		   player_of_idx_team(tm, i)->health, player_of_idx_team(tm, i)->cskill);
 	}
 
@@ -182,16 +182,27 @@ game_get_player_probs(GArray *players, gfloat *probs, gfloat *weights, gboolean
 {
     gint i;
 
-    probs[0] = (skills) ? player_get_game_skill(&g_array_index(players, Player, 1), FALSE) *
-	weights[g_array_index(players, Player, 1).pos - 1] :
-	weights[g_array_index(players, Player, 1).pos - 1] *
+    probs[0] = (skills) ? 
+	player_get_game_skill(&g_array_index(players, Player, 1), FALSE) *
+	weights[g_array_index(players, Player, 1).cpos - 1] :
+	weights[g_array_index(players, Player, 1).cpos - 1] *
 	(g_array_index(players, Player, 1).cskill != 0);
+
+    probs[0] *= (1 + (gfloat)g_array_index(players, Player, 1).streak *
+		 const_float("float_player_streak_influence_skill"));
+		 
+
     for(i=1;i<10;i++)
+    {
 	probs[i] = probs[i - 1] + 
-	    ((skills) ? player_get_game_skill(&g_array_index(players, Player, i + 1), FALSE) *
-	     weights[g_array_index(players, Player, i + 1).pos - 1] :
-	     weights[g_array_index(players, Player, i + 1).pos - 1] *
+	    ((skills) ? 
+	     player_get_game_skill(&g_array_index(players, Player, i + 1), FALSE) *
+	     weights[g_array_index(players, Player, i + 1).cpos - 1] :
+	     weights[g_array_index(players, Player, i + 1).cpos - 1] *
 	     (g_array_index(players, Player, i + 1).cskill != 0));
+	probs[i] *= (1 + (gfloat)g_array_index(players, Player, i + 1).streak *
+		     const_float("float_player_streak_influence_skill"));
+    }
 }
 
 /** Return the player who's shooting the following penalty
@@ -252,11 +263,18 @@ game_initialize(Fixture *fix)
     ticket_income = 
 	fix->attendance * const_int("int_team_stadium_ticket_price");
 
-    if(user_idx[0] != -1 && fix->home_advantage)
+    if(!sett_int("int_opt_disable_finances") && user_idx[0] != -1 && fix->home_advantage)
     {
 	usr(user_idx[0]).money += ticket_income;
 	usr(user_idx[0]).money_in[1][MON_IN_TICKET] += ticket_income;
 
+	usr(user_idx[0]).money -= 
+	    (gint)rint((gfloat)ticket_income *
+		       (gfloat)usr(user_idx[0]).youth_academy.percentage / 100);
+	usr(user_idx[0]).money_out[1][MON_OUT_YA] -= 
+	    (gint)rint((gfloat)ticket_income *
+		       (gfloat)usr(user_idx[0]).youth_academy.percentage / 100);
+
 	if(debug < 50)
 	{
 	    fix->teams[0]->stadium.safety -= 
@@ -274,13 +292,29 @@ game_initialize(Fixture *fix)
 	    {
 		if(g_array_index(fix->teams[i]->players, Player, j).cskill > 0)
 		{
-		    player_games_goals_set(&g_array_index(fix->teams[i]->players, Player, j), fix->clid,
+		    player_games_goals_set(&g_array_index(fix->teams[i]->players,
+							  Player, j), fix->clid,
 					   PLAYER_VALUE_GAMES, 1);
-		    g_array_index(fix->teams[i]->players, Player, j).career[PLAYER_VALUE_GAMES]++;
-		}
+		    g_array_index(fix->teams[i]->players, Player, j).
+			career[PLAYER_VALUE_GAMES]++;
 		
-		g_array_index(fix->teams[i]->players, Player, j).participation = TRUE;
+		    g_array_index(fix->teams[i]->players, Player, j).
+			participation = TRUE;
+
+		    if(query_player_is_youth((&g_array_index(fix->teams[i]->players,
+							     Player, j))))
+			g_array_index(fix->teams[i]->players, Player, j).lsu +=
+			    const_float("float_youth_lsu_addition_match");
+
+		    player_streak_add_to_prob(
+			&g_array_index(fix->teams[i]->players, Player, j),
+			const_float("float_player_streak_add_startup"));
+		}
 	    }
+	    else
+		player_streak_add_to_prob(
+		    &g_array_index(fix->teams[i]->players, Player, j),
+		    const_float("float_player_streak_add_no_startup"));
 	}
 
 	if(user_idx[i] != -1)
@@ -337,7 +371,9 @@ game_assign_attendance(Fixture *fix)
 	    factor *= const_float("float_game_stadium_attendance_cup_international_factor");
     }
 
-    fix->attendance = MIN((gint)rint((gfloat)tm[0]->stadium.capacity * factor), max_att);
+    factor = MIN(factor, 1);
+    fix->attendance = MIN(MIN((gint)rint((gfloat)tm[0]->stadium.capacity * factor),
+			      max_att * factor), max_att);
 
     tm[0]->stadium.average_attendance = 
 	(gint)rint((gfloat)(tm[0]->stadium.average_attendance * tm[0]->stadium.games + fix->attendance) /
@@ -402,7 +438,7 @@ game_check_live_game_resume_state(void)
 	for(j=0;j<11;j++)
 	    if(!query_integer_is_in_array(player_of_idx_team(teams[i], j)->id,
 					  usr(stat2).live_game.team_state[i].player_ids,
-					  0, 11))
+					  11))
 		subs[i]++;
 	
 	if(subs[i] > usr(stat2).live_game.subs_left[i])
@@ -431,7 +467,7 @@ game_reset_players(gint idx)
     for(i=0;i<tm->players->len;i++)
 	if(!query_integer_is_in_array(player_of_idx_team(tm, i)->id,
 				      usr(stat2).live_game.team_state[idx].player_ids,
-				      0, 11))
+				      11))
 	    g_array_append_val(players, *player_of_idx_team(tm, i));
 
     g_array_free(tm->players, TRUE);
@@ -460,7 +496,7 @@ game_get_subs(gint team_number, gint *subs_in, gint *subs_out)
 	current_players[i] = player_of_idx_team(tm, i)->id;
 	if(!query_integer_is_in_array(player_of_idx_team(tm, i)->id,
 				      usr(stat2).live_game.team_state[team_number].player_ids,
-				      0, 11))
+				      11))
 	{
 	    subs_in[cnt] = player_of_idx_team(tm, i)->id;
 	    cnt++;
@@ -470,7 +506,7 @@ game_get_subs(gint team_number, gint *subs_in, gint *subs_out)
     cnt = 0;
     for(i=0;i<11;i++)
 	if(!query_integer_is_in_array(usr(stat2).live_game.team_state[team_number].player_ids[i],
-				      current_players, 0, 11))
+				      current_players, 11))
 	{
 	    subs_out[cnt] = usr(stat2).live_game.team_state[team_number].player_ids[i];
 	    cnt++;
@@ -560,8 +596,17 @@ game_player_injury(Player *pl)
 	}
 
     if(pl->health == PLAYER_INJURY_CAREER_STOP && team_is_user(pl->team) != -1)
-	user_event_add(&usr(team_is_user(pl->team)), EVENT_TYPE_PLAYER_CAREER_STOP, pl->id, -1,
+	user_event_add(&usr(team_is_user(pl->team)), 
+		       EVENT_TYPE_PLAYER_CAREER_STOP, pl->id, -1,
 		       NULL, NULL);
+
+    player_streak_add_to_prob(
+	pl, const_float("float_player_streak_add_injury"));
+    if(pl->streak == PLAYER_STREAK_HOT)
+    {
+	pl->streak = PLAYER_STREAK_NONE;
+	player_streak_reset_count(pl);
+    }
 }
 
 /** Return a factor influencing who's fouled whom
@@ -590,30 +635,38 @@ game_get_foul_possession_factor(gint boost1, gint boost2)
 gint
 game_substitute_player(Team *tm, gint player_number)
 {
-    gint i;
+    gint i, substitute = -1;
     GPtrArray *substitutes = g_ptr_array_new();
     gboolean adapt_structure;
-    gint substitute = -1;
 
     for(i=11;i<tm->players->len;i++)
 	if(g_array_index(tm->players, Player, i).cskill > 0)
 	    g_ptr_array_add(substitutes, &g_array_index(tm->players, Player, i));
 
-    g_ptr_array_sort_with_data(substitutes, (GCompareDataFunc)player_compare_substitute_func,
-			       GINT_TO_POINTER(player_of_idx_team(tm, player_number)->cpos));
+    g_ptr_array_sort_with_data(substitutes, 
+			       (GCompareDataFunc)player_compare_substitute_func,
+			       GINT_TO_POINTER(player_of_idx_team(tm, 
+								  player_number)->cpos));
     adapt_structure = 
 	(math_get_place(team_find_appropriate_structure(tm), 1) + 
 	 math_get_place(team_find_appropriate_structure(tm), 2) + 
 	 math_get_place(team_find_appropriate_structure(tm), 3) != 10 ||
-	 (player_of_idx_team(tm, player_number)->cpos != ((Player*)g_ptr_array_index(substitutes, 0))->pos &&
+	 (player_of_idx_team(tm, player_number)->cpos != 
+	  ((Player*)g_ptr_array_index(substitutes, 0))->pos &&
 	  player_substitution_good_structure(tm->structure,
 					     player_of_idx_team(tm, player_number)->cpos,
 					     ((Player*)g_ptr_array_index(substitutes, 0))->pos)));
 
     substitute = ((Player*)g_ptr_array_index(substitutes, 0))->id;
 
-    player_swap(tm, player_number,
-		tm, player_id_index(tm, ((Player*)g_ptr_array_index(substitutes, 0))->id));
+    player_streak_add_to_prob(
+	&g_array_index(tm->players, Player, player_number),
+	const_float("float_player_streak_add_sub_out"));
+
+    player_streak_add_to_prob((Player*)g_ptr_array_index(substitutes, 0),
+			      const_float("float_player_streak_add_sub_in"));
+
+    player_swap(tm, player_number, tm, player_id_index(tm, substitute));
 
     g_ptr_array_free(substitutes, TRUE);
 
@@ -627,7 +680,7 @@ game_substitute_player(Team *tm, gint player_number)
     {
 	game_gui_write_av_skills();
 	
-	selected_row[0] = -1;	
+	selected_row = -1;	
 	treeview_show_user_player_list();
     }
 
@@ -755,7 +808,7 @@ game_substitute_player_send_off(gint clid, Team *tm, gint player_number,
     {
 	game_gui_write_av_skills();
 	
-	selected_row[0] = -1;	
+	selected_row = -1;	
 	treeview_show_user_player_list();
     }
 }
@@ -851,7 +904,6 @@ game_update_stats_player(gpointer live_game, gconstpointer live_game_unit)
 	player2 = unit->event.player2;
     const Team *tm[2] = {((LiveGame*)live_game)->fix->teams[0], 
 			 ((LiveGame*)live_game)->fix->teams[1]};
-    GString *new = NULL;
     GPtrArray *players = NULL;
     
     if(unit->event.type == LIVE_GAME_EVENT_GOAL ||
@@ -861,60 +913,57 @@ game_update_stats_player(gpointer live_game, gconstpointer live_game_unit)
 	array_index = (own_goal) ? !team : team;
 
 	if(live_game_unit_before(unit, -1)->event.type == LIVE_GAME_EVENT_PENALTY)
-	    strcpy(buf2, " (P)");
+	    /* A goal scored with penalty. */
+	    strcpy(buf2, _(" (P)"));
 	else if(live_game_unit_before(unit, -1)->event.type == LIVE_GAME_EVENT_FREE_KICK)
-	    strcpy(buf2, " (FK)");
+	    /* A goal scored with a free kick. */
+	    strcpy(buf2, _(" (FK)"));
 	else if(unit->event.type == LIVE_GAME_EVENT_OWN_GOAL)
-	    strcpy(buf2, " (OG)");
+	    strcpy(buf2, _(" (OG)"));
 	else 
 	    strcpy(buf2, "");
 
 	for(i=0;i<stats->players[array_index][LIVE_GAME_STAT_ARRAY_SCORERS]->len;i++)
 	{
-	    if(g_str_has_prefix(((GString*)g_ptr_array_index(
-				     stats->players[array_index][LIVE_GAME_STAT_ARRAY_SCORERS], i))->str,
-				player_of_id_team(tm[team], player)->name->str))
+	    if(g_str_has_prefix((gchar*)g_ptr_array_index(
+				    stats->players[array_index][LIVE_GAME_STAT_ARRAY_SCORERS], i),
+				player_of_id_team(tm[team], player)->name))
 	    {
 		sprintf(buf, "%s %d%s",
-			((GString*)g_ptr_array_index(
-			    stats->players[array_index][LIVE_GAME_STAT_ARRAY_SCORERS], i))->str,
+			(gchar*)g_ptr_array_index(
+			    stats->players[array_index][LIVE_GAME_STAT_ARRAY_SCORERS], i),
 			minute, buf2);
-		g_string_printf(((GString*)g_ptr_array_index(
-				     stats->players[array_index][LIVE_GAME_STAT_ARRAY_SCORERS], i)),
-				"%s", buf);
+		misc_string_assign((gchar**)&g_ptr_array_index(
+				       stats->players[array_index][LIVE_GAME_STAT_ARRAY_SCORERS], i), buf);
 		return;
 	    }
 	}
     
-	sprintf(buf, "%s %d%s", player_of_id_team(tm[team], player)->name->str,
+	sprintf(buf, "%s %d%s", player_of_id_team(tm[team], player)->name,
 		minute, buf2);
-	new = g_string_new(buf);
-	g_ptr_array_add(stats->players[array_index][LIVE_GAME_STAT_ARRAY_SCORERS], new);
+	g_ptr_array_add(stats->players[array_index][LIVE_GAME_STAT_ARRAY_SCORERS], g_strdup(buf));
     }
     else
     {
 	strcpy(buf, "");
 	if(unit->event.type == LIVE_GAME_EVENT_INJURY)
 	{
-	    sprintf(buf, "%s", player_of_id_team(tm[team], player)->name->str);
+	    sprintf(buf, "%s", player_of_id_team(tm[team], player)->name);
 	    players = stats->players[team][LIVE_GAME_STAT_ARRAY_INJURED];
 	}
 	else if(unit->event.type == LIVE_GAME_EVENT_FOUL_YELLOW)
 	{
-	    sprintf(buf, "%s", player_of_id_team(tm[team], player2)->name->str);
+	    sprintf(buf, "%s", player_of_id_team(tm[team], player2)->name);
 	    players = stats->players[team][LIVE_GAME_STAT_ARRAY_YELLOWS];
 	}
 	else if(unit->event.type == LIVE_GAME_EVENT_SEND_OFF)
 	{
-	    sprintf(buf, "%s", player_of_id_team(tm[team], player)->name->str);
+	    sprintf(buf, "%s", player_of_id_team(tm[team], player)->name);
 	    players = stats->players[team][LIVE_GAME_STAT_ARRAY_REDS];
 	}
 
 	if(strlen(buf) > 0)
-	{
-	    new = g_string_new(buf);
-	    g_ptr_array_add(players, new);
-	}
+	    g_ptr_array_add(players, strdup(buf));
     }
 }
 
@@ -927,6 +976,12 @@ game_post_match(Fixture *fix)
     GPtrArray *teams = NULL;
     Cup *cup = NULL;
 
+    if((debug > 100 && fixture_user_team_involved(fix) != -1) ||
+       debug > 130)
+	printf("game_post_match: %s - %s\n", 
+	       fix->teams[0]->name,
+	       fix->teams[1]->name);
+
     if(query_fixture_has_tables(fix))
 	table_update(fix);
     
@@ -936,7 +991,7 @@ game_post_match(Fixture *fix)
 	    team_update_cpu_team(fix->teams[i],
 				 (fixture_user_team_involved(fix) != -1));
 	else
-	    team_update_post_match(fix->teams[i], fix->clid);
+	    team_update_post_match(fix->teams[i], fix);
     }
 
     if(fix->clid < ID_CUP_START)
@@ -952,11 +1007,11 @@ game_post_match(Fixture *fix)
 	if(team_is_user((Team*)g_ptr_array_index(teams, 0)) != -1)
 	    user_history_add(&usr(team_is_user((Team*)g_ptr_array_index(teams, 0))),
 			     USER_HISTORY_WIN_FINAL, ((Team*)g_ptr_array_index(teams, 0))->id,
-			     fix->clid, fix->round,((Team*)g_ptr_array_index(teams, 1))->name->str);	    
+			     fix->clid, fix->round,((Team*)g_ptr_array_index(teams, 1))->name);
 	else if(team_is_user((Team*)g_ptr_array_index(teams, 1)) != -1)
 	    user_history_add(&usr(team_is_user((Team*)g_ptr_array_index(teams, 1))),
 			     USER_HISTORY_LOSE_FINAL, ((Team*)g_ptr_array_index(teams, 1))->id,
-			     fix->clid, fix->round,((Team*)g_ptr_array_index(teams, 0))->name->str);
+			     fix->clid, fix->round,((Team*)g_ptr_array_index(teams, 0))->name);
 	g_ptr_array_free(teams, TRUE);
     }
     else if(fixture_user_team_involved(fix) != -1)
@@ -992,15 +1047,18 @@ game_stadium_event(Stadium *stadium, gint type)
 /** Return the maximum possible values for defence, midfield and attack. 
     Used in the opponent preview. */
 void
-game_get_max_values(gfloat max_values[3])
+game_get_max_values(gfloat max_values[4])
 {
     gint i, j;
     Player pl;
 
     pl.skill = pl.cskill = const_float("float_player_max_skill");
     pl.fitness = 1;
+    pl.streak = 0;
 
-    for(i=0;i<3;i++)
+    max_values[0] = const_float("float_player_max_skill");
+
+    for(i=1;i<4;i++)
     {
 	max_values[i] = 0;
 	for(j=1;j<11;j++)
