@@ -15,12 +15,13 @@
 #include "window.h"
 
 /** Show the live game in the live game window.
-    @param unit The current unit we show.
-    @param sleep_factor The factor determining the live game speed. */
+    @param unit The current unit we show. */
 void
 game_gui_live_game_show_unit(const LiveGameUnit *unit)
 {
     gchar buf[SMALL];
+    gfloat sleep_factor = (unit->time == 3) ? 
+	const_float("float_game_gui_live_game_speed_penalties_factor") : 1;
     gfloat fraction = (gfloat)live_game_unit_get_minute(unit) / 90;
     GtkProgressBar *progress_bar =
 	GTK_PROGRESS_BAR(lookup_widget(window.live, "progressbar_live"));
@@ -54,9 +55,10 @@ game_gui_live_game_show_unit(const LiveGameUnit *unit)
     sprintf(buf, "%d.", live_game_unit_get_minute(unit));
     gtk_progress_bar_set_fraction(progress_bar, (fraction > 1) ? 1 : fraction);
     gtk_progress_bar_set_text(progress_bar, buf);
-    g_usleep(const_int("int_game_gui_live_game_speed_base") +
-	     (option_int("int_opt_user_live_game_speed", &usr(stat2).options) * 
-	      const_int("int_game_gui_live_game_speed_step")));
+    g_usleep((gint)rint(sleep_factor * 
+			(gfloat)(const_int("int_game_gui_live_game_speed_max") +
+				 (option_int("int_opt_user_live_game_speed", &usr(stat2).options) * 
+				  const_int("int_game_gui_live_game_speed_grad")))));
 
     while(gtk_events_pending())
 	gtk_main_iteration();
@@ -73,6 +75,7 @@ game_gui_live_game_show_unit(const LiveGameUnit *unit)
 	gtk_widget_set_sensitive(button_live_close, TRUE);
 	gtk_widget_set_sensitive(button_pause, FALSE);
 	gtk_widget_set_sensitive(button_resume, FALSE);
+	gui_set_sensitive_lg_meters(FALSE);
 	gtk_widget_grab_focus(button_live_close);
 
 	game_gui_set_main_window_sensitivity(FALSE);
@@ -122,7 +125,7 @@ game_gui_live_game_set_hscale(const LiveGameUnit *unit, GtkHScale *hscale)
 			     ((unit->possession == 0) ? 1 : -1)));
     }
     else if(unit->event.type == LIVE_GAME_EVENT_POST ||
-	    unit->event.type == LIVE_GAME_EVENT_MISSED ||
+	    unit->event.type == LIVE_GAME_EVENT_MISS ||
 	    unit->event.type == LIVE_GAME_EVENT_SAVE ||
 	    unit->event.type == LIVE_GAME_EVENT_CROSS_BAR)
 	gdk_color_parse(const_app("string_game_gui_live_game_scale_color_miss"), &color);
@@ -153,7 +156,8 @@ game_gui_live_game_set_hscale(const LiveGameUnit *unit, GtkHScale *hscale)
 /** Look up the widgets in the main window. */
 void
 game_gui_get_radio_items(GtkWidget **style, GtkWidget **scout,
-			 GtkWidget **physio, GtkWidget **boost)
+			 GtkWidget **physio, GtkWidget **boost,
+			 GtkWidget **yc, GtkWidget **ya_pos_pref)
 {
     style[0] = lookup_widget(window.main, "menu_all_out_defend");
     style[1] = lookup_widget(window.main, "menu_defend");
@@ -174,13 +178,24 @@ game_gui_get_radio_items(GtkWidget **style, GtkWidget **scout,
     boost[0] = lookup_widget(window.main, "menu_boost_anti");
     boost[1] = lookup_widget(window.main, "menu_boost_off");
     boost[2] = lookup_widget(window.main, "menu_boost_on");
+
+    yc[0] = lookup_widget(window.main, "menu_yc_best");
+    yc[1] = lookup_widget(window.main, "menu_yc_good");
+    yc[2] = lookup_widget(window.main, "menu_yc_average");
+    yc[3] = lookup_widget(window.main, "menu_yc_bad");
+
+    ya_pos_pref[0] = lookup_widget(window.main, "menu_recruit0");
+    ya_pos_pref[1] = lookup_widget(window.main, "menu_recruit1");
+    ya_pos_pref[2] = lookup_widget(window.main, "menu_recruit2");
+    ya_pos_pref[3] = lookup_widget(window.main, "menu_recruit3");
+    ya_pos_pref[4] = lookup_widget(window.main, "menu_recruit4");
 }
 
 /** Set information like season, user, week etc. into the appropriate labels. */
 void
 game_gui_set_main_window_header(void)
 {
-    gint i;
+    gint i, rank;
     gchar buf[SMALL];
     GtkLabel *label_user= GTK_LABEL(lookup_widget(window.main, "label_user")),
 	*label_season= GTK_LABEL(lookup_widget(window.main, "label_season")),
@@ -193,20 +208,34 @@ game_gui_set_main_window_header(void)
     GtkWidget *menu_users[2] = {lookup_widget(window.main, "menu_next_user"),
 				lookup_widget(window.main, "menu_previous_user")};
 
-    gtk_label_set_text(label_user, current_user.name->str);
+    gtk_label_set_text(label_user, current_user.name);
     gui_label_set_text_from_int(label_season, season, FALSE);
     gui_label_set_text_from_int(label_week, week, FALSE);
     gui_label_set_text_from_int(label_round, week_round, FALSE);
     gui_label_set_text_from_int(label_rank, week_round, FALSE);
 
-    misc_print_grouped_int(current_user.money, buf, FALSE);
-    gtk_label_set_text(label_money, buf);
+    if(!sett_int("int_opt_disable_finances"))
+    {
+	misc_print_grouped_int(current_user.money, buf);
+	gtk_label_set_text(label_money, buf);
+    }
+    else
+    {
+	gtk_widget_hide(GTK_WIDGET(label_money));
+	gtk_widget_hide(GTK_WIDGET(lookup_widget(window.main, "label34")));
+    }
 
-    gui_label_set_text_from_int(label_rank, 
-				team_get_league_rank(current_user.tm), FALSE);
+    rank = team_get_league_rank(current_user.tm);
+    if(rank != 0)
+	gui_label_set_text_from_int(label_rank, rank, FALSE);
+    else
+    {
+	gtk_widget_hide(GTK_WIDGET(label_rank));
+	gtk_widget_hide(lookup_widget(window.main, "label29"));
+    }
 
-    gtk_label_set_text(label_team, current_user.tm->name->str);
-    gtk_label_set_text(label_league, league_from_clid(current_user.tm->clid)->name->str);    
+    gtk_label_set_text(label_team, current_user.tm->name);
+    gtk_label_set_text(label_league, league_cup_get_name_string(current_user.tm->clid));
 
     for(i=0;i<2;i++)
 	gtk_widget_set_sensitive(menu_users[i], (users->len > 1));
@@ -215,7 +244,7 @@ game_gui_set_main_window_header(void)
 
     game_gui_write_radio_items();
 
-    game_gui_write_meters();
+    game_gui_write_meters(current_user.tm);
 
     game_gui_write_check_items();
 }
@@ -234,28 +263,17 @@ game_gui_write_av_skills(void)
     gtk_label_set_text(label_av_skills, buf);
 }
 
-/** Activate the appropriate radio items for
-    playing style etc. according to the user settings. */
+/** Set the images for the style and boost meters to the appropriate values
+    from the team settings. */
 void
-game_gui_write_radio_items(void)
-{
-    GtkWidget *style[5], *scout[4], *physio[4], *boost[3];
-
-    game_gui_get_radio_items(style, scout, physio, boost);
-
-    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(style[current_user.tm->style + 2]), TRUE);
-    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(scout[current_user.scout % 10]), TRUE);
-    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(physio[current_user.physio % 10]), TRUE);
-    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(boost[current_user.tm->boost + 1]), TRUE);
-}
-
-/** Set the appropriate images for the style and boost meters. */
-void
-game_gui_write_meters(void)
+game_gui_write_meters(const Team *tm)
 {
     gint i;
-    GtkImage *image_style = GTK_IMAGE(lookup_widget(window.main, "image_style")),
-	*image_boost = GTK_IMAGE(lookup_widget(window.main, "image_boost"));
+    GtkImage *image_style_main = GTK_IMAGE(lookup_widget(window.main, "image_style")),
+	*image_boost_main = GTK_IMAGE(lookup_widget(window.main, "image_boost"));
+    GtkImage *image_style_live = NULL,
+	*image_boost_live = NULL;
+
     gchar *image_style_files[5] = 
 	{file_find_support_file(const_app("string_game_gui_style_all_out_defend_icon"), TRUE),
 	 file_find_support_file(const_app("string_game_gui_style_defend_icon"), TRUE),
@@ -267,8 +285,17 @@ game_gui_write_meters(void)
 	 file_find_support_file(const_app("string_game_gui_boost_off_icon"), TRUE),
 	 file_find_support_file(const_app("string_game_gui_boost_on_icon"), TRUE)};
 
-    gtk_image_set_from_file(image_style, image_style_files[current_user.tm->style + 2]);
-    gtk_image_set_from_file(image_boost, image_boost_files[current_user.tm->boost + 1]);
+    gtk_image_set_from_file(image_style_main, image_style_files[tm->style + 2]);
+    gtk_image_set_from_file(image_boost_main, image_boost_files[tm->boost + 1]);
+
+    if(window.live != NULL)
+    {
+	image_style_live = GTK_IMAGE(lookup_widget(window.live, "image_lg_style"));
+	image_boost_live = GTK_IMAGE(lookup_widget(window.live, "image_lg_boost"));
+
+	gtk_image_set_from_file(image_style_live, image_style_files[tm->style + 2]);
+	gtk_image_set_from_file(image_boost_live, image_boost_files[tm->boost + 1]);
+    }
 
     for(i=0;i<5;i++)
 	g_free(image_style_files[i]);
@@ -276,6 +303,30 @@ game_gui_write_meters(void)
 	g_free(image_boost_files[i]);
 }
 
+/** Activate the appropriate radio items for
+    playing style etc. according to the user settings. */
+void
+game_gui_write_radio_items(void)
+{
+    GtkWidget *style[5], *scout[4], *physio[4], 
+	*boost[3], *yc[4], *ya_pos_pref[5];
+
+    game_gui_get_radio_items(style, scout, physio, boost, yc, ya_pos_pref);
+
+    gtk_check_menu_item_set_active(
+	GTK_CHECK_MENU_ITEM(style[current_user.tm->style + 2]), TRUE);
+    gtk_check_menu_item_set_active(
+	GTK_CHECK_MENU_ITEM(scout[current_user.scout % 10]), TRUE);
+    gtk_check_menu_item_set_active(
+	GTK_CHECK_MENU_ITEM(physio[current_user.physio % 10]), TRUE);
+    gtk_check_menu_item_set_active(
+	GTK_CHECK_MENU_ITEM(boost[current_user.tm->boost + 1]), TRUE);
+    gtk_check_menu_item_set_active(
+	GTK_CHECK_MENU_ITEM(yc[current_user.youth_academy.coach % 10]), TRUE);
+    gtk_check_menu_item_set_active(
+	GTK_CHECK_MENU_ITEM(ya_pos_pref[current_user.youth_academy.pos_pref]), TRUE);
+}
+
 /** Set playing style etc. variables according to
     the items.
     @param widget The widget that received a click. */
@@ -283,39 +334,75 @@ void
 game_gui_read_radio_items(GtkWidget *widget)
 {
     gint i;
-    GtkWidget *boost[3];
+    GtkWidget *boost[3], *yc[4], *ya_pos_pref[5];
     GtkWidget *style[5], *scout[4], *physio[4];
     gint old_scout = current_user.scout,
-	old_physio = current_user.physio;
+	old_physio = current_user.physio,
+	old_yc = current_user.youth_academy.coach;
 
-    game_gui_get_radio_items(style, scout, physio, boost);
+    game_gui_get_radio_items(style, scout, physio, boost, yc, ya_pos_pref);
 
     for(i=0;i<3;i++)
 	if(widget == boost[i])
+	{
 	    current_user.tm->boost = i - 1;
 
+	    if(current_user.tm->boost == 1 && sett_int("int_opt_disable_boost_on"))
+	    {
+		current_user.tm->boost = 0;
+		game_gui_print_message(_("Boost ON is disabled in this country definition."));
+	    }
+	}
+
     for(i=0;i<5;i++)
 	if(widget == style[i])
 	    current_user.tm->style = i - 2;
     
-    for(i=0;i<4;i++)
-	if(widget == scout[i])
-	    current_user.scout = 100 + i * 10 + old_scout % 10;
+    if(!sett_int("int_opt_disable_transfers"))
+    {
+	for(i=0;i<4;i++)
+	    if(widget == scout[i])
+		current_user.scout = 100 + i * 10 + old_scout % 10;
+    }
 
-    for(i=0;i<4;i++)
-	if(widget == physio[i])
-	    current_user.physio = 100 + i * 10 + old_physio % 10;
+    if(!sett_int("int_opt_disable_transfers"))
+    {
+	for(i=0;i<4;i++)
+	    if(widget == physio[i])
+		current_user.physio = 100 + i * 10 + old_physio % 10;
+    }
+
+    if(!sett_int("int_opt_disable_ya"))
+    {
+	for(i=0;i<4;i++)
+	    if(widget == yc[i])
+		current_user.youth_academy.coach = 100 + i * 10 + old_yc % 10;
+    }
     
+    if(!sett_int("int_opt_disable_ya"))
+    {
+	for(i=0;i<5;i++)
+	    if(widget == ya_pos_pref[i])
+		current_user.youth_academy.pos_pref = i;
+    }
+
     if(math_get_place(current_user.scout, 2) == old_scout % 10)
 	current_user.scout = old_scout % 10;
 
     if(math_get_place(current_user.physio, 2) == old_physio % 10)
 	current_user.physio = old_physio % 10;
 
+    if(math_get_place(current_user.youth_academy.coach, 2) == old_yc % 10)
+	current_user.youth_academy.coach = old_yc % 10;
+
     if(old_scout != current_user.scout ||
-       old_physio != current_user.physio)
+       old_physio != current_user.physio ||
+       old_yc != current_user.youth_academy.coach)
 	game_gui_print_message(_("Next week you'll fire him and hire a new one."));
 
+    game_gui_write_meters(current_user.tm);
+    game_gui_write_radio_items();
+
     treeview_show_next_opponent();
 }
 
@@ -339,14 +426,26 @@ game_gui_show_main(void)
     }
     else
 	treeview_show_next_opponent();
+
+    if(current_user.counters[COUNT_USER_NEW_SPONSOR] == 1)
+	user_show_sponsors();
+    else if(current_user.counters[COUNT_USER_NEW_SPONSOR] == -1)
+	user_show_sponsor_continue();
 }
 
-/** Print a message into the main window entry. */
+/** Print a message in the message area. */
 void
-game_gui_print_message(gchar *text)
+game_gui_print_message(gchar *format, ...)
 {
-    gtk_entry_set_text(GTK_ENTRY(lookup_widget(window.main, "entry_message")), text);
+    gchar text[SMALL];
+    va_list args;
+     
+    va_start (args, format);
+    g_vsprintf(text, format, args);
+    va_end (args);
 
+    gtk_entry_set_text(GTK_ENTRY(lookup_widget(window.main, "entry_message")), text);
+    
     if(timeout_id != -1)
 	g_source_remove(timeout_id);
 
@@ -354,6 +453,36 @@ game_gui_print_message(gchar *text)
 			       (GSourceFunc)game_gui_clear_entry_message, NULL);
 }
 
+/** Source function for the delay printing function. */
+gboolean
+game_gui_print_message_source(gpointer data)
+{
+    game_gui_print_message((gchar*)data);
+
+    g_free(data);
+
+    return FALSE;
+}
+
+/** Print a message after some seconds of delay. */
+void
+game_gui_print_message_with_delay(const gchar *format, ...)
+{
+    gchar text[SMALL];
+    va_list args;
+     
+    va_start (args, format);
+    g_vsprintf(text, format, args);
+    va_end (args);
+
+    if(timeout_id != -1)
+	g_source_remove(timeout_id);
+
+    g_timeout_add(const_int("int_game_gui_message_delay") * 1000,
+		  (GSourceFunc)game_gui_print_message_source,
+		  (gpointer)g_strdup(text));
+}
+
 /** Function that gets called from time to time. */
 gboolean
 game_gui_clear_entry_message(gpointer data)
@@ -467,15 +596,22 @@ game_gui_set_main_window_sensitivity(gboolean value)
 /** Show a window with a warning.
     @param text The text to show in the window. */
 void
-game_gui_show_warning(gchar *text)
+game_gui_show_warning(const gchar *format, ...)
 {
+    gchar text[SMALL];
+    va_list args;
+     
+    va_start (args, format);
+    g_vsprintf(text, format, args);
+    va_end (args);
+
     if(opt_int("int_opt_prefer_messages") &&
        window.main != NULL)
 	game_gui_print_message(text);
     else
     {
-	window_create(WINDOW_WARNING);	
-	gtk_label_set_text(GTK_LABEL(lookup_widget(window.warning, "label_warning")), text);    
+	window_create(WINDOW_WARNING);
+	gtk_label_set_text(GTK_LABEL(lookup_widget(window.warning, "label_warning")), text);
     }
 }
 
@@ -500,17 +636,18 @@ game_gui_show_job_offer(Team *team, gint type)
     label_money = GTK_LABEL(lookup_widget(window.job_offer, "label_money"));
     label_cap = GTK_LABEL(lookup_widget(window.job_offer, "label_cap"));
     label_saf = GTK_LABEL(lookup_widget(window.job_offer, "label_saf"));
-    label_average_skill = GTK_LABEL(lookup_widget(window.job_offer, "label_average_skill"));
+    label_average_skill = 
+	GTK_LABEL(lookup_widget(window.job_offer, "label_average_skill"));
 
     if(type == STATUS_JOB_OFFER_FIRE_FINANCE)
 	sprintf(buf, _("The team owners have fired you because of financial mismanagement. Luckily, the owners of %s have heard of your dismissal and offer you a job. Here's some information on %s:"),
-		team->name->str, team->name->str);
+		team->name, team->name);
     else if(type == STATUS_JOB_OFFER_FIRE_FAILURE)
 	sprintf(buf, _("The team owners have fired you because of unsuccessfulness. Luckily, the owners of %s have heard of your dismissal and offer you a job. Here's some information on %s:"),
-		team->name->str, team->name->str);
+		team->name, team->name);
     else if(type == STATUS_JOB_OFFER_SUCCESS)
 	sprintf(buf, _("The owners of %s are deeply impressed by your success with %s and would like to hire you. Here's some information on %s:"),
-		team->name->str, current_user.tm->name->str, team->name->str);
+		team->name, current_user.tm->name, team->name);
     
     strcpy(buf2, _("Accept?"));
     if(type != STATUS_JOB_OFFER_SUCCESS)
@@ -518,15 +655,15 @@ game_gui_show_job_offer(Team *team, gint type)
 
     gtk_label_set_text(label_text, buf);
     gtk_label_set_text(label_text2, buf2);
-    gtk_label_set_text(label_name, team->name->str);
-    gtk_label_set_text(label_league, league_from_clid(team->clid)->name->str);
+    gtk_label_set_text(label_name, team->name);
+    gtk_label_set_text(label_league, league_cup_get_name_string(team->clid));
     gui_label_set_text_from_int(label_rank, team_get_league_rank(team), FALSE);
     misc_print_grouped_int(math_round_integer(team->stadium.capacity * 
 					      math_rndi(const_int("int_initial_money_lower"),
 							const_int("int_initial_money_upper")), 2),
-			   buf, FALSE);
+			   buf);
     gtk_label_set_text(label_money, buf);
-    misc_print_grouped_int(team->stadium.capacity, buf, FALSE);
+    misc_print_grouped_int(team->stadium.capacity, buf);
     gtk_label_set_text(label_cap, buf);
     gui_label_set_text_from_int(label_saf, (gint)rint(team->stadium.safety * 100), FALSE);
 
@@ -544,19 +681,23 @@ game_gui_write_check_items(void)
 {
     GtkCheckMenuItem *menu_job_offers = 
 	GTK_CHECK_MENU_ITEM(lookup_widget(window.main, "menu_job_offers")),
-	*menu_live_game = GTK_CHECK_MENU_ITEM(lookup_widget(window.main, "menu_live_game")),
-	*menu_overwrite = GTK_CHECK_MENU_ITEM(lookup_widget(window.main, "menu_overwrite"));
+	*menu_live_game = 
+	GTK_CHECK_MENU_ITEM(lookup_widget(window.main, "menu_live_game")),
+	*menu_overwrite = 
+	GTK_CHECK_MENU_ITEM(lookup_widget(window.main, "menu_overwrite"));
 
-    gtk_check_menu_item_set_active(menu_job_offers, opt_user_int("int_opt_user_show_job_offers"));
-    gtk_check_menu_item_set_active(menu_live_game, opt_user_int("int_opt_user_show_live_game"));
-    gtk_check_menu_item_set_active(menu_overwrite, opt_int("int_opt_save_will_overwrite"));
+    gtk_check_menu_item_set_active(menu_job_offers,
+				   opt_user_int("int_opt_user_show_job_offers"));
+    gtk_check_menu_item_set_active(menu_live_game,
+				   opt_user_int("int_opt_user_show_live_game"));
+    gtk_check_menu_item_set_active(menu_overwrite,
+				   opt_int("int_opt_save_will_overwrite"));
 }
 
 /** Change the options according to the check menu widgets. */
 void
 game_gui_read_check_items(GtkWidget *widget)
 {
-    gchar buf[SMALL];
     GtkWidget *menu_job_offers = 
 	lookup_widget(window.main, "menu_job_offers"),
 	*menu_live_game = lookup_widget(window.main, "menu_live_game"),
@@ -564,29 +705,33 @@ game_gui_read_check_items(GtkWidget *widget)
 
     if(widget == menu_job_offers)
     {
-	opt_user_set_int("int_opt_user_show_job_offers", !opt_user_int("int_opt_user_show_job_offers"));
-	sprintf(buf, _("Job offers set to %s."),
-		team_attribute_to_char(TEAM_ATTRIBUTE_BOOST, 
-				       opt_user_int("int_opt_user_show_job_offers")));
+	opt_user_set_int("int_opt_user_show_job_offers", 
+			 !opt_user_int("int_opt_user_show_job_offers"));
+	game_gui_print_message(_("Job offers set to %s."),
+			       team_attribute_to_char(
+				   TEAM_ATTRIBUTE_BOOST, 
+				   opt_user_int("int_opt_user_show_job_offers")));
     }
     else if(widget == menu_live_game)
     {
-	opt_user_set_int("int_opt_user_show_live_game", !opt_user_int("int_opt_user_show_live_game"));
-	sprintf(buf, _("Live game set to %s."),
-		team_attribute_to_char(TEAM_ATTRIBUTE_BOOST, 
-				       opt_user_int("int_opt_user_show_live_game")));
+	opt_user_set_int("int_opt_user_show_live_game", 
+			 !opt_user_int("int_opt_user_show_live_game"));
+	game_gui_print_message(_("Live game set to %s."),
+			       team_attribute_to_char(
+				   TEAM_ATTRIBUTE_BOOST, 
+				   opt_user_int("int_opt_user_show_live_game")));
     }
     else if(widget == menu_overwrite)
     {
-	opt_set_int("int_opt_save_will_overwrite", !opt_int("int_opt_save_will_overwrite"));
-	sprintf(buf, _("Overwrite set to %s."),
-		team_attribute_to_char(TEAM_ATTRIBUTE_BOOST, 
-				       opt_int("int_opt_save_will_overwrite")));
+	opt_set_int("int_opt_save_will_overwrite", 
+		    !opt_int("int_opt_save_will_overwrite"));
+	game_gui_print_message(_("Overwrite set to %s."),
+			       team_attribute_to_char(
+				   TEAM_ATTRIBUTE_BOOST, 
+				   opt_int("int_opt_save_will_overwrite")));
     }
     else
 	g_warning("game_gui_read_check_items: unknown widget.");
-
-    game_gui_print_message(buf);
 }
 
 /** Set the appropriate text into the labels in the help window. 
@@ -594,7 +739,8 @@ game_gui_read_check_items(GtkWidget *widget)
 void
 game_gui_set_help_labels(void)
 {
-    GtkLabel *label_help_text1 = GTK_LABEL(lookup_widget(window.help, "label_help_text1")),
+    GtkLabel *label_help_text1 = 
+	GTK_LABEL(lookup_widget(window.help, "label_help_text1")),
 	*label_help_text2 = GTK_LABEL(lookup_widget(window.help, "label_help_text2"));
     GString *text = g_string_new("");
 
