@@ -3,10 +3,12 @@
 #include "free.h"
 #include "game.h"
 #include "game_gui.h"
+#include "league.h"
 #include "lg_commentary.h"
 #include "live_game.h"
 #include "main.h"
 #include "maths.h"
+#include "misc.h"
 #include "misc_callback_func.h"
 #include "option.h"
 #include "player.h"
@@ -18,7 +20,8 @@
 #include "window.h"
 
 /** The live game we calculate. */
-LiveGame *match;
+#define match ((LiveGame*)statp)
+
 /** Whether the events are actually shown or not. */
 gboolean show;
 
@@ -38,41 +41,19 @@ gboolean show;
 void
 live_game_calculate_fixture(Fixture *fix)
 {
-    if(stat0 != STATUS_LIVE_GAME_PAUSE)
-    {
-	match = (fixture_user_team_involved(fix) != -1) ? 
-	    &usr(fixture_user_team_involved(fix)).live_game : &live_game_temp;
-	show = (fixture_user_team_involved(fix) != -1 && 
-		option_int("int_opt_user_show_live_game", 
-			   &usr(fixture_user_team_involved(fix)).options));
-
-	stat2 = fixture_user_team_involved(fix);
-	statp = match;
-
-	live_game_reset(match, fix, TRUE);
-
-	if(show)
-	{
-	    cur_user = stat2;
-	    on_button_back_to_main_clicked(NULL, NULL);
-
-	    if(window.live == NULL)
-		window.live = window_create(WINDOW_LIVE);
-	    else
-		window_live_set_spinbuttons();
-	}
-
-	game_initialize(fix);
-    }
+    if(stat0 != STATUS_LIVE_GAME_PAUSE && 
+       stat0 != STATUS_LIVE_GAME_CHANGE)
+	live_game_initialize(fix);
     else
 	stat0 = STATUS_SHOW_LIVE_GAME;
 
     game_get_values(match->fix, match->team_values,
 		    match->home_advantage);
 
-    if(debug > 80 && fixture_user_team_involved(match->fix) != -1)
+    if((debug > 80 && fixture_user_team_involved(match->fix) != -1) ||
+       debug > 130)
 	printf("\n\nlive_game_calculate_fixture\n%04d %s %s %04d\n\n",
-	       tm0->id, tm0->name->str, tm1->name->str, tm1->id);
+	       tm0->id, tm0->name, tm1->name, tm1->id);
 
     do
     {
@@ -80,10 +61,50 @@ live_game_calculate_fixture(Fixture *fix)
 	live_game_evaluate_unit(&last_unit);
     }
     while(last_unit.event.type != LIVE_GAME_EVENT_END_MATCH &&
-	  stat0 != STATUS_LIVE_GAME_PAUSE);
+	  stat0 != STATUS_LIVE_GAME_PAUSE &&
+	  stat0 != STATUS_LIVE_GAME_CHANGE);
 
     if(last_unit.event.type == LIVE_GAME_EVENT_END_MATCH)
+    {
+	if(fixture_user_team_involved(match->fix) != -1 || stat5 < -1000)
+	    lg_commentary_post_match();
 	game_post_match(fix);
+    }
+    else if(stat0 == STATUS_LIVE_GAME_CHANGE)
+	live_game_resume();
+}
+
+/** Initialize a few things at the beginning of a live game. */
+void
+live_game_initialize(Fixture *fix)
+{
+    statp= (fixture_user_team_involved(fix) != -1) ? 
+	&usr(fixture_user_team_involved(fix)).live_game : &live_game_temp;
+    show = (fixture_user_team_involved(fix) != -1 && 
+	    option_int("int_opt_user_show_live_game", 
+		       &usr(fixture_user_team_involved(fix)).options));
+
+    stat2 = fixture_user_team_involved(fix);
+
+    live_game_reset(match, fix, TRUE);
+
+    if(show)
+    {
+	cur_user = stat2;
+	on_button_back_to_main_clicked(NULL, NULL);
+
+	if(window.live == NULL)
+	    window.live = window_create(WINDOW_LIVE);
+	else
+	    gtk_window_set_title(GTK_WINDOW(window.live),
+				 league_cup_get_name_string(((LiveGame*)statp)->fix->clid));
+	window_live_set_up();
+    }
+
+    game_initialize(fix);
+    
+    if(fixture_user_team_involved(match->fix) != -1 || stat5 < -1000)
+	lg_commentary_initialize(fix);
 }
 
 /** Create a game unit for the live game.
@@ -93,7 +114,8 @@ live_game_create_unit(void)
 {
     LiveGameUnit new;
 
-    if(debug > 100 && fixture_user_team_involved(match->fix) != -1)
+    if((debug > 100 && fixture_user_team_involved(match->fix) != -1) ||
+       debug > 130)
 	printf("\t\tlive_game_create_unit\n");
 
     if(unis->len == 0)
@@ -152,7 +174,8 @@ live_game_fill_new_unit(LiveGameUnit *new)
     gfloat possession_change, scoring_chance = 0, 
 	injury_event_prob, foul_event_prob;
 
-    if(debug > 100 && fixture_user_team_involved(match->fix) != -1)
+    if((debug > 100 && fixture_user_team_involved(match->fix) != -1) ||
+       debug > 130)
 	printf("\t\tlive_game_fill_new_unit\n");
 
     possession_change = const_float("float_live_game_event_general") *
@@ -211,13 +234,14 @@ live_game_create_start_unit(void)
 {
     LiveGameUnit new;
 
-    if(debug > 100 && fixture_user_team_involved(match->fix) != -1)
+    if((debug > 100 && fixture_user_team_involved(match->fix) != -1) ||
+       debug > 130)
 	printf("\t\tlive_game_create_start_unit\n");
     new.event.player =
 	new.event.player2 = -1;
 
-    new.event.commentary = NULL;
     new.minute = 0;
+    new.event.commentary = NULL;
     new.time = LIVE_GAME_UNIT_TIME_FIRST_HALF;
     new.possession = math_rndi(0, 1);
     new.area = LIVE_GAME_UNIT_AREA_MIDFIELD;
@@ -239,7 +263,8 @@ live_game_evaluate_unit(LiveGameUnit *unit)
 {
     gint type = unit->event.type;
 
-    if(debug > 100 && fixture_user_team_involved(match->fix) != -1)
+    if((debug > 100 && fixture_user_team_involved(match->fix) != -1) ||
+       debug > 130)
 	printf("\t\tlive_game_evaluate_unit\n");
     if(type == LIVE_GAME_EVENT_FOUL)
 	live_game_event_foul();
@@ -272,16 +297,15 @@ live_game_evaluate_unit(LiveGameUnit *unit)
 		  type);
 }
 
-/** Calculate a foul event.
-    @param general Whether to create a general event after
-    showing this one. @see live_game_event_general() */
+/** Calculate a foul event. */
 void
 live_game_event_foul(void)
 {
     gfloat rndom = math_rnd(0, 1);
     gint type, fouled_player, foul_player, foul_team;
 
-    if(debug > 100 && fixture_user_team_involved(match->fix) != -1)
+    if((debug > 100 && fixture_user_team_involved(match->fix) != -1) ||
+       debug > 130)
 	printf("\t\tlive_game_event_foul\n");
     if(math_rnd(0, 1) > const_float("float_live_game_foul_by_possession") *
        game_get_foul_possession_factor(
@@ -308,7 +332,7 @@ live_game_event_foul(void)
 			    last_unit.area, 0, -1, FALSE);
 	foul_player = last_unit.event.player2 =
 	    game_get_player(tm[last_unit.possession],
-				      last_unit.area, 0, -1, FALSE);
+			    last_unit.area, 0, -1, FALSE);
     }
 
     if(rndom < const_float("float_live_game_foul_red_injury"))
@@ -316,7 +340,12 @@ live_game_event_foul(void)
     else if(rndom < const_float("float_live_game_foul_red"))
 	type = LIVE_GAME_EVENT_FOUL_RED;
     else if(rndom < const_float("float_live_game_foul_yellow"))
+    {
 	type = LIVE_GAME_EVENT_FOUL_YELLOW;
+	player_card_set(player_of_id_team(tm[foul_team], foul_player),
+			match->fix->clid, PLAYER_VALUE_CARD_YELLOW, 1, TRUE);
+	player_of_id_team(tm[foul_team], foul_player)->career[PLAYER_VALUE_CARD_YELLOW]++;
+    }
     else
 	type = LIVE_GAME_EVENT_FOUL;
 
@@ -334,12 +363,6 @@ live_game_event_foul(void)
 	if(type == LIVE_GAME_EVENT_FOUL_RED_INJURY)
 	    live_game_event_injury(!foul_team, fouled_player, TRUE);
     }
-    else if(type == LIVE_GAME_EVENT_FOUL_YELLOW)
-    {
-	player_card_set(player_of_id_team(tm[foul_team], foul_player),
-			match->fix->clid, PLAYER_VALUE_CARD_YELLOW, 1, TRUE);
-	player_of_id_team(tm[foul_team], foul_player)->career[PLAYER_VALUE_CARD_YELLOW]++;
-    }
 
     if(last_unit.area == LIVE_GAME_UNIT_AREA_ATTACK && foul_team !=
        last_unit.possession)
@@ -360,7 +383,8 @@ live_game_event_foul(void)
 void
 live_game_event_lost_possession(void)
 {
-    if(debug > 100 && fixture_user_team_involved(match->fix) != -1)
+    if((debug > 100 && fixture_user_team_involved(match->fix) != -1) ||
+       debug > 130)
 	printf("\t\tlive_game_event_lost_possession\n");
     last_unit.event.player =
 	game_get_player(tm[last_unit.possession], 
@@ -391,7 +415,8 @@ live_game_event_injury(gint team, gint player, gboolean create_new)
     LiveGameUnit new;
     gint old_structure = -1;
 
-    if(debug > 100 && fixture_user_team_involved(match->fix) != -1)
+    if((debug > 100 && fixture_user_team_involved(match->fix) != -1) ||
+       debug > 130)
 	printf("\t\tlive_game_event_injury\n");
     if(create_new)
     {
@@ -432,7 +457,7 @@ live_game_event_injury(gint team, gint player, gboolean create_new)
     if(last_unit.event.type == LIVE_GAME_EVENT_INJURY)
     {
 	game_player_injury(player_of_id_team(tm[last_unit.event.team],
-					last_unit.event.player));
+					     last_unit.event.player));
 
 	if(match->subs_left[last_unit.event.team] > 0)
 	{
@@ -482,7 +507,8 @@ live_game_event_stadium(void)
     for(i=1;i<3;i++)
 	probs[i] += probs[i - 1];
 
-    if(debug > 100 && fixture_user_team_involved(match->fix) != -1)
+    if((debug > 100 && fixture_user_team_involved(match->fix) != -1) ||
+       debug > 130)
 	printf("\t\tlive_game_event_stadium\n");
     if(rndom <= probs[0])
 	last_unit.event.type = LIVE_GAME_EVENT_STADIUM_BREAKDOWN;
@@ -512,7 +538,8 @@ live_game_event_scoring_chance(void)
     else
 	res_idx = 0;
 
-    if(debug > 100 && fixture_user_team_involved(match->fix) != -1)
+    if((debug > 100 && fixture_user_team_involved(match->fix) != -1) ||
+       debug > 130)
 	printf("\t\tlive_game_event_scoring_chance\n");
 
     if(math_rnd(0, 1) < const_float("float_live_game_scoring_chance_is_own_goal"))
@@ -523,20 +550,26 @@ live_game_event_scoring_chance(void)
 	last_unit.event.team = !last_unit.possession;
 	match->fix->result[last_unit.possession][res_idx]++;
 	last_unit.result[last_unit.possession]++;
+
+	player_streak_add_to_prob(player_of_id_team(tm[last_unit.event.team],
+						    last_unit.event.player),
+				  const_float("float_player_streak_add_own_goal"));
     }
     else
     {
+	last_unit.event.team = last_unit.possession;
+
 	if(uni(unis->len - 2).event.player != -1 &&
 	   math_rnd(0, 1) < const_float("float_live_game_player_in_poss_shoots") &&
 	   query_player_id_in_team(uni(unis->len - 2).event.player,
-			     tm[last_unit.possession]))
+				   tm[last_unit.possession]))
 	    last_unit.event.player =
 		uni(unis->len - 2).event.player;
 	else
 	{
 	    if(uni(unis->len - 2).event.player != -1 &&
 	       query_player_id_in_team(uni(unis->len - 2).event.player,
-				 tm[last_unit.possession]))
+				       tm[last_unit.possession]))
 	    {
 		last_unit.event.player =
 		    game_get_player(tm[last_unit.possession], last_unit.area, 0, 
@@ -575,15 +608,16 @@ live_game_event_penalty(void)
 {
     LiveGameUnit new;
 
-    if(debug > 100 && fixture_user_team_involved(match->fix) != -1)
+    if((debug > 100 && fixture_user_team_involved(match->fix) != -1) ||
+       debug > 130)
 	printf("\t\tlive_game_event_penalty\n");
 
     if(last_unit.time != LIVE_GAME_UNIT_TIME_PENALTIES)
     {
 	new = last_unit;
+	new.event.commentary = NULL;
 	new.minute = -1;
 	new.event.type = LIVE_GAME_EVENT_PENALTY;
-	new.event.commentary = NULL;
 
 	g_array_append_val(unis, new);
     }
@@ -593,14 +627,14 @@ live_game_event_penalty(void)
 
     if(last_unit.time == LIVE_GAME_UNIT_TIME_PENALTIES)
     {
-	if(uni(unis->len - 2).event.type == LIVE_GAME_EVENT_PENALTIES)
+	if(live_game_penalties_taken() == 1)
 	{
 	    last_unit.possession = math_rndi(0, 1);
 	    last_unit.event.player =
 		game_get_player(tm[last_unit.possession],
 				GAME_PLAYER_TYPE_PENALTY, -1, -1, FALSE);
 	}
-	else if(uni(unis->len - 4).event.type == LIVE_GAME_EVENT_PENALTIES)
+	else if(live_game_penalties_taken() == 2)
 	{
 	    last_unit.possession = !uni(unis->len - 3).possession;
 	    last_unit.event.player =
@@ -640,14 +674,15 @@ live_game_event_general(gboolean create_new)
 {
     LiveGameUnit new;
 
-    if(debug > 100 && fixture_user_team_involved(match->fix) != -1)
+    if((debug > 100 && fixture_user_team_involved(match->fix) != -1) ||
+       debug > 130)
 	printf("\t\tlive_game_event_general\n");
     if(create_new)
     {
 	new.minute = live_game_get_minute();
 	new.time = last_unit.time;
+	new.event.commentary = NULL;
 	new.event.type = LIVE_GAME_EVENT_GENERAL;
-	new.event.commentary = NULL;	
 	new.result[0] = last_unit.result[0];
 	new.result[1] = last_unit.result[1];
 	new.event.team = -1;
@@ -674,7 +709,7 @@ live_game_event_general(gboolean create_new)
 	}
 	else if(last_unit.event.type == LIVE_GAME_EVENT_GOAL ||
 		last_unit.event.type == LIVE_GAME_EVENT_OWN_GOAL ||
-		last_unit.event.type == LIVE_GAME_EVENT_MISSED ||
+		last_unit.event.type == LIVE_GAME_EVENT_MISS ||
 		last_unit.event.type == LIVE_GAME_EVENT_SAVE ||
 		last_unit.event.type == LIVE_GAME_EVENT_POST ||
 		last_unit.event.type == LIVE_GAME_EVENT_CROSS_BAR)
@@ -723,16 +758,17 @@ live_game_event_general_get_players(void)
 	uni(unis->len - 2).event.player;
     gint type = uni(unis->len - 2).event.type;
 	
-    if(debug > 100 && fixture_user_team_involved(match->fix) != -1)
+    if((debug > 100 && fixture_user_team_involved(match->fix) != -1) ||
+       debug > 130)
 	printf("\t\tlive_game_event_general_get_players\n");
     *pl1 = *pl2 = -1;
 
     if(type == LIVE_GAME_EVENT_LOST_POSSESSION)
     {
-	*pl1 = game_get_player(tm[last_unit.possession], 
-			       last_unit.area, 0, *pl1,
-			       TRUE);
 	*pl2 = old_pl1;
+	*pl1 = game_get_player(tm[last_unit.possession], 
+			       last_unit.area, 0, *pl2,
+			       TRUE);
     }
     else if(type != LIVE_GAME_EVENT_GENERAL)
     {
@@ -756,7 +792,8 @@ live_game_event_free_kick(void)
 {
     LiveGameUnit new = last_unit;
 
-    if(debug > 100 && fixture_user_team_involved(match->fix) != -1)
+    if((debug > 100 && fixture_user_team_involved(match->fix) != -1) ||
+       debug > 130)
 	printf("\t\tlive_game_event_free_kick\n");
     new.event.player =
 	new.event.player2 = -1;
@@ -789,7 +826,8 @@ live_game_event_send_off(gint team, gint player, gboolean second_yellow)
     LiveGameUnit new = last_unit;
     gint substitute = -1, to_substitute = -1;
 
-    if(debug > 100 && fixture_user_team_involved(match->fix) != -1)
+    if((debug > 100 && fixture_user_team_involved(match->fix) != -1) ||
+       debug > 130)
 	printf("\t\tlive_game_event_send_off\n");
     new.event.player =
 	new.event.player2 = -1;
@@ -808,6 +846,15 @@ live_game_event_send_off(gint team, gint player, gboolean second_yellow)
     if(debug >= 50 && team_is_user(tm[team]) != -1)
 	return;
 
+    player_streak_add_to_prob(
+	player_of_id_team(tm[team], player),
+	const_float("float_player_streak_add_sendoff"));
+    if(player_of_id_team(tm[team], player)->streak == PLAYER_STREAK_HOT)
+    {
+	player_of_id_team(tm[team], player)->streak = PLAYER_STREAK_NONE;
+	player_streak_reset_count(player_of_id_team(tm[team], player));
+    }
+
     player_of_id_team(tm[team], player)->cskill = 0;
     if(second_yellow)
 	player_card_set(player_of_id_team(tm[team], player), match->fix->clid, PLAYER_VALUE_CARD_RED, 2, FALSE);
@@ -874,13 +921,12 @@ live_game_event_substitution(gint team_number, gint sub_in, gint sub_out)
     
     new.minute = -1;
     new.time = live_game_get_time(&last_unit);
-    new.event.commentary = NULL;
 
     new.event.type = LIVE_GAME_EVENT_SUBSTITUTION;
     new.event.team = team_number;
     new.event.player = sub_in;
     new.event.player2 = sub_out;
-
+    new.event.commentary = NULL;
 
     if(player_of_id_team(tm[team_number], sub_in)->cskill > 0)
     {
@@ -905,12 +951,12 @@ live_game_event_team_change(gint team_number, gint event_type)
     
     new.minute = -1;
     new.time = live_game_get_time(&last_unit);
-    new.event.commentary = NULL;
 
     new.event.team = team_number;
     new.event.player =
 	new.event.player2 = -1;    
     new.event.type = event_type;
+    new.event.commentary = NULL;
 
     g_array_append_val(unis, new);
     
@@ -925,20 +971,24 @@ live_game_event_duel(void)
     gfloat scoring_prob;
     gfloat duel_factor;
     LiveGameUnit new = last_unit;
-    Player *attacker, *goalie;
+    Player *attacker, *goalie, *assistant;
     gint res_idx1, res_idx2;
 
-    if(debug > 100 && fixture_user_team_involved(match->fix) != -1)
+    if((debug > 100 && fixture_user_team_involved(match->fix) != -1) ||
+       debug > 130)
 	printf("\t\tlive_game_event_duel\n");
 
     new.minute = -1;
     new.event.team = new.possession;
-
     new.event.commentary = NULL;
 	
     attacker = player_of_id_team(tm[new.possession],
-			    new.event.player);
+				 new.event.player);
     goalie = player_of_idx_team(tm[!new.possession], 0);
+
+    assistant = (new.event.player2 != -1) ? 
+	player_of_id_team(tm[new.possession], new.event.player2) : NULL;
+
     new.event.player2 = goalie->id;
 
     duel_factor = player_get_game_skill(attacker, FALSE) /
@@ -981,6 +1031,14 @@ live_game_event_duel(void)
 	    player_games_goals_set(goalie, match->fix->clid, PLAYER_VALUE_GOALS, 1);
 	    attacker->career[PLAYER_VALUE_GOALS]++;
 	    goalie->career[PLAYER_VALUE_GOALS]++;
+
+	    player_streak_add_to_prob(attacker,
+				      const_float("float_player_streak_add_goal"));
+	    player_streak_add_to_prob(goalie,
+				      const_float("float_player_streak_add_goalie_goal"));
+	    if(assistant != NULL)
+		player_streak_add_to_prob(
+		    assistant, const_float("float_player_streak_add_assist"));
 	}
     }
     else
@@ -992,6 +1050,10 @@ live_game_event_duel(void)
     {
 	player_games_goals_set(goalie, match->fix->clid, PLAYER_VALUE_SHOTS, 1);
 	goalie->career[PLAYER_VALUE_SHOTS]++;
+
+	if(new.event.type == LIVE_GAME_EVENT_SAVE)
+	    player_streak_add_to_prob(goalie,
+				      const_float("float_player_streak_add_goalie_save"));
     }
     
     g_array_append_val(unis, new);
@@ -1276,10 +1338,12 @@ live_game_finish_unit(void)
 {
     LiveGameUnit *unit = &last_unit;
 
-    if(debug > 100 && fixture_user_team_involved(match->fix) != -1)
+    if((debug > 100 && fixture_user_team_involved(match->fix) != -1) ||
+       debug > 130)
 	printf("\t\tlive_game_finish_unit\n");
 
-    if(debug > 100 && fixture_user_team_involved(match->fix) != -1)
+    if((debug > 100 && fixture_user_team_involved(match->fix) != -1) ||
+       debug > 130)
 	printf("OOOO1 idx %d type %d poss %d team %d pl %d %d\n", unis->len - 1,
 	       unit->event.type, unit->possession, unit->event.team,
 	       unit->event.player,
@@ -1297,7 +1361,7 @@ live_game_finish_unit(void)
 	    treeview_show_user_player_list();
     }
 
-    if(fixture_user_team_involved(match->fix) != -1)
+    if(fixture_user_team_involved(match->fix) != -1 || stat5 < -1000)
     {
 	if(unit->time != LIVE_GAME_UNIT_TIME_PENALTIES)
 	{
@@ -1307,7 +1371,24 @@ live_game_finish_unit(void)
 					 match);
 	}
 
-	lg_commentary_generate(unit, match->fix);
+	lg_commentary_generate(match, unit);
+
+	if(-stat5 - 1000 == unit->event.type)
+	{
+	    printf("type %d com **%s**", unit->event.type, unit->event.commentary);
+	    if(g_strrstr(unit->event.commentary, "[") ||
+	       g_strrstr(unit->event.commentary, "]") ||
+	       g_strrstr(unit->event.commentary, "<") ||
+	       g_strrstr(unit->event.commentary, ">") ||
+	       g_strrstr(unit->event.commentary, "=") ||
+	       g_strrstr(unit->event.commentary, " G ") ||
+	       g_strrstr(unit->event.commentary, " L ") ||
+	       g_strrstr(unit->event.commentary, " GE ") ||
+	       g_strrstr(unit->event.commentary, " LE "))
+		printf(" ERROR?\n");
+	    else
+		printf("\n");
+	}
 
 	unit->event.verbosity = live_game_event_get_verbosity(unit->event.type);
     }
@@ -1315,7 +1396,8 @@ live_game_finish_unit(void)
     if(show)
 	game_gui_live_game_show_unit(unit);
 
-    if(debug > 100 && fixture_user_team_involved(match->fix) != -1)
+    if((debug > 100 && fixture_user_team_involved(match->fix) != -1) ||
+       debug > 130)
 	printf("OOOO idx %d type %d poss %d team %d pl %d %d\n", unis->len - 1,
 	       unit->event.type, unit->possession, unit->event.team,
 	       unit->event.player,
@@ -1379,7 +1461,7 @@ live_game_resume(void)
     gint subs_in[3],
 	subs_out[3];
 
-    match = &usr(stat2).live_game;
+    statp = &usr(stat2).live_game;
 
     for(i=0;i<2;i++)
     {
@@ -1409,14 +1491,6 @@ live_game_resume(void)
 }
 
 
-/** Set the match variable to the live game.
-    @param live_game The live game 'match' will point to. */
-void
-live_game_set_match(LiveGame *live_game)
-{
-    match = live_game;
-}
-
 /** Reset the live game variable before we begin a new live game.
     @param live_game The pointer to the live game.
     @param fix The fixture we'll show.
@@ -1442,7 +1516,18 @@ live_game_reset(LiveGame *live_game, Fixture *fix, gboolean free_variable)
 	    live_game->stats.values[1][i] = 0;
 
     live_game->fix = fix;
-    live_game->fix_id = (fix != NULL) ? fix->id : -1;
+    if(fix != NULL)
+    {
+	live_game->fix_id = fix->id;
+	live_game->team_names[0] = g_strdup(fix->teams[0]->name);
+	live_game->team_names[1] = g_strdup(fix->teams[1]->name);
+    }
+    else
+    {
+	live_game->fix_id = -1;
+	live_game->team_names[0] = 
+	    live_game->team_names[1] = NULL;
+    }
 
     live_game->subs_left[0] = live_game->subs_left[1] = 3;
     live_game->stadium_event = -1;
@@ -1474,7 +1559,7 @@ live_game_event_get_verbosity(gint event_type)
 	    event_type == LIVE_GAME_EVENT_HEADER ||
 	    event_type == LIVE_GAME_EVENT_FREE_KICK ||
 	    event_type == LIVE_GAME_EVENT_POST ||
-	    event_type == LIVE_GAME_EVENT_MISSED ||
+	    event_type == LIVE_GAME_EVENT_MISS ||
 	    event_type == LIVE_GAME_EVENT_CROSS_BAR ||
 	    event_type == LIVE_GAME_EVENT_SAVE)
 	return_value = 1;
@@ -1514,4 +1599,41 @@ live_game_event_get_verbosity(gint event_type)
 
     return return_value;
 }
-    
+
+/** Write a result like '2:3 e.t.' into the buffer.
+    @param swap Whether to swap the scores. */
+void
+live_game_unit_result_to_buf(const LiveGameUnit *unit, gchar *buf, gboolean swap)
+{
+    gint idx = (swap) ? 1 : 0;
+
+    if(unit->time == LIVE_GAME_UNIT_TIME_PENALTIES)
+	/* A result after penalties. */
+	sprintf(buf, _("%d : %d p."), unit->result[idx],
+		unit->result[!idx]);
+    else if(unit->time == LIVE_GAME_UNIT_TIME_EXTRA_TIME)
+	/* A result after extra time. */
+	sprintf(buf, _("%d : %d e.t."), unit->result[idx],
+		unit->result[!idx]);
+    else
+	sprintf(buf, "%d : %d", unit->result[idx],
+		unit->result[!idx]);
+}
+
+/** Count the number of penalties taken during penalty shoot-out. */
+gint
+live_game_penalties_taken(void)
+{
+    gint i, penalties = 0;
+
+    for(i=unis->len - 1;i>=0;i--)
+    {
+	if(uni(i).event.type == LIVE_GAME_EVENT_PENALTIES)
+	    break;
+	else if(uni(i).time == LIVE_GAME_UNIT_TIME_PENALTIES &&
+		uni(i).event.type == LIVE_GAME_EVENT_PENALTY)
+	    penalties++;
+    }
+
+    return penalties;
+}
