@@ -3,6 +3,8 @@
 
 #include "bygfoot.h"
 
+#define misc_int_to_char(number) g_strdup_printf("%d", number)
+
 void
 misc_print_error(GError **error, gboolean abort_program);
 
@@ -19,10 +21,10 @@ GPtrArray*
 misc_randomise_g_pointer_array(GPtrArray *array);
 
 void
-misc_print_grouped_int(gint number, gchar *buf, gboolean append);
+misc_print_grouped_int(gint number, gchar *buf);
 
 gboolean
-query_integer_is_in_array(gint item, gint *array, gint min, gint max);
+query_integer_is_in_array(gint item, gint *array, gint max);
 
 gint
 misc_int_compare(gint first, gint second);
@@ -30,9 +32,6 @@ misc_int_compare(gint first, gint second);
 gint
 misc_float_compare(gfloat first, gfloat second);
 
-void
-misc_truncate_string(const gchar *src, gchar *dest, gint number_of_chars);
-
 gboolean
 query_misc_string_contains(const gchar *string, const gchar *text);
 
@@ -45,4 +44,34 @@ query_misc_integer_is_in_g_array(gint item, GArray *array);
 void
 misc_string_replace_token(gchar *string, const gchar *token, const gchar *replacement);
 
+void
+misc_replace_sums(gchar *string);
+
+gfloat
+misc_get_age_from_birth(gint birth_year, gint birth_month);
+
+const gchar*
+misc_skip_spaces(const gchar* s);
+
+const gchar*
+misc_parse_value(const gchar *s, gint *value);
+
+const gchar*
+misc_parse_expression(const gchar *s, gint *result);
+
+const gchar*
+misc_parse_comparison(const gchar *s, gint *result);
+
+const gchar*
+misc_parse_and(const gchar *s, gint *result);
+
+const gchar*
+misc_parse(const gchar *s, gint *result);
+
+GPtrArray*
+misc_copy_ptr_array(const GPtrArray *array);
+
+void
+misc_string_assign(gchar **string, const gchar *contents);
+
 #endif
