@@ -2,6 +2,7 @@
 #include "callback_func.h"
 #include "debug.h"
 #include "finance.h"
+#include "free.h"
 #include "game_gui.h"
 #include "load_save.h"
 #include "main.h"
@@ -12,6 +13,7 @@
 #include "support.h"
 #include "transfer.h"
 #include "treeview.h"
+#include "treeview2.h"
 #include "treeview_helper.h"
 #include "user.h"
 #include "window.h"
@@ -118,6 +120,9 @@ on_button_digits_ok_clicked            (GtkButton       *button,
 	    if(destroy_window && stat0 == STATUS_LIVE_GAME_PAUSE)
 		gtk_widget_set_sensitive(window.main, TRUE);
 	    break;
+	case STATUS_SET_YA_PERCENTAGE:
+	    current_user.youth_academy.percentage = values[1];
+	    break;
     }
 
     if(destroy_window)
@@ -189,6 +194,12 @@ on_button_yesno_yes_clicked            (GtkButton       *button,
 	    load_save_autosave();
 	    callback_show_next_live_game();
 	    break;
+	case STATUS_QUERY_KICK_YOUTH:
+	    free_player(&g_array_index(current_user.youth_academy.players, Player, selected_row));
+	    g_array_remove_index(current_user.youth_academy.players, selected_row);
+	    on_menu_show_youth_academy_activate(NULL, NULL);
+	    selected_row = -1;
+	    break;
     }
 
     if(stat4 == STATUS_SHOW_EVENT)
@@ -214,12 +225,23 @@ on_button_contract_offer_clicked       (GtkButton       *button,
     misc2_callback_contract_offer();
 }
 
+void
+on_button_contract_cancel_clicked      (GtkButton       *button,
+                                        gpointer         user_data)
+{
+    window_destroy(&window.contract, FALSE);
+}
+
+
 gboolean
 on_window_contract_delete_event        (GtkWidget       *widget,
                                         GdkEvent        *event,
                                         gpointer         user_data)
 {
 
+    if(GTK_WIDGET_IS_SENSITIVE(lookup_widget(window.contract, "button_contract_cancel")))
+	window_destroy(&window.contract, FALSE);    
+
     return TRUE;
 }
 
@@ -246,6 +268,7 @@ on_button_user_management_close_clicked
                                         gpointer         user_data)
 {
     window_destroy(&window.user_management, FALSE);
+    on_button_back_to_main_clicked(NULL, NULL);
 }
 
 
@@ -270,7 +293,7 @@ on_treeview_user_management_users_button_press_event
     }
 
     stat2 = idx;
-    sprintf(buf, _("Remove user %s from the game?"), usr(idx).name->str);
+    sprintf(buf, _("Remove user %s from the game?"), usr(idx).name);
     window_show_yesno(buf);
     
     return FALSE;
@@ -307,7 +330,7 @@ on_button_debug_apply_clicked          (GtkButton       *button,
     gchar buf[SMALL];
     gint value = -1;
 
-    sscanf(entry_text, "%[^-1-9]%d", buf, &value);
+    sscanf(entry_text, "%[^-0-9]%d", buf, &value);
     debug_action(buf, value);
 
     gtk_entry_set_text(entry_debug, "");
@@ -424,8 +447,16 @@ void
 on_button_transfer_no_clicked          (GtkButton       *button,
                                         gpointer         user_data)
 {
-    g_array_remove_index(trans(stat2).offers, 0);
-    if(trans(stat2).offers->len > 0)
+    gint len = 1;
+
+    transoff(stat2, 0).status = TRANSFER_OFFER_REJECTED;
+    if(transoff(stat2, 0).tm == current_user.tm)
+    {
+	g_array_remove_index(trans(stat2).offers, 0);
+	len = 0;
+    }
+
+    if(trans(stat2).offers->len > len)
 	transfer_offers_notify(&trans(stat2), FALSE);
 
     treeview_show_user_player_list();
@@ -443,3 +474,114 @@ on_button_transfer_later_clicked       (GtkButton       *button,
     window_destroy(&window.transfer_dialog, FALSE);
 }
 
+gboolean
+on_window_mmatches_delete_event        (GtkWidget       *widget,
+                                        GdkEvent        *event,
+                                        gpointer         user_data)
+{
+    on_button_mm_reload_close_clicked(NULL, NULL);
+
+    return TRUE;
+}
+
+
+gboolean
+on_treeview_mmatches_button_press_event (GtkWidget       *widget,
+					 GdkEventButton  *event,
+					 gpointer         user_data)
+{
+    GtkTreePath *path = NULL;
+    GtkTreeViewColumn *col = NULL;
+    gint col_num = -1, mmidx = -1;
+    
+    if(!gtk_tree_view_get_path_at_pos(GTK_TREE_VIEW(widget),
+				      event->x, event->y,
+				      &path, &col, NULL, NULL))
+	return TRUE;
+
+    col_num = treeview_helper_get_col_number_column(col);
+    mmidx = gtk_tree_path_get_indices(path)[0];
+
+    gtk_tree_path_free(path);
+
+    if(col_num == -1 || mmidx == -1)
+    {
+	g_warning("on_treeview_mmatches_button_press_event: column or row not valid\n");
+	return TRUE;
+    }
+
+    misc2_callback_mmatches_button_press(widget, mmidx, col_num);
+    
+    return TRUE;
+}
+
+
+void
+on_button_mm_save_close_clicked        (GtkButton       *button,
+                                        gpointer         user_data)
+{
+    const gchar *filename = 
+	gtk_entry_get_text(GTK_ENTRY(lookup_widget(window.mmatches, "entry_mm_file")));
+
+    user_mm_set_filename(filename, NULL);
+    user_mm_save_file(current_user.mmatches_file,
+		      current_user.mmatches);
+    
+    window_destroy(&window.mmatches, TRUE);
+}
+
+
+void
+on_button_mm_add_clicked               (GtkButton       *button,
+                                        gpointer         user_data)
+{
+    if(current_user.live_game.units->len == 0)
+    {
+	game_gui_show_warning(_("No match stored."));
+	return;
+    }
+
+    user_mm_add_last_match(FALSE, FALSE);
+    treeview2_show_mmatches();
+}
+
+
+void
+on_button_mm_file_clicked              (GtkButton       *button,
+                                        gpointer         user_data)
+{
+    stat5 = STATUS_SELECT_MM_FILE_LOAD;
+    window_show_file_sel();
+}
+
+
+void
+on_button_mm_reload_clicked            (GtkButton       *button,
+                                        gpointer         user_data)
+{
+    GtkWidget *treeview = lookup_widget(window.mmatches, "treeview_mmatches");
+    const gchar *filename = 
+	gtk_entry_get_text(GTK_ENTRY(lookup_widget(window.mmatches, "entry_mm_file")));
+    
+    gtk_widget_hide(treeview);
+    user_mm_load_file(filename, NULL);
+    treeview2_show_mmatches();
+    gtk_widget_show(treeview);
+}
+
+void
+on_button_mm_reload_close_clicked      (GtkButton       *button,
+                                        gpointer         user_data)
+{
+    on_button_mm_reload_clicked(NULL, NULL);
+    on_button_mm_save_close_clicked(NULL, NULL);
+}
+
+void
+on_button_mm_import_clicked            (GtkButton       *button,
+                                        gpointer         user_data)
+{
+    stat5 = STATUS_SELECT_MM_FILE_IMPORT;
+    window_show_file_sel();
+}
+
