@@ -6,6 +6,7 @@
 #include "live_game.h"
 #include "load_save.h"
 #include "main.h"
+#include "maths.h"
 #include "misc_callback_func.h"
 #include "misc_callbacks.h"
 #include "option.h"
@@ -58,7 +59,7 @@ void
 on_team_selection_load_clicked         (GtkButton       *button,
                                         gpointer         user_data)
 {
-    stat1 = STATUS_LOAD_GAME_TEAM_SELECTION;
+    stat5 = STATUS_LOAD_GAME_TEAM_SELECTION;
     window_show_file_sel();
 }
 
@@ -161,7 +162,7 @@ on_button_resume_clicked               (GtkButton       *button,
 
     if(stat1 == STATUS_SHOW_LAST_MATCH)
     {
-	callback_show_last_match(FALSE);
+	callback_show_last_match(FALSE, &current_user.live_game);
 	return;
     }
     else if(game_check_live_game_resume_state())
@@ -172,13 +173,14 @@ on_button_resume_clicked               (GtkButton       *button,
 	   LIVE_GAME_EVENT_PENALTIES)
 	{
 	    gtk_widget_set_sensitive(button_pause, TRUE);
+/* 	    gui_set_sensitive_lg_meters(TRUE); */
 	    gtk_widget_grab_focus(button_pause);
 	}
 	game_gui_set_main_window_sensitivity(FALSE);
 	live_game_resume();
     }
     else
-	game_gui_show_warning("There were too many substitutions. Only 3 per game are allowed. Player list reset.");
+	game_gui_show_warning(_("There were too many substitutions. Only 3 per game are allowed. Player list reset."));
 }
 
 void
@@ -296,12 +298,18 @@ on_spinbutton_speed_button_press_event (GtkWidget       *widget,
 {
     if(event->button == 3)
     {
-	if(option_int("int_opt_user_live_game_speed", &usr(stat2).options) == -10)
-	    option_set_int("int_opt_user_live_game_speed", &usr(stat2).options, 20);
-	else if(option_int("int_opt_user_live_game_speed", &usr(stat2).options) == 20)
+	if(option_int("int_opt_user_live_game_speed", &usr(stat2).options) == 
+	   -rint((gfloat)(const_int("int_game_gui_live_game_speed_max") - 10) /
+		 (gfloat)(const_int("int_game_gui_live_game_speed_grad"))))
 	    option_set_int("int_opt_user_live_game_speed", &usr(stat2).options, 0);
+	else if(option_int("int_opt_user_live_game_speed", &usr(stat2).options) == 0)
+	    option_set_int("int_opt_user_live_game_speed", &usr(stat2).options, 
+			   -rint((gfloat)(const_int("int_game_gui_live_game_speed_max") - 10) /
+				 (gfloat)(const_int("int_game_gui_live_game_speed_grad") * 2)));
 	else
-	    option_set_int("int_opt_user_live_game_speed", &usr(stat2).options, -10);
+	    option_set_int("int_opt_user_live_game_speed", &usr(stat2).options, 
+			   -rint((gfloat)(const_int("int_game_gui_live_game_speed_max") - 10) /
+				 (gfloat)(const_int("int_game_gui_live_game_speed_grad"))));
 
 	gtk_spin_button_set_value(
 	    GTK_SPIN_BUTTON(lookup_widget(window.live, "spinbutton_speed")),
@@ -313,3 +321,132 @@ on_spinbutton_speed_button_press_event (GtkWidget       *widget,
     return FALSE;
 }
 
+
+void
+on_button_sponsors_clicked             (GtkButton       *button,
+                                        gpointer         user_data)
+{
+    misc_callback_new_sponsor();
+
+    window_destroy(&window.sponsors, FALSE);
+}
+
+
+void
+on_treeview_sponsors_row_activated     (GtkTreeView     *treeview,
+                                        GtkTreePath     *path,
+                                        GtkTreeViewColumn *column,
+                                        gpointer         user_data)
+{
+    on_button_sponsors_clicked(NULL, NULL);
+}
+
+
+gboolean
+on_window_sponsors_delete_event        (GtkWidget       *widget,
+                                        GdkEvent        *event,
+                                        gpointer         user_data)
+{
+
+    return TRUE;
+}
+
+void
+on_button_sponsors_wait_clicked        (GtkButton       *button,
+                                        gpointer         user_data)
+{
+    if(stat1 != STATUS_SPONSOR_CONTINUE)
+    {
+	g_string_printf(current_user.sponsor.name, _("None"));
+	current_user.sponsor.contract = 0;
+	current_user.sponsor.benefit = 0;
+	current_user.counters[COUNT_USER_NEW_SPONSOR] = 
+	    math_rndi(const_int("int_sponsor_without_weeks_lower"),
+		      const_int("int_sponsor_without_weeks_upper"));
+    }
+    else
+	current_user.counters[COUNT_USER_NEW_SPONSOR] = 0;
+
+    window_destroy(&window.sponsors, FALSE);
+}
+
+gboolean
+on_eventbox_lg_style_button_press_event (GtkWidget       *widget,
+					 GdkEventButton  *event,
+					 gpointer         user_data)
+{
+    gint new_style = -1;
+
+    if(stat0 == STATUS_LIVE_GAME_PAUSE)
+    {
+	on_eventbox_style_button_press_event(NULL, event, NULL);
+	return FALSE;
+    }
+
+    if(event->type != GDK_BUTTON_PRESS)
+	return FALSE;
+
+    if(event->button == 3)
+	new_style = (usr(stat2).tm->style != 2) ?
+	    usr(stat2).tm->style + 1 : -2;
+    else if(event->button == 1)
+	new_style = (usr(stat2).tm->style != -2) ?
+	    usr(stat2).tm->style - 1 : 2;
+    else
+	return FALSE;
+
+    game_save_team_states();
+    usr(stat2).tm->style = new_style;
+
+    game_gui_write_meters(usr(stat2).tm);
+    if(&current_user == &usr(stat2))
+	game_gui_write_meters(current_user.tm);
+    
+    stat0 = STATUS_LIVE_GAME_CHANGE;
+
+    return FALSE;
+}
+
+
+gboolean
+on_eventbox_lg_boost_button_press_event (GtkWidget       *widget,
+					 GdkEventButton  *event,
+					 gpointer         user_data)
+{
+    gint new_boost = -1;
+
+    if(stat0 == STATUS_LIVE_GAME_PAUSE)
+    {
+	on_eventbox_boost_button_press_event(NULL, event, NULL);
+	return FALSE;
+    }
+
+    if(event->type != GDK_BUTTON_PRESS)
+	return FALSE;
+
+    if(event->button == 3)
+	new_boost = (usr(stat2).tm->boost != 1) ?
+	    usr(stat2).tm->boost + 1 : -1;
+    else if(event->button == 1)
+	new_boost = (usr(stat2).tm->boost != -1) ?
+	    usr(stat2).tm->boost - 1 : 1;
+    else
+	return FALSE;
+
+    if(new_boost == 1 && sett_int("int_opt_disable_boost_on"))
+    {
+	game_gui_show_warning(_("Boost ON is disabled in this country definition."));
+	return FALSE;
+    }
+
+    game_save_team_states();
+    usr(stat2).tm->boost = new_boost;
+
+    game_gui_write_meters(usr(stat2).tm);
+    if(&current_user == &usr(stat2))
+	game_gui_write_meters(current_user.tm);
+    
+    stat0 = STATUS_LIVE_GAME_CHANGE;
+
+    return FALSE;
+}
