@@ -8,8 +8,8 @@
 typedef struct
 {
     /** The file id (the part between 'player_names_' and '.xml'). */
-    GString *sid;
-    /** Arrays of GStrings holding the names. */
+    gchar *sid;
+    /** Arrays of strings holding the names. */
     GPtrArray *first_names, 
 	*last_names;
 
