@@ -1,4 +1,5 @@
 #include "main.h"
+#include "misc.h"
 #include "option.h"
 #include "variables.h"
 
@@ -13,9 +14,9 @@ option_string(const gchar *name, OptionList *optionlist)
     gpointer element = g_datalist_get_data(&optionlist->datalist, name);
     
     if(element == NULL)
-	g_warning("option_string: option named %s not found\n", name);
+	g_warning("option_string: option named %s not found\nMaybe you should delete the .bygfoot directory from your home dir", name);
     else
-	return ((Option*)element)->string_value->str;
+	return ((Option*)element)->string_value;
 
     main_exit_program(EXIT_OPTION_NOT_FOUND, NULL);
 
@@ -23,15 +24,15 @@ option_string(const gchar *name, OptionList *optionlist)
 }
 
 /** Return the GString pointer going with the option. */
-GString*
+gchar**
 option_string_pointer(const gchar *name, OptionList *optionlist)
 {
     gpointer element = g_datalist_get_data(&optionlist->datalist, name);
     
     if(element == NULL)
-	g_warning("option_string: option named %s not found\n", name);
+	g_warning("option_string: option named %s not found\nMaybe you should delete the .bygfoot directory from your home dir", name);
     else
-	return ((Option*)element)->string_value;
+	return &((Option*)element)->string_value;
 
     main_exit_program(EXIT_OPTION_NOT_FOUND, NULL);
 
@@ -49,7 +50,7 @@ option_int(const gchar *name, OptionList *optionlist)
     gpointer element = g_datalist_get_data(&optionlist->datalist, name);
     
     if(element == NULL)
-	g_warning("option_int: option named %s not found\n", name);
+	g_warning("option_int: option named %s not found\nMaybe you should delete the .bygfoot directory from your home dir", name);
     else
 	return ((Option*)element)->value;
 
@@ -65,7 +66,7 @@ option_int_pointer(const gchar *name, OptionList *optionlist)
     gpointer element = g_datalist_get_data(&optionlist->datalist, name);
     
     if(element == NULL)
-	g_warning("option_int: option named %s not found\n", name);
+	g_warning("option_int: option named %s not found\nMaybe you should delete the .bygfoot directory from your home dir", name);
     else
 	return &((Option*)element)->value;
 
@@ -85,9 +86,9 @@ option_float(const gchar *name, OptionList *optionlist)
     gpointer element = g_datalist_get_data(&optionlist->datalist, name);
     
     if(element == NULL)
-	g_warning("option_float: option named %s not found\n", name);
+	g_warning("option_float: option named %s not found\nMaybe you should delete the .bygfoot directory from your home dir", name);
     else
-	return (gfloat)((Option*)element)->value / 10000;
+	return (gfloat)((Option*)element)->value / 100000;
 
     main_exit_program(EXIT_OPTION_NOT_FOUND, NULL);
 
@@ -104,9 +105,9 @@ option_set_string(const gchar *name, OptionList *optionlist, const gchar *new_va
     gpointer element = g_datalist_get_data(&optionlist->datalist, name);
     
     if(element == NULL)
-	g_warning("option_set_string: option named %s not found\n", name);
+	g_warning("option_set_string: option named %s not found\nMaybe you should delete the .bygfoot directory from your home dir", name);
     else
-	g_string_printf(((Option*)element)->string_value, "%s", new_value);
+	misc_string_assign(&((Option*)element)->string_value, new_value);
 }
 
 /** Change the value of an int option in the array.
@@ -119,7 +120,44 @@ option_set_int(const gchar *name, OptionList *optionlist, gint new_value)
     gpointer element = g_datalist_get_data(&optionlist->datalist, name);
     
     if(element == NULL)
-	g_warning("option_set_int: option named %s not found\n", name);
+	g_warning("option_set_int: option named %s not found\nMaybe you should delete the .bygfoot directory from your home dir", name);
     else
 	((Option*)element)->value = new_value;
 }
+
+
+/** Add an option to the optionlist with the given values. */
+void
+option_add(OptionList *optionlist, const gchar *name, 
+	   gint int_value, const gchar *string_value)
+{
+    gint i;
+    Option new;
+    gpointer element = NULL;
+
+    if(optionlist->list != NULL)
+	g_datalist_get_data(&optionlist->datalist, name);
+    
+    if(element != NULL)
+    {
+	g_warning("Option named '%s' already contained in optionlist.", name);
+	main_exit_program(EXIT_GENERAL, NULL);
+    }
+
+    new.name = g_strdup(name);
+    new.value = int_value;
+    new.string_value = (string_value == NULL) ? NULL : g_strdup(string_value);
+
+    if(optionlist->list == NULL)
+    {
+	optionlist->list = g_array_new(FALSE, FALSE, sizeof(Option));
+	g_datalist_init(&optionlist->datalist);
+    }
+
+    g_array_append_val(optionlist->list, new);
+    
+    for(i=0;i<optionlist->list->len;i++)
+	g_datalist_set_data(&optionlist->datalist, 
+			    g_array_index(optionlist->list, Option, i).name,
+			    &g_array_index(optionlist->list, Option, i));
+}
