@@ -4,6 +4,7 @@
 #include "language.h"
 #include "option.h"
 #include "option_gui.h"
+#include "misc.h"
 #include "support.h"
 #include "treeview.h"
 #include "user.h"
@@ -14,7 +15,6 @@
 enum BooleanOptions
 {
     BOOL_OPT_CONF_QUIT = 0,
-    BOOL_OPT_CONF_UNFIT,
     BOOL_OPT_SAVE_OVERWRITE,
     BOOL_OPT_MAXIMIZE,
     BOOL_OPT_PREFER_MESS,
@@ -26,6 +26,8 @@ enum BooleanOptions
     BOOL_OPT_PAUSE_RED,
     BOOL_OPT_PAUSE_BREAK,
     BOOL_OPT_AUTO_SUB,
+    BOOL_OPT_CONF_UNFIT,
+    BOOL_OPT_CONF_YOUTH,
     BOOL_OPT_SHOW_JOB,
     BOOL_OPT_SWAP,
     BOOL_OPT_SHOW_OVERALL,
@@ -126,6 +128,10 @@ option_gui_write_bool_widgets(gint **bool_options, GtkToggleButton **bool_widget
 	GTK_TOGGLE_BUTTON(lookup_widget(window.options, "checkbutton_auto_sub"));
     bool_options[BOOL_OPT_AUTO_SUB] = opt_user_intp("int_opt_user_auto_sub");
 
+    bool_widgets[BOOL_OPT_CONF_YOUTH] =
+	GTK_TOGGLE_BUTTON(lookup_widget(window.options, "checkbutton_conf_youth"));
+    bool_options[BOOL_OPT_CONF_YOUTH] = opt_user_intp("int_opt_user_confirm_youth");
+
     bool_widgets[BOOL_OPT_SHOW_JOB] =
 	GTK_TOGGLE_BUTTON(lookup_widget(window.options, "checkbutton_show_job"));
     bool_options[BOOL_OPT_SHOW_JOB] = opt_user_intp("int_opt_user_show_job_offers");
@@ -304,6 +310,8 @@ enum SpinOptions
 void
 option_gui_write_spin_widgets(gint **spin_options, GtkSpinButton **spin_widgets)
 {
+    gint speed_val = 0;
+
     spin_widgets[SPIN_OPT_AUTOSAVE] =
 	GTK_SPIN_BUTTON(lookup_widget(window.options, "spinbutton_autosave"));
     spin_options[SPIN_OPT_AUTOSAVE] = opt_intp("int_opt_autosave_interval");
@@ -324,6 +332,16 @@ option_gui_write_spin_widgets(gint **spin_options, GtkSpinButton **spin_widgets)
 	GTK_SPIN_BUTTON(lookup_widget(window.options, "spinbutton_live_speed"));
     spin_options[SPIN_OPT_LIVE_SPEED] = opt_user_intp("int_opt_user_live_game_speed");
 
+    /** Note the spinbutton value so that it doesn't get lost
+	when setting the range. */    
+    speed_val = gtk_spin_button_get_value_as_int(spin_widgets[SPIN_OPT_LIVE_SPEED]);
+    gtk_spin_button_set_range(
+	spin_widgets[SPIN_OPT_LIVE_SPEED], 0,
+	-rint((gfloat)(const_int("int_game_gui_live_game_speed_max") - 10) /
+	      (gfloat)(const_int("int_game_gui_live_game_speed_grad"))));
+    gtk_spin_button_set_value(spin_widgets[SPIN_OPT_LIVE_SPEED],
+			      (gfloat)speed_val);
+
     spin_widgets[SPIN_OPT_LIVE_VERBOSITY] =
 	GTK_SPIN_BUTTON(lookup_widget(window.options, "spinbutton_live_verbosity"));
     spin_options[SPIN_OPT_LIVE_VERBOSITY] = opt_user_intp("int_opt_user_live_game_verbosity");
@@ -344,7 +362,7 @@ enum EntryOptions
 
 /** Write the pointers to the options and the corresponding widgets. */
 void
-option_gui_write_entry_widgets(GString **entry_options, GtkEntry **entry_widgets)
+option_gui_write_entry_widgets(gchar ***entry_options, GtkEntry **entry_widgets)
 {
     entry_widgets[ENTRY_OPT_CONSTANTS] = 
 	GTK_ENTRY(lookup_widget(window.options, "entry_constants_file"));
@@ -368,7 +386,7 @@ option_gui_set_up_window(void)
     gint *spin_options[SPIN_OPT_END];
 
     GtkEntry *entry_widgets[ENTRY_OPT_END];
-    GString *entry_options[ENTRY_OPT_END];
+    gchar **entry_options[ENTRY_OPT_END];
 
     treeview_show_language_combo();
 
@@ -383,7 +401,7 @@ option_gui_set_up_window(void)
 	gtk_spin_button_set_value(spin_widgets[i], (gfloat)(*(spin_options[i])));
 
     for(i=0;i<ENTRY_OPT_END;i++)
-	gtk_entry_set_text(entry_widgets[i], (entry_options[i])->str);
+	gtk_entry_set_text(entry_widgets[i], *(entry_options[i]));
 }
 
 /** Read the widget states in the options window and set the
@@ -393,19 +411,19 @@ option_gui_write_options(void)
 {
     gint i;
     gint language_index = 
-	gtk_combo_box_get_active(GTK_COMBO_BOX(lookup_widget(window.options, "comboboxentry_languages")));
+	gtk_combo_box_get_active(GTK_COMBO_BOX(lookup_widget(window.options, "combobox_languages")));
     GtkToggleButton *bool_widgets[BOOL_OPT_END];
     gint *bool_options[BOOL_OPT_END];
     GtkSpinButton *spin_widgets[SPIN_OPT_END];
     gint *spin_options[SPIN_OPT_END];
     GtkEntry *entry_widgets[ENTRY_OPT_END];
-    GString *entry_options[ENTRY_OPT_END];
-
+    gchar **entry_options[ENTRY_OPT_END];
+    
     language_set(language_index);
 
     option_gui_write_bool_widgets(bool_options, bool_widgets);
     option_gui_write_spin_widgets(spin_options, spin_widgets);
-    option_gui_write_entry_widgets(entry_options, entry_widgets);
+    option_gui_write_entry_widgets(entry_options, entry_widgets);    
 
     for(i=0;i<BOOL_OPT_END;i++)
 	*(bool_options[i]) = gtk_toggle_button_get_active(bool_widgets[i]);
@@ -415,7 +433,7 @@ option_gui_write_options(void)
 
     for(i=0;i<ENTRY_OPT_END;i++)
     {
-	g_string_printf(entry_options[i], "%s", gtk_entry_get_text(entry_widgets[i]));
+	misc_string_assign(entry_options[i], gtk_entry_get_text(entry_widgets[i]));
 
 	if(i == ENTRY_OPT_CONSTANTS && 
 	   strcmp(gtk_entry_get_text(entry_widgets[i]), opt_str("string_opt_constants_file")) != 0)
