@@ -10,24 +10,24 @@
 #include "option.h"
 #include "player.h"
 #include "team.h"
+#include "transfer.h"
 #include "user.h"
 
 /** Create and return a new player.
     @param tm The team the player will belong to.
-    @param average_skill The average skill of the team. 
-    The player's skill can deviate from this value by #CONSTANT_PLAYER_AVERAGE_SKILL_VARIANCE %
+    @param average_talent The average talent of the team. 
     @return A newly created player. */
 Player
-player_new(Team *tm, gfloat average_skill, gboolean new_id)
+player_new(Team *tm, gfloat average_talent, gboolean new_id)
 {
     gint i;
     gfloat skill_factor = 
-	math_rnd(1 - const_float("float_player_average_skill_variance"),
-		 1 + const_float("float_player_average_skill_variance"));
+	math_rnd(1 - const_float("float_player_average_talent_variance"),
+		 1 + const_float("float_player_average_talent_variance"));
     Player new;
 
     new.name = (new_id) ? 
-	name_get(tm->names_file->str) : g_string_new("");
+	name_get(tm->names_file) : NULL;
 
     new.id = (new_id) ? player_id_new : -1;
     new.pos = player_get_position_from_structure(tm->structure, tm->players->len);
@@ -42,19 +42,16 @@ player_new(Team *tm, gfloat average_skill, gboolean new_id)
 		 (new.pos == PLAYER_POS_GOALIE) * 
 		 const_float("float_player_peak_age_goalie_addition"));
 
-    new.skill = CLAMP(average_skill * skill_factor, 0, 
-		      const_float("float_player_max_skill"));
-    new.talent = player_new_talent(new.skill);
-    player_estimate_talent(&new);
+    new.peak_region = 
+	math_gauss_dist(const_float("float_player_peak_region_lower"),
+			const_float("float_player_peak_region_upper"));
 
-    if(new.peak_age - new.age > const_float("float_player_peak_age_diff_younger1") ||
-       new.peak_age - new.age < const_float("float_player_peak_age_diff_older1"))
-	new.skill = new.skill * (1 - const_float("float_player_skill_reduction1"));
-    else if(new.peak_age - new.age > const_float("float_player_peak_age_diff_younger2") ||
-	    new.peak_age - new.age < const_float("float_player_peak_age_diff_peak_older"))
-	new.skill = new.skill * (1 - const_float("float_player_skill_reduction2"));
+    new.talent = CLAMP(average_talent * skill_factor, 0, 
+		       const_float("float_player_max_skill"));
 
+    new.skill = player_skill_from_talent(&new);
     new.cskill = new.skill;
+    player_estimate_talent(&new);
 
     new.fitness = math_rnd(const_float("float_player_fitness_lower"),
 			   const_float("float_player_fitness_upper"));
@@ -75,9 +72,74 @@ player_new(Team *tm, gfloat average_skill, gboolean new_id)
     new.participation = FALSE;
     new.offers = 0;
     
+    new.streak = PLAYER_STREAK_NONE;
+    new.streak_count = new.streak_prob = 0;
+
     return new;
 }
 
+/** Complete the player structure (some values have been read
+    from a team def file). */
+void
+player_complete_def(Player *pl, gfloat average_talent)
+{
+    gint i;
+    gfloat skill_factor = 
+	math_rnd(1 - const_float("float_player_average_talent_variance"),
+		 1 + const_float("float_player_average_talent_variance"));
+
+    pl->peak_age = math_rnd(const_float("float_player_peak_age_lower") +
+			    (pl->pos == PLAYER_POS_GOALIE) * 
+			    const_float("float_player_peak_age_goalie_addition"),
+			    const_float("float_player_peak_age_upper") +
+			    (pl->pos == PLAYER_POS_GOALIE) * 
+		 const_float("float_player_peak_age_goalie_addition"));
+
+    if(opt_int("int_opt_load_defs") == 2)
+    {
+	pl->age = math_gauss_dist(const_float("float_player_age_lower"),
+				  const_float("float_player_age_upper"));
+	pl->peak_age =
+	    math_rnd(const_float("float_player_peak_age_lower") +
+		     (pl->pos == PLAYER_POS_GOALIE) * 
+		     const_float("float_player_peak_age_goalie_addition"),
+		     const_float("float_player_peak_age_upper") +
+		     (pl->pos == PLAYER_POS_GOALIE) * 
+		     const_float("float_player_peak_age_goalie_addition"));
+
+	pl->peak_region = 
+	    math_gauss_dist(const_float("float_player_peak_region_lower"),
+			    const_float("float_player_peak_region_upper"));
+
+	pl->talent = CLAMP(average_talent * skill_factor, 0, 
+			   const_float("float_player_max_skill"));
+	pl->skill = player_skill_from_talent(pl);
+    }
+
+    pl->cpos = pl->pos;
+    player_estimate_talent(pl);
+
+    pl->cskill = pl->skill;
+    pl->fitness = math_rnd(const_float("float_player_fitness_lower"),
+			   const_float("float_player_fitness_upper"));
+
+    pl->health = pl->recovery = 0;
+    pl->value = player_assign_value(pl);
+    pl->wage = player_assign_wage(pl);
+    pl->contract = math_rnd(const_float("float_player_contract_lower"),
+			    const_float("float_player_contract_upper"));
+    pl->lsu = math_rnd(const_float("float_player_lsu_lower"),
+		       const_float("float_player_lsu_upper"));
+    pl->cards = g_array_new(FALSE, FALSE, sizeof(PlayerCard));
+    pl->games_goals = g_array_new(FALSE, FALSE, sizeof(PlayerGamesGoals));
+
+    for(i=0;i<PLAYER_VALUE_END;i++)
+	pl->career[i] = 0;
+
+    pl->participation = FALSE;
+    pl->offers = 0;
+}
+
 /** Return the appropriate position for the player with the given number.
     The position depends on the team structure if the player number is < 11
     and on some constants otherwise.
@@ -112,18 +174,40 @@ player_get_position_from_structure(gint structure, gint player_number)
     return position;
 }
 
-/** Calculate the talent value of the player based on his skill.
-    @param skill The player's skill.
-    @return The new talent value. */
+/** Calculate a player's skill depending on his age
+    and talent. */
 gfloat
-player_new_talent(gfloat skill)
+player_skill_from_talent(const Player *pl)
 {
-    gfloat talent = math_gauss_dist(2 * skill - const_float("float_player_max_skill"),
-				    const_float("float_player_max_skill"));
-    if(talent < skill)
-	talent = 2 * skill - talent;
+    gfloat skill = pl->talent, cur_age = pl->peak_age;
 
-    return talent;
+    if(pl->age < pl->peak_age)
+    {
+	while(cur_age > pl->age)
+	{
+	    cur_age -= ((const_float("float_player_lsu_update_limit") + 2) * 0.0192);
+	    if(pl->peak_age - cur_age > pl->peak_region)
+		skill *= (1 - ((pl->peak_age - cur_age) * 
+			       const_float("float_player_skill_update_younger_factor") +
+			       const_float("float_player_skill_update_younger_add")));
+	}
+    }
+    else
+    {
+	while(cur_age < pl->age)
+	{
+	    cur_age += ((const_float("float_player_lsu_update_limit") + 2) * 0.0192);
+	    if(cur_age - pl->peak_age > pl->peak_region)
+		skill *= (1 - ((cur_age - pl->peak_age) * 
+			       const_float("float_player_skill_update_older_factor") +
+			       const_float("float_player_skill_update_older_add")));
+	}
+    }
+
+    if(skill > pl->talent)
+	skill = pl->talent;
+    
+    return skill;
 }
 
 /** Estimate a player's talent.
@@ -159,25 +243,20 @@ player_estimate_talent(Player *pl)
 gint
 player_assign_value(const Player *pl)
 {
-    gint value;
-    gfloat diff = pl->peak_age - pl->age;
+    gfloat value;
 
-    value = (gint)powf((const_float("float_player_value_skill_weight") * pl->skill +
-			(1 - const_float("float_player_value_skill_weight")) * pl->talent * 0.7),
-		       const_float("float_player_value_power"));
+    value = powf((const_float("float_player_value_skill_weight") * pl->skill +
+		  (1 - const_float("float_player_value_skill_weight")) * 
+		  pl->talent * 0.7), const_float("float_player_value_power"));
     
-    if(diff > const_float("float_player_peak_age_diff_older1"))
-	value = (gint)rint((gfloat)value * (1 - const_float("float_player_value_scale1")));
-    else if(diff > const_float("float_player_peak_age_diff_peak_older"))
-	value = (gint)rint((gfloat)value * (1 - const_float("float_player_value_scale2")));
-    else if(diff < const_float("float_player_peak_age_diff_younger1"))
-	value = (gint)rint((gfloat)value * (1 + const_float("float_player_value_scale1")));
-    else if(diff < const_float("float_player_peak_age_diff_younger2"))
-	value = (gint)rint((gfloat)value * (1 + const_float("float_player_value_scale2")));
+    if(pl->peak_age > pl->age)
+	value *= (1 + const_float("float_player_value_younger_factor") *
+		  (pl->peak_age - pl->age));
+    else
+	value *= (1 - const_float("float_player_value_older_factor") *
+		  (pl->age - pl->peak_age));
 
-    value = math_round_integer(value, 2);
-
-    return value;
+    return math_round_integer((gint)rint(value), 2);
 }
 
 /** Assign a wage to a player. The wage depends mainly on
@@ -210,7 +289,7 @@ player_id_index(const Team *tm, gint player_id)
 	if(g_array_index(tm->players, Player, i).id == player_id)
 	    return i;
     
-    g_warning("player_id_index: didn't find player with id %d of team %s\n", player_id, tm->name->str);
+    g_warning("player_id_index: didn't find player with id %d of team %s\n", player_id, tm->name);
     
     main_exit_program(EXIT_INT_NOT_FOUND, NULL);
 
@@ -227,7 +306,7 @@ player_of_idx_team(const Team *tm, gint number)
     if(tm->players->len <= number)
     {
 	g_warning("player_of_idx_team: Player list of team %s too short for number %d.\n",
-		  tm->name->str, number);
+		  tm->name, number);
 
 	main_exit_program(EXIT_POINTER_NOT_FOUND, NULL);
 
@@ -251,7 +330,7 @@ player_of_id_team(const Team *tm, gint id)
 	if(g_array_index(tm->players, Player, i).id == id)
 	    return &g_array_index(tm->players, Player, i);
     
-    g_warning("player_of_id_team: didn't find player with id %d of team %s\n", id, tm->name->str);
+    g_warning("player_of_id_team: didn't find player with id %d of team %s\n", id, tm->name);
     
     main_exit_program(EXIT_POINTER_NOT_FOUND, NULL);
 
@@ -419,13 +498,6 @@ player_compare_substitute_func(gconstpointer a, gconstpointer b, gpointer data)
 	return_value = 
 	    misc_float_compare(skill_for_pos1, skill_for_pos2);
 
-/*     printf("%s %d %s %d   %d\n", pl1->name->str, pl1->pos, pl2->name->str,  */
-/* 	   pl2->pos, return_value); */
-/*     printf("\t gaski %.1f %.1f skipos %.1f %.1f struc %d %d\n", */
-/* 	   (gfloat)game_skill1 / 10000, (gfloat)game_skill2 / 10000, */
-/* 	   (gfloat)skill_for_pos1 / 10000, (gfloat)skill_for_pos2 / 10000, */
-/* 	   good_structure1, good_structure2); */
-	   	   
     return return_value;
 }
 
@@ -444,7 +516,7 @@ player_substitution_good_structure(gint old_structure, gint old_pos, gint player
 	old_structure - (gint)rint(powf(10, PLAYER_POS_FORWARD - old_pos)) +
 	(gint)rint(powf(10, PLAYER_POS_FORWARD - player_pos));
 
-    return query_integer_is_in_array(new_structure, accepted_structures, 0, 5);
+    return query_integer_is_in_array(new_structure, accepted_structures, 5);
 }
 
 /** Copy a player into a team.
@@ -499,10 +571,12 @@ player_swap(Team *tm1, gint player_number1, Team *tm2, gint player_number2)
     if(stat0 == STATUS_LIVE_GAME_PAUSE)
     {
 	clid = usr(stat2).live_game.fix->clid;
-	if((player_number1 < 11 && player_is_banned(player_of_idx_team(tm1, player_number1)) > 0) ||
-	   (player_number2 < 11 && player_is_banned(player_of_idx_team(tm2, player_number2)) > 0))
+	if((player_number1 < 11 && player_is_banned(player_of_idx_team(tm1, player_number1)) > 0 &&
+	    player_of_idx_team(tm1, player_number1)->participation) ||
+	   (player_number2 < 11 && player_is_banned(player_of_idx_team(tm2, player_number2)) > 0 &&
+	    player_of_idx_team(tm2, player_number2)->participation))
 	{
-	    game_gui_show_warning("You can't replace a banned player.");
+	    game_gui_show_warning(_("You can't replace a banned player."));
 	    return;
 	}
     }
@@ -613,35 +687,30 @@ player_get_game_skill(const Player *pl, gboolean skill)
 void
 player_decrease_fitness(Player *pl)
 {
-    gint i;
-    gint age_limits[7] = {const_float("float_player_peak_age_diff_younger1"),
-			  const_float("float_player_peak_age_diff_younger2"),
-			  const_float("float_player_peak_age_diff_younger3"),
-			  const_float("float_player_peak_age_diff_peak_younger"),
-			  const_float("float_player_peak_age_diff_peak_older"),
-			  const_float("float_player_peak_age_diff_older1"),
-			  const_float("float_player_peak_age_diff_older2")};
-    gfloat reduce[8] =
-	{const_float("float_player_fitness_decrease_younger1"),
-	 const_float("float_player_fitness_decrease_younger2"),
-	 const_float("float_player_fitness_decrease_younger3"),
-	 const_float("float_player_fitness_decrease_peak_younger"),
-	 const_float("float_player_fitness_decrease_peak_older"),
-	 const_float("float_player_fitness_decrease_older1"),
-	 const_float("float_player_fitness_decrease_older2"),
-	 const_float("float_player_fitness_decrease_else")};
+    gfloat goalie_factor = 
+	1 - const_float("float_player_fitness_decrease_factor_goalie") *
+	(pl->cpos == 0);
+    gfloat boost_factor = 
+	1 + (gfloat)pl->team->boost * 
+	const_float("float_player_boost_fitness_effect");
+    gfloat streak_factor = 1 + (gfloat)pl->streak * 
+	const_float("float_player_streak_influence_fitness_decrease");
 
-    gfloat diff = pl->peak_age - pl->age;
-    gfloat reduction;
+    if(pl->age < pl->peak_age - pl->peak_region)
+	pl->fitness -= (((pl->peak_age - pl->peak_region - pl->age) *
+			 const_float("float_player_fitness_decrease_younger_factor") +
+			 const_float("float_player_fitness_decrease_add")) *
+			goalie_factor * boost_factor * streak_factor);
+    else if(pl->age > pl->peak_age + pl->peak_region)
+	pl->fitness -= (((pl->age - pl->peak_age - pl->peak_region) *
+			 const_float("float_player_fitness_decrease_older_factor") +
+			 const_float("float_player_fitness_decrease_add")) *
+			goalie_factor * boost_factor * streak_factor);
+    else
+	pl->fitness -= (const_float("float_player_fitness_decrease_add") *
+			goalie_factor * boost_factor * streak_factor);
 
-    for(i=0;i<7;i++)
-	if(diff > age_limits[i])
-	    break;
-
-    reduction = reduce[i] * (1 + (gfloat)pl->team->boost * const_float("float_player_boost_fitness_effect")) *
-	(1 - (pl->cpos == 0) * const_float("float_player_fitness_decrease_factor_goalie"));
-    
-    pl->fitness = MAX(0, pl->fitness - reduction);
+    pl->fitness = MAX(0, pl->fitness);
 }
 
 
@@ -653,26 +722,11 @@ player_decrease_fitness(Player *pl)
 void
 player_update_fitness(Player *pl)
 {
-    gint i;
-    gint age_limits[7] = {const_float("float_player_peak_age_diff_younger1"),
-			  const_float("float_player_peak_age_diff_younger2"),
-			  const_float("float_player_peak_age_diff_younger3"),
-			  const_float("float_player_peak_age_diff_peak_younger"),
-			  const_float("float_player_peak_age_diff_peak_older"),
-			  const_float("float_player_peak_age_diff_older1"),
-			  const_float("float_player_peak_age_diff_older2")};
-    gfloat increase_base[8] =
-	{const_float("float_player_fitness_increase_younger1"),
-	 const_float("float_player_fitness_increase_younger2"),
-	 const_float("float_player_fitness_increase_younger3"),
-	 const_float("float_player_fitness_increase_peak_younger"),
-	 const_float("float_player_fitness_increase_peak_older"),
-	 const_float("float_player_fitness_increase_older1"),
-	 const_float("float_player_fitness_increase_older2"),
-	 const_float("float_player_fitness_increase_else")};
-
-    gfloat diff = pl->peak_age - pl->age;
-    gfloat increase = -1;
+    gfloat variance = 
+	math_rnd(1 - const_float("float_player_fitness_increase_variance"),
+		 1 + const_float("float_player_fitness_increase_variance"));
+    gfloat streak_factor = 
+	1 + (pl->streak * const_float("float_player_streak_influence_fitness_increase"));
 
     if(pl->participation)
     {
@@ -680,14 +734,21 @@ player_update_fitness(Player *pl)
 	return;
     }
 
-    for(i=0;i<7;i++)
-	if(diff > age_limits[i])
-	    break;
+    if(pl->age < pl->peak_age - pl->peak_region)
+	pl->fitness += (((pl->peak_age - pl->peak_region - pl->age) *
+			 const_float("float_player_fitness_increase_younger_factor") +
+			 const_float("float_player_fitness_increase_add")) *
+			variance * streak_factor);
+    else if(pl->age > pl->peak_age + pl->peak_region)
+	pl->fitness += (((pl->age - pl->peak_age - pl->peak_region) *
+			 const_float("float_player_fitness_increase_older_factor") +
+			 const_float("float_player_fitness_increase_add")) *
+			variance * streak_factor);
+    else
+	pl->fitness += (const_float("float_player_fitness_increase_add") * 
+			variance * streak_factor);
 
-    increase = math_rnd(increase_base[i] - const_float("float_player_fitness_increase_variance"),
-			increase_base[i] + const_float("float_player_fitness_increase_variance"));
-
-    pl->fitness = MIN(pl->fitness + increase, 1);
+    pl->fitness = MIN(pl->fitness, 1);
 }
 
 /** Return the number of yellow cards of a player
@@ -834,39 +895,22 @@ void
 player_update_skill(Player *pl)
 {
     gint i;
-    gint age_limits[7] =
-	{const_float("float_player_peak_age_diff_younger1"),
-	 const_float("float_player_peak_age_diff_younger2"),
-	 const_float("float_player_peak_age_diff_younger3"),
-	 const_float("float_player_peak_age_diff_peak_younger"),
-	 const_float("float_player_peak_age_diff_peak_older"),
-	 const_float("float_player_peak_age_diff_older1"),
-	 const_float("float_player_peak_age_diff_older2")};
-    gfloat factor_limits[3][2] =
-	{{const_float("float_player_skill_devel_younger1_lower"),
-	  const_float("float_player_skill_devel_younger1_upper")},
-	 {const_float("float_player_skill_devel_younger2_lower"),
-	  const_float("float_player_skill_devel_younger2_upper")},
-	 {const_float("float_player_skill_devel_younger3_lower"),
-	  const_float("float_player_skill_devel_younger3_upper")}};
-    gint increase_decrease[4][2] =
-	{{const_float("float_player_skill_devel_peak_lower"),
-	  const_float("float_player_skill_devel_peak_upper")},
-	 {const_float("float_player_skill_devel_older1_lower"),
-	  const_float("float_player_skill_devel_older1_upper")},
-	 {const_float("float_player_skill_devel_older2_lower"),
-	  const_float("float_player_skill_devel_older2_upper")},
-	 {const_float("float_player_skill_devel_else_lower"),
-	  const_float("float_player_skill_devel_else_upper")}};
-    gfloat diff;
 
-    if(pl->age > pl->peak_age)
-	pl->lsu += (pl->health == 0) ? 1 : const_float("float_player_lsu_injured_old");
-    else if(pl->health == 0 &&
-	    ((gfloat)player_games_goals_get(pl, pl->team->clid, PLAYER_VALUE_GAMES) / (gfloat)week >=
-	     const_float("float_player_lsu_games_percentage") || 
-	     math_rnd(0, 1) <= const_float("float_player_lsu_increase_prob")))
-	pl->lsu++;
+    if(!query_player_is_youth(pl))
+    {
+	if(pl->age > pl->peak_age)
+	    pl->lsu += (pl->health == 0) ? 1 : const_float("float_player_lsu_injured_old");
+	else if(pl->health == 0 &&
+		((gfloat)player_games_goals_get(pl, pl->team->clid, PLAYER_VALUE_GAMES) / (gfloat)week >=
+		 const_float("float_player_lsu_games_percentage") || 
+		 math_rnd(0, 1) <= const_float("float_player_lsu_increase_prob")))
+	    pl->lsu++;
+    }
+    else
+	pl->lsu += (pl->health != 0) ? 0 :
+	    const_float("float_youth_academy_lsu_addition_best") -
+	    (user_from_team(pl->team)->youth_academy.av_coach *
+	     const_float("float_youth_academy_lsu_penalty"));
 
     if(pl->lsu < const_float("float_player_lsu_update_limit") ||
        math_rnd(0, 1) < powf(const_float("float_player_lsu_update_base_prob"),
@@ -875,20 +919,17 @@ player_update_skill(Player *pl)
 	
     pl->lsu = 0;
 
-    diff = pl->peak_age - pl->age;
-
-    for(i=0;i<7;i++)
-	if(diff > age_limits[i])
-	    break;
-
-    if(i < 3)
-	pl->skill += (pl->talent - pl->skill) * math_rnd(factor_limits[i][0], factor_limits[i][1]);
-    else if(i < 5)
-	pl->skill += math_rnd(increase_decrease[0][0], increase_decrease[0][1]);
-    else if(i < 7)
-	pl->skill += math_rnd(increase_decrease[i - 4][0], increase_decrease[i - 4][1]);
+    if(pl->age < pl->peak_age - pl->peak_region)
+	pl->skill *= (1 + ((pl->peak_age - pl->age) * 
+			   const_float("float_player_skill_update_younger_factor") +
+			   const_float("float_player_skill_update_younger_add")));
+    else if(pl->age > pl->peak_age + pl->peak_region)
+	pl->skill *= (1 - ((pl->age - pl->peak_age) * 
+			   const_float("float_player_skill_update_older_factor") +
+			   const_float("float_player_skill_update_older_add")));
     else
-	pl->skill += math_rnd(increase_decrease[3][0], increase_decrease[3][1]);
+	pl->skill *= math_rnd(1 - const_float("float_player_skill_update_peak"),
+			      1 + const_float("float_player_skill_update_peak"));
 
     pl->skill = CLAMP(pl->skill, 0, pl->talent);
     pl->cskill = player_get_cskill(pl, pl->cpos, TRUE);
@@ -945,49 +986,119 @@ player_update_injury(Player *pl)
     }
 }
 
-/** Update players in user teams (age, skill, fitness etc.)
-    @param tm The team of the player.
-    @param idx The index in the players array. */
+/** Weekly test whether a player goes on a hot/cold
+    streak; if he's on a streak, decrease the streak counter. */
 void
-player_update_weekly(Team *tm, gint idx)
+player_update_streak(Player *pl)
 {
-    gchar buf[SMALL];
-    Player *pl = player_of_idx_team(tm, idx);
+    gfloat streak_type, streak_prob, 
+	streak_length, decrease_factor = 0;
+    gfloat streak_prob_factor = 
+	const_float("float_player_streak_prob_max") - 
+	const_float("float_player_streak_prob_zero"),
+	streak_prob_add = const_float("float_player_streak_prob_zero");
+
+    /** Player streak is locked. */
+    if(pl->streak_count < 0)
+    {
+	pl->streak_count++;
+	
+	if(pl->streak_count >= 0)
+	    pl->streak_count = 0;
+
+	return;
+    }
+    else if(pl->streak_count > 0)
+    {
+	if(pl->streak == PLAYER_STREAK_HOT)
+	    decrease_factor = -1;
+	else if(pl->streak == PLAYER_STREAK_COLD)
+	    decrease_factor = 1;
+	else
+	    g_warning("player_update_streak: streak count is positive (%.1f) but player %s is not on a streak!\n", pl->streak_count, pl->name);
+
+	pl->streak_count -= 
+	    (pl->streak_prob * decrease_factor *
+	     const_float("float_player_streak_count_decrease_factor") +
+	     const_float("float_player_streak_count_decrease_add"));
     
+	/** Streak is over. */
+	if(pl->streak_count <= 0)
+	{
+	    pl->streak = PLAYER_STREAK_NONE;
+	    player_streak_reset_count(pl);
+	}
+	
+	return;
+    }
+
+    if(query_player_is_in_ya(pl))
+	return;
+
+    /** Now let's find out whether there's a new streak. */
+    streak_type = math_rnd(-1, 1);
+    streak_prob = math_rnd(0, 1);
+    streak_length = math_rnd(const_float("float_player_streak_length_lower"),
+			     const_float("float_player_streak_length_upper"));
+
+    if(streak_type < pl->streak_prob &&
+       ((pl->streak_prob > 0 &&  
+	 streak_prob < streak_prob_factor * pl->streak_prob + streak_prob_add) ||
+	(pl->streak_prob <= 0 && 
+	 streak_prob < streak_prob_add * pl->streak_prob + streak_prob_add)))
+    {
+	pl->streak = PLAYER_STREAK_HOT;
+	pl->streak_count = streak_length;
+    }
+    else if(streak_type > pl->streak_prob &&
+	    ((pl->streak_prob > 0 &&  
+	      streak_prob < -streak_prob_add * pl->streak_prob + streak_prob_add) ||
+	     (pl->streak_prob <= 0 && 
+	      streak_prob < -streak_prob_factor * pl->streak_prob + streak_prob_add)))
+    {
+	pl->streak = PLAYER_STREAK_COLD;
+	pl->streak_count = streak_length;
+    }
+}
+
+/** Update a player in a user team (age, skill etc.). */
+void
+player_update_weekly(Player *pl)
+{
+    pl->age += 0.0192;
+
     if(debug < 50)
-    {
-	pl->age += 0.0192;
-	pl->contract -= 0.0192;
-    }
+	if(!sett_int("int_opt_disable_contracts") &&
+	   pl->age > const_float("float_player_age_lower"))
+	    pl->contract -= 0.0192;
 
-    if(pl->contract * 12 <= opt_user_int("int_opt_user_contract_limit") &&
-       (pl->contract + 0.0192) * 12 > opt_user_int("int_opt_user_contract_limit") &&
-       debug < 50)
-    {
-	sprintf(buf, _("%s's contract expires in %.1f years."),
-		pl->name->str, pl->contract);
-
-	user_event_add(&usr(team_is_user(tm)), EVENT_TYPE_WARNING,
-		       -1, -1, NULL, buf);
-    }
+    if(!sett_int("int_opt_disable_contracts") && debug < 50 &&
+       pl->contract * 12 <= opt_user_int("int_opt_user_contract_limit") &&
+       (pl->contract + 0.0192) * 12 > opt_user_int("int_opt_user_contract_limit"))
+	user_event_add(user_from_team(pl->team), EVENT_TYPE_WARNING,
+		       -1, -1, NULL, _("%s's contract expires in %.1f years."),
+		       pl->name, pl->contract);
 
     if(pl->contract <= 0)
-	player_remove_contract(tm, idx);
+	player_remove_contract(pl);
 
     player_update_skill(pl);
+
     if(pl->health > 0)
 	player_update_injury(pl);
+    else
+	player_update_streak(pl);
 }
 
 /** Remove a player from a user team after the contract expired.
     @param tm The user team.
     @param idx The player index. */
 void
-player_remove_contract(Team *tm, gint idx)
+player_remove_contract(Player *pl)
 {
-    user_event_add(user_from_team(tm), EVENT_TYPE_PLAYER_LEFT, -1, -1, NULL,
-		   player_of_idx_team(tm, idx)->name->str);
-    player_remove_from_team(tm, idx);
+    user_event_add(user_from_team(pl->team), EVENT_TYPE_PLAYER_LEFT, -1, -1, NULL,
+		   pl->name);
+    player_remove_from_team(pl->team, player_id_index(pl->team, pl->id));
 }
 
 /** Remove a player from a team.
@@ -995,6 +1106,7 @@ player_remove_contract(Team *tm, gint idx)
 void
 player_remove_from_team(Team *tm, gint idx)
 {
+    transfer_remove_player_ptr(player_of_idx_team(tm, idx));
     free_player(player_of_idx_team(tm, idx));
     g_array_remove_index(tm->players, idx);    
 }
@@ -1004,20 +1116,40 @@ player_remove_from_team(Team *tm, gint idx)
     @param pl The player we update.
     @param clid The fixture clid. */
 void
-player_update_post_match(Player *pl, gint clid)
+player_update_post_match(Player *pl, const Fixture *fix)
 {
-    gint yellow_red = league_cup_get_yellow_red(clid);
+    gint yellow_red = league_cup_get_yellow_red(fix->clid);
+    gint winner = -1;
 
-    if(player_card_get(pl, clid, PLAYER_VALUE_CARD_RED) > 0)
-	player_card_set(pl, clid, PLAYER_VALUE_CARD_RED, -1, TRUE);
+    if(player_card_get(pl, fix->clid, PLAYER_VALUE_CARD_RED) > 0)
+	player_card_set(pl, fix->clid, PLAYER_VALUE_CARD_RED, -1, TRUE);
 
-    if(player_card_get(pl, clid, PLAYER_VALUE_CARD_YELLOW) >= yellow_red)
+    if(player_card_get(pl, fix->clid, PLAYER_VALUE_CARD_YELLOW) >= yellow_red)
     {
-	player_card_set(pl, clid, PLAYER_VALUE_CARD_YELLOW, 0, FALSE);
+	player_card_set(pl, fix->clid, PLAYER_VALUE_CARD_YELLOW, 0, FALSE);
 	
-	if(player_card_get(pl, clid, PLAYER_VALUE_CARD_RED) == 0 && debug < 50)
-	    player_card_set(pl, clid, PLAYER_VALUE_CARD_RED, 1, FALSE);
-    }    
+	if(player_card_get(pl, fix->clid, PLAYER_VALUE_CARD_RED) == 0 && debug < 50)
+	    player_card_set(pl, fix->clid, PLAYER_VALUE_CARD_RED, 1, FALSE);
+    }
+
+    if(pl->cpos == PLAYER_POS_GOALIE &&
+       ((fix->result[0][0] == 0 && fix->teams[1] == pl->team) ||
+	(fix->result[1][0] == 0 && fix->teams[0] == pl->team)))
+	player_streak_add_to_prob(
+	    pl, const_float("float_player_streak_add_goalie_clean"));
+
+    /** No streak change if we had a draw. */
+    if(!fix->decisive && fix->result[0][0] == fix->result[1][0])
+       return;
+
+    winner = GPOINTER_TO_INT(fixture_winner_of(fix, TRUE));
+
+    if(winner == pl->team->id)
+	player_streak_add_to_prob(
+	    pl, const_float("float_player_streak_add_win"));
+    else
+	player_streak_add_to_prob(
+	    pl, const_float("float_player_streak_add_loss"));
 }
 
 /** Replace a player by a new one in a cpu team. 
@@ -1030,7 +1162,7 @@ player_replace_by_new(Player *pl, gboolean free_player)
     gint idx = player_id_index(tm, pl->id);
     Player new = player_new(tm, team_get_average_skill(tm, FALSE), FALSE);
         
-    new.name = name_get(pl->team->names_file->str);
+    new.name = name_get(pl->team->names_file);
     new.pos = pl->pos;
     new.cpos = pl->cpos;
     new.id = pl->id;
@@ -1180,3 +1312,72 @@ player_get_last_name(const gchar *name)
 
     return rev_name;
 }
+
+/** Create a pointer array with pointers to the players
+    and return it. */
+GPtrArray*
+player_get_pointers_from_array(const GArray *players_array)
+{
+    gint i;
+    GPtrArray *players = g_ptr_array_new();
+
+    for(i=0;i<players_array->len;i++)
+	g_ptr_array_add(players, &g_array_index(players_array, Player, i));
+
+    return players;
+}
+
+
+/** Move a player from the team to the youth academy. */
+void
+player_move_to_ya(gint idx)
+{
+    Player *pl = player_of_idx_team(current_user.tm, idx);
+    Player player = *pl;
+
+    if(query_transfer_player_is_on_list(pl))
+	transfer_remove_player_ptr(pl);
+
+    player.cskill = player.skill;
+    player.cpos = player.pos;
+
+    g_array_remove_index(current_user.tm->players, idx);
+    g_array_append_val(current_user.youth_academy.players, player);
+}
+
+/** Move a player from the youth academy to the team. */
+void
+player_move_from_ya(gint idx)
+{
+    Player *pl = &g_array_index(current_user.youth_academy.players, Player, idx);
+    Player player = *pl;
+
+    g_array_remove_index(current_user.youth_academy.players, idx);
+    g_array_append_val(current_user.tm->players, player);
+}
+
+/** Wrapper for the streak probability addition operation
+    so that it always stays between -1 and 1. */
+void
+player_streak_add_to_prob(Player *pl, gfloat add)
+{
+    /** No streaks for CPU players (yet). */
+    if(team_is_user(pl->team) == -1)
+	return;
+
+    pl->streak_prob += add;
+    pl->streak_prob = CLAMP(pl->streak_prob, -1, 1);
+}
+
+/** Find out whether a player is in the youth academy. */
+gboolean
+query_player_is_in_ya(const Player *pl)
+{
+    gint i;
+
+    for(i=0;i<pl->team->players->len;i++)
+	if(pl == &g_array_index(pl->team->players, Player, i))
+	    return FALSE;
+
+    return TRUE;
+}
