@@ -1,4 +1,5 @@
 #include "cup.h"
+#include "file.h"
 #include "fixture.h"
 #include "game.h"
 #include "game_gui.h"
@@ -11,6 +12,7 @@
 #include "team.h"
 #include "transfer.h"
 #include "user.h"
+#include "xml_team.h"
 
 /**
    Generate a team with default values, e.g. 
@@ -23,15 +25,18 @@ team_new(gboolean new_id)
 {
     Team new;
 
-    new.name = g_string_new("");
-    new.names_file = g_string_new("");
-    new.symbol = g_string_new("");
+    new.name = NULL;
+    new.names_file = NULL;
+    new.symbol = NULL;
+    new.def_file = NULL;
+    new.stadium.name = NULL;
     
     new.clid = -1;
     new.id = (new_id) ? team_id_new : -1;
     new.structure = team_assign_playing_structure();
     new.style = team_assign_playing_style();
     new.boost = 0;
+    new.average_talent = 0;
 
     new.players = g_array_new(FALSE, FALSE, sizeof(Player));
 
@@ -95,33 +100,44 @@ team_generate_players_stadium(Team *tm)
     gfloat skill_factor = math_rnd(1 - const_float("float_team_skill_variance"),
 				   1 + const_float("float_team_skill_variance"));
     Player new;
-    gfloat average_skill;
-    gfloat wages = 0;
-
-    if(tm->clid < ID_CUP_START)
-	average_skill = 
-	    const_float("float_player_max_skill") * skill_factor *
-	    (((gfloat)league_from_clid(tm->clid)->average_skill) / 10000);
-    else
-	average_skill = 
-	    skill_factor * team_get_average_skills(lig(0).teams) *
-	    (1 + ((gfloat)cup_from_clid(tm->clid)->skill_diff / 10000));
-
-    average_skill = CLAMP(average_skill, 0, const_float("float_player_max_skill"));    
-
-    for(i=0;i<const_int("int_team_cpu_players");i++)
-    {
-	new = player_new(tm, average_skill, TRUE);
-	wages += new.wage;
-	g_array_append_val(tm->players, new);
-    }
+    gfloat wages = 0, average_talent;
+    gchar *def_file = team_has_def_file(tm);
 
     tm->stadium.average_attendance = tm->stadium.possible_attendance =
-	tm->stadium.games = 0;    
+	tm->stadium.games = 0;
     tm->stadium.safety = 
 	math_rnd(const_float("float_team_stadium_safety_lower"),
 		 const_float("float_team_stadium_safety_upper"));
 
+    if(tm->clid < ID_CUP_START)
+	average_talent = (tm->average_talent == 0) ?
+	    skill_factor * league_from_clid(tm->clid)->average_talent :
+	    tm->average_talent;
+    else
+	average_talent = 
+	    skill_factor * team_get_average_talents(lig(0).teams) *
+	    (1 + cup_from_clid(tm->clid)->talent_diff);
+	
+    average_talent = CLAMP(average_talent, 0, const_float("float_player_max_skill"));
+    tm->average_talent = average_talent;
+
+    if(def_file == NULL)
+    {
+	for(i=0;i<const_int("int_team_cpu_players");i++)
+	{
+	    new = player_new(tm, average_talent, TRUE);
+	    g_array_append_val(tm->players, new);
+	}
+    }
+    else
+    {
+	xml_team_read(tm, def_file);
+	g_free(def_file);
+    }
+
+    for(i=0;i<const_int("int_team_cpu_players") - 2;i++)
+	wages += g_array_index(tm->players, Player, i).wage;
+
     tm->stadium.capacity = 
 	math_round_integer((gint)rint((wages / (gfloat)const_int("int_team_stadium_ticket_price")) *
 				      const_float("float_team_stadium_size_wage_factor")), 2);
@@ -135,7 +151,7 @@ team_generate_players_stadium(Team *tm)
     @return TRUE if the team's already participating in a cup,
     FALSE otherwise. */
 gboolean
-query_team_is_in_international_cups(const Team *tm, gint group)
+query_team_is_in_cups(const Team *tm, gint group)
 {
     gint i, j;
 
@@ -145,11 +161,12 @@ query_team_is_in_international_cups(const Team *tm, gint group)
     for(i=0;i<cps->len;i++)
 	if(cp(i).group == group)
 	    for(j=0;j<cp(i).team_names->len;j++)
-		if(strcmp(tm->name->str, ((GString*)g_ptr_array_index(cp(i).team_names, j))->str) == 0)
+		if(strcmp(tm->name, 
+			  (gchar*)g_ptr_array_index(cp(i).team_names, j)) == 0)
 		{
 		    if(debug > 90)
-			printf("team %s group %d found in %s \n", tm->name->str,
-			       group, cp(i).name->str);
+			printf("team %s group %d found in %s \n", tm->name,
+			       group, cp(i).name);
 		    return TRUE;
 		}
     
@@ -166,10 +183,11 @@ query_team_is_in_cup(const Team *tm, const Cup *cup)
     gint i;
 
     for(i=0;i<cup->team_names->len;i++)
-	if(strcmp(tm->name->str, ((GString*)g_ptr_array_index(cup->team_names, i))->str) == 0)
+	if(strcmp(tm->name, 
+		  (gchar*)g_ptr_array_index(cup->team_names, i)) == 0)
 	    return TRUE;
 
-	return FALSE;
+    return FALSE;
 }
 
 /** Return a GPtrArray containing the pointers
@@ -212,21 +230,6 @@ team_of_id(gint id)
     return NULL;
 }
 
-/** Return the players of the team in a pointer array.
-    @param tm The team we examine.
-    @return The players of the team in an array. */
-GPtrArray*
-team_get_player_pointers(const Team *tm)
-{
-    gint i;
-    GPtrArray *players = g_ptr_array_new();
-
-    for(i=0;i<tm->players->len;i++)
-	g_ptr_array_add(players, &g_array_index(tm->players, Player, i));
-
-    return players;
-}
-
 /** Return a pointer to the next or last fixture the team participates in.
     @param tm The team we examine.
     @return The pointer to the fixture or NULL if none is found. */
@@ -247,7 +250,7 @@ team_get_fixture(const Team *tm, gboolean last_fixture)
     {
 	for(i=0;i<ligs->len;i++)
 	{
-	    if(lig(i).id == tm->clid)
+	    if(lig(i).active && lig(i).id == tm->clid)
 	    {
 		for(j=0;j<lig(i).fixtures->len;j++)
 		    if(g_array_index(lig(i).fixtures, Fixture, j).attendance == -1 &&
@@ -285,7 +288,7 @@ team_get_fixture(const Team *tm, gboolean last_fixture)
     {
 	for(i=0;i<ligs->len;i++)
 	{
-	    if(lig(i).id == tm->clid)
+	    if(lig(i).active && lig(i).id == tm->clid)
 	    {
 		for(j=lig(i).fixtures->len - 1;j>=0;j--)
 		    if(g_array_index(lig(i).fixtures, Fixture, j).attendance != -1 &&
@@ -332,7 +335,7 @@ team_is_user(const Team *tm)
     gint i;
 
     for(i=0;i<users->len;i++)
-	if(usr(i).tm == tm)
+	if(usr(i).team_id == tm->id)
 	    return i;
 
     return -1;
@@ -367,28 +370,59 @@ team_get_average_skill(const Team *tm, gboolean cskill)
     return (counter > 0) ? sum / (gfloat)counter : 0;
 }
 
+/** Return the overall average talent of the team's players.
+    @param tm The team we examine. */
+gfloat
+team_get_average_talent(const Team *tm)
+{
+    gint i;
+    gfloat sum = 0;
+
+    for(i=0;i<tm->players->len;i++)
+	sum += player_of_idx_team(tm, i)->talent;
+
+    return (tm->players->len > 0) ? sum / (gfloat)tm->players->len : 0;
+}
+
+
 /** Return the rank of the team in the league tables. */
 gint
 team_get_league_rank(const Team *tm)
 {
-    gint i;
-    GArray *elements = league_from_clid(tm->clid)->table.elements;
+    gint i, clid = team_get_table_clid(tm), rank = 0;
+    GArray *elements = NULL;
+    
+    if(clid < ID_CUP_START)
+    {
+	if(!league_from_clid(clid)->active)
+	    return 0;
+
+	elements = league_from_clid(tm->clid)->table.elements;
+    }
+    else
+    {
+	rank = team_get_cup_rank(
+	    tm, &g_array_index(cup_from_clid(clid)->rounds, CupRound,
+			       cup_has_tables(clid)), FALSE);
+	return (rank == -1) ? 0 : rank;
+    }
 
     for(i=0;i<elements->len;i++)
-	if(g_array_index(elements, TableElement, i).team == tm)
+	if(g_array_index(elements, TableElement, i).team_id == tm->id)
 	    return i + 1;
     
     g_warning("team_get_league_rank: no rank found for team %s in league %s. \n",
-	      tm->name->str, league_cup_get_name_string(tm->clid));
+	      tm->name, league_cup_get_name_string(tm->clid));
 
     main_exit_program(EXIT_INT_NOT_FOUND, NULL);
 
     return -1;
 }
 
-/** Return the rank of the team in the round robin stage. */
+/** Return the rank of the team in the round robin stage.
+    @param abort Whether to exit if no corresponding entry can be found. */
 gint
-team_get_cup_rank(const Team *tm, const CupRound *cupround)
+team_get_cup_rank(const Team *tm, const CupRound *cupround, gboolean abort)
 {
     gint i, j;
 
@@ -399,9 +433,12 @@ team_get_cup_rank(const Team *tm, const CupRound *cupround)
 		return j + 1;
     }
 
-    g_warning("team_get_cup_rank: no rank found for team %s. \n ", tm->name->str);
-
-    main_exit_program(EXIT_INT_NOT_FOUND, NULL);
+    if(abort)
+    {
+	g_warning("team_get_cup_rank: no rank found for team %s. \n ", tm->name);
+	
+	main_exit_program(EXIT_INT_NOT_FOUND, NULL);
+    }
 
     return -1;
 }
@@ -417,8 +454,13 @@ team_find_appropriate_structure(const Team *tm)
   gint structure = 0;
 
   for(i=0;i<11;i++)
-      if(player_of_idx_team(tm, i)->cskill > 0 && player_of_idx_team(tm, i)->pos != 0)
-	  structure += (gint)rint(powf(10, PLAYER_POS_FORWARD - player_of_idx_team(tm, i)->pos));
+      if(player_of_idx_team(tm, i)->cskill > 0 && player_of_idx_team(tm, i)->cpos != 0)
+      {
+	  if(player_of_idx_team(tm, i)->pos != 0)
+	      structure += (gint)rint(powf(10, PLAYER_POS_FORWARD - player_of_idx_team(tm, i)->pos));
+	  else
+	      structure += (gint)rint(powf(10, PLAYER_POS_FORWARD - player_of_idx_team(tm, i)->cpos));
+      }
 
   return structure;
 }
@@ -498,10 +540,13 @@ team_attribute_to_char(gint attribute, gint value)
 	    switch(value)
 	    {
 		case -1:
+		    /* Boost value. */
 		    return _("ANTI");
 		case 0:
+		    /* Boost value. */
 		    return _("OFF");
 		case 1:
+		    /* Boost value. */
 		    return _("ON");
 	    }
 	    break;
@@ -516,8 +561,6 @@ team_attribute_to_char(gint attribute, gint value)
 void
 team_change_attribute_with_message(Team *tm, gint attribute, gint new_value)
 {
-    gchar buf[SMALL];
-
     switch(attribute)
     {
 	default:
@@ -525,15 +568,13 @@ team_change_attribute_with_message(Team *tm, gint attribute, gint new_value)
 	    break;
 	case TEAM_ATTRIBUTE_STYLE:
 	    current_user.tm->style = new_value;
-	    sprintf(buf, "Team style changed to %s.", team_attribute_to_char(attribute, new_value));
+	    game_gui_print_message(_("Team style changed to %s."), team_attribute_to_char(attribute, new_value));
 	    break;
 	case TEAM_ATTRIBUTE_BOOST:
 	    current_user.tm->boost = new_value;
-	    sprintf(buf, "Boost changed to %s.", team_attribute_to_char(attribute, new_value));
+	    game_gui_print_message(_("Boost changed to %s."), team_attribute_to_char(attribute, new_value));
 	    break;
     }
-
-    game_gui_print_message(buf);
 }
 
 /** Make cpu players healthy etc.
@@ -669,19 +710,19 @@ team_update_user_team_weekly(Team *tm)
     gint i;
 
     for(i=tm->players->len - 1;i>=0;i--)
-	player_update_weekly(tm, i);
+	player_update_weekly(&g_array_index(tm->players, Player, i));
 }
 
 /** Regenerate player fitness etc. after a match. 
     @param tm The user team we examine.
     @param clid The fixture clid. */
 void
-team_update_post_match(Team *tm, gint clid)
+team_update_post_match(Team *tm, const Fixture *fix)
 {
     gint i;
 
     for(i=0;i<tm->players->len;i++)
-	player_update_post_match(player_of_idx_team(tm, i), clid);
+	player_update_post_match(player_of_idx_team(tm, i), fix);
 }
 
 /** Some updates each round.
@@ -705,7 +746,7 @@ team_get_table_value(const Team *tm, gint type)
 
     if(tm->clid >= ID_CUP_START)
     {
-	g_warning("team_get_table_value: team is not a league team: %s \n", tm->name->str);
+	g_warning("team_get_table_value: team is not a league team: %s \n", tm->name);
 
 	main_exit_program(EXIT_INT_NOT_FOUND, NULL);
 
@@ -720,7 +761,7 @@ team_get_table_value(const Team *tm, gint type)
 
     if(i == elements->len)
     {
-	g_warning("team_get_table_value: table entry not found for team %s \n", tm->name->str);
+	g_warning("team_get_table_value: table entry not found for team %s \n", tm->name);
 
 	main_exit_program(EXIT_INT_NOT_FOUND, NULL);
 
@@ -790,6 +831,9 @@ team_compare_func(gconstpointer a, gconstpointer b, gpointer data)
     }
     else if(type == TEAM_COMPARE_UNSORTED)
 	return_value = 0;
+    else if(type == TEAM_COMPARE_AV_SKILL)
+	return_value = misc_float_compare(team_get_average_skill(tm1, FALSE),
+					  team_get_average_skill(tm2, FALSE));
 
     return return_value;
 }
@@ -884,27 +928,32 @@ team_get_index(const Team *tm)
 		return i;
     }
 
-    g_warning("team_get_index: team %s not found.\n", tm->name->str);
+    g_warning("team_get_index: team %s not found.\n", tm->name);
 
     main_exit_program(EXIT_INT_NOT_FOUND, NULL);
 
     return -1;
 }
 
-/** Return the average of the average skills of the teams in the array. */
+/** Return the average of the average talents of the teams in the array. */
 gfloat
-team_get_average_skills(const GArray *teams)
+team_get_average_talents(const GArray *teams)
 {
-    gint i;
+    gint i, j, cnt = 0;
     gfloat sum = 0;
 
     if(teams->len == 0)
 	return 0;
 
     for(i=0;i<teams->len;i++)
-	sum += team_get_average_skill(&g_array_index(teams, Team, i), FALSE);
+	for(j=0;j<g_array_index(teams, Team, i).players->len;j++)
+	{
+	    sum += g_array_index(g_array_index(teams, Team, i).players,
+				 Player, j).talent;
+	    cnt++;
+	}
 
-    return sum / teams->len;
+    return sum / (gfloat)cnt;
 }
 
 /** Find out whether a team plays at a given date. */
@@ -934,9 +983,10 @@ query_team_plays(const Team *tm, gint week_number, gint week_round_number)
 
 /** Show the results of the user team against the specified team. 
     @param buf The buffer to fill with the results.
-    @param sort Whether to sort the results according to home/away. */
+    @param sort Whether to sort the results according to home/away.
+    @param cup_matches Whether to take cup matches into account as well. */
 void
-team_write_own_results(const Team *tm, gchar *buf, gboolean sort)
+team_write_own_results(const Team *tm, gchar *buf, gboolean sort, gboolean cup_matches)
 {
     gint i, res[2];
     gchar buf2[SMALL], buf3[SMALL], buf4[SMALL], neutral[SMALL];
@@ -947,61 +997,68 @@ team_write_own_results(const Team *tm, gchar *buf, gboolean sort)
     strcpy(neutral, "");
 
     for(i=0;i<matches->len;i++)
-    {
-	res[0] = math_sum_int_array(((Fixture*)g_ptr_array_index(matches, i))->result[0], 2);
-	res[1] = math_sum_int_array(((Fixture*)g_ptr_array_index(matches, i))->result[1], 2);
-	
-	if(res[((Fixture*)g_ptr_array_index(matches, i))->teams[0] != current_user.tm] >
-	   res[((Fixture*)g_ptr_array_index(matches, i))->teams[0] == current_user.tm])
-	    sprintf(buf2, _("W  %d : %d"), 
-		    res[((Fixture*)g_ptr_array_index(matches, i))->teams[0] != current_user.tm],
-		    res[((Fixture*)g_ptr_array_index(matches, i))->teams[0] == current_user.tm]);
-	else if(res[((Fixture*)g_ptr_array_index(matches, i))->teams[0] != current_user.tm] <
-		res[((Fixture*)g_ptr_array_index(matches, i))->teams[0] == current_user.tm])
-	    sprintf(buf2, _("L  %d : %d"), 
-		    res[((Fixture*)g_ptr_array_index(matches, i))->teams[0] != current_user.tm],
-		    res[((Fixture*)g_ptr_array_index(matches, i))->teams[0] == current_user.tm]);
-	else
-	    sprintf(buf2, _("Dw %d : %d"),
-		    res[((Fixture*)g_ptr_array_index(matches, i))->teams[0] != current_user.tm],
-		    res[((Fixture*)g_ptr_array_index(matches, i))->teams[0] == current_user.tm]);
-	
-	if(((Fixture*)g_ptr_array_index(matches, i))->home_advantage)
+	if(cup_matches || ((Fixture*)g_ptr_array_index(matches, i))->clid < ID_CUP_START)
 	{
-	    
-	    if(((Fixture*)g_ptr_array_index(matches, i))->teams[0] == current_user.tm)
+	    res[0] = math_sum_int_array(((Fixture*)g_ptr_array_index(matches, i))->result[0], 2);
+	    res[1] = math_sum_int_array(((Fixture*)g_ptr_array_index(matches, i))->result[1], 2);
+	
+	    if(res[((Fixture*)g_ptr_array_index(matches, i))->teams[0] != current_user.tm] >
+	       res[((Fixture*)g_ptr_array_index(matches, i))->teams[0] == current_user.tm])
+		/* a won match */
+		sprintf(buf2, _("W  %d : %d"), 
+			res[((Fixture*)g_ptr_array_index(matches, i))->teams[0] != current_user.tm],
+			res[((Fixture*)g_ptr_array_index(matches, i))->teams[0] == current_user.tm]);
+	    else if(res[((Fixture*)g_ptr_array_index(matches, i))->teams[0] != current_user.tm] <
+		    res[((Fixture*)g_ptr_array_index(matches, i))->teams[0] == current_user.tm])
+		/* a lost match */
+		sprintf(buf2, _("L  %d : %d"), 
+			res[((Fixture*)g_ptr_array_index(matches, i))->teams[0] != current_user.tm],
+			res[((Fixture*)g_ptr_array_index(matches, i))->teams[0] == current_user.tm]);
+	    else
+		/* a drawn match */
+		sprintf(buf2, _("Dw %d : %d"),
+			res[((Fixture*)g_ptr_array_index(matches, i))->teams[0] != current_user.tm],
+			res[((Fixture*)g_ptr_array_index(matches, i))->teams[0] == current_user.tm]);
+	
+	    if(((Fixture*)g_ptr_array_index(matches, i))->home_advantage)
 	    {
-		sprintf(buf3, _("%s (H) "), buf2);
-		place = 0;
+	    
+		if(((Fixture*)g_ptr_array_index(matches, i))->teams[0] == current_user.tm)
+		{
+		    /* a match at home */
+		    sprintf(buf3, _("%s (H) "), buf2);
+		    place = 0;
+		}
+		else
+		{
+		    /* a match away */
+		    sprintf(buf3, _("<span background='%s' foreground='%s'>%s (A) </span> "), 
+			    const_app("string_treeview_live_game_commentary_away_bg"),
+			    const_app("string_treeview_live_game_commentary_away_fg"),
+			    buf2);
+		    place = 1;
+		}
 	    }
 	    else
 	    {
-		sprintf(buf3, _("<span background='%s' foreground='%s'>%s (A) </span> "), 
-			const_app("string_treeview_live_game_commentary_away_bg"),
-			const_app("string_treeview_live_game_commentary_away_fg"),
-			buf2);
-		place = 1;
+		/* a match on neutral ground */
+		sprintf(buf3, _("%s (N) "), buf2);
+		place = -1;
 	    }
-	}
-	else
-	{
-	    sprintf(buf3, _("%s (N) "), buf2);
-	    place = -1;
-	}
 
-	if(!sort || place == 1)
-	    strcat(buf4, buf3);
-	else
-	{
-	    if(place == 0)
+	    if(!sort || place == 1)
+		strcat(buf4, buf3);
+	    else
 	    {
-		strcpy(buf2, buf4);
-		sprintf(buf4, "%s%s", buf3, buf2);
+		if(place == 0)
+		{
+		    strcpy(buf2, buf4);
+		    sprintf(buf4, "%s%s", buf3, buf2);
+		}
+		else if(place == -1)
+		    strcat(neutral, buf3);
 	    }
-	    else if(place == -1)
-		strcat(neutral, buf3);
 	}
-    }
 
     sprintf(buf, "%s%s", buf4, neutral);
 
@@ -1034,11 +1091,14 @@ team_write_results(const Team *tm, gchar *result_buf, gchar *goals_buf)
 				g_ptr_array_index(latest_fixtures, i))->
 			       result[(((Fixture*)g_ptr_array_index(latest_fixtures, i))->teams[0] == tm)], 2);
 	if(res[0] == res[1])
+	    /* draw */
 	    strcat(result_buf, _("Dw "));
 	else if(res[(((Fixture*)g_ptr_array_index(latest_fixtures, i))->teams[0] == tm)] >
 		res[(((Fixture*)g_ptr_array_index(latest_fixtures, i))->teams[0] != tm)])
+	    /* lost */
 	    strcat(result_buf, _("L  "));
 	else
+	    /* won */
 	    strcat(result_buf, _("W  "));
     }
 
@@ -1058,3 +1118,183 @@ query_team_is_in_teams_array(const Team *tm, const GPtrArray *teams)
 
     return FALSE;
 }
+
+/** Check whether we find a definition file for the
+    given team. */
+gchar*
+team_has_def_file(const Team *tm)
+{
+    gchar *return_value = NULL;
+    gchar buf[SMALL];
+
+    if(tm->def_file != NULL && opt_int("int_opt_load_defs") != 0)
+    {
+	sprintf(buf, "team_%s.xml", tm->def_file);
+	return_value = file_find_support_file(buf, FALSE);
+    }
+
+    return return_value;
+}
+
+/** Complete the definition of the team (add players,
+    calculate wages etc. Called after reading a team def file. */
+void
+team_complete_def(Team *tm)
+{
+    gint i, new_pos, pos_sum;
+    gint positions[4] = {0, 0, 0, 0};
+    Player new_player;
+    gint add = const_int("int_team_cpu_players") - tm->players->len;
+    gboolean is_user = (team_is_user(tm) != -1);
+
+    for(i=0;i<tm->players->len;i++)
+    {
+	player_complete_def(&g_array_index(tm->players, Player, i), 
+			    tm->average_talent);
+	positions[g_array_index(tm->players, Player, i).pos]++;
+
+	/** This is so we don't remove loaded players
+	    from the team at startup. */
+	if(is_user)
+	    g_array_index(tm->players, Player, i).recovery = 1;
+    }
+
+    for(i=0;i<add;i++)
+    {
+	pos_sum = math_sum_int_array(positions, 4);
+	new_player = player_new(tm, tm->average_talent, TRUE);
+	
+	if(positions[0] < 2)
+	    new_pos = 0;
+	else if((gfloat)positions[1] / (gfloat)pos_sum <
+		const_float("float_player_pos_bound1"))
+	    new_pos = 1;
+	else if((gfloat)positions[2] / (gfloat)pos_sum <
+		const_float("float_player_pos_bound1"))
+	    new_pos = 2;
+	else
+	    new_pos = 3;
+
+	new_player.pos = new_player.cpos = new_pos;
+	positions[new_pos]++;
+
+	g_array_append_val(tm->players, new_player);
+    }
+
+    team_complete_def_sort(tm);
+}
+
+/** Sort the players in the team according to the team structure
+    and the player positions. */
+void
+team_complete_def_sort(Team *tm)
+{
+    gint i, j;
+    gint positions[4] = {0, 0, 0, 0};
+    gint structure[4] = {1, 
+			 math_get_place(tm->structure, 3),
+			 math_get_place(tm->structure, 2),
+			 math_get_place(tm->structure, 1)};
+    Player player_tmp, player_tmp2;
+    gchar buf[SMALL];
+    
+    for(i=0;i<11;i++)
+	positions[g_array_index(tm->players, Player, i).pos]++;
+
+    for(i=0;i<4;i++)
+    {
+	while(positions[i] > structure[i])
+	{
+	    for(j=0;j<11;j++)
+		if(g_array_index(tm->players, Player, j).pos == i)
+		{
+		    player_tmp = g_array_index(tm->players, Player, j);
+		    g_array_remove_index(tm->players, j);
+		    break;
+		}
+
+	    for(j=10;j<tm->players->len;j++)
+	    {
+		if(g_array_index(tm->players, Player, j).pos != i)
+		{
+		    player_tmp2 = g_array_index(tm->players, Player, j);
+		    g_array_remove_index(tm->players, j);
+		    break;
+		}
+	    }
+
+	    if(j == tm->players->len)
+	    {
+		sprintf(buf, "team_complete_def_sort (1): cannot sort according to structure %d (team %s).",
+			tm->structure, tm->name);
+		main_exit_program(EXIT_DEF_SORT, buf);
+	    }
+	    
+	    positions[i]--;
+	    positions[player_tmp2.pos]++;
+
+	    g_array_append_val(tm->players, player_tmp);
+	    g_array_prepend_val(tm->players, player_tmp2);	    
+	}
+
+	while(positions[i] < structure[i])
+	{
+	    for(j=0;j<11;j++)
+		if(g_array_index(tm->players, Player, j).pos > i)
+		{
+		    player_tmp = g_array_index(tm->players, Player, j);
+		    g_array_remove_index(tm->players, j);
+		    break;
+		}
+
+	    if(j == 11)
+	    {
+		sprintf(buf, "team_complete_def_sort (2): cannot sort according to structure %d (team %s).",
+			tm->structure, tm->name);
+		main_exit_program(EXIT_DEF_SORT, buf);
+	    }
+
+	    for(j=10;j<tm->players->len;j++)
+		if(g_array_index(tm->players, Player, j).pos == i)
+		{
+		    player_tmp2 = g_array_index(tm->players, Player, j);
+		    g_array_remove_index(tm->players, j);
+		    break;
+		}
+
+	    if(j == tm->players->len)
+	    {
+		sprintf(buf, "team_complete_def_sort (3): cannot sort according to structure %d (team %s).",
+			tm->structure, tm->name);
+		main_exit_program(EXIT_DEF_SORT, buf);
+	    }
+	    
+	    positions[i]++;
+	    positions[player_tmp.pos]--;
+
+	    g_array_append_val(tm->players, player_tmp);
+	    g_array_prepend_val(tm->players, player_tmp2);
+	}
+    }
+
+    team_rearrange(tm);
+}
+
+/** Find out which cup or league table the team
+    belongs to. */
+gint
+team_get_table_clid(const Team *tm)
+{
+    gint i;
+
+    if(tm->clid >= ID_CUP_START ||
+       (tm->clid < ID_CUP_START && !league_from_clid(tm->clid)->active))
+    {
+	for(i = acps->len - 1; i >= 0; i--)
+	    if(cup_has_tables(acp(i)->id) != -1 && 
+	       query_team_is_in_cup(tm, acp(i)))
+		return acp(i)->id;
+    }
+
+    return tm->clid;
+}
