@@ -18,15 +18,6 @@ enum TeamCompare
     TEAM_COMPARE_END
 };
 
-/** A struct needed when managing promotions
-    and relegations. */
-typedef struct
-{
-    Team tm; /**< The team we move. */
-    gint league_idx; /** The index of the league that the team gets moved to. */
-
-} TeamMove;
-
 Team
 team_new(gboolean new_id);
 
@@ -43,7 +34,7 @@ void
 team_generate_players_stadium(Team *tm);
 
 gboolean
-query_team_is_in_international_cups(const Team *tm, gint group);
+query_team_is_in_cups(const Team *tm, gint group);
 
 gboolean
 query_team_is_in_cup(const Team *tm, const Cup *cup);
@@ -54,20 +45,20 @@ team_get_pointers_from_array(const GArray *teams);
 Team*
 team_of_id(gint id);
 
-GPtrArray*
-team_get_player_pointers(const Team *tm);
-
 Fixture*
 team_get_fixture(const Team *tm, gboolean last_fixture);
 
 gfloat
 team_get_average_skill(const Team *tm, gboolean cskill);
 
+gfloat
+team_get_average_talent(const Team *tm);
+
 gint
 team_is_user(const Team *tm);
 
 gint
-team_get_cup_rank(const Team *tm, const CupRound *cupround);
+team_get_cup_rank(const Team *tm, const CupRound *cupround, gboolean abort);
 
 gint
 team_get_league_rank(const Team *tm);
@@ -94,7 +85,7 @@ void
 team_update_cpu_team(Team *tm, gboolean reset_fitness);
 
 void
-team_update_post_match(Team *tm, gint clid);
+team_update_post_match(Team *tm, const Fixture *fix);
 
 void
 team_update_cpu_corrections(Team *tm, gboolean reset_fitness);
@@ -124,7 +115,7 @@ gint
 team_get_index(const Team *tm);
 
 gfloat
-team_get_average_skills(const GArray *teams);
+team_get_average_talents(const GArray *teams);
 
 gboolean
 query_team_plays(const Team *tm, gint week_number, gint week_round_number);
@@ -136,9 +127,21 @@ void
 team_write_results(const Team *tm, gchar *result_buf, gchar *goals_buf);
 
 void
-team_write_own_results(const Team *tm, gchar *buf, gboolean sort);
+team_write_own_results(const Team *tm, gchar *buf, gboolean sort, gboolean cup_matches);
 
 gboolean
 query_team_is_in_teams_array(const Team *tm, const GPtrArray *teams);
 
+gchar*
+team_has_def_file(const Team *tm);
+
+void
+team_complete_def(Team *tm);
+
+void
+team_complete_def_sort(Team *tm);
+
+gint
+team_get_table_clid(const Team *tm);
+
 #endif
