@@ -1,3 +1,4 @@
+#include "callbacks.h"
 #include "cup.h"
 #include "finance.h"
 #include "free.h"
@@ -19,23 +20,35 @@ transfer_update(void)
 {
     gint i, j;
 
-    if(week > transfer_get_deadline())
+    if((week > transfer_get_deadline() ||
+       sett_int("int_opt_disable_transfers")) &&
+       transfer_list->len == 0)
 	return;
 
-    if(week == transfer_get_deadline())
+    if(week >= transfer_get_deadline())
     {
 	for(i=transfer_list->len - 1;i>=0;i--)
-	    transfer_remove_player(i);
-	return;
+	    if(trans(i).offers->len == 0)
+		transfer_remove_player(i);
+
+	if(transfer_list->len == 0)
+	    return;
     }
 
     for(i=transfer_list->len - 1;i>=0;i--)
 	for(j=trans(i).offers->len - 1;j>=0;j--)
-	    if(transoff(i, j).accepted || team_is_user(transoff(i, j).tm) == -1)
+	    if(transoff(i, j).status == TRANSFER_OFFER_REJECTED2 ||
+	       team_is_user(transoff(i, j).tm) == -1)
 		g_array_remove_index(trans(i).offers, j);
+	    else if(transoff(i, j).status == TRANSFER_OFFER_REJECTED)
+		transoff(i, j).status = TRANSFER_OFFER_REJECTED2;
+
+    if(week < transfer_get_deadline())
+    {
+	transfer_add_new_players();
+	transfer_add_cpu_offers();
+    }
 
-    transfer_add_new_players();
-    transfer_add_cpu_offers();
     transfer_evaluate_offers();
 
     for(i=transfer_list->len - 1;i>=0;i--)
@@ -108,10 +121,18 @@ transfer_offer_compare_func(gconstpointer a, gconstpointer b)
 	*tr2 = (const TransferOffer*)b;    
     gint return_value;
 
-    return_value = (tr1->fee != tr2->fee) ?
-	misc_int_compare(tr1->fee, tr2->fee) :
-	misc_int_compare(tr1->wage, tr2->wage);
-
+    if(tr1->status >= TRANSFER_OFFER_REJECTED &&
+       tr2->status >= TRANSFER_OFFER_REJECTED)
+	return_value = 0;
+    else if(tr1->status >= TRANSFER_OFFER_REJECTED)
+	return_value = 1;
+    else if(tr2->status >= TRANSFER_OFFER_REJECTED)
+	return_value = -1;
+    else
+	return_value = (tr1->fee != tr2->fee) ?
+	    misc_int_compare(tr1->fee, tr2->fee) :
+	    misc_int_compare(tr1->wage, tr2->wage);
+    
     return return_value;
 }
 
@@ -126,16 +147,16 @@ transfer_offers_notify(Transfer *tr, gboolean sort)
 	g_array_sort(tr->offers, transfer_offer_compare_func);
     off = &g_array_index(tr->offers, TransferOffer, 0);
 
-    off->accepted = TRUE;
+    off->status = TRANSFER_OFFER_ACCEPTED;
 
     if(team_is_user(tr->tm) != -1)
 	user_event_add(user_from_team(tr->tm), EVENT_TYPE_TRANSFER_OFFER_USER,
-		       -1, -1, NULL, player_of_id_team(tr->tm, tr->id)->name->str);
+		       -1, -1, NULL, player_of_id_team(tr->tm, tr->id)->name);
 
     else
 	user_event_add(user_from_team(off->tm),
 		       EVENT_TYPE_TRANSFER_OFFER_CPU, -1, -1,
-		       NULL, player_of_id_team(tr->tm, tr->id)->name->str);
+		       NULL, player_of_id_team(tr->tm, tr->id)->name);
 
     if(!sort)
 	user_event_show_next();
@@ -147,44 +168,56 @@ void
 transfer_evaluate_offers(void)
 {
     gint i, j;
+    gboolean notify;
 
     for(i=transfer_list->len - 1;i>=0;i--)
 	if(trans(i).offers->len > 0)
 	{
+	    notify = FALSE;
+
 	    if(team_is_user(trans(i).tm) == -1)
 	    {
 		for(j=trans(i).offers->len - 1; j >= 0; j--)
 		{
-		    if(player_of_id_team(trans(i).tm, trans(i).id)->value > transoff(i, j).fee &&
-		       player_of_id_team(trans(i).tm, trans(i).id)->wage > transoff(i, j).wage)
+		    if(transoff(i, j).status != TRANSFER_OFFER_REJECTED2)
 		    {
-			user_event_add(user_from_team(transoff(i, j).tm),
-				       EVENT_TYPE_TRANSFER_OFFER_REJECTED_FEE_WAGE, 
-				       transoff(i, j).fee, transoff(i, j).wage,
-				       trans(i).tm, player_of_id_team(trans(i).tm, trans(i).id)->name->str);
-			g_array_remove_index(trans(i).offers, j);
-		    }
-		    else if(player_of_id_team(trans(i).tm, trans(i).id)->value > transoff(i, j).fee)
-		    {
-			user_event_add(user_from_team(transoff(i, j).tm),
-				       EVENT_TYPE_TRANSFER_OFFER_REJECTED_FEE,
-				       transoff(i, j).fee, transoff(i, j).wage,
-				       trans(i).tm, player_of_id_team(trans(i).tm, trans(i).id)->name->str);
-			g_array_remove_index(trans(i).offers, j);
-		    }
-		    else if(player_of_id_team(trans(i).tm, trans(i).id)->wage > transoff(i, j).wage)
-		    {
-			user_event_add(user_from_team(transoff(i, j).tm),
-				       EVENT_TYPE_TRANSFER_OFFER_REJECTED_WAGE,
-				       transoff(i, j).fee, transoff(i, j).wage,
-				       trans(i).tm, player_of_id_team(trans(i).tm, trans(i).id)->name->str);
-			g_array_remove_index(trans(i).offers, j);
+			if(player_of_id_team(trans(i).tm, trans(i).id)->value > transoff(i, j).fee &&
+			   player_of_id_team(trans(i).tm, trans(i).id)->wage > transoff(i, j).wage)
+			{
+			    user_event_add(user_from_team(transoff(i, j).tm),
+					   EVENT_TYPE_TRANSFER_OFFER_REJECTED_FEE_WAGE, 
+					   transoff(i, j).fee, transoff(i, j).wage,
+					   trans(i).tm, player_of_id_team(trans(i).tm, trans(i).id)->name);
+			    transoff(i, j).status = TRANSFER_OFFER_REJECTED;
+			}
+			else if(player_of_id_team(trans(i).tm, trans(i).id)->value > transoff(i, j).fee)
+			{
+			    user_event_add(user_from_team(transoff(i, j).tm),
+					   EVENT_TYPE_TRANSFER_OFFER_REJECTED_FEE,
+					   transoff(i, j).fee, transoff(i, j).wage,
+					   trans(i).tm, player_of_id_team(trans(i).tm, trans(i).id)->name);
+			    transoff(i, j).status = TRANSFER_OFFER_REJECTED;
+			}
+			else if(player_of_id_team(trans(i).tm, trans(i).id)->wage > transoff(i, j).wage)
+			{
+			    user_event_add(user_from_team(transoff(i, j).tm),
+					   EVENT_TYPE_TRANSFER_OFFER_REJECTED_WAGE,
+					   transoff(i, j).fee, transoff(i, j).wage,
+					   trans(i).tm, player_of_id_team(trans(i).tm, trans(i).id)->name);
+			    transoff(i, j).status = TRANSFER_OFFER_REJECTED;
+			}
+			else
+			    notify = TRUE;
 		    }
 		}
 	    }
+	    else
+		notify = TRUE;
 
-	    if(trans(i).offers->len > 0)
+	    if(notify)
 		transfer_offers_notify(&trans(i), TRUE);
+	    else if(week >= transfer_get_deadline())
+		transfer_remove_player(i);
 	}
 }
 
@@ -264,8 +297,10 @@ transfer_get_deadline(void)
     gint length = 0;
 
     for(i=0;i<ligs->len;i++)
-	length = MAX(length, 
-		     g_array_index(lig(i).fixtures, Fixture, lig(i).fixtures->len - 1).week_number);
+	if(lig(i).active)
+	    length = 
+		MAX(length, g_array_index(
+			lig(i).fixtures, Fixture, lig(i).fixtures->len - 1).week_number);
 
     return (gint)rint((gfloat)length * const_float("float_transfer_deadline_percentage"));
 }
@@ -336,7 +371,7 @@ transfer_player_has_offer(const Player *pl)
 	if(trans(i).tm == pl->team &&
 	   trans(i).id == pl->id &&
 	   trans(i).offers->len > 0 &&
-	   transoff(i, 0).accepted)
+	   transoff(i, 0).status == TRANSFER_OFFER_ACCEPTED)
 	    return &transoff(i, 0);
 
     return NULL;
@@ -355,35 +390,20 @@ transfer_add_offer(gint idx, Team *tm, gint fee, gint wage)
 	{
 	    transoff(idx, i).fee = fee;
 	    transoff(idx, i).wage = wage;
+	    transoff(idx, i).status = TRANSFER_OFFER_NOT_CONSIDERED;
 	    return TRUE;
 	}
 
     new.tm = tm;
     new.fee = fee;
     new.wage = wage;
-    new.accepted = FALSE;
+    new.status = TRANSFER_OFFER_NOT_CONSIDERED;
 
     g_array_append_val(trans(idx).offers, new);
 
     return FALSE;
 }
 
-/** Return the index of the transfer containing the player going with
-    the team and the id. */
-gint
-transfer_get_index(const Team *tm, gint id)
-{
-    gint i;
-
-    for(i=0;i<transfer_list->len;i++)
-	if(trans(i).tm == tm && trans(i).id == id)
-	    return i;
-
-    main_exit_program(EXIT_INT_NOT_FOUND,
-		      "transfer_get_index: didn't find transfer.");
-    return -1;
-}
-
 /** Remove any offers from the team for the given transfer player. */
 gboolean
 transfer_remove_offer(gint idx, const Team *tm)
@@ -404,23 +424,28 @@ transfer_remove_offer(gint idx, const Team *tm)
 void
 transfer_add_remove_user_player(Player *pl)
 {
-    gchar buf[SMALL];
-
     if(!query_transfer_player_is_on_list(pl))
     {
-	transfer_add_player(pl,
-			    (gint)rint(((gfloat)const_int("int_transfer_time_lower") +
-					(gfloat)const_int("int_transfer_time_upper")) / 2));
-	sprintf(buf, _("%s has been added to the transfer list for %d weeks."),
-		pl->name->str, 
-		(gint)rint(((gfloat)const_int("int_transfer_time_lower") +
-			    (gfloat)const_int("int_transfer_time_upper")) / 2));
-	game_gui_print_message(buf);
+	if(week < transfer_get_deadline())
+	{
+	    transfer_add_player(pl,
+				(gint)rint(((gfloat)const_int("int_transfer_time_lower") +
+					    (gfloat)const_int("int_transfer_time_upper")) / 2));
+	    game_gui_print_message(_("%s has been added to the transfer list for %d weeks."),
+				   pl->name, 
+				   (gint)rint(((gfloat)const_int("int_transfer_time_lower") +
+					       (gfloat)const_int("int_transfer_time_upper")) / 2));
+	}
+	else
+	{
+	    game_gui_print_message(_("The transfer deadline is over."));
+	    return;
+	}
     }
     else
 	transfer_remove_player_ptr(pl);
 
-    treeview_show_transfer_list(GTK_TREE_VIEW(lookup_widget(window.main, "treeview_right")));
+    on_button_transfers_clicked(NULL, NULL);
 }
 
 /** Find out whether the current user has to do
@@ -434,12 +459,27 @@ transfer_offers_pending(void)
     for(i=0;i<transfer_list->len;i++)
 	if((trans(i).tm == current_user.tm &&
 	    trans(i).offers->len > 0 &&
-	    transoff(i, 0).accepted) ||
+	    transoff(i, 0).status == TRANSFER_OFFER_ACCEPTED) ||
 	   (team_is_user(trans(i).tm) == -1 &&
 	    trans(i).offers->len > 0 &&
 	    transoff(i, 0).tm == current_user.tm &&
-	    transoff(i, 0).accepted))
+	    transoff(i, 0).status == TRANSFER_OFFER_ACCEPTED))
 	    return TRUE;
 
     return FALSE;
 }
+
+/** Find out whether the team has already made an offer for the player
+    and set the fee and wage accordingly. */
+void
+transfer_get_previous_offer(const Transfer *tr, const Team *tm, gint *fee, gint *wage)
+{
+    gint i;
+
+    for(i=0;i<tr->offers->len;i++)
+	if(g_array_index(tr->offers, TransferOffer, i).tm == tm)
+	{
+	    *fee = g_array_index(tr->offers, TransferOffer, i).fee;
+	    *wage = g_array_index(tr->offers, TransferOffer, i).wage;
+	}
+}
