@@ -53,9 +53,6 @@ transfer_remove_player_ptr(const Player *pl);
 Team*
 transfer_team_get_new(void);
 
-gint
-transfer_get_index(const Team *tm, gint id);
-
 void
 transfer_add_remove_user_player(Player *pl);
 
@@ -71,4 +68,7 @@ query_transfer_current_team_offer(const Player *pl);
 gboolean
 transfer_offers_pending(void);
 
+void
+transfer_get_previous_offer(const Transfer *tr, const Team *tm, gint *fee, gint *wage);
+
 #endif
