@@ -48,11 +48,11 @@ treeview_create_team_selection_list(gboolean show_cup_teams, gboolean show_user_
 	    if(team_is_user(&g_array_index(lig(i).teams, Team, j)) == -1)
 	    {
 		gtk_list_store_append(ls, &iter);
-		treeview_helper_insert_icon(ls, &iter, 1, lig(i).symbol->str);
+		treeview_helper_insert_icon(ls, &iter, 1, g_array_index(lig(i).teams, Team, j).symbol);
 		gtk_list_store_set(ls, &iter,
 				   0, cnt++,
 				   2, (gpointer)&g_array_index(lig(i).teams, Team, j),
-				   3, lig(i).name->str,
+				   3, lig(i).name,
 				   4, (gpointer)&g_array_index(lig(i).teams, Team, j),
 				   -1);
 	    }
@@ -68,11 +68,11 @@ treeview_create_team_selection_list(gboolean show_cup_teams, gboolean show_user_
 	    {
 		gtk_list_store_append(ls, &iter);
 		treeview_helper_insert_icon(ls, &iter, 1, 
-					    ((Team*)g_ptr_array_index(cp(i).teams, j))->symbol->str);
+					    ((Team*)g_ptr_array_index(cp(i).teams, j))->symbol);
 		gtk_list_store_set(ls, &iter,
 				   0, cnt++,
 				   2, g_ptr_array_index(cp(i).teams, j),
-				   3, cp(i).name->str,
+				   3, cp(i).name,
 				   4, g_ptr_array_index(cp(i).teams, j),
 				   -1);
 	    }
@@ -134,8 +134,8 @@ treeview_set_up_team_selection_treeview(GtkTreeView *treeview)
     gtk_tree_view_column_add_attribute(col, renderer,
 				       "text", 3);
 
-    /* Average skill */
     col = gtk_tree_view_column_new();
+    /* Average skill */
     gtk_tree_view_column_set_title(col, _("Av.Sk."));
     gtk_tree_view_column_set_sort_column_id(col, 4);
     gtk_tree_view_append_column(treeview, col);
@@ -162,6 +162,7 @@ treeview_show_team_list(GtkTreeView *treeview, gboolean show_cup_teams,
     GtkTreeModel *team_list = 
 	treeview_create_team_selection_list(show_cup_teams, show_user_teams);
     GtkTreeSelection *selection;
+    GtkTreePath *path = gtk_tree_path_new_from_string("0");
      
     treeview_helper_clear(treeview);
 
@@ -170,38 +171,41 @@ treeview_show_team_list(GtkTreeView *treeview, gboolean show_cup_teams,
     gtk_tree_view_set_model(treeview, team_list);
     
     selection = gtk_tree_view_get_selection(treeview);
-    gtk_tree_selection_select_path(selection,
-				   gtk_tree_path_new_from_string("0"));
+    gtk_tree_selection_select_path(selection, path);
     g_object_unref(team_list);
+    gtk_tree_path_free(path);
 }
 
 /** Create the list store for a player list. 
     @param players The array containing the players.
     @param attributes An array containing the attributes we show.
     @param max The size of the attribute array.
-    @param separator Whether we draw a blank line after the 11th player. */
+    @param separator Whether we draw a blank line after the 11th player.
+    @param status Whether player status is shown (takes two columns). */
 GtkTreeModel*
 treeview_create_player_list(GPtrArray *players, gint *attributes, gint max, 
-			    gboolean show_separator, gboolean sortable)
+			    gboolean show_separator, gboolean sortable, gboolean status)
 {
     gint i, j;
     GtkListStore  *ls;
     GtkTreeIter iter;
-    GType types[max + 1];
+    GType types[max + 1 + status];
 
     types[0] = G_TYPE_INT;
-    for(i=0;i<max;i++)
+    for(i=0;i<max + status;i++)
 	types[i + 1] = G_TYPE_POINTER;
 
-    ls = gtk_list_store_newv(max + 1, types);
+    ls = gtk_list_store_newv(max + 1 + status, types);
 
     for(i=0;i<players->len;i++)
     {
 	gtk_list_store_append(ls, &iter);
 	if(show_separator && i == 11)
 	{
-	    gtk_list_store_set(ls, &iter, 0, const_int("int_treeview_helper_int_empty"), -1);
-	    for(j=0;j<max;j++)
+	    gtk_list_store_set(ls, &iter, 0, 
+			       const_int("int_treeview_helper_int_empty"), -1);
+
+	    for(j=0;j<max + status;j++)
 		gtk_list_store_set(ls, &iter, j + 1, NULL, -1);
 	    
 	    gtk_list_store_append(ls, &iter);
@@ -210,7 +214,7 @@ treeview_create_player_list(GPtrArray *players, gint *attributes, gint max,
 	else
 	    gtk_list_store_set(ls, &iter, 0, i + 1, -1);
 
-	for(j=0;j<max;j++)
+	for(j=0;j<max + status;j++)
 	    gtk_list_store_set(ls, &iter, j + 1, g_ptr_array_index(players, i), -1);
     }
 
@@ -226,8 +230,10 @@ treeview_create_player_list(GPtrArray *players, gint *attributes, gint max,
 	       attributes[i] == PLAYER_LIST_ATTRIBUTE_ETAL ||
 	       attributes[i] == PLAYER_LIST_ATTRIBUTE_VALUE ||
 	       attributes[i] == PLAYER_LIST_ATTRIBUTE_WAGE)
-		gtk_tree_sortable_set_sort_func(GTK_TREE_SORTABLE(ls), i + 1,
-						treeview_helper_player_compare, GINT_TO_POINTER(attributes[i]), NULL);
+		gtk_tree_sortable_set_sort_func(
+		    GTK_TREE_SORTABLE(ls), 
+		    i + 1 + (status && attributes[i] > PLAYER_LIST_ATTRIBUTE_STATUS),
+		    treeview_helper_player_compare, GINT_TO_POINTER(attributes[i]), NULL);
     }
     
     return (GtkTreeModel*)ls;
@@ -238,22 +244,33 @@ void
 treeview_set_up_player_list(GtkTreeView *treeview, gint *attributes, gint max,
 			    gboolean show_separator, gboolean sortable)
 {
-    gint i;
+    gint i, cnt = 1;
     GtkTreeViewColumn   *col;
     GtkCellRenderer     *renderer;
     gchar *titles[PLAYER_LIST_ATTRIBUTE_END] =
 	{_("Name"),
+	 /* Current position of a player. */
 	 _("CPos"),
+	 /* Position of a player. */
 	 _("Pos"),
+	 /* Current skill of a player. */
 	 _("CSk"),
+	 /* Skill of a player. */
 	 _("Sk"),
+	 /* Fitness of a player. */
 	 _("Fit"),
+	 /* Games of a player. */
 	 _("Ga"),
+	 /* Shots of a player. */
 	 _("Sh"),
+	 /* Goals of a player. */
 	 _("Go"),
+	 /* Status of a player. */
 	 _("Status"),
+	 /* Yellow cards of a player. */
 	 _("YC"),
 	 _("Age"),
+	 /* Estimated talent of a player. */
 	 _("Etal"),
 	 _("Value"),
 	 _("Wage"),
@@ -306,7 +323,26 @@ treeview_set_up_player_list(GtkTreeView *treeview, gint *attributes, gint max,
 	    attributes[i] == PLAYER_LIST_ATTRIBUTE_ETAL ||
 	    attributes[i] == PLAYER_LIST_ATTRIBUTE_VALUE ||
 	    attributes[i] == PLAYER_LIST_ATTRIBUTE_WAGE))
-	    gtk_tree_view_column_set_sort_column_id(col, i + 1);	    
+	    gtk_tree_view_column_set_sort_column_id(col, cnt);
+
+	cnt++;
+
+	if(attributes[i] == PLAYER_LIST_ATTRIBUTE_STATUS)
+	{
+	    col = gtk_tree_view_column_new();
+	    gtk_tree_view_append_column(treeview, col);
+	    renderer = gtk_cell_renderer_pixbuf_new();
+	    gtk_tree_view_column_pack_start(col, renderer, TRUE);
+	    gtk_tree_view_column_set_cell_data_func(
+		col, renderer, treeview_helper_player_status_to_cell,
+		NULL, NULL);
+
+	    gtk_tree_view_column_set_alignment(col, 0.5);
+	    g_object_set(renderer, "xalign", 0.5,
+			 NULL);
+
+	    cnt++;
+	}
     }
 }
 
@@ -318,24 +354,31 @@ treeview_set_up_player_list(GtkTreeView *treeview, gint *attributes, gint max,
     @param attrib The #PlayerListAttribute that determines which attributes to show.
     @param show_separator Whether we draw a blank line after the 11th player. */
 void
-treeview_show_player_list(GtkTreeView *treeview, GPtrArray *players, PlayerListAttribute attribute,
+treeview_show_player_list(GtkTreeView *treeview, GPtrArray *players, 
+			  PlayerListAttribute attribute,
 			  gboolean show_separator)
 {
     gint i, cnt = 0;
     gint columns = math_sum_int_array(attribute.on_off, PLAYER_LIST_ATTRIBUTE_END);
     gint attributes[columns];
     GtkTreeModel *model = NULL;
-    gboolean sortable = (treeview != GTK_TREE_VIEW(lookup_widget(window.main, "player_list1")));
+    gboolean sortable = 
+	(treeview != GTK_TREE_VIEW(lookup_widget(window.main, "player_list1")));
     
     treeview_helper_clear(treeview);
 
     for(i=0;i<PLAYER_LIST_ATTRIBUTE_END;i++)
+    {
 	if(attribute.on_off[i])
 	    attributes[cnt++] = i;
+    }
 
     treeview_set_up_player_list(treeview, attributes, columns, show_separator, sortable);
 
-    model = treeview_create_player_list(players, attributes, columns, show_separator, sortable);
+    model = treeview_create_player_list(players, attributes, 
+					columns, show_separator, 
+					sortable, 
+					attribute.on_off[PLAYER_LIST_ATTRIBUTE_STATUS]);
 
     gtk_tree_view_set_model(treeview, model);
     g_object_unref(model);
@@ -356,7 +399,7 @@ treeview_show_user_player_list(void)
 
     for(i=0;i<2;i++)
     {
-	players = team_get_player_pointers(current_user.tm);
+	players = player_get_pointers_from_array(current_user.tm->players);
 	user_set_player_list_attributes(&current_user, &attribute, i + 1);
 	treeview_show_player_list(GTK_TREE_VIEW(treeview[i]), players, attribute, TRUE);
     }
@@ -367,7 +410,7 @@ treeview_show_user_player_list(void)
 void
 treeview_show_player_list_team(GtkTreeView *treeview, const Team *tm, gint scout)
 {
-    GPtrArray *players = team_get_player_pointers(tm);
+    GPtrArray *players = player_get_pointers_from_array(tm->players);
 
     treeview_show_player_list(treeview, players, 
 			      treeview_helper_get_attributes_from_scout(scout), TRUE);
@@ -393,12 +436,12 @@ treeview_live_game_show_commentary(const LiveGameUnit *unit)
 	sprintf(buf2, "<span background='%s' foreground='%s'>%s</span>",
 		const_app("string_treeview_live_game_commentary_away_bg"),
 		const_app("string_treeview_live_game_commentary_away_fg"),
-		unit->event.commentary->str);
+		unit->event.commentary);
     else
 	sprintf(buf2, "<span background='%s' foreground='%s'>%s</span>",
 		const_app("string_treeview_helper_color_default_background"),
 		const_app("string_treeview_helper_color_default_foreground"),
-		unit->event.commentary->str);
+		unit->event.commentary);
     
     gtk_list_store_prepend(ls, &iter);
     treeview_helper_insert_icon(ls, &iter, 1, treeview_helper_live_game_icon(unit->event.type));
@@ -428,7 +471,7 @@ treeview_live_game_create_init_commentary(const LiveGameUnit *unit)
 
     gtk_list_store_append(ls, &iter);
     treeview_helper_insert_icon(ls, &iter, 1, treeview_helper_live_game_icon(unit->event.type));
-    gtk_list_store_set(ls, &iter, 0, buf, 2, unit->event.commentary->str, -1);
+    gtk_list_store_set(ls, &iter, 0, buf, 2, unit->event.commentary, -1);
 
     return GTK_TREE_MODEL(ls);
 }
@@ -495,15 +538,19 @@ treeview_live_game_create_result(const LiveGameUnit *unit)
 {
     GtkListStore  *ls;
     GtkTreeIter iter;
+    gchar buf[SMALL];
 
     ls = gtk_list_store_new(3,
-			    G_TYPE_POINTER,
-			    G_TYPE_POINTER,
-			    G_TYPE_POINTER);
+			    G_TYPE_STRING,
+			    G_TYPE_STRING,
+			    G_TYPE_STRING);
 
+    live_game_unit_result_to_buf(unit, buf, FALSE);
     gtk_list_store_append(ls, &iter);
-    gtk_list_store_set(ls, &iter, 0, (gpointer)usr(stat2).live_game.fix,
-		       1, (gpointer)unit, 2, (gpointer)usr(stat2).live_game.fix, -1);
+    gtk_list_store_set(ls, &iter, 
+		       0, ((LiveGame*)statp)->team_names[0],
+		       1, buf, 
+		       2, ((LiveGame*)statp)->team_names[1], -1);
 
     return GTK_TREE_MODEL(ls);
 }
@@ -571,22 +618,22 @@ treeview_create_users(void)
     {
 	gtk_list_store_append(ls, &iter);
 	gtk_list_store_set(ls, &iter, 0, i + 1,
-			   1, usr(i).name->str,
-			   2, usr(i).tm->name->str,
+			   1, usr(i).name,
+			   2, usr(i).tm->name,
 			   -1);
 
 	if(stat0 == STATUS_TEAM_SELECTION)
 	{
 	    if(usr(i).scout == -1)
 		gtk_list_store_set(ls, &iter, 3,
-				   league_from_clid(usr(i).tm->clid)->name->str, -1);
+				   league_cup_get_name_string(usr(i).tm->clid), -1);
 	    else
 		gtk_list_store_set(ls, &iter, 3,
-				   lig(usr(i).scout).name->str, -1);
+				   lig(usr(i).scout).name, -1);
 	}
 	else
 	    gtk_list_store_set(ls, &iter, 3,
-			       league_from_clid(usr(i).tm->clid)->name->str, -1);
+			       league_cup_get_name_string(usr(i).tm->clid), -1);
     }
 
     return GTK_TREE_MODEL(ls);
@@ -650,6 +697,7 @@ treeview_create_game_stats(LiveGame *live_game)
     gchar buf[2][SMALL];
     gchar buf3[SMALL];
     gchar *categories[LIVE_GAME_STAT_VALUE_END] =
+	/* 'Normal' goals, ie. no penalties or free kicks. */
 	{_("Goals (regular)"),
 	 _("Shots"),   
 	 _("Shot %"),   
@@ -665,11 +713,11 @@ treeview_create_game_stats(LiveGame *live_game)
 			    G_TYPE_STRING,
 			    G_TYPE_STRING);
 
-    fixture_result_to_buf(live_game->fix, buf[0]);
+    fixture_result_to_buf(live_game->fix, buf[0], FALSE);
     gtk_list_store_append(ls, &iter);
-    gtk_list_store_set(ls, &iter, 0, live_game->fix->teams[0]->name->str,
+    gtk_list_store_set(ls, &iter, 0, live_game->fix->teams[0]->name,
 		       1, buf[0],
-		       2, live_game->fix->teams[1]->name->str,
+		       2, live_game->fix->teams[1]->name,
 		       -1);
 
     for(k=0;k<LIVE_GAME_STAT_ARRAY_END;k++)
@@ -703,7 +751,7 @@ treeview_create_game_stats(LiveGame *live_game)
 		if(i < stats->players[j][k]->len)
 		{
 		    sprintf(buf3, "%s%s%s", buf[0],
-			    ((GString*)g_ptr_array_index(stats->players[j][k], i))->str,
+			    (gchar*)g_ptr_array_index(stats->players[j][k], i),
 			    buf[1]);
 		    gtk_list_store_set(ls, &iter, j * 2, buf3, -1);
 		}
@@ -711,7 +759,7 @@ treeview_create_game_stats(LiveGame *live_game)
 	}
     }
 
-    misc_print_grouped_int(live_game->fix->attendance, buf[0], FALSE);
+    misc_print_grouped_int(live_game->fix->attendance, buf[0]);
     sprintf(buf[1], _("Attendance\n%s"), buf[0]);
     gtk_list_store_append(ls, &iter);
     gtk_list_store_set(ls, &iter, 0, buf[1], 1, "", 2, "", -1);
@@ -816,17 +864,17 @@ treeview_create_fixtures_header(const Fixture *fix, GtkListStore *ls, gboolean b
 
     if(fix->clid < ID_CUP_START)
     {
-	name = league_from_clid(fix->clid)->name->str;
+	name = league_cup_get_name_string(fix->clid);
 	strcpy(round_name, "");
-	symbol = league_from_clid(fix->clid)->symbol->str;
+	symbol = league_from_clid(fix->clid)->symbol;
     }
     else
     {
-	name = cup_from_clid(fix->clid)->name->str;
+	name = cup_from_clid(fix->clid)->name;
 	cup_round_name(fix, buf);
 	sprintf(round_name, "\n%s", buf);
 	strcat(buf3, "\n");
-	symbol = cup_from_clid(fix->clid)->symbol->str;
+	symbol = cup_from_clid(fix->clid)->symbol;
     }
     
     sprintf(buf, "<span background='%s' foreground='%s'>%s%s</span>", 
@@ -862,7 +910,7 @@ treeview_create_fixture(const Fixture *fix, GtkListStore *ls)
     if(fix->clid >= ID_CUP_START &&
        query_cup_is_international(fix->clid))
 	for(i=0;i<2;i++)
-	    symbol[i] = fix->teams[i]->symbol->str;
+	    symbol[i] = fix->teams[i]->symbol;
     
     if(fixture_user_team_involved(fix) != -1)
 	treeview_helper_set_user_colours(usr(fixture_user_team_involved(fix)).tm,
@@ -873,7 +921,7 @@ treeview_create_fixture(const Fixture *fix, GtkListStore *ls)
 	colour_bg = const_app("string_treeview_helper_color_default_background");
     }
 
-    fixture_result_to_buf(fix, buf_result);
+    fixture_result_to_buf(fix, buf_result, FALSE);
 
     for(i=0;i<2;i++)
 	if(query_fixture_has_tables(fix))
@@ -881,19 +929,20 @@ treeview_create_fixture(const Fixture *fix, GtkListStore *ls)
 	    if(fix->clid < ID_CUP_START)
 		rank = team_get_league_rank(fix->teams[i]);
 	    else
-		rank = team_get_cup_rank(fix->teams[i], cup_get_last_tables_round(fix->clid));
+		rank = team_get_cup_rank(fix->teams[i], 
+					 cup_get_last_tables_round(fix->clid), TRUE);
 
 	    sprintf(buf[i], "<span background='%s' foreground='%s'>%s [%d]</span>",
-		    colour_bg, colour_fg, fix->teams[i]->name->str, rank);
+		    colour_bg, colour_fg, fix->teams[i]->name, rank);
 	}
 	else if(fix->clid >= ID_CUP_START &&
 		query_cup_is_national(fix->clid))
 	    sprintf(buf[i], "<span background='%s' foreground='%s'>%s (%d)</span>",
-		    colour_bg, colour_fg, fix->teams[i]->name->str,
+		    colour_bg, colour_fg, fix->teams[i]->name,
 		    league_from_clid(fix->teams[i]->clid)->layer);
 	else
 	    sprintf(buf[i], "<span background='%s' foreground='%s'>%s</span>",
-		    colour_bg, colour_fg, fix->teams[i]->name->str);
+		    colour_bg, colour_fg, fix->teams[i]->name);
 
     sprintf(buf[2], "<span background='%s' foreground='%s'>%s</span>",
 	    colour_bg, colour_fg, buf_result);
@@ -1012,17 +1061,18 @@ treeview_table_write_header(GtkListStore *ls, const Table *table, gint number)
 
     if(table->clid < ID_CUP_START)
     {
-	symbol = league_from_clid(table->clid)->symbol->str;
-	strcpy(buf, league_from_clid(table->clid)->name->str);
+	symbol = league_from_clid(table->clid)->symbol;
+	strcpy(buf, league_cup_get_name_string(table->clid));
     }
     else
     {
-	symbol = cup_from_clid(table->clid)->symbol->str;
+	symbol = cup_from_clid(table->clid)->symbol;
 	if(g_array_index(cup_from_clid(table->clid)->rounds, CupRound,
 			 table->round).tables->len > 1)			 
-	    sprintf(buf, _("%s Group %d"), cup_from_clid(table->clid)->name->str, number);
+	    /*  A group of a round robin stage of a cup. */
+	    sprintf(buf, _("%s Group %d"), cup_from_clid(table->clid)->name, number);
 	else
-	    sprintf(buf, "%s", cup_from_clid(table->clid)->name->str);
+	    sprintf(buf, "%s", cup_from_clid(table->clid)->name);
     }
 
     gtk_list_store_append(ls, &iter);
@@ -1054,7 +1104,7 @@ treeview_create_single_table(GtkListStore *ls, const Table *table, gint number)
 	elem = &g_array_index(table->elements, TableElement, i);
 
 	if(table->clid >= ID_CUP_START)
-	    treeview_helper_insert_icon(ls, &iter, 0, elem->team->symbol->str);
+	    treeview_helper_insert_icon(ls, &iter, 0, elem->team->symbol);
 	
 	if(elem->old_rank > i)
 	    treeview_helper_insert_icon(ls, &iter, 2, 
@@ -1066,13 +1116,14 @@ treeview_create_single_table(GtkListStore *ls, const Table *table, gint number)
 	    treeview_helper_insert_icon(ls, &iter, 2, 
 					const_app("string_treeview_table_stay_icon"));
 
-	treeview_helper_get_table_element_colours(table, i, &colour_fg, &colour_bg, FALSE);
+	treeview_helper_get_table_element_colours(table, i, 
+						  &colour_fg, &colour_bg, FALSE);
 	sprintf(buf[0], "<span background='%s' foreground='%s'>%d</span>",
 		colour_bg, colour_fg, i + 1);
 
 	treeview_helper_get_table_element_colours(table, i, &colour_fg, &colour_bg, TRUE);
 	sprintf(buf[1], "<span background='%s' foreground='%s'>%s</span>", 
-		colour_bg, colour_fg, elem->team->name->str);
+		colour_bg, colour_fg, elem->team->name);
 
 	gtk_list_store_set(ls, &iter, 1, buf[0], 3, buf[1], -1);
 
@@ -1133,13 +1184,21 @@ treeview_set_up_table(GtkTreeView *treeview)
     GtkCellRenderer     *renderer;
     gchar *titles[9] =
 	{_("Team"),
+	 /* Games played (a number). */
 	 _("PL"),
+	 /* Games won. */
 	 _("W"),
+	 /* Games drawn. */
 	 _("Dw"),
+	 /* Games lost. */
 	 _("L"),
+	 /* Goals for. */
 	 _("GF"),
+	 /* Goals against. */
 	 _("GA"),
+	 /* Goal difference. */
 	 _("GD"),
+	 /* Points. */
 	 _("PTS")};
 
     gtk_tree_selection_set_mode(gtk_tree_view_get_selection(treeview),
@@ -1206,8 +1265,15 @@ treeview_create_stadium_summary(GtkListStore *ls)
     gchar buf[SMALL];
     GtkTreeIter iter;
 
+    if(current_user.tm->stadium.name != NULL)
+    {
+	gtk_list_store_append(ls, &iter);
+	gtk_list_store_set(ls, &iter, 0, _("Stadium"), 
+			   1, current_user.tm->stadium.name, 2, "", -1);
+    }
+
     gtk_list_store_append(ls, &iter);
-    misc_print_grouped_int(current_user.tm->stadium.capacity, buf, FALSE);
+    misc_print_grouped_int(current_user.tm->stadium.capacity, buf);
     gtk_list_store_set(ls, &iter, 0, _("Stadium capacity"), 1, buf, 2, "", -1);
 
     gtk_list_store_append(ls, &iter);
@@ -1239,15 +1305,18 @@ treeview_create_finances(const User* user)
 	*out = user->money_out[0];
     gchar *in_titles[MON_IN_TRANSFERS] =
 	{_("Prize money"),
-	 _("Ticket income")};
+	 _("Ticket income"),
+	 _("Sponsorship")};
     gchar *out_titles[MON_OUT_TRANSFERS] =
 	{_("Wages"),
 	 _("Physio"),
 	 _("Scout"),
+	 _("Youth coach"),
+	 _("Youth academy"),
 	 _("Journey costs"),
-	 _("Stadium improvements"),
-	 _("Stadium bills"),
+	 /* Money paid to players a user fired. */
 	 _("Compensations")};
+
     GtkTreeIter iter;
     GtkListStore *ls = 
 	gtk_list_store_new(3,
@@ -1256,32 +1325,22 @@ treeview_create_finances(const User* user)
 			   G_TYPE_STRING);
 
     gtk_list_store_append(ls, &iter);
+    /* Finances balance. */
     gtk_list_store_set(ls, &iter, 0, _("Bi-weekly balance"), 1, "", 2, "", -1);
 
     for(i=0;i<MON_IN_TRANSFERS;i++)
 	if(in[i] != 0)
 	{
-	    misc_print_grouped_int(in[i], buf, FALSE);
+	    misc_print_grouped_int(in[i], buf);
 	    gtk_list_store_append(ls, &iter);
 	    gtk_list_store_set(ls, &iter, 0, in_titles[i], 1, buf, 2, "", -1);
 	    balance += in[i];
 	}
 
-    if(in[MON_IN_TRANSFERS] != 0 || out[MON_OUT_TRANSFERS] != 0)
-    {
-	misc_print_grouped_int(in[MON_IN_TRANSFERS], buf, FALSE);
-	misc_print_grouped_int(out[MON_OUT_TRANSFERS], buf3, FALSE);
-	sprintf(buf2, "<span foreground='%s'>%s</span>",
-		const_app("string_treeview_finances_expenses_fg"), buf3);
-	gtk_list_store_append(ls, &iter);
-	gtk_list_store_set(ls, &iter, 0, _("Transfers"), 1, buf, 2, buf2, -1);
-	balance += (in[MON_IN_TRANSFERS] + out[MON_OUT_TRANSFERS]);
-    }
-
     for(i=0;i<MON_OUT_TRANSFERS;i++)
 	if(out[i] != 0)
 	{
-	    misc_print_grouped_int(out[i], buf3, FALSE);
+	    misc_print_grouped_int(out[i], buf3);
 	    sprintf(buf, "<span foreground='%s'>%s</span>",
 		    const_app("string_treeview_finances_expenses_fg"), buf3);
 	    gtk_list_store_append(ls, &iter);
@@ -1289,8 +1348,9 @@ treeview_create_finances(const User* user)
 	    balance += out[i];
 	}    
 
-    misc_print_grouped_int(balance, buf, FALSE);
+    misc_print_grouped_int(balance, buf);
     gtk_list_store_append(ls, &iter);
+    /* Finances balance. */
     gtk_list_store_set(ls, &iter, 0, _("Balance"), 1, "", 2, "", -1);
     if(balance >= 0)
 	strcpy(buf2, buf);
@@ -1302,7 +1362,30 @@ treeview_create_finances(const User* user)
     gtk_list_store_append(ls, &iter);
     gtk_list_store_set(ls, &iter, 0, "", 1, "", 2, "", -1);
 
-    misc_print_grouped_int(user->money, buf, FALSE);
+    if(in[MON_IN_TRANSFERS] != 0 || out[MON_OUT_TRANSFERS] != 0)
+    {
+	misc_print_grouped_int(in[MON_IN_TRANSFERS], buf);
+	misc_print_grouped_int(out[MON_OUT_TRANSFERS], buf3);
+	sprintf(buf2, "<span foreground='%s'>%s</span>",
+		const_app("string_treeview_finances_expenses_fg"), buf3);
+	gtk_list_store_append(ls, &iter);
+	gtk_list_store_set(ls, &iter, 0, _("Transfers"), 1, buf, 2, buf2, -1);
+    }
+
+    if(out[MON_OUT_STADIUM_IMPROVEMENT] + out[MON_OUT_STADIUM_BILLS] != 0)
+    {
+	misc_print_grouped_int(out[MON_OUT_STADIUM_IMPROVEMENT] + out[MON_OUT_STADIUM_BILLS], 
+			       buf);
+	sprintf(buf2, "<span foreground='%s'>%s</span>",
+		const_app("string_treeview_finances_expenses_fg"), buf);
+	gtk_list_store_append(ls, &iter);
+	gtk_list_store_set(ls, &iter, 0, _("Stadium expenses"), 1, "", 2, buf2, -1);
+    }
+
+    gtk_list_store_append(ls, &iter);
+    gtk_list_store_set(ls, &iter, 0, "", 1, "", 2, "", -1);
+
+    misc_print_grouped_int(user->money, buf);
     gtk_list_store_append(ls, &iter);
     if(user->money >= 0)
 	strcpy(buf2, buf);
@@ -1311,13 +1394,13 @@ treeview_create_finances(const User* user)
 		const_app("string_treeview_finances_expenses_fg"), buf);
     gtk_list_store_set(ls, &iter, 0, _("Money"), 1, buf2, 2, "", -1);
 
-    misc_print_grouped_int(finance_team_drawing_credit_loan(user->tm, FALSE), buf, FALSE);
+    misc_print_grouped_int(finance_team_drawing_credit_loan(user->tm, FALSE), buf);
     gtk_list_store_append(ls, &iter);
     gtk_list_store_set(ls, &iter, 0, _("Drawing credit"), 1, buf, 2, "", -1);
     
     if(user->debt != 0)
     {
-	misc_print_grouped_int(user->debt, buf, FALSE);
+	misc_print_grouped_int(user->debt, buf);
 	sprintf(buf2, "<span foreground='%s'>%s</span>",
 		const_app("string_treeview_finances_expenses_fg"), buf);
 	sprintf(buf, _("Debt (repay in %d weeks)"), user->counters[COUNT_USER_LOAN]);
@@ -1328,6 +1411,26 @@ treeview_create_finances(const User* user)
     gtk_list_store_append(ls, &iter);
     gtk_list_store_set(ls, &iter, 0, "", 1, "", 2, "", -1);
 
+    gtk_list_store_append(ls, &iter);
+    gtk_list_store_set(ls, &iter, 0, _("Sponsor"), 1, user->sponsor.name->str,
+		       2, "", -1);
+
+    misc_print_grouped_int(user->sponsor.benefit, buf);
+    /* Contract time and money a sponsor pays. */
+    sprintf(buf2, _("%.1f months / %s"), ((gfloat)user->sponsor.contract) / 4, buf);
+    gtk_list_store_append(ls, &iter);
+    gtk_list_store_set(ls, &iter, 0, _("Contract / Money"), 1, buf2, 2, "", -1);
+
+    gtk_list_store_append(ls, &iter);
+    gtk_list_store_set(ls, &iter, 0, "", 1, "", 2, "", -1);
+
+    sprintf(buf, "%d%%", user->youth_academy.percentage);
+    gtk_list_store_append(ls, &iter);
+    gtk_list_store_set(ls, &iter, 0, _("Youth acad. invest."), 1, buf, 2, "", -1);
+
+    gtk_list_store_append(ls, &iter);
+    gtk_list_store_set(ls, &iter, 0, "", 1, "", 2, "", -1);
+
     treeview_create_stadium_summary(ls);
 
     return GTK_TREE_MODEL(ls);
@@ -1398,10 +1501,11 @@ treeview_create_next_opponent_values(GtkListStore *ls, const Fixture *fix)
 {
     gint i, j;
     gchar buf[SMALL], buf2[SMALL];
-    gfloat max_values[3],
+    gfloat max_values[4],
 	team_values[2][GAME_TEAM_VALUE_END];
-    gchar *titles[3] =
-	{_("Defend"),
+    gchar *titles[4] =
+	{_("Goalie"),
+	 _("Defend"),
 	 _("Midfield"),
 	 _("Attack")};
     GtkTreeIter iter;
@@ -1409,7 +1513,7 @@ treeview_create_next_opponent_values(GtkListStore *ls, const Fixture *fix)
     game_get_values(fix, team_values, 0);
     game_get_max_values(max_values);
 
-    for(i=0;i<3;i++)
+    for(i=0;i<4;i++)
     {
 	strcpy(buf2, "");
 	for(j=0;j<(gint)rint((gfloat)const_int("int_treeview_max_pipes") *
@@ -1478,7 +1582,7 @@ treeview_create_next_opponent(void)
     gtk_list_store_set(ls, &iter, 0, "", 1, "", -1);
 
     gtk_list_store_append(ls, &iter);
-    gtk_list_store_set(ls, &iter, 0, _("Team"), 1, opp->name->str, -1);
+    gtk_list_store_set(ls, &iter, 0, _("Team"), 1, opp->name, -1);
     
     if(opp->clid < ID_CUP_START)
     {
@@ -1520,8 +1624,9 @@ treeview_create_next_opponent(void)
     gtk_list_store_append(ls, &iter);
     gtk_list_store_set(ls, &iter, 0, _("Goals"), 1, buf2, -1);
 
-    team_write_own_results(opp, buf, FALSE);
-    gtk_list_store_append(ls, &iter);
+    team_write_own_results(opp, buf, FALSE, TRUE);
+    gtk_list_store_append(ls, &iter);    
+    /* The user's results against a specific team. */
     gtk_list_store_set(ls, &iter, 0, _("Your results"), 1, buf, -1);
     
     return GTK_TREE_MODEL(ls);
@@ -1585,12 +1690,12 @@ treeview_create_league_results(void)
 	    if(team_is_user(g_array_index(table_elements, TableElement, i).team) != -1)
 		sprintf(name, "<span background='%s'>%s</span>",
 			const_app("string_treeview_user_bg"),
-			g_array_index(table_elements, TableElement, i).team->name->str);
+			g_array_index(table_elements, TableElement, i).team->name);
 	    else
-		strcpy(name, g_array_index(table_elements, TableElement, i).team->name->str);
+		strcpy(name, g_array_index(table_elements, TableElement, i).team->name);
 
 	    team_write_own_results(g_array_index(table_elements, TableElement, i).team,
-				   results, TRUE);
+				   results, TRUE, FALSE);
 
 	    gtk_list_store_append(ls, &iter);
 	    gtk_list_store_set(ls, &iter, 0, name,
@@ -1749,8 +1854,12 @@ treeview_create_player_info(const Player *pl)
 	 _("Wage"),
 	 _("Contract"),
 	 _("Games/Goals\n"),
+	 /* 'Limit' is the number of yellow cards until a player gets
+	  banned automatically for a match. */
 	 _("Yellow cards (limit)\n"),
 	 _("Banned\n"),
+	 /* Hot streak or cold streak of a player. */
+	 _("Streak"),
 	 _("Career values"),
 	 _("New contract\noffers")};
 
@@ -1879,7 +1988,7 @@ treeview_create_user_history(void)
 	gtk_list_store_set(ls, &iter,
 			   0, g_array_index(current_user.history, UserHistory, i).season,
 			   1, g_array_index(current_user.history, UserHistory, i).week,
-			   2, team_of_id(g_array_index(current_user.history, UserHistory, i).team_id)->name->str,
+			   2, team_of_id(g_array_index(current_user.history, UserHistory, i).team_id)->name,
 			   4, buf, -1);
 
     }
@@ -1893,7 +2002,9 @@ treeview_set_up_user_history(GtkTreeView *treeview)
     gint i;
     GtkTreeViewColumn   *col;
     GtkCellRenderer     *renderer;
-    gchar *titles[3] ={_("Sea"), _("Week"), _("Team")};
+    /* Season. */
+    gchar *titles[3] ={_("Sea"), 
+		       _("Week"), _("Team")};
 
     gtk_tree_selection_set_mode(gtk_tree_view_get_selection(treeview),
 				GTK_SELECTION_NONE);
@@ -1958,15 +2069,17 @@ treeview_create_league_stats(GtkListStore *ls, const LeagueStat *league_stat)
     gchar *team_icons[2] = {const_app("string_treeview_league_stats_off_teams_icon"),
 			    const_app("string_treeview_league_stats_def_teams_icon")};
     gchar *player_titles[2][2] = {{_("Best goal scorers"), _("Shot %")},
-				  {_("Best goalkeepers"), _("Save %")}};
+				  {_("Best goalkeepers"),
+				   /* % of saves for goalies (#goals / #shots) */
+				   _("Save %")}};
     gchar *player_icons[2] = {const_app("string_treeview_league_stats_scorers_icon"),
 			      const_app("string_treeview_league_stats_goalies_icon")};
     gchar *colour_fg = NULL, *colour_bg = NULL;
     
     gtk_list_store_append(ls, &iter);
-    treeview_helper_insert_icon(ls, &iter, 0, league_from_clid(league_stat->clid)->symbol->str);
+    treeview_helper_insert_icon(ls, &iter, 0, league_from_clid(league_stat->clid)->symbol);
     gtk_list_store_set(ls, &iter, 1, const_int("int_treeview_helper_int_empty"),
-		       2, league_from_clid(league_stat->clid)->name->str, 3, "", 4, "", 5, "", -1);
+		       2, league_cup_get_name_string(league_stat->clid), 3, "", 4, "", 5, "", -1);
 
     for(i=0;i<2;i++)
     {
@@ -1990,7 +2103,7 @@ treeview_create_league_stats(GtkListStore *ls, const LeagueStat *league_stat)
 					     &colour_bg, &colour_fg);
 
 	    sprintf(buf2, "<span background='%s' foreground='%s'>%s</span>",
-		    colour_bg, colour_fg, team_of_id(g_array_index(teams[i], Stat, j).team_id)->name->str);
+		    colour_bg, colour_fg, team_of_id(g_array_index(teams[i], Stat, j).team_id)->name);
 	    
 	    gtk_list_store_append(ls, &iter);
 	    gtk_list_store_set(ls, &iter, 0, NULL, 1, j + 1,
@@ -2009,7 +2122,9 @@ treeview_create_league_stats(GtkListStore *ls, const LeagueStat *league_stat)
 	gtk_list_store_append(ls, &iter);
 	treeview_helper_insert_icon(ls, &iter, 0, player_icons[i]);
 	gtk_list_store_set(ls, &iter, 1, const_int("int_treeview_helper_int_empty"),
+			   /* Goals. */
 			   2, player_titles[i][0], 3, _("Go"),
+			   /* Goals per game. */
 			   4, _("Go/Ga"), 5, player_titles[i][1], -1);
 
 	for(j=0;j<players[i]->len;j++)
@@ -2030,8 +2145,8 @@ treeview_create_league_stats(GtkListStore *ls, const LeagueStat *league_stat)
 					     &colour_bg, &colour_fg);
 	    sprintf(buf3, "<span background='%s' foreground='%s'>%s (%s)</span>",
 		    colour_bg, colour_fg,
-		    g_array_index(players[i], Stat, j).value_string->str,
-		    team_of_id(g_array_index(players[i], Stat, j).team_id)->name->str);
+		    g_array_index(players[i], Stat, j).value_string,
+		    team_of_id(g_array_index(players[i], Stat, j).team_id)->name);
 	    sprintf(buf4, "%d", g_array_index(players[i], Stat, j).value1);
 
 	    gtk_list_store_append(ls, &iter);
@@ -2121,8 +2236,8 @@ treeview_create_season_history_champions(GtkListStore *ls, const GArray* league_
 	{
 	    gtk_list_store_append(ls, &iter);
 	    gtk_list_store_set(ls, &iter, 0, NULL, 1, const_int("int_treeview_helper_int_empty"),
-			       2, g_array_index(champs[i], ChampStat, j).cl_name->str,
-			       3, g_array_index(champs[i], ChampStat, j).team_name->str,
+			       2, g_array_index(champs[i], ChampStat, j).cl_name,
+			       3, g_array_index(champs[i], ChampStat, j).team_name,
 			       4, "", 5, "", -1);
 	}
 
@@ -2180,14 +2295,14 @@ treeview_create_country_list(const GPtrArray *country_list)
     
     for(i=0;i<country_list->len;i++)
     {
-	sscanf(((GString*)g_ptr_array_index(country_list, i))->str, "country_%[^.]%[.xml]",
+	sscanf((gchar*)g_ptr_array_index(country_list, i), "country_%[^.]%[.xml]",
 	       buf2, trash);
 	sprintf(buf, "flag_%s.png", buf2);
 
 	gtk_list_store_append(ls, &iter);
 	treeview_helper_insert_icon(ls, &iter, 0, buf);
 	gtk_list_store_set(ls, &iter, 1, 
-			   ((GString*)g_ptr_array_index(country_list, i))->str, -1);
+			   (gchar*)g_ptr_array_index(country_list, i), -1);
     }
 
     return GTK_TREE_MODEL(ls);
@@ -2221,18 +2336,18 @@ treeview_show_contributors(const OptionList *help_list)
     {
 	gtk_list_store_append(ls, &iter);
 	
-	if(g_str_has_prefix(g_array_index(help_list->list, Option, i).name->str,
+	if(g_str_has_prefix(g_array_index(help_list->list, Option, i).name,
 			    "string_contrib_title"))
 	{
 	    sprintf(buf, "\n<span %s>%s</span>", 
 		    const_app("string_help_window_title_attribute"),
-		    g_array_index(help_list->list, Option, i).string_value->str);
+		    g_array_index(help_list->list, Option, i).string_value);
 	    gtk_list_store_set(ls, &iter, 0, buf, -1);
 	}
-	else if(g_str_has_prefix(g_array_index(help_list->list, Option, i).name->str,
+	else if(g_str_has_prefix(g_array_index(help_list->list, Option, i).name,
 				 "string_contrib_"))
 	{
-	    strcpy(buf, g_array_index(help_list->list, Option, i).string_value->str);
+	    strcpy(buf, g_array_index(help_list->list, Option, i).string_value);
 	    gtk_list_store_set(ls, &iter, 0, buf, -1);
 	}
 
@@ -2255,7 +2370,7 @@ treeview_create_league_list(void)
     for(i=0;i<ligs->len;i++)
     {
 	gtk_list_store_append(ls, &iter);
-	gtk_list_store_set(ls, &iter, 0, lig(i).name->str, -1);
+	gtk_list_store_set(ls, &iter, 0, lig(i).name, -1);
     }
 
     return GTK_TREE_MODEL(ls);
@@ -2274,9 +2389,6 @@ treeview_show_leagues_combo(void)
     gtk_combo_box_set_model(combo_leagues, model);
     g_object_unref(model);
 
-    if(gtk_combo_box_entry_get_text_column(GTK_COMBO_BOX_ENTRY(combo_leagues)) == -1)
-	gtk_combo_box_entry_set_text_column(GTK_COMBO_BOX_ENTRY(combo_leagues), 0);
-
     gtk_cell_layout_clear(GTK_CELL_LAYOUT(combo_leagues));
 
     renderer = treeview_helper_cell_renderer_text_new();
@@ -2298,18 +2410,19 @@ treeview_create_language_list(void)
 	misc_separate_strings(const_str("string_language_symbols"));
 
     gtk_list_store_append(ls, &iter);
+    /* Language is system-set (not user chosen). */
     gtk_list_store_set(ls, &iter, 0, NULL, 1, _("System"), -1);
 
     for(i=0;i<names->len;i++)
     {
 	gtk_list_store_append(ls, &iter);
-	treeview_helper_insert_icon(ls, &iter, 0, ((GString*)g_ptr_array_index(symbols, i))->str);
+	treeview_helper_insert_icon(ls, &iter, 0, (gchar*)g_ptr_array_index(symbols, i));
 	gtk_list_store_set(ls, &iter, 1, 
-			   ((GString*)g_ptr_array_index(names, i))->str, -1);
+			   (gchar*)g_ptr_array_index(names, i), -1);
     }
 
-    free_g_string_array(&names);
-    free_g_string_array(&symbols);
+    free_gchar_array(&names);
+    free_gchar_array(&symbols);
 
     return GTK_TREE_MODEL(ls);
 }
@@ -2320,15 +2433,10 @@ treeview_show_language_combo(void)
 {
     GtkTreeModel *model = treeview_create_language_list();
     GtkComboBox *combo_languages =
-	GTK_COMBO_BOX(lookup_widget(window.options, "comboboxentry_languages"));
+	GTK_COMBO_BOX(lookup_widget(window.options, "combobox_languages"));
     GtkCellRenderer *renderer = NULL;
     gint idx = language_get_current_index();
 
-    gtk_combo_box_set_model(combo_languages, model);
-    g_object_unref(model);
-
-    gtk_combo_box_entry_set_text_column(GTK_COMBO_BOX_ENTRY(combo_languages), 1);
-
     gtk_cell_layout_clear(GTK_CELL_LAYOUT(combo_languages));
 
     renderer = gtk_cell_renderer_pixbuf_new();
@@ -2339,5 +2447,85 @@ treeview_show_language_combo(void)
     gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(combo_languages), renderer, FALSE);
     gtk_cell_layout_set_attributes(GTK_CELL_LAYOUT(combo_languages), renderer, "text", 1, NULL);
 
+    gtk_combo_box_set_model(combo_languages, model);
+    g_object_unref(model);
+
     gtk_combo_box_set_active(GTK_COMBO_BOX(combo_languages), idx + 1);
 }
+
+GtkTreeModel*
+treeview_create_sponsors(const GArray *sponsors)
+{
+    gint i;
+    GtkListStore *ls = gtk_list_store_new(3, G_TYPE_STRING, G_TYPE_INT, 
+					  G_TYPE_INT);
+    GtkTreeIter iter;
+
+    for(i=0;i<sponsors->len;i++)
+    {
+	gtk_list_store_append(ls, &iter);
+	gtk_list_store_set(ls, &iter, 
+			   0, g_array_index(sponsors, UserSponsor, i).name->str,
+			   1, g_array_index(sponsors, UserSponsor, i).contract / 4,
+			   2, g_array_index(sponsors, UserSponsor, i).benefit,
+			   -1);
+    }
+
+    for(i=1;i<3;i++)
+	gtk_tree_sortable_set_sort_func(GTK_TREE_SORTABLE(ls), i,
+					treeview_helper_int_compare, GINT_TO_POINTER(i), NULL);
+
+    return GTK_TREE_MODEL(ls);
+}
+
+void
+treeview_set_up_sponsors(GtkTreeView *treeview)
+{
+    gint i;
+    GtkTreeViewColumn   *col;
+    GtkCellRenderer     *renderer;
+    gchar *titles[3] = 
+	{_("Company"),
+	 _("Contract length\n(Months)"),
+	 _("Money / week")};
+    
+    gtk_tree_selection_set_mode(gtk_tree_view_get_selection(treeview),
+				GTK_SELECTION_BROWSE);
+    gtk_tree_view_set_rules_hint(treeview, TRUE);
+    gtk_tree_view_set_headers_visible(treeview, TRUE);
+
+    for(i=0;i<3;i++)
+    {
+	col = gtk_tree_view_column_new();
+	gtk_tree_view_column_set_title(col, titles[i]);
+	gtk_tree_view_append_column(treeview, col);	
+	renderer = treeview_helper_cell_renderer_text_new();
+	gtk_tree_view_column_pack_start(col, renderer, TRUE);
+	gtk_tree_view_column_add_attribute(col, renderer,
+					   "markup", i);
+
+	if(i > 0)
+	{
+	    g_object_set(renderer, "xalign", 0.5, NULL);
+	    gtk_tree_view_column_set_sort_column_id(col, i);
+	}
+    }
+}
+
+/** Show a list of sponsor offers. */
+void
+treeview_show_sponsors(const GArray *sponsors)
+{
+    GtkTreeView *treeview = GTK_TREE_VIEW(lookup_widget(window.sponsors,
+							"treeview_sponsors"));
+    GtkTreeModel *model = treeview_create_sponsors(sponsors);
+    GtkTreePath *path = gtk_tree_path_new_from_string("0");
+
+    treeview_helper_clear(treeview);
+    
+    treeview_set_up_sponsors(treeview);
+    gtk_tree_view_set_model(treeview, GTK_TREE_MODEL(model));
+    gtk_tree_selection_select_path(gtk_tree_view_get_selection(treeview), path);
+    g_object_unref(model);
+    gtk_tree_path_free(path);
+}
