@@ -10,6 +10,7 @@
 #include "team.h"
 #include "transfer.h"
 #include "treeview.h"
+#include "treeview2.h"
 #include "treeview_helper.h"
 #include "user.h"
 #include "variables.h"
@@ -186,8 +187,8 @@ treeview_helper_live_game_icon(gint event_type)
 	return const_app("string_live_game_event_cross_bar_icon");
     else if(event_type == LIVE_GAME_EVENT_SAVE)
 	return const_app("string_live_game_event_save_icon");
-    else if(event_type == LIVE_GAME_EVENT_MISSED)
-	return const_app("string_live_game_event_missed_icon");
+    else if(event_type == LIVE_GAME_EVENT_MISS)
+	return const_app("string_live_game_event_miss_icon");
     else if(event_type == LIVE_GAME_EVENT_FOUL)
 	return const_app("string_live_game_event_foul_icon");
     else if(event_type == LIVE_GAME_EVENT_FOUL_YELLOW)
@@ -421,7 +422,8 @@ treeview_helper_player_compare(GtkTreeModel *model,
     table would participate in an international cup and set the
     colours accordingly. */
 gboolean
-treeview_helper_get_table_element_colour_cups(const League *league, gint idx, gchar **colour_bg)
+treeview_helper_get_table_element_colour_cups(const League *league, 
+					      gint idx, gchar **colour_bg)
 {
     gint i, j, k;
     const CupRound *cup_round = NULL;
@@ -440,16 +442,82 @@ treeview_helper_get_table_element_colour_cups(const League *league, gint idx, gc
 	    {
 		cup_round = &g_array_index(cp(i).rounds, CupRound, k);
 		for(j=0;j<cup_round->choose_teams->len;j++)
-		    if(strcmp(g_array_index(cup_round->choose_teams, CupChooseTeam, j).sid->str, buf) == 0 ||
-		       strcmp(g_array_index(cup_round->choose_teams, CupChooseTeam, j).sid->str, league->sid->str) == 0)
+		    if(strcmp(g_array_index(cup_round->choose_teams, 
+					    CupChooseTeam, j).sid, buf) == 0 ||
+		       strcmp(g_array_index(cup_round->choose_teams, 
+					    CupChooseTeam, j).sid, league->sid) == 0)
 		    {
-			if((idx + 1 >= g_array_index(cup_round->choose_teams, CupChooseTeam, j).start_idx &&
-			    idx + 1 <= g_array_index(cup_round->choose_teams, CupChooseTeam, j).end_idx && 
-			    g_array_index(cup_round->choose_teams, CupChooseTeam, j).randomly) ||
-			   (idx + 1 >= g_array_index(cup_round->choose_teams, CupChooseTeam, j).start_idx &&
-			    idx + 1 < g_array_index(cup_round->choose_teams, CupChooseTeam, j).start_idx + 
-			    g_array_index(cup_round->choose_teams, CupChooseTeam, j).number_of_teams &&
-			    !g_array_index(cup_round->choose_teams, CupChooseTeam, j).randomly))
+			if((idx + 1 >= g_array_index(cup_round->choose_teams, 
+						     CupChooseTeam, j).start_idx &&
+			    idx + 1 <= g_array_index(cup_round->choose_teams, 
+						     CupChooseTeam, j).end_idx && 
+			    g_array_index(cup_round->choose_teams, 
+					  CupChooseTeam, j).randomly) ||
+			   (idx + 1 >= g_array_index(cup_round->choose_teams, 
+						     CupChooseTeam, j).start_idx &&
+			    idx + 1 < g_array_index(cup_round->choose_teams, 
+						    CupChooseTeam, j).start_idx + 
+			    g_array_index(cup_round->choose_teams, 
+					  CupChooseTeam, j).number_of_teams &&
+			    !g_array_index(cup_round->choose_teams, 
+					   CupChooseTeam, j).randomly))
+			{			    			    
+			    *colour_bg = cup_highlight_colour;
+			    return TRUE;
+			}
+		    }
+	    }
+    }
+    
+    return FALSE;
+}
+
+/** Find out whether the team given 
+    would participate in an international cup and set the
+    colour accordingly. */
+gboolean
+treeview_helper_get_table_element_colour_cups_cup(const Cup *cup, 
+						  const Team *tm, gchar **colour_bg)
+{
+    gint i, j, k, idx = -1;
+    const CupRound *cup_round = NULL;
+    gchar *cup_highlight_colour = NULL;
+    GPtrArray *teams = cup_get_teams_sorted(cup);
+
+    for(i=0;i<teams->len;i++)
+	if((Team*)g_ptr_array_index(teams, i) == tm)
+	{
+	    idx = i;
+	    break;
+	}
+
+    for(i=0;i<cps->len;i++)
+    {
+	cup_highlight_colour = cup_get_highlight_colour(&cp(i));
+
+	if(cup_highlight_colour != NULL)
+	    for(k=0;k<cp(i).rounds->len;k++)
+	    {
+		cup_round = &g_array_index(cp(i).rounds, CupRound, k);
+		for(j=0;j<cup_round->choose_teams->len;j++)
+		    if(strcmp(g_array_index(
+				  cup_round->choose_teams, CupChooseTeam, j).sid,
+			      cup->sid) == 0)
+		    {
+			if((idx + 1 >= g_array_index(cup_round->choose_teams, 
+						     CupChooseTeam, j).start_idx &&
+			    idx + 1 <= g_array_index(cup_round->choose_teams, 
+						     CupChooseTeam, j).end_idx && 
+			    g_array_index(cup_round->choose_teams, 
+					  CupChooseTeam, j).randomly) ||
+			   (idx + 1 >= g_array_index(cup_round->choose_teams, 
+						     CupChooseTeam, j).start_idx &&
+			    idx + 1 < g_array_index(cup_round->choose_teams, 
+						    CupChooseTeam, j).start_idx + 
+			    g_array_index(cup_round->choose_teams, 
+					  CupChooseTeam, j).number_of_teams &&
+			    !g_array_index(cup_round->choose_teams, 
+					   CupChooseTeam, j).randomly))
 			{			    			    
 			    *colour_bg = cup_highlight_colour;
 			    return TRUE;
@@ -466,8 +534,9 @@ treeview_helper_get_table_element_colour_cups(const League *league, gint idx, gc
     @param idx The index of the element we're looking at.
     @param user Whether to take into account user colours. */
 void
-treeview_helper_get_table_element_colours(const Table *table, gint idx, gchar **colour_fg, 
-					  gchar **colour_bg, gboolean user)
+treeview_helper_get_table_element_colours(const Table *table, gint idx, 
+					  gchar **colour_fg, gchar **colour_bg,
+					  gboolean user)
 {
     gint i;
     const TableElement *elem = &g_array_index(table->elements, TableElement, idx);
@@ -518,9 +587,16 @@ treeview_helper_get_table_element_colours(const Table *table, gint idx, gchar **
 	    fixture_get_round_robin_advance(cup_from_clid(table->clid), table->round);
 	for(i=0;i<cup_advance->len;i++)
 	    if((Team*)g_ptr_array_index(cup_advance, i) == elem->team)
-		*colour_bg = const_app("string_treeview_table_promotion");
-	
+	    {
+		*colour_bg = const_app("string_treeview_table_promotion");	
+		free_g_ptr_array(&cup_advance);
+		return;
+	    }
+
 	free_g_ptr_array(&cup_advance);
+
+	treeview_helper_get_table_element_colour_cups_cup(
+	    cup_from_clid(table->clid), elem->team, colour_bg);
     }
 }
 
@@ -591,7 +667,7 @@ treeview_helper_team_selection(GtkTreeViewColumn *col,
     gtk_tree_model_get(model, iter, column, &team_pointer, -1);
 
     if(column == 2)
-	g_object_set(renderer, "text", ((Team*)team_pointer)->name->str, NULL);
+	g_object_set(renderer, "text", ((Team*)team_pointer)->name, NULL);
     else if(column == 4)
     {
 	sprintf(buf, "%.1f", team_get_average_skill((Team*)team_pointer, FALSE));
@@ -707,6 +783,9 @@ treeview_helper_player_ext_info_to_cell(GtkTreeViewColumn *col,
 	case PLAYER_INFO_ATTRIBUTE_BANNED:
 	    treeview_helper_player_info_banned_to_cell(renderer, pl->cards);
 	    break;
+	case PLAYER_INFO_ATTRIBUTE_STREAK:
+	    treeview_helper_player_info_streak_to_cell(renderer, pl->streak);
+	    break;
 	case PLAYER_INFO_ATTRIBUTE_CAREER:
 	    treeview_helper_player_info_career_to_cell(renderer, pl);
 	    break;
@@ -776,6 +855,8 @@ treeview_helper_player_info_banned_to_cell(GtkCellRenderer *renderer, const GArr
     for(i=0;i<cards->len;i++)
 	if(g_array_index(cards, PlayerCard, i).red > 0)
 	{
+	    /* Ban info of a player in the format:
+	       'Cup/league name: Number of weeks banned' */
 	    sprintf(buf2, _("%s: %d weeks\n"),
 		    league_cup_get_name_string(g_array_index(cards, PlayerCard, i).clid),
 		    g_array_index(cards, PlayerCard, i).red);
@@ -811,6 +892,8 @@ treeview_helper_player_info_yellow_to_cell(GtkCellRenderer *renderer, const GArr
 			g_array_index(cards, PlayerCard, i).yellow, yellow_red);
 	    }
 	    else
+		/* Yellow cards of a player in a cup/league. No limit means there isn't a limit
+		   after which the player gets banned for a match automatically. */
 		sprintf(buf2, _("%s: %d (no limit)\n"),
 			league_cup_get_name_string(g_array_index(cards, PlayerCard, i).clid),
 			g_array_index(cards, PlayerCard, i).yellow);
@@ -842,6 +925,15 @@ treeview_helper_player_info_games_goals_to_cell(GtkCellRenderer *renderer, const
     g_object_set(renderer, "text", buf, NULL);
 }
 
+void
+treeview_helper_player_info_streak_to_cell(GtkCellRenderer *renderer, gint streak)
+{
+    if(streak == PLAYER_STREAK_HOT)
+	g_object_set(renderer, "text", _("The player is on a hot streak"), NULL);
+    else if(streak == PLAYER_STREAK_COLD)
+	g_object_set(renderer, "text", _("The player is on a cold streak"), NULL);
+}
+
 void
 treeview_helper_player_info_health_to_cell(GtkCellRenderer *renderer, const Player *pl)
 {
@@ -849,13 +941,14 @@ treeview_helper_player_info_health_to_cell(GtkCellRenderer *renderer, const Play
 
     if(pl->health != 0)
     {
-	sprintf(buf, "%s (expected recovery in %d weeks)",
+	sprintf(buf, _("%s (expected recovery in %d weeks)"),
 		player_injury_to_char(pl->health), pl->recovery);
 	g_object_set(renderer, "background",
 		     const_app("string_treeview_helper_color_player_injury"), NULL);
     }
     else
-	strcpy(buf, "OK");
+	/* Player health: ok. */
+	strcpy(buf, _("OK"));
 
     g_object_set(renderer, "text", buf, NULL);
 }
@@ -863,10 +956,10 @@ treeview_helper_player_info_health_to_cell(GtkCellRenderer *renderer, const Play
 /** Render a player list cell. */
 void
 treeview_helper_player_to_cell(GtkTreeViewColumn *col,
-			     GtkCellRenderer   *renderer,
-			     GtkTreeModel      *model,
-			     GtkTreeIter       *iter,
-			     gpointer           user_data)
+			       GtkCellRenderer   *renderer,
+			       GtkTreeModel      *model,
+			       GtkTreeIter       *iter,
+			       gpointer           user_data)
 {
     gint column = treeview_helper_get_col_number_column(col);
     gint attribute = GPOINTER_TO_INT(user_data);
@@ -893,10 +986,12 @@ treeview_helper_player_to_cell(GtkTreeViewColumn *col,
 	    treeview_helper_player_name_to_cell(renderer, buf, pl);
 	    break;
 	case PLAYER_LIST_ATTRIBUTE_CPOS:
-	    treeview_helper_player_pos_to_cell(renderer, buf, pl->cpos);
+	    treeview_helper_player_pos_to_cell(renderer, buf, pl, 
+					       PLAYER_LIST_ATTRIBUTE_CPOS);
 	    break;
 	case PLAYER_LIST_ATTRIBUTE_POS:
-	    treeview_helper_player_pos_to_cell(renderer, buf, pl->pos);
+	    treeview_helper_player_pos_to_cell(renderer, buf, pl,
+					       PLAYER_LIST_ATTRIBUTE_POS);
 	    break;
 	case PLAYER_LIST_ATTRIBUTE_CSKILL:
 	    treeview_helper_player_cskill_to_cell(renderer, buf, pl);
@@ -917,7 +1012,8 @@ treeview_helper_player_to_cell(GtkTreeViewColumn *col,
 	    treeview_helper_player_games_goals_to_cell(buf, pl, PLAYER_VALUE_SHOTS);
 	    break;
 	case PLAYER_LIST_ATTRIBUTE_STATUS:
-	    treeview_helper_player_status_to_cell(renderer, buf, pl);
+	    treeview_helper_player_status_to_cell(NULL, renderer, 
+						  NULL, NULL, (gpointer)pl);
 	    break;
 	case PLAYER_LIST_ATTRIBUTE_CARDS:
 	    treeview_helper_player_cards_to_cell(buf, pl);
@@ -930,23 +1026,24 @@ treeview_helper_player_to_cell(GtkTreeViewColumn *col,
 		    pl->etal[current_user.scout % 10]);
 	    break;
 	case PLAYER_LIST_ATTRIBUTE_VALUE:
-	    misc_print_grouped_int(pl->value, buf, FALSE);
+	    misc_print_grouped_int(pl->value, buf);
 	    break;
 	case PLAYER_LIST_ATTRIBUTE_WAGE:
-	    misc_print_grouped_int(pl->wage, buf, FALSE);
+	    misc_print_grouped_int(pl->wage, buf);
 	    break;
 	case PLAYER_LIST_ATTRIBUTE_CONTRACT:
 	    treeview_helper_player_contract_to_cell(renderer, buf, pl->contract);
 	    break;
 	case PLAYER_LIST_ATTRIBUTE_TEAM:
-	    sprintf(buf, "%s", pl->team->name->str);
+	    sprintf(buf, "%s", pl->team->name);
 	    break;
 	case PLAYER_LIST_ATTRIBUTE_LEAGUE_CUP:
 	    strcpy(buf, league_cup_get_name_string(pl->team->clid));
 	    break;
     }
 
-    g_object_set(renderer, "text", buf, NULL);
+    if(attribute != PLAYER_LIST_ATTRIBUTE_STATUS)
+	g_object_set(renderer, "text", buf, NULL);
 }
 
 /** Render a cell of a player name. */
@@ -957,9 +1054,10 @@ treeview_helper_player_name_to_cell(GtkCellRenderer *renderer, gchar *buf, const
     const gchar *colour_fg = const_app("string_treeview_helper_color_default_foreground"),
 	*colour_bg = const_app("string_treeview_helper_color_default_background");
 
-    strcpy(buf, pl->name->str);
+    strcpy(buf, pl->name);
     if(pl->team == current_user.tm &&
        opt_user_int("int_opt_user_penalty_shooter") == pl->id)
+	/* Penalty shooter. */
 	strcat(buf, _(" (P)"));
     
     if(off != NULL)
@@ -1059,31 +1157,96 @@ treeview_helper_player_cards_to_cell(gchar *buf, const Player *pl)
     @param buf The string the cell will contain.
     @param pl The pointer to the player. */
 void
-treeview_helper_player_status_to_cell(GtkCellRenderer *renderer, gchar *buf, const Player *pl)
+treeview_helper_player_status_to_cell(GtkTreeViewColumn *col,
+				      GtkCellRenderer   *renderer,
+				      GtkTreeModel      *model,
+				      GtkTreeIter       *iter,
+				      gpointer           user_data)
 {
-    gint ban = player_is_banned(pl);
+    const Player *pl = NULL;
+    gchar buf[SMALL];
+    gint ban = 0, column = -1;
+    GdkPixbuf *symbol = NULL;
+    gboolean render_icon = (user_data == NULL);
 
-    if(pl->health != PLAYER_INJURY_NONE)
+    if(render_icon)
     {
-	sprintf(buf, _("INJ(%d)"), pl->recovery);
-	g_object_set(renderer, "background", 
-		     const_app("string_treeview_helper_color_player_injury"), NULL);
+	column = treeview_helper_get_col_number_column(col);
+	gtk_tree_model_get(model, iter, column, &pl, -1);
+    }
+    else
+	pl = (const Player*)user_data;	
 
+    if(pl == NULL)
+    {
+	if(render_icon)
+	    g_object_set(renderer, "pixbuf", NULL, NULL);
 	return;
     }
 
-    if(ban > 0)
+    ban = player_is_banned(pl);
+    
+    if(pl->health != PLAYER_INJURY_NONE)
     {
-	sprintf(buf, _("BAN(%d)"), ban);
-	g_object_set(renderer, "background",
-		     const_app("string_treeview_helper_color_player_banned"), NULL);
+	/* Injury info. */
+	if(!render_icon)
+	{
+	    sprintf(buf, _("INJ(%d)"), pl->recovery);
+	    g_object_set(renderer, "background",
+			 const_app("string_treeview_helper_color_player_injury"), NULL);
+	}
+	else
+	    symbol =
+		treeview_helper_pixbuf_from_filename(
+		    const_app("string_treeview_helper_player_status_injury"));
+    }
+    else if(ban > 0)
+    {
+	/* Red card info (how long the player is banned). */
+	if(!render_icon)
+	{
+	    sprintf(buf, _("BAN(%d)"), ban);
+	    g_object_set(renderer, "background",
+			 const_app("string_treeview_helper_color_player_banned"), NULL);
+	}
+	else
+	    symbol =
+		treeview_helper_pixbuf_from_filename(
+		    const_app("string_treeview_helper_player_status_ban"));
     }
     else
-	strcpy(buf, _("OK"));
+    {
+	if(!render_icon)
+	    /* Player status: ok. */
+	    strcpy(buf, _("OK"));
+	else
+	{
+	    if(ban == -1)
+		symbol =
+		    treeview_helper_pixbuf_from_filename(
+			const_app("string_treeview_helper_player_status_yellow_danger"));
+	    else if(pl->streak == PLAYER_STREAK_HOT)
+		symbol =
+		    treeview_helper_pixbuf_from_filename(
+			const_app("string_treeview_helper_player_status_hot_streak"));
+	    else if(pl->streak == PLAYER_STREAK_COLD)
+		symbol =
+		    treeview_helper_pixbuf_from_filename(
+			const_app("string_treeview_helper_player_status_cold_streak"));
+	    else
+		symbol =
+		    treeview_helper_pixbuf_from_filename(
+			const_app("string_treeview_helper_player_status_ok"));
+	}
+    }    
 
-    if(ban == -1)
-	g_object_set(renderer, "background",
-		     const_app("string_treeview_helper_color_player_yellow_danger"), NULL);
+    if(render_icon)
+    {
+	g_object_set(renderer, "pixbuf", symbol, NULL);
+	treeview_helper_unref(G_OBJECT(symbol));
+    }
+    else
+	g_object_set(renderer, "text", buf, NULL);
 }
 
 /** Render a cell of player games or goals.
@@ -1141,11 +1304,16 @@ treeview_helper_player_fitness_to_cell(GtkCellRenderer *renderer, gchar *buf, gf
     @param buf The string the cell will contain.
     @param pos The position of the player. */
 void
-treeview_helper_player_pos_to_cell(GtkCellRenderer *renderer, gchar *buf, gint pos)
+treeview_helper_player_pos_to_cell(GtkCellRenderer *renderer, gchar *buf, 
+				   const Player *pl, gint type)
 {
+    gint pos = (type == PLAYER_LIST_ATTRIBUTE_POS) ?
+	pl->pos : pl->cpos;
+
     switch(pos)
     {
 	default:
+	    /* Goalie */
 	    strcpy(buf, _("G"));
 	    g_object_set(renderer, "background", 
 			 const_app("string_treeview_helper_color_player_pos_goalie_bg"),
@@ -1153,6 +1321,7 @@ treeview_helper_player_pos_to_cell(GtkCellRenderer *renderer, gchar *buf, gint p
 			 const_app("string_treeview_helper_color_player_pos_goalie_fg"), NULL);
 	    break;
 	case PLAYER_POS_DEFENDER:
+	    /* Defender */
 	    strcpy(buf, _("D"));
 	    g_object_set(renderer, "background", 
 			 const_app("string_treeview_helper_color_player_pos_defender_bg"),
@@ -1160,6 +1329,7 @@ treeview_helper_player_pos_to_cell(GtkCellRenderer *renderer, gchar *buf, gint p
 			 const_app("string_treeview_helper_color_player_pos_defender_fg"), NULL);
 	    break;
 	case PLAYER_POS_MIDFIELDER:
+	    /* Midfielder */
 	    strcpy(buf, _("M"));
 	    g_object_set(renderer, "background", 
 			 const_app("string_treeview_helper_color_player_pos_midfielder_bg"),
@@ -1167,6 +1337,7 @@ treeview_helper_player_pos_to_cell(GtkCellRenderer *renderer, gchar *buf, gint p
 			 const_app("string_treeview_helper_color_player_pos_midfielder_fg"), NULL);
 	    break;
 	case PLAYER_POS_FORWARD:
+	    /* Forward */
 	    strcpy(buf, _("F"));
 	    g_object_set(renderer, "background", 
 			 const_app("string_treeview_helper_color_player_pos_forward_bg"),
@@ -1174,6 +1345,12 @@ treeview_helper_player_pos_to_cell(GtkCellRenderer *renderer, gchar *buf, gint p
 			 const_app("string_treeview_helper_color_player_pos_forward_fg"), NULL);
 	    break;
     }
+
+    if(pl->cskill == 0)
+	g_object_set(renderer, "background", 
+		     const_app("string_treeview_helper_color_player_pos_disabled_bg"),
+		     "foreground", 
+		     const_app("string_treeview_helper_color_player_pos_disabled_fg"), NULL);
 }
 
 /** Render a cell of player cskill.
@@ -1202,34 +1379,185 @@ treeview_helper_player_cskill_to_cell(GtkCellRenderer *renderer, gchar *buf, con
 /** Render the result (team names and goals) in the live game view. */
 void
 treeview_helper_live_game_result(GtkTreeViewColumn *col,
+				 GtkCellRenderer   *renderer,
+				 GtkTreeModel      *model,
+				 GtkTreeIter       *iter,
+				 gpointer           user_data)
+{    
+    gint column = treeview_helper_get_col_number_column(col);
+    gchar  buf[SMALL];
+    gchar *data = NULL;
+
+    strcpy(buf, "");
+
+    gtk_tree_model_get(model, iter, column, &data, -1);
+    sprintf(buf, "<span %s>%s</span>", 
+	    const_app("string_treeview_helper_live_game_result_attributes"),
+	    data);
+
+    g_object_set(renderer, "markup", buf, NULL);
+}
+
+gint
+treeview_helper_int_compare(GtkTreeModel *model,
+			    GtkTreeIter *a,
+			    GtkTreeIter *b,
+			    gpointer user_data)
+{
+    gint column = GPOINTER_TO_INT(user_data);
+    gint value1, value2;
+
+    gtk_tree_model_get(model, a, column, &value1, -1);
+    gtk_tree_model_get(model, b, column, &value2, -1);
+
+    return misc_int_compare(value1, value2);
+}
+
+void
+treeview_helper_mm_teams(GtkTreeViewColumn *col,
+			 GtkCellRenderer   *renderer,
+			 GtkTreeModel      *model,
+			 GtkTreeIter       *iter,
+			 gpointer           user_data)
+{
+    gint column = treeview_helper_get_col_number_column(col);
+    const MemMatch *mm = NULL;
+    const gchar *text = NULL;
+
+    g_object_set(renderer, "background",
+		 const_app("string_treeview_helper_color_default_background"), NULL);
+    g_object_set(renderer, "foreground",
+		 const_app("string_treeview_helper_color_default_foreground"), NULL);
+    g_object_set(renderer, "text", "", NULL);
+
+    if(column == 1 || column == 2)
+    {
+	gtk_tree_model_get(model, iter, column, &mm, -1);
+	if(mm != NULL)
+	{
+	    if(column == 1)
+		g_object_set(renderer, "text", mm->lg.team_names[mm->user_team], NULL);
+	    else
+		g_object_set(renderer, "text", mm->lg.team_names[!mm->user_team], NULL);
+
+	    if(!mm->neutral &&
+	       ((column == 1 && mm->user_team == 1) ||
+		(column == 2 && mm->user_team == 0)))
+		g_object_set(renderer, "background",
+			     const_app("string_treeview_live_game_commentary_away_bg"), NULL);
+	}
+	else if(column == 1)
+	{
+	    g_object_set(renderer, "text", _("ADD LAST MATCH"), NULL);
+	    g_object_set(renderer, "background", 
+			 const_app("string_treeview_helper_mmatches_add_bg"), NULL);
+	    g_object_set(renderer, "foreground", 
+			 const_app("string_treeview_helper_mmatches_add_fg"), NULL);
+	}
+    }
+    else if(column >= TREEVIEW_MMATCH_COL_REPLAY)
+    {
+	gtk_tree_model_get(model, iter, column, &text, -1);
+	g_object_set(renderer, "text", text, NULL);
+    }
+    
+    gtk_tree_model_get(model, iter, 1, &mm, -1);
+
+    if(mm != NULL)
+    {
+	if(column == TREEVIEW_MMATCH_COL_REPLAY)
+	{
+	    g_object_set(renderer, "background",
+			 const_app("string_treeview_helper_mmatches_replay_bg"), NULL);
+	    g_object_set(renderer, "foreground",
+			 const_app("string_treeview_helper_mmatches_replay_fg"), NULL);
+	}
+	else if(column == TREEVIEW_MMATCH_COL_REMOVE)
+	{
+	    g_object_set(renderer, "background",
+			 const_app("string_treeview_helper_mmatches_remove_bg"), NULL);
+	    g_object_set(renderer, "foreground",
+			 const_app("string_treeview_helper_mmatches_remove_fg"), NULL);
+	}
+	else if(column == TREEVIEW_MMATCH_COL_EXPORT)
+	{
+	    g_object_set(renderer, "background",
+			 const_app("string_treeview_helper_mmatches_export_bg"), NULL);
+	    g_object_set(renderer, "foreground",
+			 const_app("string_treeview_helper_mmatches_export_fg"), NULL);
+	}
+    }
+}
+
+void
+treeview_helper_season_results(GtkTreeViewColumn *col,
 			       GtkCellRenderer   *renderer,
 			       GtkTreeModel      *model,
 			       GtkTreeIter       *iter,
 			       gpointer           user_data)
 {
-    
     gint column = treeview_helper_get_col_number_column(col);
-    gchar  buf[SMALL];    
-    Fixture *fix = NULL;
-    LiveGameUnit *unit = NULL;
+    gchar buf[SMALL];
+    const Fixture *fix = NULL;
+    gint user_idx = -1;
 
-    strcpy(buf, "");
+    gtk_tree_model_get(model, iter, column, &fix, -1);
 
-    if(column == 0 || column == 2)
+    if(fix == NULL)
+	return;
+
+    user_idx = (fix->team_ids[1] == current_user.tm->id);
+
+    if(column == 3)
     {
-	gtk_tree_model_get(model, iter, column, &fix, -1);
-	sprintf(buf, "<span %s>%s</span>", 
-		const_app("string_treeview_helper_live_game_result_attributes"),
-		fix->teams[column == 2]->name->str);
+	strcpy(buf, fix->teams[!user_idx]->name);
+	if(user_idx == 1)
+	    g_object_set(renderer, "background",
+			 const_app("string_treeview_live_game_commentary_away_bg"), NULL);
+	else
+	    g_object_set(renderer, "background",
+			 const_app("string_treeview_helper_color_default_background"), NULL);
     }
-    else if(column == 1)
+    else if(column == 4)
     {
-	gtk_tree_model_get(model, iter, column, &unit, -1);
-	sprintf(buf, "<span %s>%d : %d</span>", 
-		const_app("string_treeview_helper_live_game_result_attributes"),
-		unit->result[0], unit->result[1]);
-
+	if(fix->attendance == -1)
+	{
+	    strcpy(buf, "");
+	    g_object_set(renderer, "background",
+			 const_app("string_treeview_helper_color_default_background"), NULL);
+	}
+	else if((fix->result[0][2] + fix->result[1][2] > 0 && 
+		 fix->result[user_idx][2] > fix->result[!user_idx][2]) ||
+		(fix->result[0][2] + fix->result[1][2] == 0 && 
+		 math_sum_int_array(fix->result[user_idx], 2) >
+		 math_sum_int_array(fix->result[!user_idx], 2)))
+	{
+	    /* Won. */
+	    strcpy(buf, _("W"));
+	    g_object_set(renderer, "background",
+			 const_app("string_treeview_helper_season_results_win_bg"), NULL);
+	}
+	else if((fix->result[0][2] + fix->result[1][2] > 0 && 
+		 fix->result[user_idx][2] < fix->result[!user_idx][2]) ||
+		(fix->result[0][2] + fix->result[1][2] == 0 && 
+		 math_sum_int_array(fix->result[user_idx], 2) <
+		 math_sum_int_array(fix->result[!user_idx], 2)))
+	{
+	    /* Lost. */
+	    strcpy(buf, _("L"));
+	    g_object_set(renderer, "background",
+			 const_app("string_treeview_helper_season_results_loss_bg"), NULL);
+	}
+	else
+	{
+	    /* Draw. */
+	    strcpy(buf, _("Dw"));
+	    g_object_set(renderer, "background",
+			 const_app("string_treeview_helper_season_results_draw_bg"), NULL);
+	}
     }
+    else if(column == 5)
+	fixture_result_to_buf(fix, buf, (user_idx == 1));
 
-    g_object_set(renderer, "markup", buf, NULL);
+    g_object_set(renderer, "text", buf, NULL);
 }
