@@ -1,4 +1,6 @@
 #include "cup.h"
+#include "file.h"
+#include "finance.h"
 #include "fixture.h"
 #include "free.h"
 #include "game_gui.h"
@@ -7,13 +9,17 @@
 #include "main.h"
 #include "maths.h"
 #include "misc.h"
+#include "name.h"
 #include "option.h"
 #include "player.h"
+#include "support.h"
 #include "team.h"
 #include "transfer.h"
 #include "treeview.h"
 #include "user.h"
 #include "window.h"
+#include "xml_mmatches.h"
+#include "youth_academy.h"
 
 /** Create a new user with default values. */
 User
@@ -21,7 +27,7 @@ user_new(void)
 {
     User new;
 
-    new.name = g_string_new("NONAME");
+    new.name = g_strdup("NONAME");
     new.tm = NULL;
     new.team_id = -1;
     
@@ -32,6 +38,16 @@ user_new(void)
     new.options.list = NULL;
     new.options.datalist = NULL;
 
+    new.sponsor.name = g_string_new("");
+    new.sponsor.contract = new.sponsor.benefit = -1;
+
+    new.youth_academy.players = g_array_new(FALSE, FALSE, sizeof(Player));
+    new.youth_academy.pos_pref = PLAYER_POS_ANY;
+    new.youth_academy.coach = QUALITY_AVERAGE;
+
+    new.mmatches_file = NULL;
+    new.mmatches = g_array_new(FALSE, FALSE, sizeof(MemMatch));
+
     return new;
 }
 
@@ -55,10 +71,10 @@ user_set_up_team_new_game(User *user)
 	while(team_is_user(&g_array_index(lig(user->scout).teams, Team, rndom)) != -1)
 	    rndom = math_rndi(0, lig(user->scout).teams->len - 1);
       
-	sprintf(buf, "%s", g_array_index(lig(user->scout).teams, Team, rndom).name->str);
-	g_string_printf(g_array_index(lig(user->scout).teams, Team, rndom).name, "%s",
-			user->tm->name->str);
-	g_string_printf(user->tm->name, "%s", buf);
+	sprintf(buf, "%s", g_array_index(lig(user->scout).teams, Team, rndom).name);
+	misc_string_assign(&g_array_index(lig(user->scout).teams, Team, rndom).name,
+			   user->tm->name);
+	misc_string_assign(&user->tm->name, buf);
 
 	user->tm = &g_array_index(lig(user->scout).teams, Team, rndom);
 	user->team_id = g_array_index(lig(user->scout).teams, Team, rndom).id;
@@ -70,7 +86,7 @@ user_set_up_team_new_game(User *user)
 }
 
 /** Set up finances, remove some players etc. for a new user team.
-    @param user The user whose team we set up. */
+    @par am user The user whose team we set up. */
 void
 user_set_up_team(User *user)
 {
@@ -78,18 +94,31 @@ user_set_up_team(User *user)
 
     for(i=PLAYER_POS_DEFENDER; i<=PLAYER_POS_FORWARD; i++)
 	for(j=user->tm->players->len - 1; j > 10; j--)
-	    if(g_array_index(user->tm->players, Player, j).pos == i)
+	    if(g_array_index(user->tm->players, Player, j).pos == i &&
+	       g_array_index(user->tm->players, Player, j).recovery != 1)
 	    {
 		player_remove_from_team(user->tm, j);
 		break;
 	    }
 
+    for(i=0;i<user->tm->players->len;i++)
+	g_array_index(user->tm->players, Player, i).recovery = 0;
+
     user->scout = user->physio = QUALITY_AVERAGE;
+
+    if(sett_int("int_opt_disable_transfers"))
+	user->physio = QUALITY_GOOD;
     
     user->tm->style = 0;
     
     user_set_up_finances(user);
     user_set_up_counters(user);
+
+    if(!sett_int("int_opt_disable_ya"))
+	youth_academy_new(user);
+
+    user->counters[COUNT_USER_NEW_SPONSOR] = 
+	(sett_int("int_opt_disable_finances")) ? -5 : 1;
 }
 
 
@@ -164,7 +193,7 @@ user_set_player_list_attributes(const User *user, PlayerListAttribute *attribute
     sprintf(prefix, "int_opt_user_pl%d_att", list_number);
 
     for(i=0;i<user->options.list->len;i++)
-	if(g_str_has_prefix(g_array_index(user->options.list, Option, i).name->str, prefix))
+	if(g_str_has_prefix(g_array_index(user->options.list, Option, i).name, prefix))
 	{
 	    attribute->on_off[cnt] = g_array_index(user->options.list, Option, i).value;
 	    cnt++;
@@ -221,7 +250,7 @@ user_from_team(const Team *tm)
 	    return &usr(i);
 
     g_warning("User going with team %s not found.\n",
-	      tm->name->str);
+	      tm->name);
 
     main_exit_program(EXIT_POINTER_NOT_FOUND, NULL);
 
@@ -233,7 +262,6 @@ user_from_team(const Team *tm)
 void
 user_job_offer(User *user)
 {
-    gchar buf[SMALL];
     Team *new_team = NULL;
 
     if(math_rnd(0, 1) > const_float("float_user_success_counter_check") ||
@@ -243,8 +271,8 @@ user_job_offer(User *user)
     if(user->counters[COUNT_USER_SUCCESS] < -(gfloat)const_int("int_user_success_offer_limit") * 0.9 &&
        !user->counters[COUNT_USER_WARNING])
     {
-	sprintf(buf, _("The owners of %s are not satisfied with the recent performance of the team. There are rumours they're looking for a new manager."), user->tm->name->str);
-	user_event_add(user, EVENT_TYPE_WARNING, -1, -1, NULL, buf);
+	user_event_add(user, EVENT_TYPE_WARNING, -1, -1, NULL,
+		       _("The owners of %s are not satisfied with the recent performance of the team. There are rumours they're looking for a new manager."), user->tm->name);
 	user->counters[COUNT_USER_WARNING] = 1;
 	return;
     }
@@ -342,18 +370,25 @@ user_event_new(void)
 /** Add an event with the specified values to the event array of the user. */
 void
 user_event_add(User *user, gint type, gint value1, gint value2, 
-	       gpointer value_pointer, gchar *value_string)
+	       gpointer value_pointer, gchar *format, ...)
 {
     Event new = user_event_new();
-
+    gchar text[SMALL];
+    va_list args;
+     
     new.user = user;
     new.type = type;
     new.value1 = value1;
     new.value2 = value2;
     new.value_pointer = value_pointer;
     
-    if(value_string != NULL)
-	new.value_string = g_string_new(value_string);
+    if(format != NULL)
+    {
+	va_start (args, format);
+	g_vsprintf(text, format, args);
+	va_end (args);
+	new.value_string = g_strdup(text);
+    }
     else
 	new.value_string = NULL;    
 
@@ -391,12 +426,11 @@ user_event_show_next(void)
 	    g_warning("user_event_show_next: unknown event type %d\n", event->type);
 	    break;
 	case EVENT_TYPE_PLAYER_LEFT:
-	    sprintf(buf, _("%s has left your team because his contract expired."),
-		    event->value_string->str);
-	    game_gui_show_warning(buf);
+	    game_gui_show_warning(_("%s has left your team because his contract expired."),
+		    event->value_string);
 	    break;
 	case EVENT_TYPE_WARNING:
-	    game_gui_show_warning(event->value_string->str);
+	    game_gui_show_warning(event->value_string);
 	    break;
 	case EVENT_TYPE_FIRE_FINANCE:
 	    stat2 = STATUS_JOB_OFFER_FIRE_FINANCE;
@@ -415,59 +449,53 @@ user_event_show_next(void)
 	    break;
 	case EVENT_TYPE_OVERDRAW:
 	    if(event->value1 == 1)
-		sprintf(buf, _("You have overdrawn your bank account. The team owners give you %d weeks to get above your drawing credit limit."), const_int("int_finance_overdraw_positive"));
+		game_gui_show_warning(_("You have overdrawn your bank account. The team owners give you %d weeks to get above your drawing credit limit."), const_int("int_finance_overdraw_positive"));
 	    else
-		sprintf(buf, _("You have overdrawn your bank account once again. Bear in mind that after the fourth time you get fired.\nThe team owners give you %d weeks to get above your drawing credit limit."), const_int("int_finance_overdraw_positive"));
-	    game_gui_show_warning(buf);
+		game_gui_show_warning(_("You have overdrawn your bank account once again. Bear in mind that after the fourth time you get fired.\nThe team owners give you %d weeks to get above your drawing credit limit."), const_int("int_finance_overdraw_positive"));
 	    break;
 	case EVENT_TYPE_TRANSFER_OFFER_USER:
-	    sprintf(buf, _("Have a look at the transfer list, there's an offer for %s."),
-		    event->value_string->str);
-	    game_gui_show_warning(buf);
+	    game_gui_show_warning(_("Have a look at the transfer list, there's an offer for %s."),
+		    event->value_string);
 	    break;
 	case EVENT_TYPE_TRANSFER_OFFER_CPU:
-	    sprintf(buf, _("Your offer for %s has been accepted. If you still want to buy him, go to the transfer list and left click on the player."),
-		    event->value_string->str);
-	    game_gui_show_warning(buf);
+	    game_gui_show_warning(_("Your offer for %s has been accepted. If you still want to buy him, go to the transfer list and left click on the player."),
+		    event->value_string);
 	    break;
 	case EVENT_TYPE_TRANSFER_OFFER_REJECTED_BETTER_OFFER:
-	    misc_print_grouped_int(event->value1, buf2, FALSE);
-	    misc_print_grouped_int(event->value2, buf3, FALSE);	    
-	    sprintf(buf, _("The owners of %s have rejected your offer (%s / %s) for %s. There was a better offer for the player than yours."), ((Team*)event->value_pointer)->name->str, buf2, buf3, event->value_string->str);
-	    game_gui_show_warning(buf);
+	    misc_print_grouped_int(event->value1, buf2);
+	    misc_print_grouped_int(event->value2, buf3);	    
+	    game_gui_show_warning(_("The owners of %s have rejected your offer (%s / %s) for %s. There was a better offer for the player than yours."), ((Team*)event->value_pointer)->name, buf2, buf3, event->value_string);
 	    break;
 	case EVENT_TYPE_TRANSFER_OFFER_REJECTED_FEE_WAGE:
-	    misc_print_grouped_int(event->value1, buf2, FALSE);
-	    misc_print_grouped_int(event->value2, buf3, FALSE);	    
-	    sprintf(buf, _("The owners of %s have rejected your offer (%s / %s) for %s. Neither the fee nor the wage you offered were acceptable, they say."), ((Team*)event->value_pointer)->name->str, buf2, buf3, event->value_string->str);
-	    game_gui_show_warning(buf);
+	    misc_print_grouped_int(event->value1, buf2);
+	    misc_print_grouped_int(event->value2, buf3);	    
+	    game_gui_show_warning(_("The owners of %s have rejected your offer (%s / %s) for %s. Neither the fee nor the wage you offered were acceptable, they say."), ((Team*)event->value_pointer)->name, buf2, buf3, event->value_string);
 	    break;
 	case EVENT_TYPE_TRANSFER_OFFER_REJECTED_FEE:
-	    misc_print_grouped_int(event->value1, buf2, FALSE);
-	    misc_print_grouped_int(event->value2, buf3, FALSE);	    
-	    sprintf(buf, _("The owners of %s have rejected your offer (%s / %s) for %s. The team owners weren't satisfied with the fee you offered."), ((Team*)event->value_pointer)->name->str, buf2, buf3, event->value_string->str);
-	    game_gui_show_warning(buf);
+	    misc_print_grouped_int(event->value1, buf2);
+	    misc_print_grouped_int(event->value2, buf3);	    
+	    game_gui_show_warning(_("The owners of %s have rejected your offer (%s / %s) for %s. The team owners weren't satisfied with the fee you offered."), ((Team*)event->value_pointer)->name, buf2, buf3, event->value_string);
 	    break;
 	case EVENT_TYPE_TRANSFER_OFFER_REJECTED_WAGE:
-	    misc_print_grouped_int(event->value1, buf2, FALSE);
-	    misc_print_grouped_int(event->value2, buf3, FALSE);	    
-	    sprintf(buf, _("%s of %s has rejected your offer (%s / %s). He wasn't satisfied with the wage you offered."),
-		    event->value_string->str,
-		    ((Team*)event->value_pointer)->name->str, buf2, buf3);
-	    game_gui_show_warning(buf);
+	    misc_print_grouped_int(event->value1, buf2);
+	    misc_print_grouped_int(event->value2, buf3);	    
+	    /* A player from a team has rejected a transfer offer. */
+	    game_gui_show_warning(_("%s of %s has rejected your offer (%s / %s). He wasn't satisfied with the wage you offered."),
+		    event->value_string,
+		    ((Team*)event->value_pointer)->name, buf2, buf3);
 	    break;
 	case EVENT_TYPE_TRANSFER_OFFER_MONEY:
-	    sprintf(buf, _("You didn't have enough money to buy %s from %s."),
-		    event->value_string->str, ((Team*)event->value_pointer)->name->str);
-	    game_gui_show_warning(buf);
+	    /* Buy a player from a team. */
+	    game_gui_show_warning(_("You didn't have enough money to buy %s from %s."),
+		    event->value_string, ((Team*)event->value_pointer)->name);
 	    break;
 	case EVENT_TYPE_TRANSFER_OFFER_ROSTER:
-	    sprintf(buf, _("Your roster is full. You couldn't buy %s from %s."),
-		    event->value_string->str, ((Team*)event->value_pointer)->name->str);
-	    game_gui_show_warning(buf);
+	    /* Buy a player from a team. */
+	    game_gui_show_warning(_("Your roster is full. You couldn't buy %s from %s."),
+		    event->value_string, ((Team*)event->value_pointer)->name);
 	    break;
 	case EVENT_TYPE_PLAYER_CAREER_STOP:
-	    sprintf(buf, _("%s's injury was so severe that he can't play football on a professional level anymore. He leaves your team."), player_of_id_team(event->user->tm, event->value1)->name->str);
+	    sprintf(buf, _("%s's injury was so severe that he can't play football on a professional level anymore. He leaves your team."), player_of_id_team(event->user->tm, event->value1)->name);
 	    if(event->user->tm->players->len < 12)
 	    {
 		strcat(buf, _(" Fortunately he's got a cousin who can help your team out."));
@@ -493,6 +521,11 @@ user_change_team(User *user, Team *tm)
     user->team_id = tm->id;
 
     user_set_up_team(user);
+
+    user->counters[COUNT_USER_NEW_SPONSOR] = 0;
+    g_string_free(user->sponsor.name, TRUE);
+    user->sponsor = user_get_sponsor(&current_user);    
+
     for(i=user->events->len - 1; i >= 0; i--)
 	user_event_remove(user, i);    
 }
@@ -629,12 +662,12 @@ user_history_add(User *user, gint type, gint team_id,
     
     if(replace)
     {
-	g_string_printf(his->value_string, "%s", string);
+	misc_string_assign(&his->value_string, string);
 	g_array_sort(user->history, (GCompareFunc)user_history_compare);
     }
     else
     {
-	his->value_string = g_string_new(string);
+	his->value_string = g_strdup(string);
 	g_array_prepend_val(user->history, *his);
     }
 }
@@ -652,58 +685,69 @@ user_history_to_string(const UserHistory *history, gchar *buf)
 	    g_warning("user_history_to_string: unknown history type %d.\n", history->type);
 	    strcpy(buf, "FIXME!!!");
 	case USER_HISTORY_START_GAME:
+	    /* Buy a team in a league. */
 	    sprintf(buf, _("You start the game with %s in the %s."),
-		    team_of_id(history->team_id)->name->str,
+		    team_of_id(history->team_id)->name,
 		    league_cup_get_name_string(history->value1));
 	    break;
     	case USER_HISTORY_FIRE_FINANCES:
+	    /* Team fires, team in a league. */
 	    sprintf(buf, _("%s fires you because of financial mismanagement.\nYou find a new job with %s in the %s."),
-		    team_of_id(history->team_id)->name->str,
-		    team_of_id(history->value1)->name->str,
+		    team_of_id(history->team_id)->name,
+		    team_of_id(history->value1)->name,
 		    league_cup_get_name_string(history->value2));
 	    break;
     	case  USER_HISTORY_FIRE_FAILURE:
+	    /* Team fires, team in a league. */
 	    sprintf(buf, _("%s fires you because of unsuccessfulness.\nYou find a new job with %s in the %s."),
-		    team_of_id(history->team_id)->name->str,
-		    team_of_id(history->value1)->name->str,
+		    team_of_id(history->team_id)->name,
+		    team_of_id(history->value1)->name,
 		    league_cup_get_name_string(history->value2));
 	    break;
     	case  USER_HISTORY_JOB_OFFER_ACCEPTED:
+	    /* Team in a league. Leave team. */
 	    sprintf(buf, _("%s offer you a job in the %s.\nYou accept the challenge and leave %s."),
-		    team_of_id(history->value1)->name->str,
+		    team_of_id(history->value1)->name,
 		    league_cup_get_name_string(history->value2),
-		    team_of_id(history->team_id)->name->str);
+		    team_of_id(history->team_id)->name);
 	    break;
     	case  USER_HISTORY_END_SEASON:
+	    /* League name. */
 	    sprintf(buf, _("You finish the season in the %s on rank %d."),
 		    league_cup_get_name_string(history->value1),
 		    history->value2);
 	    break;
     	case  USER_HISTORY_PROMOTED:
+	    /* League name. */
 	    sprintf(buf, _("You get promoted to the %s."),
 		    league_cup_get_name_string(history->value1));
 	    break;	    
     	case  USER_HISTORY_RELEGATED:
+	    /* League name. */
 	    sprintf(buf, _("You get relegated to the %s."),
 		    league_cup_get_name_string(history->value1));
 	    break;	    
     	case  USER_HISTORY_WIN_FINAL:
+	    /* Cup name, team name. */
 	    sprintf(buf, _("You win the %s final against %s."),
 		    league_cup_get_name_string(history->value1),
-		    history->value_string->str);
+		    history->value_string);
 	    break;
     	case  USER_HISTORY_LOSE_FINAL:
+	    /* Cup name, team name. */
 	    sprintf(buf, _("You lose in the %s final against %s."),
 		    league_cup_get_name_string(history->value1),
-		    history->value_string->str);
+		    history->value_string);
 	    break;
     	case USER_HISTORY_REACH_CUP_ROUND:	    
 	    cup_get_round_name(cup_from_clid(history->value1), history->value2, buf2);
+	    /* Cup round name (e.g. Last 32), number, cup name. */
 	    sprintf(buf, _("You reach the %s (round %d) of the %s."), buf2,
 		    history->value2 + 1,
 		    league_cup_get_name_string(history->value1));
 	    break;
     	case USER_HISTORY_CHAMPION:	    
+	    /* League name. */
 	    sprintf(buf, _("You are champion of the %s!"),
 		    league_cup_get_name_string(history->value1));
 	    break;
@@ -723,3 +767,309 @@ query_user_no_turn(void)
 
     return FALSE;
 }
+
+/** Get a random sponsor for the user. */
+UserSponsor
+user_get_sponsor(const User *user)
+{
+    UserSponsor new;
+    gint suc_factor = (current_user.counters[COUNT_USER_SUCCESS] < 0) ? -1 : 1;
+    gchar *names[18] =
+	/* Company name. */
+	{_(" Systems"),
+	 /* Company name. */
+	 _(" Communications"),
+	 /* Company name. */
+	 _(" Holdings"),
+	 /* Company name. */
+	 _(" Industries"),
+	 /* Company name. */
+	 _(" Company"),
+	 /* Company name. */
+	 _(" Telecommunications"),
+	 /* Company name. */
+	 _(" Labs"),
+	 /* Company name. */
+	 _(" Technologies"),
+	 /* Company name. */
+	 _(" Chemicals"),
+	 /* Company name. */
+	 _(" Energy"),
+	 /* Company name. */
+	 _(" Bank"),
+	 /* Company name. */
+	 _(" Products"),
+	 /* Company name. */
+	 _(" Software"),
+	 /* Company name. */
+	 _(" Scientific"),
+	 /* Company name. */
+	 _(" Financial"),
+	 /* Company name. */
+	 _(" Petroleum"),
+	 /* Company name. */
+	 _(" Restaurants"),
+	 /* Company name. */
+	 _(" Data Systems")};
+    gchar *short_names[7] =
+	/* Company short name, leading to things like 'Marshall Data Systems Ltd.' */
+	{_(" Ltd."),
+	 /* Company short name. */
+	 _(" Assoc."),
+	 /* Company short name. */
+	 _(" Co."),
+	 /* Company short name. */
+	 _(" Ent."),
+	 /* Company short name. Copy the '&amp;'. */
+	 _(" &amp; Co."),
+	 /* Company short name. */
+	 _(" Corp."),
+	 /* Company short name. */
+	 _(" Group")};
+    gchar *name_add[3] =
+	/* Company addition, leading to 'Marshall & Sons Petroleum Co.'. Copy the '&amp;'. */
+	{_(" &amp; Sons"),
+	 /* Company addition. Copy the '&amp;'. */
+	 _(" &amp; Daughters"),
+	/* Company addition. */
+	 _(" Bros.")};
+   
+    new.name = g_string_new(name_get_random_last_name(name_get_list_from_sid(user->tm->names_file)));
+    if(math_rnd(0, 1) < 0.2)
+	g_string_append(new.name, name_add[math_rndi(0, 2)]);
+    g_string_append(new.name, names[math_rndi(0, 17)]);
+    if(math_rnd(0, 1) < 0.7)
+	g_string_append(new.name, short_names[math_rndi(0, 6)]);
+
+    new.contract = 4 * math_rndi(9, 36);
+    new.benefit = (gint)rint((((const_float("float_sponsor_wage_percentage_upper") -
+			       const_float("float_sponsor_wage_percentage_lower")) /
+			      (2 * (gfloat)const_int("int_user_success_offer_limit"))) *
+			     (gfloat)current_user.counters[COUNT_USER_SUCCESS]
+			     + ((const_float("float_sponsor_wage_percentage_upper") -
+				 const_float("float_sponsor_wage_percentage_lower")) / 2)) *
+			     (1 + suc_factor * 
+			      ((gfloat)new.contract / 52 - 1) * const_float("float_sponsor_contract_length_factor")) *
+			     (finance_wage_unit(current_user.tm) * current_user.tm->players->len));
+    new.benefit = math_round_integer(new.benefit, 2);
+
+    return new;
+}
+
+/** Show a list of sponsors for the user to choose from. */
+void
+user_show_sponsors(void)
+{
+    gint i, num_offers;
+    GArray *sponsors = g_array_new(FALSE, FALSE, sizeof(UserSponsor));
+    UserSponsor sponsor;
+
+    if(ABS(current_user.counters[COUNT_USER_SUCCESS]) >= 
+       const_int("int_user_success_offer_limit"))
+	num_offers = (current_user.counters[COUNT_USER_SUCCESS] < 0) ?
+	    const_int("int_sponsor_offers_lower") :
+	    const_int("int_sponsor_offers_upper");
+    else
+	num_offers = (gint)rint(((gfloat)(const_int("int_sponsor_offers_upper") -
+					  const_int("int_sponsor_offers_lower")) /
+				 (2 * (gfloat)const_int("int_user_success_offer_limit"))) *
+				(gfloat)current_user.counters[COUNT_USER_SUCCESS]
+				+ ((gfloat)(const_int("int_sponsor_offers_upper") -
+					    const_int("int_sponsor_offers_lower")) / 2) + 1);
+
+    num_offers += math_rndi(-1, 1);
+    if(num_offers <= 0)
+	num_offers = 1;
+
+    for(i=0;i<num_offers;i++)
+    {
+	sponsor = user_get_sponsor(&current_user);
+	g_array_append_val(sponsors, sponsor);
+    }
+    
+    window_create(WINDOW_SPONSORS);
+
+    treeview_show_sponsors(sponsors);
+
+    for(i=0;i<num_offers;i++)
+	g_string_free(g_array_index(sponsors, UserSponsor, i).name, TRUE);
+    g_array_free(sponsors, TRUE);
+}
+
+/** The current sponsor offers to continue. */
+void
+user_show_sponsor_continue(void)
+{
+    GArray *sponsors = g_array_new(FALSE, FALSE, sizeof(UserSponsor));
+    UserSponsor sponsor;
+    gchar buf[SMALL];
+
+    sponsor = user_get_sponsor(&current_user);
+
+    g_string_printf(sponsor.name, current_user.sponsor.name->str);
+    
+    g_array_append_val(sponsors, sponsor);
+
+    window_create(WINDOW_SPONSORS);
+
+    sprintf(buf, _("Your current sponsor is satisfied with your results and would like to renew the contract. Currently they're paying you %d a week."), current_user.sponsor.benefit);
+    gtk_label_set_text(GTK_LABEL(lookup_widget(window.sponsors, "label_sponsors")), buf);
+
+    treeview_show_sponsors(sponsors);
+
+    g_string_free(g_array_index(sponsors, UserSponsor, 0).name, TRUE);
+    g_array_free(sponsors, TRUE);
+
+    stat1 = STATUS_SPONSOR_CONTINUE;
+}
+
+/** Load a MM file. */
+void
+user_mm_load_file(const gchar *filename, GArray *mmatches)
+{
+    gchar prefix[SMALL], filename_local[SMALL],
+	matches_file[SMALL];
+    GArray *mm_array = (mmatches == NULL) ?
+	current_user.mmatches : mmatches;
+    
+    strcpy(filename_local, filename);
+    strncpy(prefix, filename_local, strlen(filename_local) - 8);
+    prefix[strlen(filename_local) - 8] = '\0';
+
+    sprintf(matches_file, "%s___mmatches", prefix);
+
+    file_decompress(filename_local);
+    
+    if(mmatches == NULL)
+	free_mmatches(&mm_array, TRUE);
+
+    xml_mmatches_read(matches_file, mm_array);
+
+    strcat(prefix, "___*");
+    file_remove_files(prefix);
+
+    if(mmatches == NULL)
+	misc_string_assign(&current_user.mmatches_file, filename_local);
+}
+
+/** Add the last match to the MM file.
+    @param load_file Whether to load the matches from the file first.
+    @param save_file Whether to save the MM array
+    to file afterwards. */
+void
+user_mm_add_last_match(gboolean load_file, gboolean save_file)
+{
+    MemMatch new;
+    const Fixture *fix = current_user.live_game.fix;
+    gchar buf[SMALL];
+
+    if(fix->clid < ID_CUP_START)
+	new.competition_name = g_string_new(league_cup_get_name_string(fix->clid));
+    else
+    {
+	cup_round_name(fix, buf);
+	new.competition_name = g_string_new("");
+	g_string_printf(new.competition_name, "%s %s", 
+			league_cup_get_name_string(fix->clid), buf);
+    }
+
+    new.country_name = g_strdup(country.name);
+    new.neutral = !(fix->home_advantage);
+    new.user_team = (fix->team_ids[0] != current_user.team_id);
+    new.lg = current_user.live_game;
+    
+    /** This will tell the free function NOT
+	to free the user live game. */
+    current_user.live_game.started_game = -1;
+
+    current_user.live_game.units = 
+	g_array_new(FALSE, FALSE, sizeof(LiveGameUnit));
+
+    if(load_file)
+	user_mm_load_file(current_user.mmatches_file, NULL);
+
+    g_array_append_val(current_user.mmatches, new);
+    game_gui_print_message(_("Memorable match added."));
+
+    if(save_file)
+	user_mm_save_file(current_user.mmatches_file,
+			  current_user.mmatches);
+}
+
+/** Save an MM file. */
+void
+user_mm_save_file(const gchar *filename, const GArray *mmatches)
+{
+    gchar prefix[SMALL];
+
+    strncpy(prefix, filename, strlen(filename) - 8);
+    prefix[strlen(filename) - 8] = '\0';
+
+    strcat(prefix, "___");
+
+    xml_mmatches_write(prefix, mmatches);
+
+    file_compress_files(filename, prefix);
+}
+
+/** Set an appropriate filename for the memorable
+    matches file. */
+void
+user_mm_set_filename(const gchar *filename, gchar *dest)
+{
+    gchar buf[SMALL];
+
+    if(g_str_has_suffix(filename, ".bmm.zip"))
+    {
+	if(dest == NULL)
+	    misc_string_assign(&current_user.mmatches_file, filename);
+	else
+	    strcpy(dest, filename);
+	return;
+    }
+
+    strcpy(buf, filename);
+
+    while(g_str_has_suffix(buf, ".bmm") ||
+	  g_str_has_suffix(buf, ".zip"))
+	buf[strlen(buf) - 4] = '\0';
+
+    if(dest == NULL)
+    {
+	strcat(buf, ".bmm.zip");
+	misc_string_assign(&current_user.mmatches_file, buf);
+    }
+    else
+	sprintf(dest, "%s.bmm.zip", buf);
+}
+
+/** Import an MM file into the user's MMs. */
+void
+user_mm_import_file(const gchar *filename)
+{
+    GArray *mm_array = g_array_new(FALSE, FALSE, sizeof(MemMatch));
+    gint i;
+
+    user_mm_load_file(filename, mm_array);
+
+    for(i=0;i<mm_array->len;i++)
+	g_array_append_val(current_user.mmatches, g_array_index(mm_array, MemMatch, i));
+
+    g_array_free(mm_array, TRUE);
+}
+
+/** Export a match to a file. */
+void
+user_mm_export_file(const gchar *filename)
+{
+    gchar buf[SMALL];
+    GArray *mmatches = g_array_new(FALSE, FALSE, sizeof(MemMatch));
+
+    user_mm_set_filename(filename, buf);
+
+    g_array_append_val(mmatches, g_array_index(current_user.mmatches, MemMatch, stat4));
+
+    user_mm_save_file(buf, mmatches);
+
+    g_array_free(mmatches, TRUE);
+}
