@@ -7,13 +7,15 @@
 #include "option_struct.h"
 #include "player_struct.h"
 #include "team_struct.h"
+#include "youth_academy_struct.h"
 
 /** Indices for the money_in array. */
 enum MonIn
 {
     MON_IN_PRIZE = 0,
     MON_IN_TICKET,
-    MON_IN_TRANSFERS,
+    MON_IN_SPONSOR,
+    MON_IN_TRANSFERS,    
     MON_IN_END
 };
 
@@ -23,11 +25,13 @@ enum MonOut
     MON_OUT_WAGE = 0,
     MON_OUT_PHYSIO,
     MON_OUT_SCOUT,
+    MON_OUT_YC,
+    MON_OUT_YA,
     MON_OUT_JOURNEY,
-    MON_OUT_STADIUM_IMPROVEMENT,
-    MON_OUT_STADIUM_BILLS,
     MON_OUT_COMPENSATIONS,
     MON_OUT_TRANSFERS,
+    MON_OUT_STADIUM_IMPROVEMENT,
+    MON_OUT_STADIUM_BILLS,
     MON_OUT_END
 };
 
@@ -46,14 +50,74 @@ enum CounterValue
     COUNT_USER_STADIUM_SAFETY, /**< Counter for increasing stadium safety. */
     COUNT_USER_SHOW_RES, /**< Whether the latest result is shown when the main window gets refreshed. */
     COUNT_USER_TOOK_TURN, /**< Whether the user took his turn in a week round. */
+    COUNT_USER_NEW_SPONSOR, /**< A new sponsor offer has to be shown. */
     COUNT_USER_END
 };
 
+/** User-related things that get recorded. */
+enum UserHistoryType
+{
+    USER_HISTORY_START_GAME = 0,
+    USER_HISTORY_FIRE_FINANCES,
+    USER_HISTORY_FIRE_FAILURE,
+    USER_HISTORY_JOB_OFFER_ACCEPTED,
+    USER_HISTORY_END_SEASON,
+    USER_HISTORY_PROMOTED,
+    USER_HISTORY_RELEGATED,
+    USER_HISTORY_WIN_FINAL,
+    USER_HISTORY_LOSE_FINAL,
+    USER_HISTORY_REACH_CUP_ROUND,
+    USER_HISTORY_CHAMPION,
+    USER_HISTORY_END
+};
+
+/** A memorable match (a recording of a live game) of a user. */
+typedef struct
+{
+    /** Name of the country the user was playing with. */
+    gchar *country_name;
+    /** The name of the competition, including
+	the cup round name. */
+    GString *competition_name;
+    /** Whether the match was on neutral ground. Only
+	relevant for MM list display. */
+    gboolean neutral;
+    /** 0 or 1, indicating which team was the user's. */
+    gint user_team;
+    /** The recording. */
+    LiveGame lg;
+} MemMatch;
+
+/** A structure holding an element of a user's history,
+    e.g. the event of being fired. */
+typedef struct
+{
+    /** When the event happened. */
+    gint season, week;
+    
+    /** The type (see #UserHistoryType) and team
+	of the user. */
+    gint type, team_id,
+	/** These can hold various information like
+	    team or league/cup ids. */
+	value1, value2;
+    gchar *value_string;
+
+} UserHistory;
+
+/** A user sponsor. */
+typedef struct
+{
+    GString *name;
+    gint benefit; /**< Money per week. */
+    gint contract; /**< Contract length in weeks. */
+} UserSponsor;
+
 /** A structure representing a human player. */
 typedef struct
 {
     /** Username. */
-    GString *name;
+    gchar *name;
     /** The team the user manages. */
     Team *tm;
     /** The team id (needed when the team pointer gets invalid). */
@@ -77,6 +141,14 @@ typedef struct
     gint scout, physio;
     /** The variable for the latest user live game. @see #Game */
     LiveGame live_game;
+    /** Sponsor of the user. */
+    UserSponsor sponsor;
+    /** Youth academy of the user. */
+    YouthAcademy youth_academy;
+    /** The currently used MM file. */
+    gchar *mmatches_file;
+    /** The array of MMs. */
+    GArray *mmatches;
 } User;
 
 enum EventType
@@ -113,43 +185,8 @@ typedef struct
     /** A pointer for different purposes. */
     gpointer value_pointer;
     /** A string for different purposes. */
-    GString *value_string;
-
+    gchar *value_string;
 } Event;
 
 
-/** User-related things that get recorded. */
-enum UserHistoryType
-{
-    USER_HISTORY_START_GAME = 0,
-    USER_HISTORY_FIRE_FINANCES,
-    USER_HISTORY_FIRE_FAILURE,
-    USER_HISTORY_JOB_OFFER_ACCEPTED,
-    USER_HISTORY_END_SEASON,
-    USER_HISTORY_PROMOTED,
-    USER_HISTORY_RELEGATED,
-    USER_HISTORY_WIN_FINAL,
-    USER_HISTORY_LOSE_FINAL,
-    USER_HISTORY_REACH_CUP_ROUND,
-    USER_HISTORY_CHAMPION,
-    USER_HISTORY_END
-};
-
-/** A structure holding an element of a user's history,
-    e.g. the event of being fired. */
-typedef struct
-{
-    /** When the event happened. */
-    gint season, week;
-    
-    /** The type (see #UserHistoryType) and team
-	of the user. */
-    gint type, team_id,
-	/** These can hold various information like
-	    team or league/cup ids. */
-	value1, value2;
-    GString *value_string;
-
-} UserHistory;
-
 #endif
