@@ -18,10 +18,28 @@
 #include "support.h"
 #include "transfer.h"
 #include "treeview.h"
+#include "treeview2.h"
 #include "treeview_helper.h"
 #include "user.h"
 #include "window.h"
 
+/** Show the window with the news. */
+/* void */
+/* window_show_news(void) */
+/* { */
+/*     GtkNotebook *nb = NULL; */
+
+/*     window_create(WINDOW_HELP); */
+
+/*     nb = GTK_NOTEBOOK(lookup_widget(window.help, "notebook1")); */
+
+/*     gtk_notebook_remove_page(nb, 0); */
+/*     gtk_notebook_remove_page(nb, -1); */
+
+/*     gtk_label_set_text(GTK_LABEL(lookup_widget(window.help, "label_contributors")), */
+/* 		       _("News")); */
+/* } */
+
 /** Show the help/about window. 
     @param page Which notebook page to display. */
 void
@@ -33,7 +51,7 @@ window_show_help(gint page)
 
     if(help_file == NULL)
     {
-	game_gui_show_warning("Didn't find file 'bygfoot_help'.");
+	game_gui_show_warning(_("Didn't find file 'bygfoot_help'."));
 	return;
     }
 
@@ -76,11 +94,6 @@ window_show_startup(void)
     if(country_files->len == 0)
 	main_exit_program(EXIT_NO_COUNTRY_FILES, "Didn't find any country definition files in the support directories.");
 
-    model = treeview_create_country_list(country_files);
-    gtk_combo_box_set_model(GTK_COMBO_BOX(combo_country), model);
-    g_object_unref(model);
-
-    gtk_combo_box_entry_set_text_column(GTK_COMBO_BOX_ENTRY(combo_country), 1);
     gtk_cell_layout_clear(GTK_CELL_LAYOUT(combo_country));
 
     renderer = gtk_cell_renderer_pixbuf_new();
@@ -91,9 +104,13 @@ window_show_startup(void)
     gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(combo_country), renderer, TRUE);
     gtk_cell_layout_set_attributes(GTK_CELL_LAYOUT(combo_country), renderer, "text", 1, NULL);
 
+    model = treeview_create_country_list(country_files);
+    gtk_combo_box_set_model(GTK_COMBO_BOX(combo_country), model);
+    g_object_unref(model);
+
     gtk_combo_box_set_active(GTK_COMBO_BOX(combo_country), 0);
 
-    free_g_string_array(&country_files);
+    free_gchar_array(&country_files);
 }
 
 /** Show the file selection window. */
@@ -103,38 +120,107 @@ window_show_file_sel(void)
     gchar buf[SMALL];
     const gchar *home = g_get_home_dir();
     gchar *filename = NULL;
+    GtkFileFilter *filter;
+    gboolean mm_file_exists = FALSE;
 
     window_create(WINDOW_FILE_CHOOSER);
 
-    if(stat1 == STATUS_SAVE_GAME)
+    if(stat5 != STATUS_LOAD_GAME &&
+       stat5 != STATUS_LOAD_GAME_TEAM_SELECTION)
 	gtk_file_chooser_set_action(GTK_FILE_CHOOSER(window.file_chooser),
 				    GTK_FILE_CHOOSER_ACTION_SAVE);
     else
 	gtk_file_chooser_set_action(GTK_FILE_CHOOSER(window.file_chooser),
 				    GTK_FILE_CHOOSER_ACTION_OPEN);
 
-    if(strlen(save_file->str) > 0)
-	gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(window.file_chooser),
-				      save_file->str);
+    filter = gtk_file_filter_new();
+    if(stat5 == STATUS_SAVE_GAME || 
+       stat5 == STATUS_LOAD_GAME ||
+       stat5 == STATUS_LOAD_GAME_TEAM_SELECTION)
+    {
+	gtk_file_filter_set_name(filter, _("Bygfoot Save Files"));
+	gtk_file_filter_add_pattern(filter, "*.zip");
+	gtk_file_filter_add_pattern(filter, "last_save");
+    }
     else
     {
-	sprintf(buf, "%s/%s/saves/.", home, HOMEDIRNAME);
-	gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(window.file_chooser), buf);
+	gtk_file_filter_set_name(filter, _("Bygfoot Memorable Matches"));
+	gtk_file_filter_add_pattern(filter, "*.bmm.zip");
+    }
+    gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(window.file_chooser), filter);
+
+    filter = gtk_file_filter_new ();
+    gtk_file_filter_set_name(filter, _("All Files"));
+    gtk_file_filter_add_pattern(filter, "*");
+    gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(window.file_chooser), filter);
+
+    if((stat5 == STATUS_SAVE_GAME || 
+	stat5 == STATUS_LOAD_GAME ||
+	stat5 == STATUS_LOAD_GAME_TEAM_SELECTION) &&
+       save_file != NULL)
+	gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(window.file_chooser),
+				      save_file);
+    else if(users->len > 0 && current_user.mmatches_file != NULL)
+	gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(window.file_chooser),
+				      current_user.mmatches_file);
+    else
+    {
+        if(os_is_unix)
+            sprintf(buf, "%s%s%s%ssaves", home, G_DIR_SEPARATOR_S, 
+		    HOMEDIRNAME, G_DIR_SEPARATOR_S);
+        else
+        {
+            gchar *pwd = g_get_current_dir();
+            sprintf(buf, "%s%ssaves", pwd, G_DIR_SEPARATOR_S);
+            g_free(pwd);
+	}
+
+	gtk_file_chooser_set_current_folder(
+	    GTK_FILE_CHOOSER(window.file_chooser), buf);
     }
 
     if(gtk_dialog_run(GTK_DIALOG(window.file_chooser)) == GTK_RESPONSE_OK)
     {
 	filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(window.file_chooser));
 
-	if(stat1 == STATUS_LOAD_GAME)
-	    load_save_load_game(filename);
-	else if(stat1 == STATUS_LOAD_GAME_TEAM_SELECTION)
+	if(stat5 == STATUS_LOAD_GAME)
+	    load_save_load_game(filename, FALSE);
+	else if(stat5 == STATUS_LOAD_GAME_TEAM_SELECTION)
 	    misc_callback_startup_load(filename);
-	else if(stat1 == STATUS_SAVE_GAME)
+	else if(stat5 == STATUS_SAVE_GAME)
 	    load_save_save_game(filename);
+	else if(stat5 == STATUS_SELECT_MM_FILE_LOAD)
+	{
+	    mm_file_exists = g_file_test(filename, G_FILE_TEST_EXISTS);
+	    if((mm_file_exists && g_str_has_suffix(filename, ".bmm.zip")) ||
+	       !mm_file_exists)
+	    {
+		if(mm_file_exists)
+		    user_mm_load_file(filename, NULL);
+		else
+		    user_mm_set_filename(filename, NULL);
+		window_show_mmatches();
+	    }
+	    else
+		game_gui_show_warning(_("Not a valid Bygfoot Memorable Matches filename."));
+	}
+	else if(stat5 == STATUS_SELECT_MM_FILE_ADD)
+	{
+	    user_mm_set_filename(filename, NULL);
+	    mm_file_exists = g_file_test(current_user.mmatches_file,
+					 G_FILE_TEST_EXISTS);
+	    user_mm_add_last_match(mm_file_exists, TRUE);
+	}
+	else if(stat5 == STATUS_SELECT_MM_FILE_IMPORT)
+	{
+	    user_mm_import_file(filename);
+	    window_show_mmatches();
+	}
+	else if(stat5 == STATUS_SELECT_MM_FILE_EXPORT)
+	    user_mm_export_file(filename);
 
-	if(stat1 == STATUS_LOAD_GAME ||
-	   stat1 == STATUS_LOAD_GAME_TEAM_SELECTION)
+	if(stat5 == STATUS_LOAD_GAME ||
+	   stat5 == STATUS_LOAD_GAME_TEAM_SELECTION)
 	{
 	    cur_user = 0;
 	    on_button_back_to_main_clicked(NULL, NULL);
@@ -147,6 +233,18 @@ window_show_file_sel(void)
     window_destroy(&window.file_chooser, FALSE);
 }
 
+/** Show window with memorable matches list. */
+void
+window_show_mmatches(void)
+{
+    if(window.mmatches == NULL)
+	window_create(WINDOW_MMATCHES);
+    treeview2_show_mmatches();
+
+    gtk_entry_set_text(GTK_ENTRY(lookup_widget(window.mmatches, "entry_mm_file")),
+		       current_user.mmatches_file);
+}
+
 /**  Show the options window. */
 void
 window_show_options(void)
@@ -167,14 +265,19 @@ window_show_menu_player(GdkEvent *event)
 
     gtk_menu_popup(GTK_MENU(window.menu_player), NULL, NULL, NULL, NULL, 
 		   ((GdkEventButton*)event)->button, gdk_event_get_time(event));
+}
 
-    if(week >= transfer_get_deadline())
-    {
-	gtk_widget_set_sensitive(lookup_widget(window.menu_player,
-					       "player_menu_put_on_transfer_list"), FALSE);
-	gtk_widget_set_sensitive(lookup_widget(window.menu_player,
-					       "player_menu_remove_from_transfer_list"), FALSE);
-    }
+/** Show the youth academy context menu, triggered by 'event'. */
+void
+window_show_menu_youth(GdkEvent *event)
+{
+    if(window.menu_youth != NULL)
+	window_destroy(&window.menu_youth, FALSE);
+
+    window.menu_youth = create_menu_youth();
+
+    gtk_menu_popup(GTK_MENU(window.menu_youth), NULL, NULL, NULL, NULL, 
+		   ((GdkEventButton*)event)->button, gdk_event_get_time(event));
 }
 
 /** Show the digits window with the labels and values set 
@@ -195,9 +298,12 @@ window_show_digits(const gchar *text_main, const gchar* text1, gint value1,
     spinbutton1 = GTK_SPIN_BUTTON(lookup_widget(window.digits, "spinbutton1"));
     spinbutton2 = GTK_SPIN_BUTTON(lookup_widget(window.digits, "spinbutton2"));
 
-    if(stat0 == STATUS_GET_LOAN ||
-       stat0 == STATUS_PAY_LOAN)
+    if(stat1 == STATUS_GET_LOAN ||
+       stat1 == STATUS_PAY_LOAN)
 	gtk_spin_button_set_range(spinbutton1, (gdouble)1, (gdouble)value1);
+    else if(stat1 == STATUS_SET_YA_PERCENTAGE)
+	gtk_spin_button_set_range(spinbutton2, (gdouble)0,
+				  (gdouble)const_int("int_youth_academy_max_percentage"));
 
     gtk_spin_button_set_value(spinbutton1, (gdouble)value1);
     gtk_spin_button_set_value(spinbutton2, (gdouble)value2);
@@ -222,7 +328,8 @@ window_show_stadium(void)
     gchar buf[SMALL];
     const Team *tm = current_user.tm;
     GtkLabel *label_capacity,
-	*label_stadium_status, *label_average_attendance;
+	*label_stadium_status, *label_average_attendance,
+	*label_name;
     GtkProgressBar *progressbar_safety,
 	*progressbar_average_attendance;
     gfloat average_attendance_perc = 0;
@@ -232,6 +339,7 @@ window_show_stadium(void)
     label_capacity = GTK_LABEL(lookup_widget(window.stadium, "label_capacity"));
     label_stadium_status = GTK_LABEL(lookup_widget(window.stadium, "label_stadium_status"));
     label_average_attendance = GTK_LABEL(lookup_widget(window.stadium, "label_average_attendance"));
+    label_name = GTK_LABEL(lookup_widget(window.stadium, "label_stadium_name"));
     
     progressbar_safety = GTK_PROGRESS_BAR(lookup_widget(window.stadium, "progressbar_safety"));
     progressbar_average_attendance = GTK_PROGRESS_BAR(lookup_widget(window.stadium, "progressbar_average_attendance"));
@@ -239,6 +347,10 @@ window_show_stadium(void)
     gui_label_set_text_from_int(label_capacity, tm->stadium.capacity, FALSE);
     gui_label_set_text_from_int(label_average_attendance, tm->stadium.average_attendance, FALSE);
     
+    if(tm->stadium.name != NULL)
+	gtk_label_set_text(label_name, tm->stadium.name);
+    else
+	gtk_widget_hide(GTK_WIDGET(label_name));
 
     if(tm->stadium.games > 0)
 	average_attendance_perc =
@@ -296,17 +408,101 @@ window_show_transfer_dialog(const gchar *text)
     gtk_label_set_text(GTK_LABEL(lookup_widget(window.transfer_dialog, "label_transfer_dialog")), text);
 }
 
-/** Set the spinbuttons in the live window
-    to the correct values. */
+/** Set up the spinbuttons and boost/style images in the
+    live window. */
 void
-window_live_set_spinbuttons(void)
+window_live_set_up(void)
 {
-    gtk_spin_button_set_value(
-	GTK_SPIN_BUTTON(lookup_widget(window.live, "spinbutton_speed")),
-	(gfloat)option_int("int_opt_user_live_game_speed", &usr(stat2).options));
+    GtkSpinButton *sb_speed = 
+	GTK_SPIN_BUTTON(lookup_widget(window.live, "spinbutton_speed"));
+    gfloat user_option = (gfloat)option_int("int_opt_user_live_game_speed", 
+					    &usr(stat2).options);
+    
+    gtk_spin_button_set_range(
+	sb_speed, 0, -rint((gfloat)(const_int("int_game_gui_live_game_speed_max") - 10) /
+			   (gfloat)(const_int("int_game_gui_live_game_speed_grad"))));
+    
+    gtk_spin_button_set_value(sb_speed, user_option);
+    
     gtk_spin_button_set_value(
 	GTK_SPIN_BUTTON(lookup_widget(window.live, "spinbutton_verbosity")),
 	(gfloat)option_int("int_opt_user_live_game_verbosity", &usr(stat2).options));
+
+    game_gui_write_meters(usr(stat2).tm);
+}
+
+/** Save main window size and position into a file.*/
+void
+window_main_save_geometry(void)
+{
+    gchar filename[SMALL];
+    const gchar *home = g_get_home_dir();
+    gchar *pwd = g_get_current_dir();
+    FILE *fil = NULL;
+    gint width, height, pos_x, pos_y, paned_pos;
+    
+    if(os_is_unix)
+	sprintf(filename, "%s%s%s%swindow_settings",
+		home, G_DIR_SEPARATOR_S, HOMEDIRNAME, G_DIR_SEPARATOR_S);
+    else
+	sprintf(filename, "%s%swindow_settings",
+		pwd, G_DIR_SEPARATOR_S);
+
+    g_free(pwd);
+
+    if(window.main != NULL && file_my_fopen(filename, "w", &fil, FALSE))
+    {
+	gtk_window_get_size(GTK_WINDOW(window.main), &width, &height);
+	gtk_window_get_position(GTK_WINDOW(window.main), &pos_x, &pos_y);
+	paned_pos = gtk_paned_get_position(
+	    GTK_PANED(lookup_widget(window.main, "hpaned2")));
+
+	fprintf(fil, "int_window_settings_width\t%d\n", width);
+	fprintf(fil, "int_window_settings_height\t%d\n", height);
+	fprintf(fil, "int_window_settings_pos_x\t%d\n", pos_x);
+	fprintf(fil, "int_window_settings_pos_y\t%d\n", pos_y);
+	fprintf(fil, "int_window_settings_paned_pos\t%d\n", paned_pos);
+    
+	fclose(fil);
+    }
+}
+
+/** Set the main window geometry according to the file
+    settings. */
+void
+window_main_load_geometry(void)
+{
+    gchar filename[SMALL];
+    const gchar *home = g_get_home_dir();
+    gchar *pwd = g_get_current_dir();
+    OptionList optionlist;
+    
+    if(os_is_unix)
+	sprintf(filename, "%s%s%s%swindow_settings",
+		home, G_DIR_SEPARATOR_S, HOMEDIRNAME, G_DIR_SEPARATOR_S);
+    else
+	sprintf(filename, "%s%swindow_settings",
+		pwd, G_DIR_SEPARATOR_S);
+
+    g_free(pwd);
+
+    if(g_file_test(filename, G_FILE_TEST_EXISTS))
+    {
+	optionlist.list = NULL;
+	optionlist.datalist = NULL;
+	file_load_opt_file(filename, &optionlist);
+
+	gtk_window_resize(GTK_WINDOW(window.main),
+			  option_int("int_window_settings_width", &optionlist),
+			  option_int("int_window_settings_height", &optionlist));
+	gtk_window_move(GTK_WINDOW(window.main),
+			option_int("int_window_settings_pos_x", &optionlist),
+			option_int("int_window_settings_pos_y", &optionlist));
+	gtk_paned_set_position(GTK_PANED(lookup_widget(window.main, "hpaned2")),
+			       option_int("int_window_settings_paned_pos", &optionlist));
+
+	free_option_list(&optionlist, FALSE);
+    }
 }
 
 /** Create and show a window. Which one depends on the argument.
@@ -336,8 +532,8 @@ window_create(gint window_type)
 	    {
 		window.main = create_main_window();
 		wind = window.main;
-		sprintf(buf, _("Welcome to Bygfoot %s"), VERS);
-		game_gui_print_message(buf);
+		window_main_load_geometry();
+		game_gui_print_message(_("Welcome to Bygfoot %s"), VERS);
 		sprintf(buf, "Bygfoot Football Manager %s", VERS);
 	    }
 	    else
@@ -365,7 +561,7 @@ window_create(gint window_type)
 	    }
 	    strcpy(buf, league_cup_get_name_string(((LiveGame*)statp)->fix->clid));
 	    wind = window.live;
-	    window_live_set_spinbuttons();
+	    window_live_set_up();
 	    break;
 	case WINDOW_WARNING:
 	    if(window.warning != NULL)
@@ -373,7 +569,7 @@ window_create(gint window_type)
 	    else
 		window.warning = create_window_warning();
 	    wind = window.warning;
-	    strcpy(buf, "Erm...");
+	    strcpy(buf, _("Erm..."));
 	    break;
 	case WINDOW_PROGRESS:
 	    if(window.progress != NULL)
@@ -436,7 +632,7 @@ window_create(gint window_type)
 		window.options = create_window_options();
 	    }
 	    wind = window.options;
-	    strcpy(buf, "Options");
+	    strcpy(buf, _("Options"));
 	    break;
 	case WINDOW_FONT_SEL:
 	    if(window.font_sel != NULL)
@@ -444,7 +640,7 @@ window_create(gint window_type)
 	    else
 		window.font_sel = create_window_font_sel();
 	    wind = window.font_sel;
-	    strcpy(buf, "Select font");
+	    strcpy(buf, _("Select font"));
 	    break;
 	case WINDOW_FILE_CHOOSER:
 	    if(window.file_chooser != NULL)
@@ -459,7 +655,7 @@ window_create(gint window_type)
 	    else
 		window.contract = create_window_contract();
 	    wind = window.contract;
-	    strcpy(buf, "Contract offer");
+	    strcpy(buf, _("Contract offer"));
 	    break;
 	case WINDOW_USER_MANAGEMENT:
 	    if(window.user_management != NULL)
@@ -467,7 +663,7 @@ window_create(gint window_type)
 	    else
 		window.user_management = create_window_user_management();
 	    wind = window.user_management;
-	    strcpy(buf, "User management");
+	    strcpy(buf, _("User management"));
 	    break;
 	case WINDOW_DEBUG:
 	    if(window.wdebug != NULL)
@@ -490,7 +686,26 @@ window_create(gint window_type)
 	    else
 		window.transfer_dialog = create_window_transfer_dialog();
 	    wind = window.transfer_dialog;
-	    strcpy(buf, "Transfer offer");
+	    strcpy(buf, _("Transfer offer"));
+	    break;
+	case WINDOW_SPONSORS:
+	    if(window.sponsors != NULL)
+		g_warning("window_create: called on already existing window\n");
+	    else
+		window.sponsors = create_window_sponsors();
+	    wind = window.sponsors;
+	    strcpy(buf, _("Sponsorship offers"));
+	    break;
+	case WINDOW_MMATCHES:
+	    if(window.mmatches != NULL)
+		g_warning("window_create: called on already existing window\n");
+	    else
+	    {
+		window.mmatches = create_window_mmatches();
+		popups_active++;
+	    }
+	    wind = window.mmatches;
+	    strcpy(buf, _("Memorable matches"));
 	    break;
     }
 
