@@ -24,6 +24,8 @@ enum Windows
     WINDOW_DEBUG,
     WINDOW_HELP,
     WINDOW_TRANSFER_DIALOG,
+    WINDOW_SPONSORS,
+    WINDOW_MMATCHES,
     WINDOW_END
 };
 
@@ -59,9 +61,24 @@ void
 window_show_menu_player(GdkEvent *event);
 
 void
-window_live_set_spinbuttons(void);
+window_show_menu_youth(GdkEvent *event);
+
+void
+window_live_set_up(void);
 
 void
 window_show_transfer_dialog(const gchar *text);
 
+void
+window_show_news(void);
+
+void
+window_show_mmatches(void);
+
+void
+window_main_save_geometry(void);
+
+void
+window_main_load_geometry(void);
+
 #endif
