@@ -21,16 +21,6 @@
 #include "xml_loadsave_transfers.h"
 #include "xml_loadsave_users.h"
 
-gint
-xml_get_tag_from_name(const gchar *element_name)
-{
-    gchar buf[SMALL];
-
-    misc_truncate_string(element_name, buf, -1);
-
-    return (gint)g_ascii_strtod(buf, NULL);
-}
-
 void
 xml_load_users(const gchar *dirname, const gchar *basename)
 {
@@ -64,15 +54,15 @@ xml_load_leagues(const gchar *dirname, const gchar *basename)
     free_leagues_array(&ligs, TRUE);
     
     for(i=0;i<dir_contents->len;i++)
-	if(!g_str_has_suffix(((GString*)g_ptr_array_index(dir_contents, i))->str, "_table.xml") && 
-	   !g_str_has_suffix(((GString*)g_ptr_array_index(dir_contents, i))->str, "_fixtures.xml") &&
-	   !g_str_has_suffix(((GString*)g_ptr_array_index(dir_contents, i))->str, "_teams.xml") &&
-	   !g_str_has_suffix(((GString*)g_ptr_array_index(dir_contents, i))->str, "_stat.xml") &&
-	   !query_misc_string_contains(((GString*)g_ptr_array_index(dir_contents, i))->str, "_promcup"))
-	    xml_load_league(dirname, ((GString*)g_ptr_array_index(dir_contents, i))->str,
+	if(!g_str_has_suffix((gchar*)g_ptr_array_index(dir_contents, i), "_table.xml") && 
+	   !g_str_has_suffix((gchar*)g_ptr_array_index(dir_contents, i), "_fixtures.xml") &&
+	   !g_str_has_suffix((gchar*)g_ptr_array_index(dir_contents, i), "_teams.xml") &&
+	   !g_str_has_suffix((gchar*)g_ptr_array_index(dir_contents, i), "_stat.xml") &&
+	   !g_strrstr((gchar*)g_ptr_array_index(dir_contents, i), "_promcup"))
+	    xml_load_league(dirname, (gchar*)g_ptr_array_index(dir_contents, i),
 			    dir_contents);
     
-    free_g_string_array(&dir_contents);
+    free_gchar_array(&dir_contents);
 }
 
 void
@@ -87,12 +77,15 @@ xml_load_league(const gchar *dirname, const gchar *basename, const GPtrArray *di
 
     g_array_append_val(ligs, new);
 
-    sprintf(buf, "Loading league: %s",
-	    new.name->str);
+    sprintf(buf, _("Loading league: %s"),
+	    new.name);
     gui_show_progress(
 	gtk_progress_bar_get_fraction(
 	    GTK_PROGRESS_BAR(lookup_widget(window.progress, "progressbar"))), buf);
 
+    if(debug > 80)
+	printf("%s\n", buf);
+
     sprintf(buf, "%s/%s_teams.xml", dirname, prefix);
     xml_loadsave_teams_read(buf, lig(ligs->len - 1).teams);
 
@@ -123,19 +116,19 @@ xml_load_cups(const gchar *dirname, const gchar *basename)
     
     for(i=0;i<dir_contents->len;i++)
     {
-	if(!query_misc_string_contains(((GString*)g_ptr_array_index(dir_contents, i))->str, "_table") &&
-	   !g_str_has_suffix(((GString*)g_ptr_array_index(dir_contents, i))->str, "_fixtures.xml") &&
-	   !g_str_has_suffix(((GString*)g_ptr_array_index(dir_contents, i))->str, "_teams.xml"))
+	if(!g_strrstr((gchar*)g_ptr_array_index(dir_contents, i), "_table") &&
+	   !g_str_has_suffix((gchar*)g_ptr_array_index(dir_contents, i), "_fixtures.xml") &&
+	   !g_str_has_suffix((gchar*)g_ptr_array_index(dir_contents, i), "_teams.xml"))
 	{
 	    new_cup = cup_new(FALSE);
 	    g_array_append_val(cps, new_cup);
 	    xml_load_cup(&g_array_index(cps, Cup, cps->len - 1), dirname,
-			 ((GString*)g_ptr_array_index(dir_contents, i))->str,
+			 (gchar*)g_ptr_array_index(dir_contents, i),
 			 dir_contents);
 	}
     }
     
-    free_g_string_array(&dir_contents);
+    free_gchar_array(&dir_contents);
 }
 
 void
@@ -147,12 +140,15 @@ xml_load_cup(Cup *cup, const gchar *dirname, const gchar *basename, const GPtrAr
     sprintf(buf, "%s/%s", dirname, basename);
     xml_loadsave_cup_read(buf, cup);
 
-    sprintf(buf, "Loading cup: %s",
-	    cup->name->str);
+    sprintf(buf, _("Loading cup: %s"),
+	    cup->name);
     gui_show_progress(
 	gtk_progress_bar_get_fraction(
 	    GTK_PROGRESS_BAR(lookup_widget(window.progress, "progressbar"))), buf);
 
+    if(debug > 80)
+	printf("%s\n", buf);
+
     sprintf(buf, "%s/%s_fixtures.xml", dirname, prefix);
     xml_loadsave_fixtures_read(buf, cup->fixtures);
 
@@ -171,3 +167,12 @@ xml_load_transfers(const gchar *dirname, const gchar *basename)
 
     xml_loadsave_transfers_read(buf);
 }
+
+void
+xml_write_string(FILE *fil, const gchar *string, gint tag, const gchar* indent)
+{
+    if(string == NULL)
+	return;
+
+    fprintf(fil, "%s<_%d>%s</_%d>\n", indent, tag, string, tag);
+}
